/* brinkmateState.h
 */
#ifndef OSL_BRINKMATE_BRINKMATESTATE_H
#define OSL_BRINKMATE_BRINKMATESTATE_H

#include "osl/checkmate/dualCheckmateSearcher.h"
#include "osl/state/hashEffectState.h"
#include "osl/apply_move/doUndoMoveStack.h"
#include "osl/container/moveStack.h"
#include "osl/stl/stack.h"

namespace osl
{
  namespace brinkmate
  {
    class BrinkmateState
    {
    public:
      typedef DualCheckmateSearcher<> checkmate_t;
    private:
      HashEffectState m_state;
      checkmate_t *checkmator;	// acquaintance
      DoUndoMoveStack<HashEffectState> move_stack;
      stack<int> defense_check;
      MoveStack moves;
      PathEncoding path;
    public:
      BrinkmateState();
      void init(const HashEffectState&);
      void init(checkmate_t&);
      
      void push(Move, bool is_defense_check);
      void pop();

      const checkmate::ProofDisproof testThreatmateState(int node);
      const checkmate::ProofDisproof testWinningState(int node);
      bool isEffectiveDefense(Move);
      bool isThreatmateState(int node);
      bool isWinningState(int node);
      void setEmergency() { move_stack.setEmergency(); }

      int depth() const { return path.getDepth(); }
      int defenseCheckCount() const { return defense_check.top(); }
      int effectiveDepth() const { 
	return path.getDepth() - defenseCheckCount();
      }
      const HashEffectState& state() const { return m_state; }
      Player turn() const { return m_state.getTurn(); }
      const Move lastMove() const { return moves.lastMove(); }
    };
  } // namespace brinkmate
} // namespace osl

#endif /* OSL_BRINKMATE_BRINKMATESTATE_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
