/* orderOfMove.h
 */
#ifndef CATEGORY_ORDEROFMOVE_H
#define CATEGORY_ORDEROFMOVE_H

#include "osl/category/categoryEnv.h"
#include "osl/category/categoryList.h"
#include "osl/category/classify.h"
#include "osl/category/moveEvaluation.h"
#include "osl/container/moveLogProbVector.h"
namespace osl
{
  namespace category
  {
    template <class T> class CategoryUtil;
    namespace analyzer
    {
      /**
       * Ϳ줿̤ǻؼ꤬ܤ뤫Ĵ٤
       */
      class OrderOfMove
      {
	const CategoryEnv& env;
	Move target;
      public:
	struct Output
	{
	  int order;
	  int generated;
	  const char *name;
	  Output() : order(0), generated(0), name(0)
	  {
	  }
	  virtual ~Output()
	  {
	  }
	  virtual void generatorOfEachMove(const char *n, const MoveLogProb&)
	  {
	    name = n;		// Ǹ == target ä
	  }
	};
      private:
	Output& out;
      public:
	OrderOfMove(const CategoryEnv& e, Move t, Output& o)
	  : env(e), target(t), out(o)
	{
	}
	template <typename Head, typename Tail>
	MoveLogProb generate(CategoryList<Head, Tail>);
	MoveLogProb generate(NullCategory)
	{
	  return MoveLogProb();
	}
	template <typename SubCategories>
	const char *getCategoryName(Classify<SubCategories>, MoveLogProb move)
	{
	  Position8 sendoffs;
	  CategoryUtilBase::findSendOff(env, sendoffs);
	  const int diff = 
	    MoveEvaluation::evalDetailed(env, move.getMove(), sendoffs);
	  return getNameInClassify(SubCategories(), move, diff);
	}
	template <typename T>
	const char *getCategoryName(T, MoveLogProb)
	{
	  return T::getName();
	}
	template <typename Head, typename Tail>
	const char* getNameInClassify(CategoryList<Head,Tail>, MoveLogProb move,
				      int pieceValue)
	{
	  const std::pair<bool,int> r = 
	    CategoryUtil<Head>::getProbIf(env, move.getMove(), pieceValue);
	  if (r.first && (r.second == move.getLogProb()))
	    return Head::getName();
	  return getNameInClassify(Tail(), move, pieceValue);
	}
	const char *getNameInClassify(NullCategory, MoveLogProb, int)
	{
	  return "null?!";
	}
      };
    } // namespace analyzer
  } // namespace category
} // namespace osl

template <typename Head, typename Tail>
osl::MoveLogProb osl::category::analyzer::OrderOfMove::
generate(CategoryList<Head, Tail>)
{
  MoveLogProbVector moves;
  CategoryUtil<Head>::generate(env,moves);
  out.generated += moves.size();
  for (size_t i=0; i<moves.size(); ++i, ++(out.order))
  {
    out.generatorOfEachMove(getCategoryName(Head(), moves[i]), moves[i]);
    if (moves[i].getMove() == target)
      return moves[i];
  }
  return generate(Tail());
}

#endif /* CATEGORY_ORDEROFMOVE_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
