/* capture.h
 */
#ifndef _CAPTURE_H
#define _CAPTURE_H

#include "osl/category/categoryEnv.h"
#include "osl/category/categoryTraits.h"
#include "osl/category/probTable.h"
#include "osl/move_classifier/trapRook.h"
#include "osl/move_classifier/moveAdaptor.h"
#include "osl/container/moveStack.h"
#include <boost/static_assert.hpp>

namespace osl
{
  namespace category
  {
    /**
     * ̤ capture
     * @param PTYPE μζ
     */
    template <Ptype PTYPE>
    struct Capture : public StandardProbabilityCategory
    {
      static Interpolator<PieceValueTable> probTable;
      BOOST_STATIC_ASSERT(PtypeTraits<PTYPE>::isBasic);	// ȴ
      static const char *getName();
      static bool isInCategory(const CategoryEnv& /*env*/, Move move)
      {
	const Ptype target = move.capturePtype();
	return (target != PTYPE_EMPTY) && (unpromote(target) == PTYPE);
      }
      static void generate(const CategoryEnv& env, MoveVector& out);
    };

    /**
     * ľľμʬλؼ³ΤΤȤ
     */
    struct ContinueCapture : public StandardProbabilityCategory
    {
      static Interpolator<PieceValueTable> probTable;
      static const char *getName() { return "ContinueCapture"; }
      static bool isEffective(const CategoryEnv& env)
      {
	return env.history->lastMove(2).isNormal();
      }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (! isEffective(env))
	  return false;
	const Move last_last_move=env.history->lastMove(2);
	assert(last_last_move.isNormal());
	return (move.from() == last_last_move.to()) 
	  && (move.capturePtype() != PTYPE_EMPTY);
      }
      struct PositionAction;
      static void generate(const CategoryEnv& env, MoveVector& out);
    };

    /**
     * ֤
     */
    struct TrapRook : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "TrapRook"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	using move_classifier::PlayerMoveAdaptor;
	return PlayerMoveAdaptor<move_classifier::TrapRook>::
	  isMember(*env.state, move);
      }
    };
    
  } // namespace category
} // namespace osl

#endif /* _CAPTURE_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
