/* categoryListUtil.tcc
 */
#ifndef _CATEGORYLISTUTIL_TCC
#define _CATEGORYLISTUTIL_TCC

#include "osl/category/categoryListUtil.h"
#include "osl/category/categoryUtil.h"
#include "osl/move_classifier/moveAdaptor.h"
#include "osl/move_classifier/safeMove.h"
#include "osl/container/moveLogProbSet.h"

namespace osl
{
  namespace category
  {
    template <class MoveSet>
    struct CategoryListUtilGatherAll
    {
      MoveSet *out;
      const NumEffectState& state;
      CategoryListUtilGatherAll(MoveSet *o, const NumEffectState& s)
	: out(o), state(s)
      {
      }
      template <bool is_king_escape>
      bool operator()(const char */*category_name*/,
		      const MoveLogProbVector& moves)
      {
	for (MoveLogProbVector::const_iterator p=moves.begin(); 
	     p!=moves.end(); ++p)
	{
	  using namespace osl::move_classifier;
	  const Move m = p->getMove();
	  if (m.isPass() || m.isDrop()
	      || PlayerMoveAdaptor<SafeMove>::isMember(state, m))
	  {
	    out->assignIfBetter(*p);
	  }
	}
	const bool king_escape_generated = is_king_escape && (! moves.empty());
	return ! king_escape_generated;
      }
    };
  } // namespace category
} // namespace osl

template <typename Categories, typename MoveSet>
void osl::category::CategoryListUtil::
gatherAllMoves(const CategoryEnv& env, MoveSet& out)
{
  forEachCategory(Categories(), env, 
		  CategoryListUtilGatherAll<MoveSet>(&out, *env.state));
}

template <class Generator>
void osl::category::CategoryListUtil::
generate(const CategoryEnv& env, MoveLogProbVector& out)
{
  boost::function_requires<category::CategoryConcept<Generator> >();
  CategoryUtil<Generator>::generate(env, out);
}

#endif /* _CATEGORYLISTUTIL_TCC */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
