/* classify.tcc
 */
#ifndef OSL_CATEGORY_CLASSIFY_TCC
#define OSL_CATEGORY_CLASSIFY_TCC
      
#include "osl/category/classify.h"
#include "osl/category/categoryUtil.h"
#include "osl/category/moveEvaluation.h"
#include "osl/move_generator/allMoves.h"
#include "osl/move_action/store.h"
#include <algorithm>
#include <cassert>

template <class SubCategories>
void osl::category::Classify<SubCategories>::
generate(const CategoryEnv& env, MoveLogProbVector& out)
{
  MoveVector moves;
  move_action::Store store(moves);
  move_generator::AllMoves<NumEffectState,move_action::Store>::
    generateMoves(env.state->getTurn(),*env.state, store);

  Position8 sendoffs;
  CategoryUtilBase::findSendOff(env, sendoffs);
  bool threatmate =
    (env.record &&
     env.record->threatmate().threatmateMove(env.state->getTurn()).isNormal());
  for (MoveVector::const_iterator p=moves.begin(); p!=moves.end(); ++p)
  {
    // TODO: ʬ/ΨĴ٤륳ɤΨ
    // 㤨Сdrop moveǤCapturePromoteϤʤʤ
    const int max = env.limit+1;
    const int diff = MoveEvaluation::evalDetailed(env, *p, sendoffs);
    
    const int prob = getMinProb(SubCategories(), env, *p, diff, max);
    assert(prob);
    if (prob < max)
      out.push_back(*p, prob);
    else if (threatmate)
      out.push_back(*p, std::min(prob, env.limit));
  }
}
      
template <class SubCategories>
template <class Head, class Tail>
int osl::category::Classify<SubCategories>::
getProb(CategoryList<Head,Tail>, const CategoryEnv& env, Move m, 
	int pieceValue, int cur)
{
  if (MinProb<Head>::minProb() < cur)
  {
    const std::pair<bool,int> r = CategoryUtil<Head>::getProbIf(env,m,pieceValue);
    if (r.first)
      return r.second;
    // fall thru
  }
  return getProb(Tail(), env, m, pieceValue, cur);
}

template <class SubCategories>
template <class Head>
int osl::category::Classify<SubCategories>::
getProb(CategoryList<Head,NullCategory>, const CategoryEnv& env, Move m, 
	int pieceValue, int cur)
{
  if (MinProb<Head>::minProb() < cur)
  {
    const std::pair<bool,int> 
      r = CategoryUtil<Head>::getProbIf(env,m,pieceValue);
    if (r.first)
      return r.second;
  }
  return cur;
}

template <class SubCategories>
template <class Head, class Tail>
int osl::category::Classify<SubCategories>::
getMinProb(CategoryList<Head,Tail>, const CategoryEnv& env, Move m, 
	   int pieceValue, int cur)
{
  if (MinProb<Head>::minProb() < cur)
  {
    const std::pair<bool,int> r
      = CategoryUtil<Head>::getProbIf(env,m,pieceValue);
    if (r.first)
      return getMinProb(Tail(), env, m, pieceValue, std::min(cur,r.second));
    // fall thru
  }
  return getMinProb(Tail(), env, m, pieceValue, cur);
}

template <class SubCategories>
template <class Head>
int osl::category::Classify<SubCategories>::
getMinProb(CategoryList<Head,NullCategory>, 
	   const CategoryEnv& env, Move m, int pieceValue, int cur)
{
  if (MinProb<Head>::minProb() < cur)
  {
    const std::pair<bool,int> r = CategoryUtil<Head>::getProbIf(env,m,pieceValue);
    if (r.first)
      return std::min(cur,r.second);
  }
  return cur;
}

#endif /* OSL_CATEGORY_CLASSIFY_TCC */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
