/* lance.h
 */
#ifndef OSL_CATEGORY_LANCE_H
#define OSL_CATEGORY_LANCE_H

#include "osl/category/categoryTraits.h"
#include "osl/category/categoryEnv.h"
#include "osl/container/moveStack.h"
#include "osl/container/moveVector.h"

namespace osl
{
  namespace category
  {
    struct LanceAttackKing : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "LanceAttackKing"; }
      static bool isInCategory(const CategoryEnv&env, Move move)
      {
	if (move.ptype() != LANCE || !move.isDrop())
	  return false;

	const Player player = move.player();
	const Position king = env.state->getKingPosition(alt(player));

	if (move.to().x() != king.x() ||
	    env.state->hasEffectBy(alt(player), move.to()))
	  return false;

	Position up = Board_Table.nextPosition(player, move.to(), U);
	Piece piece = env.state->getPieceAt(up);
	while (up.isOnBoard() && piece.isEmpty())
	{
	  up = Board_Table.nextPosition(player, up, U);
	  piece = env.state->getPieceAt(up);
	}

	if (!piece.isPiece() || piece.ptype() != PAWN)
	  return false;

	if (piece.owner() == alt(player) &&
	    ((player == BLACK &&
	      piece.position().y() == king.y() + 1) ||
	     (player == WHITE &&
	      piece.position().y() == king.y() - 1)))
	  return true;

	if (piece.owner() == player &&
	    abs(piece.position().y() - king.y()) <= 3)
	  return true;

	return false;
      }
    };

    // ι
    struct LanceSacrifice : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "LanceSacrifice"; }
      static bool isInCategory(const CategoryEnv &env, Move move)
      {
	if (move.ptype() != LANCE || ! move.isDrop())
	  return false;

	const Player player = move.player();
	const Position king = env.state->getKingPosition(alt(player));

	return (env.progress32.value() > 15 &&
		abs(move.to().x() - king.x()) <= 2 &&
		abs(move.to().y() - king.y()) <= 2 &&
		(move.to().x() == 3 || move.to().x() == 7) &&
		(move.to().y() == 3 || move.to().y() == 7) &&
		env.state->countEffect(alt(player),  move.to()) > 1);
      }
    };
  } // namespace category
} // namespace osl

#endif /* OSL_CATEGORY_LANCE_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
