/* mate.h
 */
#ifndef _CATEGORY_MATE_H
#define _CATEGORY_MATE_H

#include "osl/category/categoryEnv.h"
#include "osl/category/categoryTraits.h"
#include "osl/effect_util/neighboring8Direct.h"

/**
 * @file
 * 󤻴طΥƥ
 */

namespace osl
{
  namespace category
  {
    class PieceValueTable;
    /**
     * ư(dropޤ)ξ꤫ư̤8˵
     */
    struct DirectAttackToKing8 : public StandardProbabilityCategory
    {
      static Interpolator<PieceValueTable> probTable;
      static const char *getName() { return "DirectAttackToKing8"; }
    public:
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	const PtypeO ptypeo = move.ptypeO();
	const Position to = move.to();
	const Position op_king =
	  env.state->getKingPosition(alt(env.state->getTurn()));
	return Neighboring8Direct::hasEffect(*env.state, ptypeo, to, 
					     op_king);
      }
    };

    /**
     * ȶΤϤ ʤ
     */
    struct SlowApproach : public StandardProbabilityCategory
    {
      static Interpolator<PieceValueTable> probTable;
      static const char *getName() { return "SlowApproach"; }

      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (move.capturePtype() != PTYPE_EMPTY)
	  return false;
	if ((move.ptype() != PPAWN)
	    && (move.ptype() != PLANCE)
	    && (move.ptype() != PKNIGHT)
	    && (move.ptype() != KNIGHT)
	    && (move.ptype() != GOLD)
	    && (move.ptype() != SILVER)
	    && (move.ptype() != PSILVER)
	    && (move.ptype() != PBISHOP)
	    && (move.ptype() != PROOK)
	  )
	  return false;
	const Position from = move.from();
	if (from == Position::STAND())
	  return false;
	const Position to = move.to();
	const Position op_king =
	  env.state->getKingPosition(alt(env.state->getTurn()));
	return approaching(from, to, op_king)
	  || (Board_Table.nextPosition(move.player(), from, D) == to);
      }
      /** ̤˶դƤ뤫ɤ */
      static bool approaching(Position from, Position to, Position target)
      {
	// FIXME: sloow
	int fromX = from.x();
	int fromY = from.y();
	int toX = to.x();
	int toY = to.y();
	int targetX = target.x();
	int targetY = target.y();
	return(abs(toY-targetY) < 2
	       && ((abs(toX-targetX) < abs(fromX-targetX))
		   || (toX-targetX == fromX-targetX // ŪˤŨؤľ
		       && abs(toY-targetY) <= abs(fromY-targetY))));
      }
    };

    /**
     * ̤ξ岼2Υ줿ǤϩʶǤļꡣ
     * ϼ礤ߤƥ뤵٤⤷ʤʹ٤
     * ̣褦ʵ⤹롣
     */
    struct DropPieceOnEscapeRoute : public FixedProbabilityCategory
    {
      static FixedValueTable probTable; // 799
      static const char *getName() { return "DropPieceOnEscapeRoute"; }
      static bool isInCategory(const CategoryEnv &env, Move move)
      {
	if (!move.isDrop())
	  return false;
	const Position king =
	  env.state->getKingPosition(alt(env.state->getTurn()));
	if (abs(king.x() - move.to().x()) != 2 ||
	    abs(king.y() - move.to().y()) > 1 )
	  return false;

	const Player p = move.player();
	const Position up = Board_Table.nextPosition(p, move.to(), U);
	const Position down = Board_Table.nextPosition(p, move.to(), D);
	const Piece piece1 = up.isOnBoard() ? env.state->getPieceAt(up)
	  : env.state->getPieceAt(Board_Table.nextPosition(p, down, D));
	// ξ̤ƤΤ false ˤʤ褦פФƤƤ
	// 
	const Piece piece2 = env.state->getPieceAt(down);
	return (piece1.isPiece() &&
		piece1.owner() == alt(p) &&
		piece2.isPiece() &&
		piece2.owner() == alt(p));
      }
    };

    /**
     * ̤ϲʤȤ
     */
    struct LureKingToBottom : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "LureKingToBottom"; }
      static bool isInCategory(const CategoryEnv &env, Move move)
      {
	Player player = move.player();
	if ((player == BLACK && move.to().y() == 1) ||
	    (player == WHITE && move.to().y() == 9))
	{
	  return (env.state->hasEffectByPtype<KING>(alt(player), move.to()) &&
		  env.state->countEffect(alt(player), move.to()) == 1);
	}
	return false;
      }
    };
  } // namespace category
} // namespace osl

#endif /* _CATEGORY_MATE_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
