/* probTable.tcc
 */
#ifndef _PROBTABLE_TCC
#define _PROBTABLE_TCC

#include "osl/category/probTable.h"
#include "osl/category/categoryTraits.h"

#include <string>
#include <algorithm>
#include <iostream>
#include <stdexcept>
#include <cstdio>

template <int IndexMax>
osl::category::ProbTable<IndexMax>::
ProbTable(const char *oslHome, const char *name)
{
  filename[0] = '\0';
  std::string path(oslHome);
  path += "/data/category/";
  path += name;
  path += ".txt";
  init(path.c_str());
}

template <int IndexMax>
osl::category::ProbTable<IndexMax>::
ProbTable()
{
  filename[0] = '\0';
  std::fill(prob.begin(), prob.end(), CategoryTraits::maxProb());
}

template <int IndexMax>
osl::category::ProbTable<IndexMax>::
~ProbTable()
{
  for (size_t i=0; i<indexMax(); ++i)
  {
    if (prob[i] == 0)
      std::cerr << i << " th elem was 0 in " << &filename[0] << "\n";
  }
}

template <int IndexMax>
void osl::category::ProbTable<IndexMax>::
interpolate()
{
  for (size_t i=0; i<indexMax(); ++i)
  {
    if (prob[i] == 0)
    {
      // и 0 äۤȤɻؤʤ
      prob[i] = CategoryTraits::maxProb();
    }
  }
}

template <int IndexMax>
void osl::category::ProbTable<IndexMax>::
init(const char *filename)
{
  strncpy(&this->filename[0], filename, 128);
  this->filename[127] = '\0';

  min = CategoryTraits::maxProb();
  size_t i=0;
  FILE *fp = fopen(filename, "r");
  if (! fp)
    goto error;
  for (; i+4<indexMax(); i+=4)
  {
    int v0,v1,v2,v3;
    if (fscanf(fp, "%d %d %d %d", &v0,&v1,&v2,&v3) < 4)
      goto error;
    v0 = roundUp(v0);
    v1 = roundUp(v1);
    v2 = roundUp(v2);
    v3 = roundUp(v3);
    if (v0)
      min = std::min(min, v0);
    if (v1)
      min = std::min(min, v1);
    if (v2)
      min = std::min(min, v2);
    if (v3)
      min = std::min(min, v3);
    prob[i  ] = v0;
    prob[i+1] = v1;
    prob[i+2] = v2;
    prob[i+3] = v3;
  }
  for (; i<indexMax(); ++i)
  {
    int val;
    if (fscanf(fp, "%d", &val) < 1)
      goto error;
    val = roundUp(val);
    if (val)
      min = std::min(min, val);
    prob[i] = val;
  }
  goto close;
error:
  {
    std::cerr << "read failure in PieceValueTable::init"
	      << "\n" << filename << "\n";
    throw std::runtime_error(std::string(filename)
			     + " read failure in PieceValueTable::init");
  }
close:
  if (fp)
    fclose(fp);
}

#endif /* _PROBTABLE_TCC */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
