/* silver.h
 */
#ifndef _SILVER_H
#define _SILVER_H

#include "osl/category/categoryTraits.h"
#include "osl/category/categoryEnv.h"
#include "osl/container/moveVector.h"
#include "osl/container/moveStack.h"
namespace osl
{
  namespace category
  {
    /**
     * Ǥ
     */
    struct StabInTheBack : public FixedProbabilityCategory
    {
      static FixedValueTable probTable; // { return 274; } // 0.149371
      static const char *getName() { return "StabInTheBack"; }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	const CategoryEnv::effect_state_t& state = *env.state;
	const Player P = move.player();
	const Piece backL = 
	  state.getPieceAt(Board_Table.nextPosition(P, move.to(), DL));
	const Piece backR = 
	  state.getPieceAt(Board_Table.nextPosition(P, move.to(), DR));
	return (move.ptype() == SILVER)
	  && move.isDrop()
	  && backR.isPiece()
	  && (backR.owner() == alt(P))
	  && backL.isPiece()
	  && (backL.owner() == alt(P))
	  && ((backL.ptype() == GOLD)
	      || (backL.ptype() == ROOK))
	  && ((backR.ptype() == GOLD)
	      || (backR.ptype() == ROOK))
	  && (! state.hasEffectBy(alt(P), move.to()));
      }
    };

    /**
     * ζ
     */
    struct SilverHeadOfKnight : public FixedProbabilityCategory
    {
      static FixedValueTable probTable; 
      static const char *getName() { return "SilverHeadOfKnight"; }

      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (! ((move.ptype() == SILVER)
	       && move.isDrop()))
	  return false;
	const Player P = move.player();
	const Position to = move.to();
	const Piece front = 
	  env.state->getPieceAt(Board_Table.nextPosition(P, to, U));
	return front.isPiece()
	  && (front.owner() == alt(P))
	  && (front.ptype() == KNIGHT);
      }
    };

    /**
     * ľηζ
     */
    struct SilverHeadOfLastKnight : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "SilverHeadOfKnight!"; }

      static bool isEffective(const CategoryEnv& env)
      {
	return env.history->lastMove().isNormal();
      }
      static bool isInCategory(const CategoryEnv& env, Move move)
      {
	if (! isEffective(env))
	  return false;
	const Player P = move.player();
	const Move last_move=env.history->lastMove();
	assert(last_move.isNormal());
	const Position to = move.to();
	return (last_move.to() == Board_Table.nextPosition(P, to, U))
	  && SilverHeadOfKnight::isInCategory(env, move);
      }
      static void generate(const CategoryEnv& env, MoveVector& moves)
      {
	if (! isEffective(env))
	  return;
	const Move last_move=env.history->lastMove();
	assert(last_move.isNormal());
	if (last_move.ptype() != KNIGHT)
	  return;
	const Player P = env.state->getTurn();
	if (! env.state->hasPieceOnStand(P, SILVER))
	  return;
	const Position to
	  = Board_Table.nextPosition(alt(P), last_move.to(), U);
	if (env.state->getPieceOnBoard(to) != Piece::EMPTY())
	  return;
	moves.push_back(Move(to, SILVER, P));
      }
    };

    /**
     * ʢ
     */ 
    struct SilverNextToKing : public FixedProbabilityCategory
    {
      static FixedValueTable probTable;
      static const char *getName() { return "SilverNextToKing"; }

      static bool isInCategory(const CategoryEnv &env, Move move)
      {
	if (move.ptype() != SILVER)
	  return false;

	const Player player = move.player();
	const Piece king = env.state->getKingPiece(alt(player));
	if (move.to().y() != king.position().y() ||
	    !env.state->hasEffectByPtype<KING>(alt(player), move.to()) ||
	    env.state->countEffect(alt(player), move.to()) != 1)
	  return false;
	if (((player == BLACK && move.to().y()) == 2 ||
	     (player == WHITE && move.to().y() == 8)) &&
	    ((move.to().x() < 4 && move.to().x() == king.position().y() + 1) ||
	     (move.to().x() > 6 && move.to().x() + 1 == king.position().y())))
	{
	  return
	    env.state->hasEffectBy(player,
				   Board_Table.nextPosition(player, move.to(), U)) ||
	    env.state->hasEffectBy(player,
				   Board_Table.nextPosition(player, move.to(), D));
	}
	return false;
      }
    };
  } // namespace category
} // namespace osl


#endif /* _SILVER_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
