/* checkMoveGenerator.tcc
 */
#ifndef _CHECKMOVEGENERATOR_TCC
#define _CHECKMOVEGENERATOR_TCC

#include "osl/checkmate/checkMoveGenerator.h"
#include "osl/checkmate/checkMoveList.h"
#include "osl/checkmate/checkAssert.h"
#include "osl/checkmate/pawnCheckmateMoves.h"
#include "osl/move_generator/escape.h"
#include "osl/move_generator/addEffect.h"
#include "osl/move_action/store.h"
#include "osl/move_action/safeFilter.h"
#include "osl/move_action/checkFilter.h"
#include "osl/move_classifier/check.h"
#include "osl/move_classifier/moveAdaptor.h"
#include "osl/move_classifier/pawnDropCheckmate.h"
#include "osl/container/moveVector.h"
#include <cassert>

// TODO: Ѥ move_action äƹ®

template <osl::Player P>
void osl::checkmate::
CheckMoveGenerator<P>::generateAttack(const NumEffectState& state, CheckMoveList& out,
				      Move last_move)
{
  using namespace move_classifier;
  check_assert(out.empty());
  const Position targetKing
    = state.template getKingPosition<PlayerTraits<P>::opponent>();
  Piece attackerPiece;
  const bool counter_check = 
    state.template lastMoveIsCheck<P>(last_move, attackerPiece);
  MoveVector moves;
  {
    if (counter_check)
    {
#if 0
      // ®Ȼפ뤬ɥ1
      move_action::Store store(moves);
      typedef move_action::CheckFilter<P,NumEffectState,move_action::Store> check_filter_t;
      check_filter_t checkFilter(state,store);
      move_generator::Escape<P,NumEffectState,check_filter_t>::
	generateKingEscape(state, last_move, checkFilter);
#else
      MoveVector escape;
      move_action::Store store(escape);
      move_generator::Escape<P,NumEffectState,move_action::Store>::
	generateKingEscape(state, last_move, store);
      for (size_t i=0; i<escape.size(); ++i)
      {
	if (MoveAdaptor<Check<P> >::isMember(state, escape[i]))
	  moves.push_back(escape[i]);
      }
#endif
    }
    else
    {
      move_action::Store store(moves);
      typedef move_action::SafeFilter<P,NumEffectState,move_action::Store> safe_filter_t;
      safe_filter_t safeFilter(state,store);
      move_generator::AddEffect<P,true>::generate
	(state,targetKing,safeFilter);
    }
  }
  
  out.reserve(moves.size());
  for (size_t i=0; i<moves.size(); ++i)
  {
    CheckMove new_move(moves[i]);
    const Position from = moves[i].from();
    if (! from.isPieceStand())
    {
      const Ptype ptype = moves[i].ptype();
      const Position to = moves[i].to();
      // filterѤ
#ifdef PAWN_CHECKMATE_SENSITIVE
      if (PawnCheckmateMoves::effectiveOnlyIfPawnCheckmate<P>(ptype, from, to))
      {
	new_move.flags = MoveFlags::NoPromote;
      }
#endif
    }
#ifdef DELAY_SACRIFICE
    const Position to = moves[i].to();
    const Ptype capturePtype = moves[i].capturePtype();
    if (capturePtype == PTYPE_EMPTY)
    { 
      const int defense = state.countEffect(alt(P),to);
      int offense = state.countEffect(P,to);
      if (from.isPieceStand())
	--offense;
      if (defense > offense)
      {
	new_move.flags.set(MoveFlags::SacrificeAttack);
      }
    }
#endif
    check_assert(std::find(moves.begin(), moves.begin()+=i, moves[i])
		 == (moves.begin()+=i)); // unique moves
    out.push_back(new_move);
  }
}

namespace osl
{
  namespace checkmate
  {
    template <class NumEffectState>
    Ptype getCheapestDrop(Player turn, const NumEffectState& state)
    {
      if (state.hasPieceOnStand(turn, PAWN))
	return PAWN;
      if (state.hasPieceOnStand(turn, LANCE))
	return LANCE;
      if (state.hasPieceOnStand(turn, KNIGHT))
	return KNIGHT;
      if (state.hasPieceOnStand(turn, SILVER))
	return SILVER;
      if (state.hasPieceOnStand(turn, GOLD))
	return GOLD;
      if (state.hasPieceOnStand(turn, BISHOP))
	return BISHOP;
      if (state.hasPieceOnStand(turn, ROOK))
	return ROOK;
      return PTYPE_EMPTY;
    }
    template <class NumEffectState>
    Ptype getSecondCheapestDrop(Player turn, const NumEffectState& state,
				Ptype cheapest)
    {
      if (cheapest == PAWN)
      {
	if (state.hasPieceOnStand(turn, LANCE))
	  return LANCE;
	if (state.hasPieceOnStand(turn, KNIGHT))
	  return KNIGHT;
	if (state.hasPieceOnStand(turn, SILVER))
	  return SILVER;
	if (state.hasPieceOnStand(turn, GOLD))
	  return GOLD;
	if (state.hasPieceOnStand(turn, BISHOP))
	  return BISHOP;
	if (state.hasPieceOnStand(turn, ROOK))
	  return ROOK;
      }
      return PTYPE_EMPTY;
    }
  }
}

template <osl::Player P>
unsigned int osl::checkmate::
CheckMoveGenerator<P>::generateEscape(const NumEffectState& state, CheckMoveList& out,
				      Move last_move)
{
  using namespace move_classifier;
  const Player Defense = PlayerTraits<P>::opponent;

  unsigned int simple_king_moves = 0;

  MoveVector moves;
  {
    move_action::Store store(moves);
    move_generator::Escape<Defense,NumEffectState,move_action::Store>::
      generateKingEscape(state, last_move, store);
  }
  assert(out.empty());
  out.reserve(moves.size());
#ifdef DELAY_INTERPOSE
  const Ptype cheapest = getCheapestDrop(alt(P), state);
  const Ptype secondCheapest = getSecondCheapestDrop(alt(P), state, cheapest);
#endif
  for (size_t i=0; i<moves.size(); ++i)
  {
    const Move m = moves[i];
    CheckMove cm(m);
    const Position to = m.to();
    const Position from = m.from();
    const Ptype ptype = m.ptype();
    if (from.isPieceStand())
    {
      if (PawnDropCheckmate<Defense>::isMember(state, ptype, from, to))
	continue;		// ͤᲦ
#ifdef DELAY_INTERPOSE
      if (((ptype != cheapest)
	   && (ptype != secondCheapest))
	  || ((! state.hasEffectBy(alt(P), to))	// ʬʤ
	      && (! state.hasMultipleEffectBy(P, to))))	// Ǥʤ
	cm.flags = MoveFlags::BlockingBySacrifice;
#endif
    }
    else 
    {  
      assert(move_classifier::SafeMove<PlayerTraits<P>::opponent>
	     ::isMember(state, ptype, from, to));
#ifdef DELAY_INTERPOSE
      if ((ptype != KING)
	  && (m.capturePtype() == PTYPE_EMPTY)
	  && (! state.hasMultipleEffectBy(alt(P), to)))
      {
	// ư
	cm.flags = MoveFlags::BlockingBySacrifice;
      }
#endif
      if (ptype == KING && m.capturePtype() == PTYPE_EMPTY)
	++simple_king_moves;
    }
    out.push_back(cm);
  }
  return simple_king_moves;
}

#endif /* _CHECKMOVEGENERATOR_TCC */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
