/* fixedDepthSearcher.tcc
 */
#ifndef _CHECKMATE_FIXED_DEPTH_SERCHER_TCC
#define _CHECKMATE_FIXED_DEPTH_SERCHER_TCC
#include "osl/checkmate/fixedDepthSearcher.h"
#include "osl/checkmate/immediateCheckmate.h"
#include "osl/checkmate/proofPieces.h"
#include "osl/checkmate/proofNumberTable.h"
#include "osl/state/numEffectState.h"
#include "osl/container/moveVector.h"
#include "osl/move_action/store.h"
#include "osl/move_action/count.h"
#include "osl/move_action/safeFilter.h"
#include "osl/move_generator/addEffect.h"
#include "osl/move_generator/escape.h"
#include "osl/move_classifier/check.h"
#include "osl/effect_util/effectUtil.h"
#include "osl/apply_move/applyMove.h"
#include "osl/effect/liberty8.h"
#include "osl/neighboring8.h"
#include "osl/stat/ratio.h"

namespace osl
{
  namespace checkmate
  {
    template<Player P, bool SetPieces>
    struct FixedAttackHelper{
      FixedDepthSearcher &searcher;
      Move move;
      int depth;
      ProofDisproof& pdp;
      PieceStand& pieces;
      FixedAttackHelper(FixedDepthSearcher &s,int d,ProofDisproof& p,
			PieceStand& pi)
	: searcher(s), depth(d), pdp(p), pieces(pi)
      {
      }
      void operator()(Position)
      {
	assert(move.isNormal());
	pdp=searcher.defense<P,SetPieces>(move,depth-1,pieces);
      }
    };
   /**
    * Pư¦ǤϤʤ¦
    */
    template<Player P, bool SetPieces, bool MayUnsafe=false>
    struct FixedDefenseHelper{
      FixedDepthSearcher &searcher;
      int depth;
      ProofDisproof& pdp;
      PieceStand& pieces;
      Move best_move;
      FixedDefenseHelper(FixedDepthSearcher &s,int d,ProofDisproof& p,
			 PieceStand& pi) 
	: searcher(s), depth(d), pdp(p), pieces(pi)
      {
      }
      void operator()(Position)
      {
	if (MayUnsafe)
	  pdp=searcher.attackMayUnsafe<P,SetPieces,false>(depth-1, best_move, pieces);
	else
	  pdp=searcher.attack<P,SetPieces,false>(depth-1, best_move, pieces);
      }
    };
  }
}

template <osl::Player P, bool SetPieces, bool HasGuide>
const osl::checkmate::ProofDisproof 
osl::checkmate::FixedDepthSearcher::
attackMayUnsafe(int depth, Move& best_move, PieceStand& proof_pieces)
{
  assert(state->getTurn() == P);
  const Position target_king
    = state->template getKingPosition<PlayerTraits<P>::opponent>();
  if (state->hasEffectBy<P>(target_king))
    return ProofDisproof::NoEscape();
  return attack<P,SetPieces,HasGuide>(depth, best_move, proof_pieces);
}

template <osl::Player P, bool SetPieces, bool HasGuide>
const osl::checkmate::ProofDisproof 
osl::checkmate::FixedDepthSearcher::
attack(int depth, Move& best_move, PieceStand& proof_pieces)
{
  assert(state->getTurn() == P);
  assert ((! HasGuide)
	  || (state->isAlmostValidMove(best_move)
	      && move_classifier::
	      Check<P>::isMember(*state, best_move.ptype(), best_move.from(), 
				 best_move.to())));
  addCount();
  const Position target_king
    = state->template getKingPosition<PlayerTraits<P>::opponent>();
  assert(! state->hasEffectBy<P>(target_king));
  const King8Info info = King8Info::make<P>(*state, target_king);
  if ((! EffectUtil::isKingInCheck(P, *state))
      && ImmediateCheckmate::hasCheckmateMove<P>(*state, info, target_king,
						 best_move))
  {
    if (SetPieces)
    {
      proof_pieces = PieceStand();
      if (best_move.isDrop())
	proof_pieces.add(best_move.ptype());
    }
    return ProofDisproof::Checkmate();
  }
  if (depth <= 0) 
  {
    return Proof_Number_Table.attackEstimation(*state, P, info, target_king);
  }

  ProofDisproof pdp;
  typedef FixedAttackHelper<P,SetPieces> helper_t;
  helper_t helper(*this,depth,pdp,proof_pieces);
  int minProof = ProofDisproof::PROOF_MAX;
  int sumDisproof=0;
  if (HasGuide)
  {
    helper.move=best_move;
    ApplyMove<P>::doUndoMove(*state,best_move,helper);
    if (pdp.isCheckmateSuccess())
    {
      if (SetPieces)
	proof_pieces = ProofPieces::attack(proof_pieces, best_move, stand(P));
      return ProofDisproof::Checkmate();
    }
    minProof = pdp.proof();
    sumDisproof += pdp.disproof();
  }
  
  const Position targetKing
    = state->template getKingPosition<PlayerTraits<P>::opponent>();
  MoveVector moves;
  move_action::Store store(moves);
  move_generator::AddEffect<P,true>::generate
    (*state,targetKing,store);
  if (moves.size()==0)
    return ProofDisproof::NoCheckmate();
  for(size_t i=0;i<moves.size();i++) {
    if (HasGuide && moves[i] == best_move)
      continue;
    helper.move=moves[i];
    ApplyMove<P>::doUndoMove(*state,moves[i],helper);
    int proof=pdp.proof();
    if (proof<minProof){
      if (proof==0){
	best_move=moves[i];
	if (SetPieces)
	{
	  proof_pieces = ProofPieces::attack(proof_pieces, best_move, stand(P));
	}
	return ProofDisproof::Checkmate();
      }
      minProof=proof;
    }
    sumDisproof+=pdp.disproof();
  }
  return ProofDisproof(minProof,sumDisproof);
}

template <osl::Player P, bool SetPieces>
inline
const osl::checkmate::ProofDisproof 
osl::checkmate::FixedDepthSearcher::
defenseEstimation(Move last_move, PieceStand& proof_pieces,
		  Piece attacker_piece, Position target_position) const
{
  assert(state->getTurn() == alt(P));
  const Player Opponent = PlayerTraits<P>::opponent;
  effect::Liberty8<Opponent> libertyKing(*state,target_position);
  int count=libertyKing.count();
  // multiple checkʤΤǡpawn dropǤϤʤ
  if (attacker_piece.isEmpty())
  {
    if (count>0){
      return ProofDisproof(count,1);
    }
    return ProofDisproof::NoEscape();
  }
  const Position attack_from=attacker_piece.position();
  count += state->countEffect(alt(P), attack_from);
  if (Neighboring8::isNeighboring8(attack_from, target_position))
    --count;
  const EffectContent effect
    = Ptype_Table.getEffect(attacker_piece.ptypeO(), 
			    attack_from, target_position);
  if (! effect.hasUnblockableEffect())
  {
    // this is better approximation than naive enumeration of blocking moves,
    // for counting of disproof number in df-pn,
    ++count;
  }

  if (count==0){
    if (last_move.isDrop() && last_move.ptype()==PAWN)
      return ProofDisproof::PawnCheckmate();
    if (SetPieces)
    {
      proof_pieces = ProofPieces::leaf(*state, P, stand(P));
    }
    return ProofDisproof::NoEscape();
  }
  return ProofDisproof(count, 1);
}

template <osl::Player Defense>
void osl::checkmate::FixedDepthSearcher::
generateBlockingWhenLiberty0(Piece defense_king, Position attack_from,
			     MoveVector& moves) const
{
  assert(state->getKingPiece(Defense) == defense_king);
  using namespace move_generator;
  using namespace move_action;
  MoveVector all_moves;
  Store store(all_moves);
  NotKingOpenFilter<Defense,NumEffectState,Store> safe_filter(*state,store);
  
  Escape<Defense,NumEffectState,Store>::
    generateBlockingKing(*state, defense_king, attack_from,store,safe_filter);

  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end(); ++p)
  {
    if (p->isDrop())
    {
      if (! state->hasEffectBy<Defense>(p->to()))
	continue;
    }
    else
    {
      // move
      if (! Neighboring8::isNeighboring8(p->from(), defense_king.position()))
      {
	if (! state->hasMultipleEffectBy(Defense, p->to()))
	  continue;
      }
    }
    moves.push_back(*p);
  }
}

template <osl::Player Defense> inline
int osl::checkmate::FixedDepthSearcher::
blockEstimation(Position attack_from, Position defense_king) const
{
  // Τޥ褦Ȼפä̤ʤä
  return 1;
}

template <osl::Player P, bool SetPieces>
const osl::checkmate::ProofDisproof 
osl::checkmate::FixedDepthSearcher::
defense(Move last_move, int depth, PieceStand& proof_pieces)
{
  assert(state->getTurn() == alt(P));
  addCount();
  const Player Defense = PlayerTraits<P>::opponent;
  const Position attackerKing
    = state->template getKingPosition<P>();
  /**
   * ľιμ꤬
   */
  if (state->hasEffectBy<Defense>(attackerKing))
    return ProofDisproof::NoCheckmate();
  const Piece target_king
    = state->template getKingPiece<Defense>();
  const Position target_position = target_king.position();
  assert(state->hasEffectBy<P>(target_position));
  Piece attacker_piece;
  state->template lastMoveIsCheck<Defense>(last_move,attacker_piece);
  if (depth <= 0)
  {
    return defenseEstimation<P, SetPieces>
      (last_move, proof_pieces, attacker_piece, target_position);
  }

  assert(depth > 0);
  MoveVector moves;
  move_action::Store store(moves);
  bool blockable_check = false;
  int nonblock_moves;
  {
    using namespace move_generator;
    using namespace move_action;
    if (attacker_piece.isEmpty()) {
      Escape<Defense,NumEffectState,Store>::template
	generateEscape<KING>(*state,target_king,store);
      nonblock_moves = moves.size();
    }
    else {
      const Position attack_from=attacker_piece.position();
      NotKingOpenFilter<Defense,NumEffectState,Store> safe_filter(*state,store);
      Escape<Defense,NumEffectState,Store>::
	generateCaptureKing(*state, target_king, attack_from, safe_filter);
      const int num_captures = moves.size();
      Escape<Defense,NumEffectState,Store>::template
	generateEscape<KING>(*state, target_king, store);
      nonblock_moves = moves.size();
      blockable_check =
	! effect_util::UnblockableCheck::isMember(alt(P), *state);
      if ((depth <= 1) && num_captures && (nonblock_moves > 2))
      {
	if (nonblock_moves > 3)
	{
	  const int block_estimaate = blockable_check 
	    ? blockEstimation<Defense>(attack_from, target_position) 
	    : 0;
	  return ProofDisproof(nonblock_moves + block_estimaate, 1);
	}
      } 
      if (moves.empty())
	generateBlockingWhenLiberty0<Defense>(target_king, attack_from, moves);
    }
  }

  if (moves.empty()) {
    if (last_move.isDrop() && last_move.ptype()==PAWN)
      return ProofDisproof::PawnCheckmate();
    if (SetPieces)
    {
      proof_pieces = ProofPieces::leaf(*state, P, stand(P));
    }
    return ProofDisproof::NoEscape();
  }
  const bool cut_candidate = (depth <= 1)
    && (nonblock_moves - (state->hasPieceOnStand<P,GOLD>()
			  || state->hasPieceOnStand<P,SILVER>()) > 4);
  if (cut_candidate)
    return ProofDisproof(nonblock_moves, 1);

  typedef FixedDefenseHelper<P,SetPieces> helper_t;
  if (SetPieces)
    proof_pieces = PieceStand();
  PieceStand child_proof;
  ProofDisproof pdp;
  helper_t helper(*this, depth, pdp, child_proof);
  int minDisproof = ProofDisproof::DISPROOF_MAX;
  int sumProof = 0;
  size_t i=0;
start_examine:
  for (;i<moves.size();i++){
    ApplyMove<PlayerTraits<P>::opponent>::doUndoMove(*state,moves[i],helper);
    const int disproof=pdp.disproof();
    if (disproof<minDisproof){
      if (disproof==0)
      {
	return pdp;		// maybe PawnCheckmate
      }
      minDisproof=disproof;
    }
    sumProof+=pdp.proof();
    if (sumProof == 0)
    {
      if (SetPieces)
	proof_pieces = proof_pieces.max(child_proof);
    }
    else
    {
      if (i+1 < moves.size())
      {
	minDisproof = 1;
	if ((int)i < nonblock_moves)
	  sumProof += nonblock_moves - (i+1);
	if (blockable_check)
	  ++sumProof;
      }
      return ProofDisproof(sumProof,minDisproof);
    }
  }
  if ((sumProof == 0) && blockable_check 
      && ((int)moves.size() == nonblock_moves))
  {
    using namespace move_generator;
    using namespace move_action;
    const Position attack_from=attacker_piece.position();
    NotKingOpenFilter<Defense,NumEffectState,Store> safe_filter(*state,store);
    Escape<Defense,NumEffectState,Store>::
      generateBlockingKing(*state, target_king, attack_from,store,safe_filter);
    if ((int)moves.size() > nonblock_moves)
      goto start_examine;
  }
  
  if (SetPieces && (sumProof == 0) && blockable_check)
  {
    ProofPiecesUtil::addMonopolizedPieces(*state, P, stand(P), proof_pieces);
  }
  return ProofDisproof(sumProof,minDisproof);
}

template <osl::Player P>
const osl::checkmate::ProofDisproof 
osl::checkmate::FixedDepthSearcher::
hasEscapeByMove(Move next_move, int depth, Move& check_move,
		PieceStand& proof_pieces)
{
  typedef FixedDefenseHelper<P,true,true> helper_t;
  proof_pieces = PieceStand();
  ProofDisproof pdp;
  helper_t helper(*this, depth+1, pdp, proof_pieces);
  ApplyMove<PlayerTraits<P>::opponent>::doUndoMove(*state,next_move,helper);
  check_move = helper.best_move;
  return pdp;
}

template <osl::Player P>
const osl::checkmate::ProofDisproof 
osl::checkmate::FixedDepthSearcher::
hasEscapeByMove(Move next_move, int depth)
{
  typedef FixedDefenseHelper<P,false,true> helper_t;
  PieceStand proof_pieces;
  ProofDisproof pdp;
  helper_t helper(*this, depth+1, pdp, proof_pieces);
  ApplyMove<PlayerTraits<P>::opponent>::doUndoMove(*state,next_move,helper);
  return pdp;
}

template <osl::Player P>
const osl::checkmate::ProofDisproof 
osl::checkmate::FixedDepthSearcher::
hasCheckmateWithGuide(int depth, Move& guide, PieceStand& proof_pieces)
{
  assert(guide.isNormal());
  if (! guide.isDrop())
  {
    const Ptype existing = state->getPieceOnBoard(guide.to()).ptype();
    if (existing != KING)
      guide.setCapturePtype(existing);
  }
  if (state->template isAlmostValidMove<false>(guide)
      && move_classifier::Check<P>
      ::isMember(*state, guide.ptype(), guide.from(), guide.to()))
    return attack<P,true,true>(depth, guide, proof_pieces);
  return attack<P,true,false>(depth, guide, proof_pieces);
}

#endif /* _CHECKMATE_FIXED_DEPTH_SERCHER_TCC */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
