/* proofNumberTable.h
 */
#ifndef OSL_CHECKMATE_PROOF_NUMBER_TABLE_H
#define OSL_CHECKMATE_PROOF_NUMBER_TABLE_H
#include "osl/checkmate/king8Info.h"
#include "osl/checkmate/proofDisproof.h"
#include "osl/state/numEffectState.h"
#include "osl/effect_util/additionalEffect.h"
#include "osl/boardTable.h"
#include "osl/neighboring8.h"
#include "osl/misc/carray.h"
#include "osl/misc/carray2d.h"
#include "osl/misc/bitOp.h"
#include <boost/cstdint.hpp>
#include <boost/scoped_ptr.hpp>

namespace osl
{
  namespace checkmate
  {
    class ProofNumberTable
    {
    public:
      struct Liberty
      {
	/** ̤μͳ٤ͽ¬. ξ 0 ΤȤ롥*/
	uint8_t liberty;
	/** fale ξɬ */
	bool has_effect;
	explicit Liberty(uint8_t l=0, bool e=false) : liberty(l), has_effect(e)
	{
	}
      };
    private:
      struct Table
      {
	/** Ĥβ -> Liberty:
	 * long direction 1ޥ̣ */
	CArray2d<CArray<Liberty,DIRECTION_SIZE>,0x100u,PTYPE_SIZE> liberties;
	/** Ƥͭdrop -> Ǿliberty.
	 * liberty 8bit + ͭdrop 8bit + Ǿ -> piece mask
	 */
	CArray2d<PieceMask,0x10000u,8> drop_liberty;
	/** ζϤǲ򤫤liberty: [liberty][move_mask] */
	CArray2d<uint8_t,0x100u,0x100u> pmajor_liberty;
	/** 1,2ܤˤΰưˤliberty: [liberty][move_mask].
	 * ʳU˶बưǤ⤳ɤ
	 */
	CArray2d<uint8_t,0x100u,0x100u> promote_liberty;
	/** ʳΰưliberty: [liberty][move_mask] */
	CArray2d<uint8_t,0x100u,0x100u> other_move_liberty;
      };
      boost::scoped_ptr<Table> table;
    public:
      ProofNumberTable();

      /**
       * dir β򤫤libertyͽ
       */
      const Liberty countLiberty(Ptype ptype, Direction d, unsigned int liberty_mask) const
      {
	assert((d != UUL) && (d != UUR));
	assert(liberty_mask <= 0xff);
	return (table->liberties)[liberty_mask][ptype][d];
      }
      /**
       * 8˵ؤdropޤϼʤưlibertyͽ¬֤ͤ.
       * ̤ϼ֤Ǥ֤ʤ͡
       * ϤɽҤɬפʤΤǸƤӽФ¦ǽ롥
       * @return ξ 0 ΤȤ롥
       */
      const Liberty countLibertyShortNotKnight(Player player, Position to, Ptype ptype, 
					       Position king, King8Info info) const
      {
	assert(Neighboring8::isNeighboring8(to, king));
	assert(ptype != KNIGHT);
	const unsigned int liberty_mask = info.liberty();
	const Direction d = 
	  longToShort((player == BLACK)
		      ? Board_Table.getLongDirection<BLACK>(to, king)
		      : Board_Table.getLongDirection<WHITE>(to, king));
	return countLiberty(ptype, d, liberty_mask);
      }
      const Liberty countLibertyLong(Player player, Position to, Ptype ptype, 
				     Position king, King8Info info) const
      {
	assert(! Neighboring8::isNeighboring8(to, king));
	const unsigned int liberty_mask = info.liberty();
	const Offset32 offset32(king,to);
	const Offset offset = Board_Table.getShortOffsetNotKnight(offset32);
	if (offset.zero())
	  return Liberty(0, false);
	if (to + offset + offset != king) // 2ޥʾ
	{
	  if (isMajor(ptype))
	    ptype = unpromote(ptype);
	  else if (ptype != LANCE)
	    return Liberty(0, false);
	}
	const Direction d = 
	  (player == BLACK)
	  ? Board_Table.getLongDirection<BLACK>(offset32)
	  : Board_Table.getLongDirection<WHITE>(offset32);
	assert(isLong(d));
	return countLiberty(ptype, d, liberty_mask);
      }
      /**
       * move ϲǤɬפ
       */
      int countLiberty(const NumEffectState& state, int liberty_count,
		       Move move, Position king, King8Info info) const
      {
	assert(liberty_count == misc::BitOp::countBit(info.liberty()));
	const Player attack = move.player();
	const Player defense = alt(attack);
	const Position to = move.to();
	const Ptype ptype = move.ptype();
	if (ptype == KNIGHT)
	  return std::max(1,liberty_count + state.countEffect(defense, to));

	const bool neighboring = Neighboring8::isNeighboring8(to, king);
	Liberty liberty = neighboring 
	  ? countLibertyShortNotKnight(attack, to, ptype, king, info)
	  : countLibertyLong(attack, to, ptype, king, info);
	if (liberty.liberty == 0)
	  return std::max(liberty_count-1,1);
	if (! neighboring && liberty.has_effect)
	{
	  // TODO: ;ȶĴǤʤ liberty.liberty <=
	  // liberty_count ݤĤ褦Ĵ
	  ++liberty.liberty;	// ʬ⤷ξξ
	}
	  
	liberty.liberty += state.countEffect(defense, to);
	if (move.isDrop())
	{
	  if (neighboring)
	  {
	    if (state.countEffect(attack, to))
	      --liberty.liberty;		// adjust king capture
	  }
	  assert(liberty.liberty);
	  return liberty.liberty;
	}
	// ư: ΤƤʤɤϤ
	if (neighboring)
	{
	  if (state.countEffect(attack, to) >= 2
	      || effect_util::AdditionalEffect::hasEffect(state, to, attack))
	    --liberty.liberty;		// adjust king capture
	}
	assert(liberty.liberty);
	return liberty.liberty;
      }
      /** ƥ */
      int countLiberty(const NumEffectState& state, Move move) const;

      /** drop Τ */
      int libertyAfterAllDrop(const NumEffectState& state) const;
      int libertyAfterAllDrop(const NumEffectState& state, Player attack,
			      King8Info info, Position king) const
      {
	assert(state.getTurn() == attack);
	int result = info.libertyCount()-1;
	if (result < 2)
	  return 1;
	const unsigned int ld_mask = info.libertyDropMask();
	const PieceMask stand_mask=state.standMask(attack);
	for (; 
	     result > 1
	       && (stand_mask & table->drop_liberty[ld_mask][result-1]).any();
	     --result)
	{
	}
	return result;	
      }
      /** ư Τ */
      int libertyAfterAllMove(const NumEffectState& state) const;
      int libertyAfterAllMove(const NumEffectState& state, Player attack,
			      King8Info info, Position king) const;
      /**  */
      int libertyAfterAllCheck(const NumEffectState& state) const;

      int disproofAfterAllCheck(const NumEffectState&, Player, King8Info) const;
      /**  */
      const ProofDisproof attackEstimation(const NumEffectState& state) const;
      const ProofDisproof attackEstimation(const NumEffectState& state,
					   Player attack,
					   King8Info info, Position king) const;
    };
    extern const ProofNumberTable Proof_Number_Table;
  }
}

#endif /* _CHECKMATE_IMMEDIATE_CHECKMATE_TABLE_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:

