#ifndef _NUM_SIMPLE_EFFECT_H
#define _NUM_SIMPLE_EFFECT_H

#include "osl/effect/numBitmapEffect.h"
#include "osl/misc/carray.h"
#include "osl/apply_move/doUndoMoveLockForward.h"
#include "osl/apply_move/applyDoUndoXMove.h"
#include "osl/ptype.h"
#include "osl/piece.h"

#include <boost/static_assert.hpp>

namespace osl 
{
  namespace state
  {
    class SimpleState;
  }
  namespace effect
  {
    template<class State>
    class NumSimpleEffectTable;
    template<class State>
    bool operator==(const NumSimpleEffectTable<State>&,const NumSimpleEffectTable<State>&);
    template<class State>
    std::ostream& operator<<(std::ostream&, const NumSimpleEffectTable<State>&);


    struct NumSimpleEffectUtil
    {
      static const mask_t longEffectMask() {
#if OSL_WORDSIZE == 64
	return mask_t::makeDirect(0xff0000000000uLL);
#elif OSL_WORDSIZE == 32
	return mask_t::makeDirect(0xff00u);
#endif  
      }
#if OSL_WORDSIZE == 64
      static const int longToNumOffset=-8;
#elif OSL_WORDSIZE == 32
      static const int longToNumOffset=32-8;
#endif
      static const mask_t makeLongMask(int num)
      {
	return mask_t::makeDirect(0x101) << PieceMask::numToOffset(num);
      }
      template<Player P>
      static NumBitmapEffect makeLongEffect(int num){
	assert(32<=num && num<=39);
	NumBitmapEffect effect=NumBitmapEffect::playerEffect<P>();
	effect.orMask(1,makeLongMask(num));
	return effect;
      }
      static NumBitmapEffect makeLongEffect(Player pl,int num){
	assert(32<=num && num<=39);
	NumBitmapEffect effect=NumBitmapEffect::playerEffect(pl);
	effect.orMask(1,makeLongMask(num));
	return effect;
      }
    };
    /**
     * Τǡ.
     */
    template<class State>
    class NumSimpleEffectTable
    {
    protected:
      CArray<NumBitmapEffect, Position::SIZE> effects;
    public:
      /**
       * ֤餢ûˡ򹹿.
       * @param P(template) - ֤ˤνͭ
       * @param T(template) - ֤ˤμ
       * @param D(template) - νͭԤΩ줫鸫
       * @param OP(template) - ­餹
       * @param pos - ΰ
       * @param num - ֹ
       */
      template<Player P,Ptype T,Direction Dir,NumBitmapEffect::Op OP>
      void doEffectShort(Position pos,int num)
      {
	if ((PtypeTraits<T>::moveMask & DirectionTraits<Dir>::mask)!=0)
	  {
	    const Position target = pos+DirectionPlayerTraits<Dir,P>::offset();
	    effects[target.index()].template opEqual<OP>(NumBitmapEffect::makeEffect<P>(num));
	  }
      }
      /**
       * ֤餢Ĺˡ򹹿.
       * @param P(template) - ֤ˤνͭ
       * @param T(template) - ֤ˤμ
       * @param D(template) - Ω줫鸫
       * @param OP(template) - ­餹
       * @param state - (ư)
       * @param pos - ΰ
       * @param num - ֹ
       */
      template<Player P,Ptype T,Direction Dir,NumBitmapEffect::Op OP>
      void doEffectLong(const State& state,Position pos,int num)
      {
	if ((PtypeTraits<T>::moveMask & DirectionTraits<DirectionPlayerTraits<Dir,P>::directionByBlack>::mask)!=0)
	  {

	    Offset offset=DirectionPlayerTraits<Dir,BLACK>::offset();
	    assert(!offset.zero());
	    NumBitmapEffect effect=NumSimpleEffectUtil::makeLongEffect<P>(num);

	    for (;;)
	      {
		pos=pos+offset;
		effects[pos.index()].template opEqual<OP>(effect);
		// effectemptyޤ褦ˤûʤ
		if (! state.getPieceAt(pos).isEmpty()) break;
	      }
	  }
      }
      /**
       * ζ𤬻򹹿.
       * @param P(template) - ֤ˤνͭ
       * @param T(template) - ֤ˤμ
       * @param OP(template) - ­餹
       * @param state - (ư)
       * @param pos - ΰ
       * @param num - ֹ
       */
      template<Player P,Ptype T,NumBitmapEffect::Op OP>
      void doEffectBy(const State& state,Position pos,int num);
      /**
       * ζ𤬻򹹿.
       * @param OP(template) - ­餹
       * @param state - (ư)
       * @param ptypeo - μ
       * @param pos - ΰ
       * @param num - ֹ
       */
      template<NumBitmapEffect::Op OP>
      void doEffect(const State& state,PtypeO ptypeo,Position pos,int num);

      /**
       * 𤬻򹹿.
       * @param OP(template) - ­餹
       * @param state - (ư)
       * @param p - 
       */
      template<NumBitmapEffect::Op OP>
      void doEffect(const State& state,Piece p)
      {
	doEffect<OP>(state,p.ptypeO(),p.position(),p.number());
      }
      /**
       * ̤Υǡ򸵤˽.
       * @param state - 
       */
      void init(const State& state);
      /**
       * 󥹥ȥ饯.
       */
      NumSimpleEffectTable(const State& state)
      {
	init(state);
      }
      /**
       * ֤ǡФ.
       * @param pos - 
       */
      const NumBitmapEffect getEffect(Position pos) const
      {
	return effects[pos.index()];
      }
      /**
       * pos˶/Ĺ֥å/Ĺݤǡι.
       * @param OP(template) - ­餹
       * @param state - ̤ξ pos˶֤ǤǤ褤
       * @param pos - Ѳ
       */
      template<NumBitmapEffect::Op OP>
      void doBlockAt(const State& state,Position pos);
      friend bool operator== <>(const NumSimpleEffectTable& et1,const NumSimpleEffectTable& et2);
    };

    template<class State>
    inline bool operator!=(const NumSimpleEffectTable<State>& et1,const NumSimpleEffectTable<State>& et2)
    {
      return !(et1==et2);
    }

    template<class State>
    class NumSimpleEffect;
    template<class State>
    bool operator==(const NumSimpleEffect<State>& st1, const NumSimpleEffect<State>& st2);

    /**
     * Ķ
     * - effects (NumSimpleEffectTable) 
     * - onBoardMask (PieceMask) ׾ˤ
     */
    template<class State>
    class NumSimpleEffect: public State
    {
    public:
      /** Ѿ饹 effect ĤФ˻Ѥ */
      typedef NumSimpleEffect<State> effect_state_t;
    private:
      NumSimpleEffectTable<typename State::simple_state_t> effects;
      CArray<PieceMask,2> onBoardMask;

      friend bool operator==<>(const NumSimpleEffect& st1,
			       const NumSimpleEffect& st2);
      typedef NumSimpleEffect state_t;
      friend class apply_move::ApplyDoUndoSimpleMove<BLACK,NumSimpleEffect>;
      friend class apply_move::ApplyDoUndoCaptureMove<BLACK,NumSimpleEffect>;
      friend class apply_move::ApplyDoUndoDropMove<BLACK,NumSimpleEffect>;
      friend class apply_move::ApplyDoUndoSimpleMove<WHITE,NumSimpleEffect>;
      friend class apply_move::ApplyDoUndoCaptureMove<WHITE,NumSimpleEffect>;
      friend class apply_move::ApplyDoUndoDropMove<WHITE,NumSimpleEffect>;
      PieceMask& getOnBoardMask(Player p) {
	return onBoardMask[playerToIndex(p)];
      }
    public:
      const PieceMask& getOnBoardMask(Player p) const {
	return onBoardMask[playerToIndex(p)];
      }
      bool isOnBoardNum(int num) const
      {
	return getOnBoardMask(BLACK).test(num) ||
	  getOnBoardMask(WHITE).test(num);
      }

      Piece getPieceOf(int num) const
      {
	return State::getPieceOf(num);
      }
      /** T  isBasic ǤʤƤư unpromote(T) η̤Ʊ.
       * 64bitǤʤϤ
       */ 
      template<Player P,Ptype T,typename F>
      void forEachOnBoard(F& func) const {
	mask_t onMask=getOnBoardMask(P).getMask(PtypeFuns<T>::indexNum) 
	  & mask_t::makeDirect(PtypeFuns<T>::indexMask);
    
	while (onMask.any())
	  {
	    int num=onMask.takeOneBit()+((PtypeFuns<T>::indexNum)<<5);
	    Piece p = getPieceOf(num);
	    func(p);
	  }
      }

      explicit NumSimpleEffect(const state::SimpleState& st);
      NumBitmapEffect getEffect(Position pos) const
      {
	return effects.getEffect(pos);
      }
      bool isConsistent(bool showError) const;
      // move methods
      void doSimpleMove(Position from, Position to, int promoteMask);
      void doDropMove(Position to,Ptype ptype);
      void doCaptureMove(Position from, Position to, Piece target,int promoteMask);
      /**
       * ޥˤDirectionǤĹ뤫ɤ.
       * 64bitбѤ
       */
      template<Direction Dir,Player P>
      bool hasEffectDir(Position to) const {
	BOOST_STATIC_ASSERT( (DirectionTraits<Dir>::isLong) );
	const PieceMask& onBoardMask=getOnBoardMask(P);
	mask_t mask1=onBoardMask.getMask(1);
	mask1 &= ((PtypeDirectionTraits<LANCE,Dir>::canMove 
		   ? mask_t::makeDirect(PtypeFuns<LANCE>::indexMask) 
		   : mask_t::makeDirect(0)) 
		  | (PtypeDirectionTraits<BISHOP,Dir>::canMove 
		     ?  mask_t::makeDirect(PtypeFuns<BISHOP>::indexMask) 
		     :  mask_t::makeDirect(0))
		  | (PtypeDirectionTraits<ROOK,Dir>::canMove 
		     ? mask_t::makeDirect(PtypeFuns<ROOK>::indexMask)
		     : mask_t::makeDirect(0)));
	mask1 <<= 8;
	// ûӽ
	mask1&=getEffect(to).getMask(1)& NumSimpleEffectUtil::longEffectMask();
	while (mask1.any())
	  {
	    int num=mask1.takeOneBit()+NumSimpleEffectUtil::longToNumOffset;
	    Position from = getPieceOf(num).position();
	    Direction dir=Board_Table.getLongDirection<BLACK>(Offset32(to,from));
	    if (dir==DirectionPlayerTraits<Dir,P>::directionByBlack)
	      return true;
	  }
	return false;
      }
      /** 
       * оݤȤޥˤץ쥤䡼뤫ɤ.
       * @param player ¦
       * @param target оݤΥޥ
       */
      bool hasEffectBy(Player player,Position target) const {
	assert(target.isOnBoard());
	mask_t mask=getEffect(target).getMask(1);
	mask&=NumBitmapEffect::playerEffectMask(player);
	return !mask.none();
      }


      /** 
       * оݤȤޥˤץ쥤䡼(ʳ)뤫ɤ.
       * ꤷ뤳Ȥ.
       * @param player ¦
       * @param piece ¦ζ
       * @param target оݤΥޥ
       */
      bool hasEffectNotBy(Player player,Piece piece,Position target) const {
	assert(piece.owner()==player);
	PieceMask onBoardMask=getOnBoardMask(player);
	int num=piece.number();
	onBoardMask.reset(num);
	return (onBoardMask&getEffect(target)).any();
      }
      /** 
       * İʾζ𤫤.
       * ⤽ʤƤФ
       * @param player ¦
       * @param target оݤΥޥ
       */
      bool hasMultipleEffectBy(Player player,Position target) const 
      {
	mask_t mask=getEffect(target).getMask(1);
	mask&=NumBitmapEffect::playerEffectMask(player);
	return NumBitmapEffect::playerEffect(player).getMask(1) < mask;
      }

      /** 
       * attack  target Ĥ
       * @param target оݤΥޥ
       */
      bool hasEffectByPiece(Piece attack, Position target) const 
      {
	assert(target.isOnBoard());
	return getEffect(target).test(attack.number());
      }

      /**
       * target  ptype 뤫?
       */
      template <Ptype PTYPE>
      bool hasEffectByPtype(Player attack, Position target) const
      {
	mask_t mask=getEffect(target).getMask(PtypeFuns<PTYPE>::indexNum);
	mask&=mask_t::makeDirect(PtypeFuns<PTYPE>::indexMask);
	const PieceMask& onBoardMask=getOnBoardMask(attack);
	mask&=onBoardMask.getMask(PtypeFuns<PTYPE>::indexNum);
	return mask.any();
      }

      /**
       * ο2ޤǿ
       * @return 0,1,2 (2ξ2ʾ)
       */
      int countEffect2(Player player,Position target) const 
      {
	assert(target.isOnBoard());
	return std::min(2,countEffect(player,target));
      }
      /**
       * ο. 
       * targetפϤ߽ФƤϤʤ
       */
      int countEffect(Player player,Position target) const 
      {
	assert(target.isOnBoard());
	return getEffect(target).countEffect(player);
      }
      /**
       * ο. 
       * targetפϤ߽ФƤϤʤ
       * @param pins ζϿʤ
       */
      int countEffect(Player player,Position target, PieceMask pins) const 
      {
	assert(target.isOnBoard());
	const NumBitmapEffect effect = getEffect(target);
	const int all = effect.countEffect(player);
	pins &= effect;
	return all - pins.countBit();
      }
    private:
      int countKingEffect(Player P, PieceMask pieces) const 
      {
	if (pieces.test(PtypeTable::getKingIndex(P)))
	  return pieces.countBit2();
	return pieces.none() ? 0 : 2;
      }
    public:
      /**
       * ̵̤ͭȤʳζ2ޤǿ
       * @return 0 ʤˤʤ, 1 ̤Τ, 2 ʳ1İʾ夢
       */
      int countKingEffect(Player P, Position target) const 
      {
	assert(target.isOnBoard());
	const PieceMask pieces
	  = getOnBoardMask(P) & getEffect(target);
	return countKingEffect(P, pieces);
      }
      int countKingEffect(Player P, Position target, PieceMask dont_use) const 
      {
	assert(target.isOnBoard());
	PieceMask pieces = getOnBoardMask(P);
	pieces ^= dont_use;
	return countKingEffect(P, pieces & getEffect(target));
      }
      /**
       * 1Ĥpiece֤
       * ʤPiece::EMPTY()֤
       * @param P(template) ¦
       * @param p åȤΰ
       */
      template<Player P> 
      Piece hasEffectPiece(Position p) const
      {
	assert(p.isOnBoard());
#if OSL_WORDSIZE == 64
	const PieceMask pieceMask=getOnBoardMask(P)&getEffect(p);
	if (pieceMask.none()) return Piece::EMPTY();
	return getPieceOf(pieceMask.getMask(0).bsf());
#elif OSL_WORDSIZE == 32
	const PieceMask& onBoardMask=getOnBoardMask(P);
	mask_t mask0=onBoardMask.getMask(0);
	NumBitmapEffect effect=getEffect(p);
	mask0&=effect.getMask(0);
	if (mask0.any())
	  {
	    return getPieceOf(mask0.bsf());
	  }
	else{
	  mask_t mask1=effect.getMask(1);
	  mask1|=(mask1>>8);
	  mask1&=onBoardMask.getMask(1);
	  if (mask1.any())
	    {
	      int num=mask1.bsf()+32;
	      assert(32<=num && num<=39);
	      return getPieceOf(num);
	    }
	  else{
	    return Piece::EMPTY();
	  }
	}
#endif
      }
      Piece hasEffectPiece(Player pl,Position pos) const
      {
	if (pl==BLACK) return hasEffectPiece<BLACK>(pos);
	else return hasEffectPiece<WHITE>(pos);
      }
      /**
       * @param P ¦
       * @param Type 
       */
      template<Player P,Ptype Type,class Action>
      void forEachEffectPtype(Position pos,Action & action) const {
	mask_t mask=getEffect(pos).getMask(PtypeFuns<Type>::indexNum);
	mask&=mask_t::makeDirect(PtypeFuns<Type>::indexMask);
	const PieceMask& onBoardMask=getOnBoardMask(P);
	mask&=onBoardMask.getMask(PtypeFuns<Type>::indexNum);
	while (mask.any())
	  {
	    int num=mask.takeOneBit()+(PtypeFuns<Type>::indexNum<<5);
	    action.template doActionPtype<P,Type>(getPieceOf(num),pos);
	  }
      }

      /** 
       * pos ˤ move  action  member ƤӽФ.
       * @param action Ȥ AlwayMoveAction
       */
      template<Player P,class Action>
      void forEachEffect(Position pos,Action & action) const
      {
	const PieceMask pieceMask=getOnBoardMask(P)&getEffect(pos);
#if OSL_WORDSIZE == 64
	mask_t mask=pieceMask.getMask(0);
	while (mask.any())
	  {
	    const int num=mask.takeOneBit();
	    action.template doAction<P>(getPieceOf(num),pos);
	  }
#elif OSL_WORDSIZE == 32
	mask_t mask0=pieceMask.getMask(0);
	while (mask0.any())
	  {
	    const int num=mask0.takeOneBit();
	    action.template doAction<P>(getPieceOf(num),pos);
	  }
	mask_t mask1=pieceMask.getMask(1);
	while (mask1.any())
	  {
	    const int num=mask1.takeOneBit()+32;
	    action.template doAction<P>(getPieceOf(num),pos);
	  }
#endif
      }

      /** 
       * pos ˤ move  action  member ƤӽФ
       * @param action Ȥ AlwayMoveAction
       * @param piece  ʳζȤ
       */
      template<Player P,class Action>
      void forEachEffectNotBy(Position pos,Piece piece,Action & action) const {
	PieceMask pieceMask=getOnBoardMask(P);
	pieceMask.reset(piece.number());
	PieceMask effectMask=(PieceMask)getEffect(pos);
	pieceMask=(pieceMask&effectMask);
#if OSL_WORDSIZE == 64
	mask_t mask=pieceMask.getMask(0);
	while (mask.any())
	  {
	    int num=mask.takeOneBit();
	    action.template doAction<P>(getPieceOf(num),pos);
	  }
#elif OSL_WORDSIZE == 32
	mask_t mask0=pieceMask.getMask(0);
	while (mask0.any())
	  {
	    int num=mask0.takeOneBit();
	    action.template doAction<P>(getPieceOf(num),pos);
	  }
	mask_t mask1=pieceMask.getMask(1);
	while (mask1.any())
	  {
	    int num=mask1.takeOneBit()+32;
	    action.template doAction<P>(getPieceOf(num),pos);
	  }
#endif
      }
    

      /** ®ٽŻǤϤʤֽŻ 
       * pos ˤ move  action  member ƤӽФ
       * @param action Ȥ AlwayMoveAction
       */
      template<Player P,class Action>
      void forSortedEffect(Position pos,Action & action) const ;
      /**
       * attackerPiecetargetPositionĤɤ.
       * targetPosition϶Ǥɤ
       * @param attackerPiece 
       * @param targetPosition
       */
      bool hasEffectTo(Piece attackerPiece,Position targetPosition) const
      {
	assert(attackerPiece.isPiece() && targetPosition.isOnBoard());
	return getEffect(targetPosition).test(attackerPiece.number());
      }

      /** 
       * оݤȤޥˤץ쥤䡼뤫ɤ.
       * @param player ¦
       * @param target оݤΥޥ
       */
      template<Player P>
      bool hasEffectBy(Position target) const {
	assert(target.isOnBoard());
	mask_t mask=getEffect(target).getMask(1);
	mask&=NumBitmapEffect::playerEffectMask<P>();
	return !mask.none();
      }
    
      bool hasEffectBy(Player P, Position target,Piece& attackerPiece) const 
      {
	if (P == BLACK)
	  return hasEffectBy<BLACK>(target, attackerPiece);
	else
	  return hasEffectBy<WHITE>(target, attackerPiece);
      }
      /**
       * @param P(template) - ĤƤ¦Υץ쥤
       * @param target - Ĥ줿
       * @param attackerPiece - multiple attackξPiece::EMPTY()
       *        ǤʤʤĤƤ֤
       */
      template<Player P>
      bool hasEffectBy(Position target,Piece& attackerPiece) const {
	attackerPiece=Piece::EMPTY();
	const PieceMask& pieceMask=getOnBoardMask(P)&getEffect(target);
#if OSL_WORDSIZE == 64
	mask_t mask=pieceMask.getMask(0);
	if (mask.none()) return false;
	/**
	 * mask|=0x8000000000000000ll;
	 * if((mask&(mask-1))!=0x8000000000000000ll) ʤ1ĤifʸǺѤ
	 */
	if (mask.hasMultipleBit())
	  return true;
	int num=mask.bsf();
	attackerPiece=getPieceOf(num);
	return true;
#elif OSL_WORDSIZE == 32
	mask_t mask0=pieceMask.getMask(0);
	mask_t mask1=pieceMask.getMask(1);
	if (mask0.any())
	  {
	    if (mask1.any())
	      return true;
	    int num=mask0.bsf();
	    if (mask0 == PieceMask::numToMask(num))
	      attackerPiece=getPieceOf(num);
	    return true;
	  }
	else if (mask1.any())
	  {
	    int num=mask1.bsf();
	    if (mask1==PieceMask::numToMask(num))
	      attackerPiece=getPieceOf(num+32);
	    return true;
	  }
	else 
	  return false;
#endif
      }
      /**
       * μ꤬꤫ɤ
       * Ĥζˤ벦ξattckPieceˤζ
       * ʣζˤ벦ξPiece::EMPTY()
       * @param P(template) - ߤμ(ä, movealt(P))
       */
      template<Player P>
      bool lastMoveIsCheck(Move /*last_move*/,  Piece& attack_piece) const
      {
	return hasEffectBy<PlayerTraits<P>::opponent>
	  (getPieceOf(KingTraits<P>::index).position(),attack_piece);
      }
      Player getTurn() const{
	return State::getTurn();
      }
      Piece getPieceAt(Position pos) const{
	return State::getPieceAt(pos);
      }
      template<Player P,Ptype T>
      bool hasPieceOnStand() const {
	return State::template hasPieceOnStand<P,T>();
      }
      bool hasPieceOnStand(Player pl,Ptype ptype) const{
	return State::hasPieceOnStand(pl,ptype);
      }
      bool isValidMoveByRule(Move move,bool showError) const;
      /**
       * ˡ꤫ɤñ˸롥̤˰¸åΤߡ
       * 롼ؤʤǤǽϡisValidMove Ѥ롥
       *
       * PtemplateˤǤȾ®ʤꤽ
       * ̤˰¸븡Ǥ⡤̤ȴ䲦ɤǤ뤫
       * ꡤͤɤϸʤ
       * @param showError(template) - false֤ˤϲɽ.
       * @param move - 
       */
      template <bool show_error>
      bool isAlmostValidMove(Move move) const;
      bool isAlmostValidMove(Move move,bool show_error=true) const;
    };

    template<class State>
    template<Player P,class Action>
    void NumSimpleEffect<State>::forSortedEffect(Position pos,Action & action) const {
      forEachEffectPtype<P,PAWN,Action>(pos,action);
      forEachEffectPtype<P,LANCE,Action>(pos,action);
      forEachEffectPtype<P,KNIGHT,Action>(pos,action);
      forEachEffectPtype<P,SILVER,Action>(pos,action);
      forEachEffectPtype<P,GOLD,Action>(pos,action);
      forEachEffectPtype<P,BISHOP,Action>(pos,action);
      forEachEffectPtype<P,ROOK,Action>(pos,action);
      forEachEffectPtype<P,KING,Action>(pos,action);
    }

    template<class State>
    inline bool operator!=(const NumSimpleEffect<State>& s1,
			   const NumSimpleEffect<State>& s2)
    {
      return !(s1==s2);
    }  
  } // namespace effect
  using effect::NumSimpleEffect;
  using effect::NumBitmapEffect;

  namespace apply_move
  {
    template<Player P,typename BaseState>
    struct ApplyDoUndoSimpleMove<P,NumSimpleEffect<BaseState> >
    {
      typedef NumSimpleEffect<BaseState> state_t;
      template <typename F>
      static void doUndoSimpleMove(state_t& s, 
				   Position from, Position to, int promoteMask,F& func);

      static 
      void prologue(state_t& s, Position from, Position to, int promoteMask,
		    Piece& oldPiece, int& num, 
		    PtypeO& oldPtypeO, PtypeO& newPtypeO)
      {
	oldPiece=s.getPieceAt(from);
	Piece newPiece=oldPiece.promoteWithMask(promoteMask);
	newPiece+=(to-from);
	num=oldPiece.number();

	oldPtypeO=oldPiece.ptypeO();
	newPtypeO=newPiece.ptypeO();
	// ʬȤθ򳰤
	s.setPieceOf(num,newPiece);
	s.effects.template doEffect<NumBitmapEffect::Sub>(s,oldPtypeO,from,num);
	// ʬȤ֥åƤpromoteαĹ
	// 뤤ϼʬȤΥ֥å
	s.setBoard(to,newPiece);
	s.effects.template doBlockAt<NumBitmapEffect::Sub>(s,to);
	s.setBoard(from,Piece::EMPTY());
	s.effects.template doBlockAt<NumBitmapEffect::Add>(s,from);
	s.effects.template doEffect<NumBitmapEffect::Add>(s,newPtypeO,to,num);
      }
      static 
      void epilogue(state_t& s, Position from, Position to, Piece oldPiece, 
		    int num, PtypeO oldPtypeO, PtypeO newPtypeO)
      {
	s.setPieceOf(num,oldPiece);
	s.effects.template doEffect<NumBitmapEffect::Sub>(s,newPtypeO,to,num);
	s.setBoard(from,oldPiece);
	s.effects.template doBlockAt<NumBitmapEffect::Sub>(s,from);
	s.setBoard(to,Piece::EMPTY());
	s.effects.template doBlockAt<NumBitmapEffect::Add>(s,to);
	s.effects.template doEffect<NumBitmapEffect::Add>(s,oldPtypeO,from,num);
      }
    };
  
  
    template<Player P,typename BaseState>
    template <typename F>
    void ApplyDoUndoSimpleMove<P,NumSimpleEffect<BaseState> >::
    doUndoSimpleMove(state_t& s, 
		     Position from, Position to, int promoteMask,F& func)
    {
      Piece oldPiece;
      int num;
      PtypeO oldPtypeO, newPtypeO;
      prologue(s, from, to, promoteMask, oldPiece, num, oldPtypeO, newPtypeO);

      if (promoteMask!=0 && num < PtypeTraits<PAWN>::indexLimit)
	{
	  s.clearPawn(P,from);
	  s.changeTurn();
	  func(to);
	  s.changeTurn();
	  s.setPawn(P,from);
	}
      else
	{
	  s.changeTurn();
	  func(to);
	  s.changeTurn();
	}

      epilogue(s, from, to, oldPiece, num, oldPtypeO, newPtypeO);
    }

    template<Player P,typename BaseState>
    struct ApplyDoUndoDropMove<P,NumSimpleEffect<BaseState> >
    {
      typedef NumSimpleEffect<BaseState> state_t;
      template <typename F>
      static void doUndoDropMove(state_t& s, 
				 Position to, Ptype ptype,F& func);
      static 
      void prologue(state_t& s, Position to, Ptype ptype,
		    Piece& oldPiece, int& num, PtypeO& ptypeO, 
		    int& numIndex, mask_t& numMask)
      {
#if OSL_WORDSIZE == 64
	numIndex=0;
#elif OSL_WORDSIZE == 32
	numIndex=Ptype_Table.getIndex(ptype);
#endif
	const mask_t ownMochigoma=
	  s.standMask(P).getMask(numIndex) & Ptype_Table.getMaskLow(ptype);
	assert(ownMochigoma.any());
	numMask=ownMochigoma.lowestBit();
	int numLow = ownMochigoma.bsf();
	num = numLow|(numIndex<<5);
	oldPiece=s.getPieceOf(num);
	Piece newPiece=oldPiece;
	newPiece+=to-Position::STAND();
	ptypeO=newPiece.ptypeO();
	s.setPieceOf(num,newPiece);
	s.effects.template doEffect<NumBitmapEffect::Add>(s,ptypeO,to,num);
	s.setBoard(to,newPiece);
	s.effects.template doBlockAt<NumBitmapEffect::Sub>(s,to);
	s.standMask(P).xorMask(numIndex,numMask);
	s.getOnBoardMask(P).xorMask(numIndex,numMask);
      }

      static 
      void epilogue(state_t& s, Position to, Piece oldPiece, 
		    int num, PtypeO ptypeO, int numIndex, mask_t numMask)
      {
	s.standMask(P).xorMask(numIndex,numMask);
	s.getOnBoardMask(P).xorMask(numIndex,numMask);
	s.setBoard(to,Piece::EMPTY());
	s.effects.template doEffect<NumBitmapEffect::Sub>(s,ptypeO,to,num);
	s.effects.template doBlockAt<NumBitmapEffect::Add>(s,to);
	s.setPieceOf(num,oldPiece);
      }
    };
  
    template<Player P,typename BaseState>
    template <typename F>
    void ApplyDoUndoDropMove<P,NumSimpleEffect<BaseState> >::
    doUndoDropMove(state_t& s, 
		   Position to, Ptype ptype,F& func)
    {
      Piece oldPiece;
      PtypeO ptypeO;
      int num, numIndex;
      mask_t numMask;
      prologue(s, to, ptype, oldPiece, num, ptypeO, numIndex, numMask);

      if (ptype==PAWN)
	{
	  s.setPawn(P,to);
	  s.changeTurn();
	  func(to);
	  s.changeTurn();
	  s.clearPawn(P,to);
	}
      else
	{
	  s.changeTurn();
	  func(to);
	  s.changeTurn();
	}
      epilogue(s, to, oldPiece, num, ptypeO, numIndex, numMask);
    
    }

    template<Player P,typename BaseState>
    struct ApplyDoUndoCaptureMove<P,NumSimpleEffect<BaseState> >
    {
      typedef NumSimpleEffect<BaseState> state_t;
      template <typename F>
      static void doUndoCaptureMove(state_t& s, Position from,Position to, 
				    Piece target, int promoteMask,F& func);
      static
      void prologue(state_t& s, Position from, Position to, Piece target, 
		    int promoteMask,
		    Piece& oldPiece, PtypeO& oldPtypeO, PtypeO& capturePtypeO, 
		    PtypeO& newPtypeO, int& num0, int& num1, 
		    int& num1Index, mask_t& num1Mask)
      {
	num1=target.number();
	num1Index=PieceMask::numToIndex(num1);
	num1Mask=PieceMask::numToMask(num1);
	s.getOnBoardMask(PlayerTraits<P>::opponent).xorMask(num1Index,num1Mask);
	s.standMask(P).xorMask(num1Index,num1Mask);
	oldPiece=s.getPieceAt(from);
	Piece newPiece=oldPiece.promoteWithMask(promoteMask);
	newPiece+=(to-from);
	num0=oldPiece.number();
	s.setPieceOf(num0,newPiece);
	s.setPieceOf(num1,target.captured());
      
	oldPtypeO=oldPiece.ptypeO();
	newPtypeO=newPiece.ptypeO();
	capturePtypeO=target.ptypeO();
	s.effects.template doEffect<NumBitmapEffect::Sub>(s,capturePtypeO,to,num1);
	s.effects.template doEffect<NumBitmapEffect::Sub>(s,oldPtypeO,from,num0);
	s.setBoard(to,newPiece);
	s.setBoard(from,Piece::EMPTY());
	s.effects.template doBlockAt<NumBitmapEffect::Add>(s,from);
	s.effects.template doEffect<NumBitmapEffect::Add>(s,newPtypeO,to,num0);
      }
      static
      void epilogue(state_t& s, Position from, Position to, Piece target, 
		    Piece oldPiece, PtypeO oldPtypeO, PtypeO capturePtypeO, 
		    PtypeO newPtypeO, int num0, int num1, 
		    int num1Index, mask_t num1Mask)
      {
	s.standMask(P).xorMask(num1Index,num1Mask);
	s.getOnBoardMask(PlayerTraits<P>::opponent).xorMask(num1Index,num1Mask);
	s.setPieceOf(num0,oldPiece);
	s.setPieceOf(num1,target);
	s.effects.template doEffect<NumBitmapEffect::Sub>(s,newPtypeO,to,num0);
	s.effects.template doEffect<NumBitmapEffect::Add>(s,capturePtypeO,to,num1);
	s.setBoard(from,oldPiece);
	s.effects.template doBlockAt<NumBitmapEffect::Sub>(s,from);
	s.setBoard(to,target);
	s.effects.template doEffect<NumBitmapEffect::Add>(s,oldPtypeO,from,num0);
      }
    
    };
  
    template<Player P,typename BaseState>
    template <typename F>
    void ApplyDoUndoCaptureMove<P,NumSimpleEffect<BaseState> >::
    doUndoCaptureMove(state_t& s, Position from,Position to, Piece target, 
		      int promoteMask,F& func)
    {
      Piece oldPiece;
      PtypeO oldPtypeO, capturePtypeO, newPtypeO;
      int num0, num1, num1Index;
      mask_t num1Mask;
      prologue(s, from, to, target, promoteMask, oldPiece, oldPtypeO, 
	       capturePtypeO, newPtypeO, num0, num1, num1Index,num1Mask);

      s.changeTurn();
      const Ptype capturePtype=target.ptype();
      if (capturePtype==PAWN)
	{
	  s.clearPawn(PlayerTraits<P>::opponent,to);
	  if (promoteMask!=0 && num0<PtypeTraits<PAWN>::indexLimit)
	    {
	      s.clearPawn(P,from);
	      func(to);
	      s.setPawn(P,from);
	    }
	  else
	    {
	      func(to);
	    }
	  s.setPawn(PlayerTraits<P>::opponent,to);
	}
      else if (promoteMask!=0 && num0<PtypeTraits<PAWN>::indexLimit)
	{
	  s.clearPawn(P,from);
	  func(to);
	  s.setPawn(P,from);
	}
      else
	{
	  func(to);
	}
      s.changeTurn();

      epilogue(s, from, to, target, oldPiece, oldPtypeO, capturePtypeO, newPtypeO, 
	       num0, num1, num1Index,num1Mask);
    }

    template <class BaseState>
    class DoUndoMoveLockSimple<NumSimpleEffect<BaseState> >;
    template <class BaseState>
    class DoUndoMoveLockDrop<NumSimpleEffect<BaseState> >;
    template <class BaseState>
    class DoUndoMoveLockCapture<NumSimpleEffect<BaseState> >;
    
  } // namespace apply_move

} // namespace osl
#endif // _NUM_SIMPLE_EFFECT_H
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
