/* eachPlayerAttackDefense.h
 */
#ifndef EVAL_EACHPLAYERATTACKDEFENSE_H
#define EVAL_EACHPLAYERATTACKDEFENSE_H

#include "osl/eval/eachPlayerPieceEval.h"
#include "osl/eval/ppair/piecePairRawEval.h"
#include "osl/eval/endgame/attackDefense.h"
#include "osl/progress/effect5x3.h"
#include "osl/state/numEffectState.h"
namespace osl
{
  namespace eval
  {
    /**
     * progress򸫤ʤ鹶βͤɸβͤmax롥
     */
    class EachPlayerAttackDefense
    {
    public:
      typedef PiecePairRawEval additional_eval_t;
      typedef PieceEval opening_eval_t;
      typedef endgame::AttackKing endgame_attack_t;
      typedef endgame::DefenseKing endgame_defense_t;
      typedef progress::Effect5x3 progress_t;
    private:
      int main_value;
      additional_eval_t additional_eval;
      progress_t current_progress;

      void addValue(const CArray<Piece,2>& kings,
		    const CArray<Progress16,2>& dangers,
		    Piece target)
      {
	main_value += valueOf(kings, dangers, target);
      }
    public:
      EachPlayerAttackDefense(const NumEffectState& state);

      static bool initialized() 
      { 
	return additional_eval_t::initialized();
      } 
      static bool setUp(const char *filename)
      {
	return additional_eval_t::setUp(filename);
      }

      const Progress16 progress16(Player p) const 
      { 
	return current_progress.progress16(p);
      }
      int value() const
      {
	return main_value 
	  + compose(current_progress.progress16(),
		    additional_eval.value(), 0); 
      }

      static int compose(Progress16 progress16, int opening, int endgame)
      {
	assert(progress16.isValid());
	assert(opening % 2 == 0);
	assert(endgame % 2 == 0);
	const int result = opening*(16-progress16.value())
	  + endgame*progress16.value();
	assert(result % 2 == 0);
	return result;
      }
      static int valueOf(const CArray<Piece,2>& kings,
			 const CArray<Progress16,2>& dangers,
			 Piece target)
      {
	const Player player = target.owner();
	return valueOf(kings[playerToIndex(player)],
		       kings[playerToIndex(alt(player))],
		       dangers[playerToIndex(player)], 
		       dangers[playerToIndex(alt(player))],
		       target.ptypeO(), target.position());
      }
      static int valueOf(Piece my_king, Piece op_king,
			 Progress16 my_danger, Progress16 op_danger,
			 PtypeO ptypeo, Position position)
      {	  
	const Player player = getOwner(ptypeo);
	assert(my_king.owner() == player);
	assert(op_king.owner() == alt(player));

	const int opening_value = opening_eval_t::value(ptypeo);
	const int endgame_attack
	  = endgame_attack_t::valueOf(op_king, ptypeo, position);
	const int endgame_defense
	  = endgame_defense_t::valueOf(my_king, ptypeo, position);

	// δȼ̤
	int attack = compose(op_danger, opening_value, endgame_attack);
	if (op_danger < my_danger)
	{
	  // ʬȵޤǹɬפ
	  const int attack_my = compose(my_danger, opening_value, endgame_attack);
	  attack = min(player, attack, attack_my);
	}
	// δȼ
	const int defense = compose(my_danger, opening_value, endgame_defense);

	return max(player, attack, defense);
      }

      static void setValues(const SimpleState&, 
			    Progress16 black_progress16,
			    Progress16 white_progress16,
			    container::PieceValues&);
      static void setValues(const SimpleState& s, container::PieceValues& o);


      private:
	void updateAll(const SimpleState&, Position from, Position to,
		       int promote_mask);
	void updateAll(const SimpleState&, Position to, Ptype ptype);
	void updateAll(const SimpleState&, Position from, Position to,
		       Piece target, int promote_mask);
    public:
      // ApplyMoveOfTurn Ȥinterface
      void doSimpleMove(const NumEffectState& state,
			Position from, Position to, int promote_mask)
      {
	const CArray<Progress16,2> old_progress
	  = {{ progress16(BLACK), progress16(WHITE) }};

	additional_eval.doSimpleMove(state,from,to,promote_mask);
	current_progress.doSimpleMove(state,from,to,promote_mask);

	const Piece old_piece = state.getPieceOnBoard(from);
	if (old_piece.ptype() == KING)
	{
	  updateAll(state, from, to, promote_mask);
	  return;
	}
	const CArray<Progress16,2> new_progress
	  = {{ progress16(BLACK), progress16(WHITE) }};
	if (! (old_progress == new_progress))
	{
	  updateAll(state, from, to, promote_mask);
	  return;
	}

	const PtypeO new_ptypeo 
	  = promoteWithMask(old_piece.ptypeO(), promote_mask);
	const Player player = old_piece.owner();
	const Piece my_king = state.getKingPiece(player);
	const Piece op_king = state.getKingPiece(alt(player));

	const int inc = valueOf(my_king, op_king, 
				new_progress[playerToIndex(player)],
				new_progress[playerToIndex(alt(player))],
				new_ptypeo, to);
	const int dec = valueOf(my_king, op_king, 
				old_progress[playerToIndex(player)],
				old_progress[playerToIndex(alt(player))],
				old_piece.ptypeO(), from);
	main_value += (inc - dec);
      }
      void doDropMove(const NumEffectState& state,Position to,Ptype ptype)
      {
	assert(ptype != KING);
	const CArray<Progress16,2> old_progress
	  = {{ progress16(BLACK), progress16(WHITE) }};

	additional_eval.doDropMove(state,to,ptype);
	current_progress.doDropMove(state,to,ptype);

	const CArray<Progress16,2> new_progress
	  = {{ progress16(BLACK), progress16(WHITE) }};
	if (! (old_progress == new_progress))
	{
	  updateAll(state, to, ptype);
	  return;
	}

	const Player player = state.getTurn();
	const PtypeO ptypeo = newPtypeO(player, ptype);
	
	const Piece my_king = state.getKingPiece(player);
	const Piece op_king = state.getKingPiece(alt(player));

	const int inc = valueOf(my_king, op_king, 
				new_progress[playerToIndex(player)],
				new_progress[playerToIndex(alt(player))],
				ptypeo, to);
	const int dec = valueOf(my_king, op_king, 
				old_progress[playerToIndex(player)],
				old_progress[playerToIndex(alt(player))],
				ptypeo, Position::STAND());
	main_value += (inc - dec);
      }
      void doCaptureMove(const NumEffectState& state,
			 Position from, Position to, 
			 Piece victim,int promote_mask)
      {
	const CArray<Progress16,2> old_progress
	  = {{ progress16(BLACK), progress16(WHITE) }};

	additional_eval.doCaptureMove(state,from,to,victim,promote_mask);
	current_progress.doCaptureMove(state,from,to,victim,promote_mask);

	const Piece old_piece = state.getPieceOnBoard(from);
	if (old_piece.ptype() == KING)
	{
	  updateAll(state, from, to, victim, promote_mask);
	  return;
	}
	const CArray<Progress16,2> new_progress
	  = {{ progress16(BLACK), progress16(WHITE) }};
	if (! (old_progress == new_progress))
	{
	  updateAll(state, from, to, victim, promote_mask);
	  return;
	}

	const PtypeO new_ptypeo 
	  = promoteWithMask(old_piece.ptypeO(), promote_mask);
	const Player player = old_piece.owner();
	const Piece my_king = state.getKingPiece(player);
	const Piece op_king = state.getKingPiece(alt(player));

	const int inc_capture = valueOf(my_king, op_king, 
					new_progress[playerToIndex(player)],
					new_progress[playerToIndex(alt(player))],
					captured(victim.ptypeO()), 
					Position::STAND());
	const int dec_capture = valueOf(op_king, my_king,
					old_progress[playerToIndex(alt(player))],
					old_progress[playerToIndex(player)],
					victim.ptypeO(), to);
	main_value += (inc_capture - dec_capture);
	const int inc = valueOf(my_king, op_king, 
				new_progress[playerToIndex(player)],
				new_progress[playerToIndex(alt(player))],
				new_ptypeo, to);
	const int dec = valueOf(my_king, op_king, 
				old_progress[playerToIndex(player)],
				old_progress[playerToIndex(alt(player))],
				old_piece.ptypeO(), from);
	main_value += (inc - dec);
      }
    public:
      static int infty()
      {
	return opening_eval_t::infty()*2 * 16; // 2̤Υܡʥ
      }
      static int captureValue(PtypeO ptypeO)
      {
	return opening_eval_t::captureValue(ptypeO) * 16;
      }
      void dump() const;
    };

  } // namespace eval
} // namespace osl

#endif /* EVAL_EACHPLAYERATTACKDEFENSE_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
