/**
 * attackDefense.h
 */
#ifndef EVAL_ENDGAME_ATTACKtDEFENSE_H
#define EVAL_ENDGAME_ATTACKtDEFENSE_H

#include "osl/eval/endgame/attackKing.h"
#include "osl/eval/endgame/defenseKing.h"
#include "osl/eval/pieceEval.h"
#include "osl/eval/evalTraits.h"
#include "osl/misc/carray.h"

namespace osl
{
  namespace container
  {
    class PieceValues;
  } // namespace container
  namespace eval
  {
    namespace endgame
    {
      /**
       * max(AttackKing, DefenseKing).
       * ޤư褦ʤ礷ɽ
       */
      class AttackDefense
      {
	CArray<int,2> values;
	void reset() { values.fill(0); }
	void addValue(Player owner, int value)
	{
	  values[playerToIndex(owner)] += value;
	}
	void addValue(Piece king_black, Piece king_white, Piece target)
	{
	  assert(king_black.ptype() == KING);
	  assert(king_white.ptype() == KING);
	  assert(king_black.owner() == BLACK);
	  assert(king_white.owner() == WHITE);
	  addValue(target.owner(), valueOf(king_black, king_white, target));
	}
      public:
	explicit AttackDefense(const SimpleState&);
	static bool initialized() { return true; }

	int value() const { return values[0] + values[1]; }
	int value(Player p) const { return values[playerToIndex(p)]; }

	// ApplyMoveOfTurn Ȥinterface
	void doSimpleMove(const SimpleState& state,
			  Position from, Position to, int promote_mask)
	{
	  const Piece old_piece = state.getPieceOnBoard(from);
	  if (old_piece.ptype() == KING)
	  {
	    updateKingMove(state, from, to);
	    return;
	  }
	  
	  const PtypeO new_ptypeo 
	    = promoteWithMask(old_piece.ptypeO(), promote_mask);
	  const Piece black_king = state.getKingPiece<BLACK>();
	  const Piece white_king = state.getKingPiece<WHITE>();

	  const int inc = valueOf(black_king, white_king, 
				  new_ptypeo, to);
	  const int dec = valueOf(black_king, white_king, 
				  old_piece.ptypeO(), from);
	  const Player player = old_piece.owner();
	  addValue(player, inc - dec);
	}
	void doDropMove(const SimpleState& state, Position to, Ptype ptype)
	{
	  assert(ptype != KING);
	  const Piece black_king = state.getKingPiece<BLACK>();
	  const Piece white_king = state.getKingPiece<WHITE>();
	  const Player player = state.getTurn();
	  const PtypeO ptypeo = newPtypeO(player, ptype);
	  const int inc = valueOf(black_king, white_king, ptypeo, to);
	  const int dec = valueOf(black_king, white_king, ptypeo, 
				  Position::STAND());
	  addValue(player, inc - dec);
	}
	void doCaptureMove(const SimpleState& state, Position from, 
			   Position to, Piece target, int promote_mask)
	{
	  const Piece old_piece = state.getPieceOnBoard(from);
	  if (old_piece.ptype() == KING)
	  {
	    updateKingMove(state, from, to, target);
	    return;
	  }

	  const Piece black_king = state.getKingPiece<BLACK>();
	  const Piece white_king = state.getKingPiece<WHITE>();
	  const int inc_capture
	    = valueOf(black_king, white_king, captured(target.ptypeO()), 
		      Position::STAND());
	  const int dec_capture
	    = valueOf(black_king, white_king, target.ptypeO(), to);
	  const Player player = old_piece.owner();
	  addValue(player, inc_capture);
	  addValue(alt(player), -dec_capture);

	  const PtypeO new_ptypeo 
	    = promoteWithMask(old_piece.ptypeO(), promote_mask);
	  const int inc 
	    = valueOf(black_king, white_king, new_ptypeo, to);
	  const int dec 
	    = valueOf(black_king, white_king, old_piece.ptypeO(), from);
	  addValue(player, inc - dec);
	}
      private:
	void updateKingMove(const SimpleState&, Position from, Position to);
	void updateKingMove(const SimpleState&, Position from, Position to,
			    Piece target);
      public:
	static int infty()
	{
	  return PieceEval::infty()*2; // 2̤Υܡʥ
	}

	static int valueOf(Piece black_king, Piece white_king,
			   Piece target)
	{
	  return valueOf(black_king, white_king,
			 target.ptypeO(), target.position());
	}
	static int valueOf(Piece black_king, Piece white_king,
			   PtypeO ptypeo, Position position)
	{
	  assert(black_king.owner() == BLACK);
	  assert(white_king.owner() == WHITE);
	  
	  const Player player = getOwner(ptypeo);
	  const Piece my_king 
	    = (player == BLACK) ? black_king : white_king;
	  const Piece op_king 
	    = (player == BLACK) ? white_king : black_king;

	  const int attack = AttackKing::valueOf(op_king, ptypeo, position);
	  const int defense = DefenseKing::valueOf(my_king, ptypeo, position);

	  return max(player, attack, defense);
	}
	static void setValues(const SimpleState&, container::PieceValues&);
      };
    } // namespace endgame
  } // namespace endgame
} // namespace osl

#endif /* EVAL_ENDGAME_ATTACKKING_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
