/* pieceEval.h
 */
#ifndef _PIECEEVAL_H
#define _PIECEEVAL_H

#include "osl/eval/ptypeEvalTraits.h"
#include "osl/eval/evalTraits.h"
#include "osl/progress/progress32.h"
#include "osl/state/simpleState.h"
#include "osl/misc/carray.h"
#include <cassert>

namespace osl
{
  namespace eval
  {
    class PtypeEvalTable
    {
    private:
      CArray<int, PTYPEO_SIZE> ptypeO2Val;
      CArray<int, PTYPEO_SIZE> promoteVal;
      CArray<int, PTYPEO_SIZE> captureVal;
    public:
      PtypeEvalTable();
      ~PtypeEvalTable();
    private:
      void init();
    public:
      /**
       * ꤫鸫ptypeOζβ
       */
      int value(PtypeO ptypeO) const{
	assert(isValidPtypeO(ptypeO));
	return ptypeO2Val[ptypeO-PTYPEO_MIN];
      }
      /**
       * ptypeζβ
       */
      int value(Ptype ptype) const{
	assert(isValid(ptype));
	return ptypeO2Val[ptype-PTYPEO_MIN];
      }
      /**
       * ptypeOpromoteɾͤ
       */
      int promoteValue(PtypeO ptypeO) const{
	assert(isPromoted(ptypeO));
	return promoteVal[ptypeO-PTYPEO_MIN];
      }
      /**
       * ownerptypeOcapture줿ɾͤ
       */
      int captureValue(PtypeO ptypeO) const{
	assert(isValidPtypeO(ptypeO));
	return captureVal[ptypeO-PTYPEO_MIN];
      }
    };
    extern const PtypeEvalTable Ptype_Eval_Table;

    /**
     * βͥ١ɾؿ.
     * ɬ
     * ͭ +, ͭ -
     *  PtypeEvalTraits<PAWN>::val 
     */
    class PieceEval
    {
      int val;
    public:
      explicit PieceEval(const SimpleState& state);
      explicit PieceEval(int v) : val(v) {}
      static bool initialized() { return true; }

      int value() const
      { 
	assert(isConsistentValueForNormalState<PieceEval>(val)); 
	return val; 
      }
      template <class State>
      static int diffWithMove(const State&, Move move)
      {
	const PtypeO capturePtypeO=newPtypeO(alt(move.player()),
					     move.capturePtype());
	int retVal=Ptype_Eval_Table.captureValue(capturePtypeO);
	if (move.isPromote())
	{
	  retVal+=Ptype_Eval_Table.promoteValue(move.ptypeO());
	}
	return retVal;
      }
      static int infty() { return 57984; }
      /**
       * move ɾ֤̣ͤƵ
       */
      template<class State,Player P>
      int computeValWithMove(const State& state,Move move) const
      {
	typedef typename State::effect_state_t effect_state_t;
	return computeDiffAfterMove<effect_state_t,P>(state,move) + value();
      }
      template<class State>
      int computeValWithMove(const State& state,Move move) const
      {
	if (move.player() == BLACK)
	  return computeValWithMove<State,BLACK>(state,move);
	else
	  return computeValWithMove<State,WHITE>(state,move);
      }

      /**
       * EvalState ͳõüǸƤФɾؿ
       *
       */
      template<class State,Player P>
      int computeValWithTakeBack(const State& state, Position /*focus*/) const
      {
	return computeValWithTakeBackMulti<State,P>(state);
      }

      /**
       * ɾͤܤ֤˴ؤ֤̣Ƶ
       *
       * move.to() == focus ä computeValWithMove(state,move)
       * η̤ state.doMove(move), computeValWithTakeBack(focus)
       * η̤ϰ
       * @param P focus ˤνͭ alt(P)֤ǼϤ
       *   Ȥߤʤ(ºݤstate.getTurn() ȰפƤɬפϤʤ)
       */
      template<class State,Player P>
      int computeValWithTakeBackLimited(const State& state, Position focus) const
      {
	typedef typename State::effect_state_t effect_state_t;
	return computeDiffAfterTakeBackLimited<effect_state_t,P>(state,focus) 
	  + value();
      }
      template<class State>
      int computeValWithTakeBackLimited(Player P, const State& state, Position focus) const
      {
	if (P == BLACK)
	  return computeValWithTakeBackLimited<State,BLACK>(state,focus);
	else
	  return computeValWithTakeBackLimited<State,WHITE>(state,focus);
      }

      template<class State,Player P>
      static int computeDiffAfterTakeBackLimited(const State& state, Position focus);
      template<class State>
      static int computeDiffAfterTakeBackLimited(Player P, const State& state, Position focus)
      {
	if (P == BLACK)
	  return PieceEval::template 
	    computeDiffAfterTakeBackLimited<State,BLACK>(state, focus);
	else
	  return PieceEval::template 
	    computeDiffAfterTakeBackLimited<State,WHITE>(state, focus);
      }

      /**
       * alt(P)ˤȤäưʾǤμ礤
       *
       * computeValWithTakeBack 㴳ΤΤϤ
       * @param P focus ˤνͭ alt(P)֤ǼϤ
       *   (ºݤstate.getTurn() ȰפƤɬפϤʤ)
       */
      template<class State,Player P>
      int computeValWithTakeBackMulti(const State& state) const
      {
	typedef typename State::effect_state_t effect_state_t;
	return computeDiffAfterTakeBackMulti<effect_state_t,P>(state) + value();
      }
      template<class State>
      int computeValWithTakeBackMulti(Player P, const State& state) const
      {
	if (P == BLACK)
	  return computeValWithTakeBackMulti<State,BLACK>(state);
	else
	  return computeValWithTakeBackMulti<State,WHITE>(state);
      }
    
      /** 
       * move ˤ֤ͤѲ (SOMA)
       *
       * - move Υޥͤ
       * - ܤϲͤξĴ٤
       * - ֤˴ؤơPROMOTE ̵ͭϹͤƤʤ
       * - ߡ֤move θˤΤӤʤ
       * - ?? ̣ζӱۤͤ
       * - ͤβͤ Player ˤȤäơ
       * - ʤɤϵˤʤ
       */
      template<class State,Player P>
      static int computeDiffAfterMove(const State& state,Move move);
      template<class State>
      static int computeDiffAfterMove(Player P, const State& state,Move move)
      {
	if (P == BLACK)
	  return computeDiffAfterMove<State,BLACK>(state,move);
	else
	  return computeDiffAfterMove<State,WHITE>(state,move);
      }
      template<class State,Player P>
      static int computeDiffAfterTakeBackMulti(const State& state);

      /** 
       * move ˤ֤ͤѲ 
       *
       * - move ΥޥǤʤ¾Υޥͤưʾ
       *   move.to()˴ؤƤΤ߿Ӥ
       * - ʤɤϵˤʤ
       * - move θĹϼ礤˹θʤȤ
       */
      template<class State,Player P>
      static int computeDiffAfterMoveMulti(const State& state,Move move);
      template<class State>
      static int computeDiffAfterMoveMulti(Player P, const State& state,Move move)
      {
	if (P == BLACK)
	  return computeDiffAfterMoveMulti<State,BLACK>(state,move);
	else
	  return computeDiffAfterMoveMulti<State,WHITE>(state,move);
      }
    
      /**
       * ¸ΨõѼ֤
       * 
       * ߤζ̤ɾͤ move ζ̤κʬ(P礬Ȥ
       * 褦)֤
       */
      template<Player P,class State>
      static int computeDiffAfterMoveForRP(const State& state,Move move)
      {
	assert(move.player()==P);
	typedef typename State::effect_state_t effect_state_t;
	const int diff = computeDiffAfterMove<effect_state_t,P>(state,move);
	return (P==BLACK) ? diff : -diff;
      }
      template<class State>
      static int computeDiffAfterMoveForRP(const State& state, Move move)
      {
	if (move.player()==BLACK)
	  return computeDiffAfterMoveForRP<BLACK,State>(state,move);
	else
	  return computeDiffAfterMoveForRP<WHITE,State>(state,move);
      }
    private:
      void addVal(int d) { val+=d; }
    public:
      // Ϥʤ
      // promoteѲ
      void doSimpleMove(const SimpleState& state,Position from, Position /*to*/, 
			int promote_mask)
      {
	const Piece p0=state.getPieceOnBoard(from);
	const Piece new_piece=p0.promoteWithMask(promote_mask);
	addVal(Ptype_Eval_Table.value(new_piece.ptypeO())
	       - Ptype_Eval_Table.value(p0.ptypeO()));
      }
      void doDropMove(const SimpleState&, Position, Ptype)
      {
      }
      void doCaptureMove(const SimpleState& state,Position from, Position /*to*/, 
			 Piece target, int promote_mask)
      {
	const Piece p0=state.getPieceOnBoard(from);
	const Piece new_piece=p0.promoteWithMask(promote_mask);
	const PtypeO capturePtypeO=target.ptypeO();
	addVal(Ptype_Eval_Table.value(new_piece.ptypeO())-
	       Ptype_Eval_Table.value(p0.ptypeO())+
	       Ptype_Eval_Table.value(captured(capturePtypeO))-
	       Ptype_Eval_Table.value(capturePtypeO));
      }

      const Progress32 progress32() const { return Progress32(0); }
      /**
       * QuiescenceSearch λ޴ǻ
       */
      static int captureValue(PtypeO ptypeO)
      {
	return Ptype_Eval_Table.captureValue(ptypeO);
      }
      static int value(PtypeO ptypeO)
      {
	return Ptype_Eval_Table.value(ptypeO);
      }
    };

  } // namespace eval
  using eval::PieceEval;
} // namespace osl

#endif /* _PIECEEVAL_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
