/* piecePairEval.h
 */
#ifndef _PIECE_PAIR_EVAL_H
#define _PIECE_PAIR_EVAL_H

#include "osl/eval/ppair/piecePairIndex.h"
#include "osl/eval/pieceEval.h"
#include "osl/eval/evalTraits.h"
#include "osl/state/simpleState.h"
#include "osl/move.h"
#include "osl/misc/carray.h"
#include <iosfwd>

namespace osl
{
  namespace container
  {
    class PieceValues;
  } // container
  
  namespace eval
  {
    namespace ppair
    {
      /**
       * PiecePairEval Ρtemplate parameter˰¸ʤʬ
       * ̤μ.
       */
      class PiecePairEvalBase
      {
      protected:
	int val;
	PiecePairEvalBase() : val(0)
	{
	}
	~PiecePairEvalBase()
	{
	}
      public:
	/** roundup  2^n Ǥ뤳 */
	static const int ROUND_UP = 2;
	static int roundUp(int v)
	{
	  return v & (~(ROUND_UP-1)); 
	}
	int value() const	{ return roundUp(val); }
	int rawValue() const { return val; }
	static int infty()
	{
	  // ProgressEval ­Ƥoverflowʤͤˤ
	  // PieceEval::infty() + 100*40*39/2
	  return 150000;
	}

	static int captureValue(PtypeO ptypeo)
	{
	  return PieceEval::captureValue(ptypeo); 
	}
      };

      template <class Table>
      class PiecePairEvalTableBase : public PiecePairEvalBase
      {
      protected:
	// ɬѾƻȤ
	explicit PiecePairEvalTableBase(const SimpleState& state);
	~PiecePairEvalTableBase() {}
      public:
	/**
	 *  old_index  new_index ưȤͤκʬ
	 * @param state ưζ
	 * @param old_index +ư
	 * @param new_index +ư
	 */
	static int adjustPairs(const SimpleState& state,
			       unsigned int new_index);    
	static int adjustPairs(const SimpleState& state,
			       unsigned int old_index, unsigned int new_index);    
	static int adjustPairs(const SimpleState& state, 
			       unsigned int old_index, unsigned int old_index2, 
			       unsigned int new_index);
	static int diffAfterSimpleMove(const SimpleState& state,
				       Position from, Position to, 
				       int promote_mask)
	{
	  const Piece old_piece=state.getPieceAt(from);
	  const PtypeO newPtypeO = promoteWithMask(old_piece.ptypeO(), promote_mask);
	  const unsigned int old_index = PiecePairIndex::indexOf(old_piece);
	  const unsigned int new_index = PiecePairIndex::indexOf(to, newPtypeO);
	  return adjustPairs(state, old_index, new_index);
	}
	static int diffAfterDropMove(const SimpleState& state,Position to,PtypeO ptypeo)
	{
	  const unsigned int new_index = PiecePairIndex::indexOf(to, ptypeo);
	  return adjustPairs(state, new_index);
	}
	static int diffAfterCaptureMove(const SimpleState& state,
					Position from, Position to, 
					PtypeO victim,int promote_mask)
	{
	  const Piece old_piece=state.getPieceAt(from);
	  const PtypeO newPtypeO = promoteWithMask(old_piece.ptypeO(), promote_mask);
	  const unsigned int old_index = PiecePairIndex::indexOf(old_piece);
	  const unsigned int new_index = PiecePairIndex::indexOf(to, newPtypeO);

	  const unsigned int indexVictim = PiecePairIndex::indexOf(to, victim);
	  return adjustPairs(state, old_index, indexVictim, 
			     new_index);
	}
	/**
	 * طͤPiece βͤѴ.
	 * - Aβͤ r(A,A) + \sum_B(r(A+B)/2)
	 * - âA,BΤɤ餫(Τ)̤λˤϡ̤Ǥʤ˴ط򽸤
	 */
	static void setValues(const SimpleState&, container::PieceValues&);

      private:
	static bool& initializationFlag();
      public:
	static bool initialized() { return initializationFlag(); } // read only
	static bool setUp(const char *filename);
	static bool setUp();
      };

      /**
       * Υڥ׾򸵤ˤɾؿζʬ.
       * - ɬ
       * - ͭ +, ͭ -
       * @param Table PiecePairTable Τɤ줫instatiation
       */
      template <class Eval,class Table>
      class PiecePairEval : public PiecePairEvalTableBase<Table>
      {
      protected:
	// ɬѾƻȤ
	explicit PiecePairEval(const SimpleState& state);
      public:
	typedef PiecePairEvalTableBase<Table> base_t;
	/** λ state  move 
	 */
	void doSimpleMove(const SimpleState& state,Position from, Position to, 
			  int promote_mask);
	void doDropMove(const SimpleState& state,Position to,Ptype ptype);
	void doCaptureMove(const SimpleState& state,Position from, Position to, 
			   Piece victim,int promote_mask);
	static int diffWithMove(const SimpleState& state, Move move)
	{
	  // TRICK: Eval::diffXXX Ȥ뤳Ȥsubclassˤoverrideǽˤ
	  const Position from=move.from();
	  const Position to=move.to();
	  if (from.isPieceStand())
	    return Eval::diffAfterDropMove(state, to, move.ptypeO());

	  const Ptype ptypeCaptured=move.capturePtype();
	  if (ptypeCaptured != PTYPE_EMPTY)
	    return Eval::diffAfterCaptureMove(state, from, to,
					      newPtypeO(alt(move.player()),
							ptypeCaptured),
					      move.promoteMask());

	  return Eval::diffAfterSimpleMove(state, from,to,move.promoteMask());
	}
	
      };

    } // namespace ppair
  } // namespace eval
} // namespace osl

#endif /* _PIECE_PAIR_EVAL_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
