/* piecePairEval.tcc
 */
#ifndef EVAL_PIECE_PAIR_EVAL_TCC
#define EVAL_PIECE_PAIR_EVAL_TCC

#include "osl/eval/ppair/piecePairEval.h"
#include "osl/eval/ppair/piecePairAssert.h"
#include "osl/container/pieceValues.h"
#include "osl/pieceTable.h"
#include "osl/oslConfig.h"

template <class Table>
osl::eval::ppair::
PiecePairEvalTableBase<Table>::PiecePairEvalTableBase(const SimpleState& state) 
{
  for (int i=0; i<Piece::SIZE; i++) {
    for (int j=i; j<Piece::SIZE; j++) {
      val += Table::Table.valueOf
	(PiecePairIndex::indexOfPieceNum(state, i), 
	 PiecePairIndex::indexOfPieceNum(state, j));
    }
  }
}

template <class Table>
bool& osl::eval::ppair::
PiecePairEvalTableBase<Table>::initializationFlag()
{
  static bool flag = false;
  return flag;
}

template <class Table>
bool osl::eval::ppair::
PiecePairEvalTableBase<Table>::setUp(const char *filename)
{
  bool& result = initializationFlag();
  result = Table::Table.setUp(filename);
  return result;
}

template <class Table>
bool osl::eval::ppair::
PiecePairEvalTableBase<Table>::setUp()
{
  std::string filename = OslConfig::home();
  filename += "/data/sibling-attack.pair";
  return setUp(filename.c_str());
}

template <class Table>
int osl::eval::ppair::PiecePairEvalTableBase<Table>::
adjustPairs(const SimpleState& state,
	    unsigned int new_index)
{
  int diff = 0;
  for (int i=0; i<Piece::SIZE; i++) 
  {
    const Piece p=state.getPieceOf(i);
    if(p.isOnBoard()){
      const unsigned int target = 
	PiecePairIndex::indexOf(p);
      // ʲ target==old_index 1ʾ()ޥå
      diff += Table::Table.valueOf(target, new_index);
    }
  }
  diff += Table::Table.valueOf(new_index, new_index);
  return diff;
}

template <class Table>
int osl::eval::ppair::PiecePairEvalTableBase<Table>::
adjustPairs(const SimpleState& state,
	    unsigned int old_index, unsigned int new_index)
{
  int diff = 0;
  for (int i=0; i<Piece::SIZE; i++) 
  {
    const Piece p=state.getPieceOf(i);
    if(p.isOnBoard()){
      const unsigned int target = 
	PiecePairIndex::indexOf(p);
      diff -= Table::Table.valueOf(target, old_index);
      // ʲ target==old_index 1ʾ()ޥå
      diff += Table::Table.valueOf(target, new_index);
    }
  }
  diff -= Table::Table.valueOf(old_index, new_index); // ­ʬ
  diff += Table::Table.valueOf(new_index, new_index);
  return diff;
}

template <class Table>
int osl::eval::ppair::PiecePairEvalTableBase<Table>::
adjustPairs(const SimpleState& state, 
	    unsigned int old_index, unsigned int old_index2, 
	    unsigned int new_index)
{
  int diff = 0;
  for (int i=0;i<Piece::SIZE; i++) 
  {
    const Piece p=state.getPieceOf(i);
    if(p.isOnBoard()){
      const unsigned int target = 
	PiecePairIndex::indexOf(p);
      diff += Table::Table.valueOf(target, new_index);
      diff -= Table::Table.valueOf(target, old_index);
      diff -= Table::Table.valueOf(target, old_index2);
    }
  }

  diff -= Table::Table.valueOf(old_index, new_index);
  diff -= Table::Table.valueOf(old_index2, new_index);
  diff += Table::Table.valueOf(old_index, old_index2);

  diff += Table::Table.valueOf(new_index, new_index);
  return diff;
}

template <class Table>
void osl::eval::ppair::PiecePairEvalTableBase<Table>::
setValues(const SimpleState& state, container::PieceValues& values) 
{
  values.fill(0);
  // ®٤̵
  for (int i=0; i<Piece::SIZE; i++) {
    const bool i_is_king = (Piece_Table.getPtypeOf(i) == KING);
    for (int j=0; j<Piece::SIZE; j++) {
      if (i==j)
	continue;
      const bool j_is_king = (Piece_Table.getPtypeOf(j) == KING);
      
      const int relation_value = Table::Table.valueOf
	(PiecePairIndex::indexOfPieceNum(state, i), 
	 PiecePairIndex::indexOfPieceNum(state, j));
      if (i_is_king && (! j_is_king))
      {
	values[j] += relation_value;
      }
      else
      {
	values[i] += relation_value;
      }
    }
  }
  for (int i=0; i<Piece::SIZE; i++) 
  {
    values[i] /= 2;

    const unsigned int index = PiecePairIndex::indexOfPieceNum(state, i);
    const int self_value = Table::Table.valueOf(index, index);
    values[i] += self_value;
  }
}

/* ------------------------------------------------------------------------- */

template <class Eval, class Table>
osl::eval::ppair::
PiecePairEval<Eval,Table>::PiecePairEval(const SimpleState& state) 
  : PiecePairEvalTableBase<Table>(state)
{
}

template <class Eval, class Table>
void osl::eval::ppair::PiecePairEval<Eval,Table>::
doSimpleMove(const SimpleState& state,Position from, Position to, 
	     int promote_mask)
{
  piece_pair_assert(state.getPieceAt(from) != Piece::EMPTY());
  piece_pair_assert(state.getPieceAt(to) == Piece::EMPTY());
  base_t::val += Eval::diffAfterSimpleMove(state, from, to, promote_mask);
}


template <class Eval, class Table>
void osl::eval::ppair::PiecePairEval<Eval,Table>::
doDropMove(const SimpleState& state,Position to,Ptype ptype)
{
  piece_pair_assert(state.getPieceAt(to) == Piece::EMPTY());
  const Player player = state.getTurn();
  base_t::val += Eval::diffAfterDropMove(state, to, newPtypeO(player, ptype));
}

template <class Eval, class Table>
void osl::eval::ppair::PiecePairEval<Eval,Table>::
doCaptureMove(const SimpleState& state,
	      Position from, Position to, Piece victim,int promote_mask)
{
  piece_pair_assert(state.getPieceAt(from) != Piece::EMPTY());
  piece_pair_assert(state.getPieceAt(to) != Piece::EMPTY());
  base_t::val += Eval::diffAfterCaptureMove(state, from, to, victim.ptypeO(), promote_mask);
}

#endif /* EVAL_PIECE_PAIR_EVAL_TCC */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
