/* progressEval.h
 */
#ifndef EVAL_PROGRESSEVAL_H
#define EVAL_PROGRESSEVAL_H

#include "osl/eval/pieceEval.h"
#include "osl/eval/minorPieceBonus.h"
#include "osl/eval/ppair/piecePairPieceEval.h"
#include "osl/eval/endgame/attackDefense.h"
#include "osl/progress/effect5x3.h"
#include "osl/progress/effect5x3d.h"
#include "osl/progress/progress32.h"
#include "osl/state/numEffectState.h"
#include <boost/static_assert.hpp>
namespace osl
{
  namespace eval
  {
    /**
     * פȽפʬȤɾؿ.
     * ɬפǤСפȽפɾؿtemplate parameterˤ롥
     */
    class ProgressEval
    {
    public:
      typedef PiecePairPieceEval opening_eval_t;
      typedef endgame::AttackDefense endgame_eval_t;
      typedef progress::Effect5x3 progress_t;
      typedef progress::Effect5x3d defense_t;
    private:
      opening_eval_t opening_eval;
      endgame_eval_t endgame_eval;
      progress_t current_progress;
      defense_t defense_effect;
      MinorPieceBonus minor_piece_bonus;
      enum{
	INVALID=EvalTraits<BLACK>::MAX_VALUE+1,
      };
      mutable int cache;
    public:
      ProgressEval(const NumEffectState& state) 
	: opening_eval(state), endgame_eval(state),
	  current_progress(state), defense_effect(state),
	  minor_piece_bonus(state), cache(INVALID)
      {
      }
      static bool initialized() 
      { 
	return opening_eval_t::initialized();
      } 
      static bool setUp(const char *filename)
      {
	return opening_eval_t::setUp(filename);
      }
      static bool setUp()
      {
	return opening_eval_t::setUp();
      }

      /** roundup  2^n Ǥ뤳 */
      static const int ROUND_UP = 64;
      static int composeValue(int value_opening, int value_endgame, 
			      Progress16 progress16,
			      Progress16 black,
			      Progress16 white,
			      Progress16 black_defense,
			      Progress16 white_defense,
			      int minor_piece_bonus)
      {
	BOOST_STATIC_ASSERT(((PtypeEvalTraits<BISHOP>::val * 2 + PtypeEvalTraits<PAWN>::val) %  16) == 0);
	BOOST_STATIC_ASSERT((PtypeEvalTraits<PAWN>::val % 32) == 0);
	assert(progress16.isValid());
	assert(black.isValid());
	assert(white.isValid());
	assert(value_opening % 2 == 0);
	assert(value_endgame % 2 == 0);
	
	/*
	  value_opening * (16 - progress16) + value_endgame * progress16
	  + bonus * (white_progress - black_progress) * prorgress16 / 16 */
	int result = value_opening*16 + 
	  progress16.value() * (value_endgame-value_opening +
				((white.value() - black.value()) * (PtypeEvalTraits<BISHOP>::val * 2 + PtypeEvalTraits<PAWN>::val * 4) +
				 (black_defense.value() - white_defense.value()) *
				 PtypeEvalTraits<BISHOP>::val) / 16);

	result += minor_piece_bonus;
	result &= ~(ROUND_UP-1);
	assert(result % 2 == 0);
	return result;
      }
      const Progress16 progress16() const { return current_progress.progress16(); }
      void invalidateCache(){ 
	cache=INVALID;
      }
      int value() const
      {
	if(cache!=INVALID){
	  return cache;
	}
	else{
	  cache=composeValue(opening_eval.value(), endgame_eval.value(), 
			     progress16(), current_progress.progress16(BLACK),
			     current_progress.progress16(WHITE),
			     defense_effect.progress16(BLACK),
			     defense_effect.progress16(WHITE),
			     minor_piece_bonus.value(progress16()));
	  return cache;
	}
      }
      const Progress32 progress32() const { 
	return Progress32(current_progress.progress16(BLACK).value()
			  + current_progress.progress16(WHITE).value()); 
      }
      static void setValues(const SimpleState&, Progress16 progress16,
			    container::PieceValues&);
      static void setValues(const SimpleState& s, container::PieceValues& o);

      void doSimpleMove(const NumEffectState& state,
			Position from, Position to,  int promote_mask)
      {
	opening_eval.doSimpleMove(state,from,to,promote_mask);
	endgame_eval.doSimpleMove(state,from,to,promote_mask);
	current_progress.doSimpleMove(state,from,to,promote_mask);
	defense_effect.doSimpleMove(state,from,to,promote_mask);
	minor_piece_bonus.doSimpleMove(state,from,to,promote_mask);
	invalidateCache();
      }
      void doDropMove(const NumEffectState& state,Position to,Ptype ptype)
      {
	opening_eval.doDropMove(state,to,ptype);
	endgame_eval.doDropMove(state,to,ptype);
	current_progress.doDropMove(state,to,ptype);
	defense_effect.doDropMove(state,to,ptype);
	minor_piece_bonus.doDropMove(state,to,ptype);
	invalidateCache();
      }
      void doCaptureMove(const NumEffectState& state,
			 Position from, Position to, 
			 Piece victim,int promote_mask)
      {
	opening_eval.doCaptureMove(state,from,to,victim,promote_mask);
	endgame_eval.doCaptureMove(state,from,to,victim,promote_mask);
	current_progress.doCaptureMove(state,from,to,victim,promote_mask);
	defense_effect.doCaptureMove(state,from,to,victim,promote_mask);
	minor_piece_bonus.doCaptureMove(state,from,to,victim,promote_mask);
	invalidateCache();
      }
    public:
      static int infty()
      {
	assert(endgame_eval_t::infty() <= opening_eval_t::infty());
	// פ
	return composeValue(opening_eval_t::infty(), 0, Progress16(0),
			    Progress16(0), Progress16(0), Progress16(0),
			    Progress16(0), 0);
      }

      static int captureValue(PtypeO ptypeO)
      {
	// פ
	return composeValue(opening_eval_t::captureValue(ptypeO), 0, 
			    Progress16(0), Progress16(0), Progress16(0),
			    Progress16(0), Progress16(0), 0);
      }
    };

  } // namespace eval
} // namespace osl

#endif /* EVAL_PROGRESSEVAL_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
