#ifndef GAME_PLAYING_SEARCHPLAYER_H
#define GAME_PLAYING_SEARCHPLAYER_H

#include "osl/game_playing/computerPlayer.h"
#include "osl/search/historyTable.h"
#include "osl/search/hasTimer.h"
#include "osl/misc/alarm.h"
#include "osl/container/moveVector.h"
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

namespace osl
{
  namespace misc
  {
    class RealTime;
  }
  namespace search
  {
    class SearchRecorder;
    class SimpleHashTable;
    class BigramKillerMove;
  }
  namespace game_playing
  {
    class TableStack;
    /**
     * MtdfPlayer  AlphaBetaPlayer ζʬ
     */
    class SearchPlayer : public ComputerPlayer
    {
    public:
      struct CheckmateSearcher;
      struct NtesukiThread;
      struct Config
      {
	int limit;
	int node_limit;
	int table_size;
	int table_record_limit;
	int initial_limit;
	int deepening_step;
	size_t total_checkmate_limit;
	bool verbose;
	/** SearchBase::next_iteration_coefficient ꤹ */
	double next_iteration_coefficient;
	Config();
	friend bool operator==(const Config& l, const Config& r);
      };
    protected:
      Config config;
      boost::scoped_ptr<search::SimpleHashTable> table_ptr;
      boost::shared_ptr<CheckmateSearcher> checkmate_ptr;
      boost::scoped_ptr<search::SearchRecorder> recorder_ptr;
      boost::scoped_ptr<search::BigramKillerMove> bigram_ptr;
      boost::scoped_ptr<TableStack> table_stack_ptr;
      class KeyOfMove;
      boost::scoped_ptr<KeyOfMove> key_of_move;
      AlarmSwitch stop_flag;
      HistoryTable history_table;
      bool copy_history_table;
      boost::scoped_ptr<search::HasTimer> searcher;
      /** õ˻ߤ */
      volatile bool plan_stop;
    public:
      SearchPlayer();
      SearchPlayer(const SearchPlayer&);
      ~SearchPlayer();

      /** ;õפξ */
      void setTotalCheckmateLimit(int limit);

      void setDepthLimit(int limit, int initial_limit, int deepening_step);
      void setNodeLimit(int node_limit);
      void setTableLimit(int size, int record_limit);
      void setVerbose(bool verbose=true);
      void setNextIterationCoefficient(double new_value);

      void enableTableStack(bool enable=true);
      /** ͭž */
      void resetRecorder(search::SearchRecorder *new_recorder);

      void pushMove(Move m);
      void popMove();

      /**
       * other ζɽȼؤ
       */
      void swapTable(SearchPlayer& other);

      const search::SimpleHashTable* table() const { return table_ptr.get(); }
      const search::SearchRecorder& recorder() const { return *recorder_ptr; }

      bool stopSearchNow();
      bool canStopSearch();	// ƽФάʤ褦ǰΰ const Ǥʤ
      /**
       * searchWithSecondsForThisMove ƤӽФ
       */
      const MoveWithComment selectBestMove(const GameState&, int, int);
    protected:
      template <class Searcher>
      ComputerPlayer* cloneIt(const Searcher&) const;
      /** @return time consumed in seconds */
      int setUpTable(const GameState&, int draw);
      template <class Searcher>
      const MoveWithComment search(const GameState&, int);
      template <class Searcher>
      bool isReasonableMoveBySearch(Searcher&, Move move, int pawn_sacrifice);
    public:
      virtual const MoveWithComment searchWithSecondsForThisMove(const GameState&, int)=0;
      /**
       * state ͭϤʼ򤤤Ĥ
       */
      void pickUpBestMoves(const GameState& state, MoveVector&) const;

      const Config& getConfig() const { return config; }

      static int secondsForThisMove(const GameState& state,
				    int total_seconds_left, int byoyomi);

      void setCopyHistoryTable(bool new_value=true) { 
	copy_history_table = new_value;
      }
      const HistoryTable& historyTable() const { return history_table; }
      void setStopSchedule(const misc::RealTime& new_timer);
    private:
      void fillMoveComment(MoveWithComment&, const GameState&, 
			   const search::SimpleHashTable *, int pawn_value);
    };

  } // namespace game_playing
} // namespace osl

#endif /* GAME_PLAYING_SEARCHPLAYER_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
