/* speculativeAllMoves.h
 */
#ifndef OSL_SPECULATIVEALLMOVES_H
#define OSL_SPECULATIVEALLMOVES_H

#include "osl/game_playing/computerPlayer.h"
#include "osl/game_playing/speculativeModel.h"
#include "osl/misc/lightMutex.h"
#include "osl/misc/fixedCapacityVector.h"
#include <boost/thread/thread.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

namespace osl
{
  namespace misc
  {
    class RealTime;
  }
  namespace game_playing
  {
    class SearchPlayer;
    /**
     * 1threadƤμ֤굡Ūõ򤹤
     */
    class SpeculativeAllMoves : public SpeculativeModel
    {
    public:
      struct SearchAllMoves;
      struct ResultVector;
    private:
      boost::shared_ptr<SearchAllMoves> searcher;
      boost::scoped_ptr<boost::thread> thread;
      boost::scoped_ptr<ResultVector> results;
      int last_search_seconds;
      bool has_byoyomi;
      bool allowed;
    public:
      SpeculativeAllMoves();
      ~SpeculativeAllMoves();

      void startSpeculative(const boost::shared_ptr<GameState> state,
			    const SearchPlayer& main_player);
      void stopOtherThan(Move);
      void stopAll();

      void setMaxThreads(int new_max_threads)
      {
	allowed = (new_max_threads > 0);
      }

      const MoveWithComment waitResult(Move last_move, int wait_for,
				       SearchPlayer& main_player, int byoyomi);

      void selectBestMoveCleanUp();
      void clearResource();
    private:
      class Runner;
    };

    class SpeculativeAllMoves::ResultVector
    {
      typedef FixedCapacityVector<std::pair<Move,MoveWithComment>,Move::MaxUniqMoves> vector_t;
      vector_t data;
      typedef LightMutex Mutex;
      mutable Mutex mutex;
    public:
      ResultVector();
      ~ResultVector();
      
      void add(Move prediction, const MoveWithComment& result);
      const MoveWithComment* find(Move prediction) const;
      void clear();
    };

    /**
     * ؼ̤resultsˤ롥
     * run threadưȤꤷƤ뤬༡ǤƥȲ
     */
    class SpeculativeAllMoves::SearchAllMoves
    {
    public:
      enum Status { 
	INITIAL, RUNNING, PREDICTION1, PREDICTION2, SEARCH1, SEARCH2, FINISHED
      };
      class Generator;
    private:
      boost::shared_ptr<GameState> state;
      boost::shared_ptr<SearchPlayer> player;
      boost::scoped_ptr<Generator> generator;
      SpeculativeAllMoves::ResultVector& results;
      Move current_move;
      volatile Status status;
      int seconds;
      typedef LightMutex Mutex;
      mutable Mutex mutex;
      /** true ʤ鼡ͽõˤϤʤ */
      volatile bool stop_flag;
    public:
      explicit SearchAllMoves(SpeculativeAllMoves::ResultVector&);
      ~SearchAllMoves();

      void setUp(const GameState&, const SearchPlayer&, int standard_seconds,
		 bool has_byoyomi);

      void run();
      
      void stopNow();
      void stopOtherThan(Move);
      void waitRunning();
      bool isFinished() const { return status == FINISHED; }

      void setStopSchedule(const misc::RealTime& timer);
      const Move currentMove() const;

      SearchPlayer* currentPlayer() { return player.get(); }
    private:
      const MoveWithComment testMove(Move);
      class StatusLock;
    };
  } // game_playing
} // osl

#endif /* OSL_SPECULATIVEALLMOVES_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
