#ifndef _HASH_TCC
#define _HASH_TCC

#include "osl/hash/hashState.h"
#include "osl/apply_move/doUndoMoveLock.h"
#include "osl/apply_move/applyMove.h"
#include "osl/pieceTable.h"
#include <iostream>

namespace osl
{
  template <class BaseState>
  class apply_move::DoUndoMoveLockSimple<hash::HashState<BaseState> >
    : public apply_move::DoUndoMoveLockCleanUp
  {
    typedef hash::HashState<BaseState> state_t;
    state_t& state;
    HashKey oldHash;
    typedef apply_move::DoUndoMoveLockSimple<BaseState> baselock_t;
    boost::scoped_ptr<baselock_t> baselock;
  public:
    DoUndoMoveLockSimple(state_t& s, 
			 Position from, Position to, int promoteMask);
    ~DoUndoMoveLockSimple();
  };

  template <class BaseState>
  class apply_move::DoUndoMoveLockDrop<hash::HashState<BaseState> >
    : public apply_move::DoUndoMoveLockCleanUp
  {
    typedef hash::HashState<BaseState> state_t;
    state_t& state;
    HashKey oldHash;
    typedef apply_move::DoUndoMoveLockDrop<BaseState> baselock_t;
    boost::scoped_ptr<baselock_t> baselock;
  public:
    DoUndoMoveLockDrop(state_t& s, Position to, Ptype ptype);
    ~DoUndoMoveLockDrop();
  };
  template <class BaseState>
  class apply_move::DoUndoMoveLockCapture<hash::HashState<BaseState> >
    : public apply_move::DoUndoMoveLockCleanUp
  {
    typedef hash::HashState<BaseState> state_t;
    state_t& state;
    HashKey oldHash;
    typedef apply_move::DoUndoMoveLockCapture<BaseState> baselock_t;
    boost::scoped_ptr<baselock_t> baselock;
  public:
    DoUndoMoveLockCapture(state_t& s, Position from, Position to, Piece target,
			  int promoteMask);
    ~DoUndoMoveLockCapture();
  };
} // namespace osl

template <class BaseState>
osl::apply_move::DoUndoMoveLockSimple<osl::hash::HashState<BaseState> >::
DoUndoMoveLockSimple(state_t& s, Position from, Position to, int promoteMask)
  : state(s), oldHash(s.hash)
{
  state.hash.doSimpleMove(s, from, to, promoteMask);
  state.hash.changeTurn();
  baselock.reset(new baselock_t(s, from, to, promoteMask));
}
template <class BaseState>
osl::apply_move::DoUndoMoveLockSimple<osl::hash::HashState<BaseState> >::
~DoUndoMoveLockSimple()
{
  baselock.reset();
  state.hash = oldHash;
}

template <class BaseState>
osl::apply_move::DoUndoMoveLockDrop<osl::hash::HashState<BaseState> >::
DoUndoMoveLockDrop(state_t& s, Position to, Ptype ptype)
  : state(s), oldHash(s.hash)
{
  state.hash.doDropMove(s, to, ptype);
  state.hash.changeTurn();
  baselock.reset(new baselock_t(s, to, ptype));
}
template <class BaseState>
osl::apply_move::DoUndoMoveLockDrop<osl::hash::HashState<BaseState> >::
~DoUndoMoveLockDrop()
{
  baselock.reset();
  state.hash = oldHash;
}
template <class BaseState>
osl::apply_move::DoUndoMoveLockCapture<osl::hash::HashState<BaseState> >::
DoUndoMoveLockCapture(state_t& s, Position from, Position to, Piece target,
		      int promoteMask)
  : state(s), oldHash(s.hash)
{
  state.hash.doCaptureMove(s, from, to, target, promoteMask);
  state.hash.changeTurn();
  baselock.reset(new baselock_t(s, from, to, target, promoteMask));
}
template <class BaseState>
osl::apply_move::DoUndoMoveLockCapture<osl::hash::HashState<BaseState> >::
~DoUndoMoveLockCapture()
{
  baselock.reset();
  state.hash = oldHash;
}

template<class State>
bool osl::hash::HashState<State>::isConsistent(bool showError) const
{
  if (! State::isConsistent(showError)) 
  {
    if (showError)
      std::cerr << "error before hash" << std::endl;
    return false;
  }
  if (hash != HashKey::calcHash(*this))
  {
    if(showError) 
      std::cerr << "hash mismatch" << std::endl;
    return false;
  }
  PieceStand blackStand;
  for (int num=0;num<40;num++)
  {
    if (State::standMask(BLACK).test(num))
      blackStand.add(Piece_Table.getPtypeOf(num));
  }
  if (hash.blackStand() != blackStand)
  {
    if(showError) 
      std::cerr << "stand mismatch" << std::endl;
    return false;
  }
  return true;
}

template<class State>
void osl::hash::HashState<State>::print(std::ostream& os) const {
  os << hash << std::endl;
  os<<static_cast<State>(*this);
}

template<class State>
void osl::hash::HashState<State>::
doSimpleMove(Position from, Position to, int promoteMask)
{
#ifndef NDEBUG
  PtypeO newPtypeo = 
#endif
    hash.doSimpleMove(*this, from,to,promoteMask);
  State::doSimpleMove(from,to,promoteMask);
  assert(newPtypeo==State::getPieceAt(to).ptypeO());
}

template<class State>
void osl::hash::HashState<State>::
doDropMove(Position to,Ptype ptype)
{
  hash.doDropMove(*this, to,ptype);
  State::doDropMove(to,ptype);
}

template<class State>
void osl::hash::HashState<State>::
doCaptureMove(Position from, Position to, Piece p1,int promoteMask)
{
  hash.doCaptureMove(*this, from,to,p1,promoteMask);
  State::doCaptureMove(from,to,p1,promoteMask);
}

template<class State>
std::ostream& operator<<(std::ostream& os,const osl::hash::HashState<State>& state)
{
  state.print(os);
  return os;
}

#endif /* _HASH_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
