/* atomicCounter.h
 */
#ifndef OSL_ATOMICCOUNTER_H
#define OSL_ATOMICCOUNTER_H

#include "osl/misc/lightMutex.h"
#include <algorithm>

namespace osl
{
  namespace misc
  {
    class AtomicCounter
    {
      typedef LightMutex Mutex;

      mutable Mutex m;
      int count;
    public:
      explicit AtomicCounter(int count=0) :count(count){}
      void inc(){
	Mutex::scoped_lock lk(m);
	count++;
      }
      void dec(){
	Mutex::scoped_lock lk(m);
	count--;
      }
      void max(int val){
	Mutex::scoped_lock lk(m);
	count=std::max(count,val);
      }
      int value() const{ 
	Mutex::scoped_lock lk(m);
	return count; 
      }
    };
  }
  using misc::AtomicCounter;
}

#endif /* OSL_ATOMICCOUNTER_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:

