/* carray.h
 */
#ifndef _CARRAY_H
#define _CARRAY_H
#include <cstddef>
#include <cassert>
#include <algorithm>
#include <iterator>
namespace osl
{
  namespace misc
  {
  /**
   * CArray  iterator.
   * ñT*Ȥȡ if (iter == end()) Ƚ񤯤٤Ȥ
   * ݥ󥿤ΤĤ if (iter) Ƚ񤤤ƤޤäƤⵤդʤᡥ
   * TODO: boost ˤʤ?
   */
  template <typename T>
  struct CArrayIterator
  {
    typedef std::random_access_iterator_tag iterator_category;
    typedef T   value_type;
    typedef int difference_type;
    typedef T*  pointer;
    typedef T&  reference;

    T *ptr;
    CArrayIterator(T *p) : ptr(p) {}
    T& operator*() const { return *ptr; }
    T* operator->() const { return ptr; }
    CArrayIterator& operator+=(int diff)
    {
      ptr += diff;
      return *this;
    }
    CArrayIterator& operator-=(int diff) { return operator+=(-diff); }
    CArrayIterator operator+(int diff) const
    {
      CArrayIterator result(*this);
      result += diff;
      return result;
    }
    CArrayIterator operator-(int diff) const { return operator+(-diff); }
    CArrayIterator& operator++() { return operator+=(1); }
    CArrayIterator operator++(int)
    {
      const CArrayIterator result = *this;
      operator++();
      return result;
    }
    CArrayIterator& operator--() { return operator+=(-1); }
    CArrayIterator operator--(int)
    {
      const CArrayIterator result = *this;
      operator--();
      return result;
    }
    operator CArrayIterator<const T>() const
    {
      return CArrayIterator<const T>(ptr);
    }
  private:
    operator bool();		// not implemented
  };
  // T  const T ΰ㤤ۼʳcompile error ˤʤϤ
  template <class T, class T2>
  inline int operator-(CArrayIterator<T> l, CArrayIterator<T2> r)
  {
    return l.ptr - r.ptr;
  };
  template <class T, class T2>
  inline bool operator==(CArrayIterator<T> l, CArrayIterator<T2> r)
  {
    return l.ptr == r.ptr;
  };
  template <class T, class T2>
  inline bool operator!=(CArrayIterator<T> l, CArrayIterator<T2> r)
  {
    return l.ptr != r.ptr;
  };
  template <class T, class T2>
  inline bool operator<(CArrayIterator<T> l, CArrayIterator<T2> r)
  {
    return l.ptr < r.ptr;
  };

  /**
   * boost::array Τޤ͡operator[]  assert 򤤤줿äΤǡ
   */
  template <typename T, size_t Capacity>
  class CArray
  {
  public:
    /** {} ˤ public ˤƤ */
    T elements[Capacity];
  public:
    typedef CArrayIterator<T> iterator;
    iterator begin() { return &elements[0]; }
    iterator end() { return &elements[Capacity]; }

    void fill(T value=T())
    {
      std::fill(begin(), end(), value);
    }
    T& operator[] (size_t i) 
    {
      assert(i < Capacity);
      return elements[i];
    }

    static size_t size() { return Capacity; }
    
    T const& operator[] (size_t i) const
    {
      assert(i < Capacity);
      return elements[i];
    }

    typedef CArrayIterator<const T> const_iterator;
    const_iterator begin() const { return &elements[0]; }
    const_iterator end()   const { return &elements[Capacity]; }

    bool operator==(const CArray& other) const
    {
      return std::equal(begin(), end(), other.begin());
    }
  };
  } // namespace misc
  using misc::CArray;
} // namespace osl


#endif /* _FIXED_CAPACITY_VECTOR_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
