#ifndef _MOVE_GENERATOR_ADD_EFFECT_TCC
#define _MOVE_GENERATOR_ADD_EFFECT_TCC

#include "osl/move_generator/addEffect.h"
#include "osl/move_action/safeFilter.h"
#include "osl/move_generator/open.h"
#include <boost/type_traits.hpp>

namespace osl
{
  namespace move_generator
  {
    namespace without_effect
    {
      /**
       * @param P(template) - ¦Υץ쥤䡼
       * @param T(template) - ¦ζμ
       * @param state - 
       * @param from - ⤷褦Ȥΰ
       * @param target - ⤷褦Ȥޥ
       * Ǥ˹⤷褦Ȥޥϲ⤷ʤ?
       * ʤ顤ʬưƤȤΰ֤ĤʤɤϹͤʤ
       */
      template <class State,Player P,Ptype T,class Action,bool isPromote>
      void generateMove(State const& state,Position from,Position target, NearMask nearMask,Action& action)
      {
	NearMask target_mask = 
	  (isPromote 
	   ? Add_Effect_Table.getNearMaskWithPromote<P>(PtypeTraits<T>::moveType,from,target)
	   : Add_Effect_Table.getNearMask<P>(PtypeTraits<T>::moveType,from,target));
	target_mask &= nearMask;
	while (target_mask.any())
	{
	  const int n =target_mask.takeOneBit();
	  Direction dir=static_cast<Direction>(n);
	  Position to=target-Board_Table.template getOffset<P>(dir);
	  if (!isPromote || 
	      (from.canPromote<P>()||to.canPromote<P>()))
	    action.unknownMove(from,to,state.getPieceAt(to),(isPromote ? PtypeFuns<T>::promotePtype : T) ,isPromote,P);
	}
      }

      template <class State,Player P,Ptype T,class Action>
      void generateLongMoveDirectNoPromote(State const& state,Position from,Position target, NearMask nearMask,Action& action)
      {
	NearMask target_mask=Add_Effect_Table.getNearMask<P>(T,from,target) &nearMask;
	while (target_mask.any())
	{
	  const int n = target_mask.takeOneBit();
	  Direction dir=static_cast<Direction>(n);
	  Position to=target-Board_Table.template getOffset<P>(dir);
	  /** 
	   * ֤٤ƶ
	   */
	  if (state.isEmptyBetween(from,to))
	    action.unknownMove(from,to,state.getPieceAt(to),T,false,P);
	}
      }

      template <class State,Player P,Ptype T,class Action>
      void generateLongMoveDirectCanPromote(State const& state,Position from,Position target, NearMask nearMask,Action& action)
      {
	NearMask target_mask=Add_Effect_Table.getNearMaskWithPromote<P>(T,from,target) &nearMask;
	NearMask target_mask_no_promote=Add_Effect_Table.getNearMask<P>(T,from,target) &nearMask;
	while (target_mask.any())
	{
	  const int n = target_mask.takeOneBit();
	  Direction dir=static_cast<Direction>(n);
	  Position to=target-Board_Table.template getOffset<P>(dir);
	  /** 
	   * ֤٤ƶ
	   */
	  if (state.isEmptyBetween(from,to))
	  {
	    if (to.canPromote<P>()||from.canPromote<P>())
	      action.unknownMove(from,to,state.getPieceAt(to),PtypeFuns<T>::promotePtype,true,P);
	    if ((target_mask_no_promote.isSet(n))!=0)
	      action.unknownMove(from,to,state.getPieceAt(to),T,false,P);
	  }
	}
      }
      /**
       *
       */
      template<class State,bool isAttackToKing>
      bool isEmptyOrAdditional(State const& state,Position from,Position to,int& count)
      {
	count=0;
	if(isAttackToKing){
	  return state.isEmptyBetween(from,to);
	}
	Offset o=Board_Table.getShortOffset(Offset32(to,from));
	assert(!o.zero());
	Piece p;
	Position pos;
	for(pos=from+o;pos!=to && (p=state.getPieceAt(pos)).isEmpty();pos+=o);
	if(pos==to) return true;
	assert(!p.isEdge());
	count=1;
	return state.hasEffectTo(p,to);
      }
      template <class State,Player P,Ptype T,class Action,bool canPromote,bool isAttackToKing>
      void generateLongMove(State const& state,Position from,Position target, NearMask nearMask,Action& action)
      {
	// static assert?
	assert(T==LANCE || T==BISHOP || T==PBISHOP || T==ROOK || T==PROOK);
	/** longofset */
	const EffectContent effect_content=
	  Ptype_Table.getEffect(newPtypeO(BLACK,unpromote(T)),Offset32(target,from).blackOffset32<P>());
	if (isAttackToKing)
	{
	  assert(! effect_content.hasUnblockableEffect());
	}
	else if (effect_content.hasUnblockableEffect())
	  return;
	if (effect_content.hasEffect())
	{
	  /** 
	   * ƱlineǤǽϤ
	   */
	  if (T==PROOK)
	  {
	    generateMove<State,P,T,Action,false>(state,from,target,nearMask,action);
	  }
	  Offset o1=effect_content.offset().template blackOffset<P>();
	  assert(!o1.zero());
	  Piece p;
	  Position pos1=target-o1;
	  for(;(p=state.getPieceAt(pos1)).isEmpty();pos1-=o1) ;
	  // ưʤ
	  if (isAttackToKing)
	  {
	    assert(pos1!=from);
	  }
	  else if (pos1==from) return;
	  assert(!p.isEdge());
	  if(!isAttackToKing && state.hasEffectTo(p,target)){
	    Piece p1;
	    Position pos3=pos1;
	    for(pos1-=o1;(p1=state.getPieceAt(pos1)).isEmpty();pos1-=o1) ;
	    if(pos1==from){
	      if(p.isOnBoardByOwner<P>()) return;
	      pos1=pos3;
	    }
	    else{
	      p=p1;
	      Position pos2=pos1-o1;
	      for(;(p1=state.getPieceAt(pos2)).isEmpty();pos2-=o1) ;
	      if (pos2!=from) return;
	    }
	  }
	  else{
	    Position pos2=pos1-o1;
	    Piece p1;
	    for(;(p1=state.getPieceAt(pos2)).isEmpty();pos2-=o1) ;
	    if (pos2!=from) return;
	  }
	  if (p.isOnBoardByOwner<P>())
	  { 
	    Open<P>::generate
	      (state,from,p,action,target,
	       longToShort(Board_Table.getLongDirection<BLACK>(Offset32(target,from).blackOffset32<P>())));
	    /** open attack */
	  
	  }
	  else
	  {
	    /** capture attack */
	    if (canPromote &&          
		(from.canPromote<P>()||pos1.canPromote<P>()) &&
		/** LANCEξľʳäƤϥ */
		(T!=LANCE || pos1==target-o1))
	      action.unknownMove(from,pos1,p,PtypeFuns<T>::promotePtype,true,P);
	    action.unknownMove(from,pos1,p,T,false,P);
	  }
	}
	else
	{
	  /**
	   * ƱlineǤʤ
	   * 8˵˰ưƤβ
	   */
	  if (!PtypeTraits<T>::isBasic)
	  {
	    generateLongMoveDirectNoPromote<State,P,T,Action>(state,from,target,nearMask,action);
	  }
	  else
	  {
	    generateLongMoveDirectCanPromote<State,P,T,Action>(state,from,target,nearMask,action);
	  }
	  if (T!=LANCE)
	  {
	    /** 1position */
	    {
	      NearMask target_mask=Add_Effect_Table.getNearMaskLong<P>(T,from,target);
	      if(isAttackToKing) target_mask&=nearMask;
	      while (target_mask.any())
	      {
		const int n = target_mask.takeOneBit();
		Offset offset=Add_Effect_Table.getOffsetLong<P>(T,from,target,n&15);
		Position mid=target+offset.blackOffset<P>();
		/** ϰϳΤȤ⤢ */
		if (! mid.isOnBoardRegion()) 
		  continue;
		Piece p=state.getPieceAt(mid);
		int count=0;
		if (p.canMoveOn<P>() 
		    && state.isEmptyBetween(from,mid)
		    && isEmptyOrAdditional<State,isAttackToKing>(state,mid,target,count))
		{
		  if (canPromote &&
		      (from.canPromote<P>()||mid.canPromote<P>()))
		    action.unknownMove(from,mid,p,PtypeFuns<T>::promotePtype,true,P);
		  action.unknownMove(from,mid,p,T,false,P);
		}
	      }
	    }
	    /** 2position */
	    if (T==PBISHOP)
	    {
	      NearMask target_mask=Add_Effect_Table.getNearMaskPBISHOP<P>(from,target);
	      if(isAttackToKing) target_mask&=nearMask;
	      while (target_mask.any())
	      {
		const int n = target_mask.takeOneBit();
		Offset offset=Add_Effect_Table.getOffsetPBISHOP<P>(from,target,n&15);
		Position mid=target+offset.blackOffset<P>();
		/** */
		int count=0;
		if (isEmptyOrAdditional<State,isAttackToKing>(state,mid,target,count))
		{
		  for(int i=0;i<2;i++){
		    Piece p=state.getPieceAt(mid);
		    if(p.canMoveOn<P>()){
		      if (canPromote &&
			  (from.canPromote<P>()||mid.canPromote<P>()))
			action.unknownMove(from,mid,p,PtypeFuns<T>::promotePtype,true,P);
		      action.unknownMove(from,mid,p,T,false,P);
		    }
		    if (!p.isEmpty()){
		      if (isAttackToKing || count>0 || p.isEdge() || !state.hasEffectTo(p,target)) break;
		      count++;
		    }
		    mid=mid+Board_Table.getShortOffset(Offset32(mid,target));
		  }
		}
	      }
	    }
	    /** 3position */
	    if (T==PROOK)
	    {
	      NearMask target_mask=Add_Effect_Table.getNearMaskPROOK<P>(from,target);
	      if(isAttackToKing) target_mask&=nearMask;
	      while (target_mask.any())
	      {
		const int n = target_mask.takeOneBit();
		Offset offset=Add_Effect_Table.getOffsetPROOK<P>(from,target,n&15);
		Position mid=target+offset.blackOffset<P>();
		/** */
		int count=0;
		if (isEmptyOrAdditional<State,isAttackToKing>(state,mid,target,count))
		{
		  for(int i=0;i<3;i++)
		  {
		    Piece p=state.getPieceAt(mid);
		    if(p.canMoveOn<P>()){
		      if (canPromote &&
			  (from.canPromote<P>()||mid.canPromote<P>()))
			action.unknownMove(from,mid,p,PtypeFuns<T>::promotePtype,true,P);
		      action.unknownMove(from,mid,p,T,false,P);
		    }
		    if (!p.isEmpty()){
		      if (isAttackToKing || count>0 || p.isEdge() || !state.hasEffectTo(p,target)) break;
		      count++;
		    }
		    mid=mid+Board_Table.getShortOffset(Offset32(mid,target));
		  }
		}
	      }
	    }
	  }
	}
      }

      template <class State,Player P,Ptype T,class Action,bool isLong,bool isAttackToKing>
      void generateDrop(State const& state,Position target, NearMask nearMask,Action& action)
      {
	NearMask target_mask
	  = (isLong ? NearMask::makeDirect(PtypeTraits<T>::moveMask>>10) :
	     NearMask::makeDirect(PtypeTraits<T>::moveMask<<16) & nearMask);
	if(isAttackToKing && isLong) target_mask&=nearMask;
	while (target_mask.any())
	{
	  const int n = target_mask.takeOneBit();
	  Direction dir=static_cast<Direction>(n&15);
	  Offset offset=Board_Table.template getOffset<P>(dir);
	  Position to=target-offset;
	  if (isLong)
	  {
	    Piece p;
	    Position pos1;
	    for(pos1=to;(p=state.getPieceAt(pos1)).isEmpty();pos1-=offset)
	      action.dropMove(pos1,T,P);
	    if(isAttackToKing || p.isEdge() || !state.hasEffectTo(p,target)) continue;
	    for(pos1-=offset;state.getPieceAt(pos1).isEmpty();pos1-=offset)
	      action.dropMove(pos1,T,P);
	    
	  }
	  /** ζػ */
	  else if (T!=PAWN || !state.template isPawnMaskSet<P>(to.x()))
	    action.dropMove(to,T,P);
	} 
      }

      /**
       * promoteԲǽ­ûˤ Functor
       */
      template <class State,Player P,Ptype T,class Action>
      class GoldKingAction
      {
	State const& state;
	Position target;
	Action& action;
	NearMask nearMask;
      public:
	GoldKingAction(State const& s,Position p,Action& a,NearMask n)
	  :state(s),target(p),action(a),nearMask(n)
	{}
	/**
	 * forEachOnBoardƤФ
	 */
	void operator()(Piece p)
	{
	  Position from=p.position();
	  generateMove<State,P,T,Action,false>(state,from,target,nearMask,action);
	}
      };
  
      /**
       * promoteǽ­ûˤ Functor
       */
      template <class State,Player P,Ptype T,class Action>
      class ShortPieceAction
      {
	State const& state;
	Position target;
	Action& action;
	NearMask nearMask;
      public:
	ShortPieceAction(State const& s,Position p,Action& a,NearMask n)
	  :state(s),target(p),action(a),nearMask(n)
	{}
	/**
	 * forEachOnBoardƤФ
	 */
	void operator()(Piece p)
	{
	  Position from=p.position();
	  if (p.isPromotedNotKingGold())
	  {
	    generateMove<State,P,PtypeFuns<T>::promotePtype,Action,false>(state,from,target,nearMask,action);
	  }
	  else
	  {
	    generateMove<State,P,T,Action,true>(state,from,target,nearMask,action);
	    generateMove<State,P,T,Action,false>(state,from,target,nearMask,action);
	  }  
	}
      };

      /**
       * ­Ĺˤ Functor
       */
      template <class State,Player P,Ptype T,class Action,bool isAttackToKing>
      class LongPieceAction
      {
	State const& state;
	Position target;
	Action& action;
	NearMask nearMask;
      public:
	LongPieceAction(State const& s,Position p,Action& a,NearMask n)
	  :state(s),target(p),action(a),nearMask(n)
	{}
	/**
	 * forEachOnBoardƤФ
	 */
	void operator()(Piece p)
	{
	  Position from=p.position();
	  if (p.isPromotedNotKingGold())
	  {
	    if (T==LANCE)
	      generateMove<State,P,PtypeFuns<T>::promotePtype,Action,false>(state,from,target,nearMask,action);
	    else
	    {
	      generateLongMove<State,P,PtypeFuns<T>::promotePtype,Action,false,isAttackToKing>(state,from,target,nearMask,action);
	    }
	  }
	  else
	  {
	    generateLongMove<State,P,T,Action,true,isAttackToKing>(state,from,target,nearMask,action);
	  }  
	}
      };

    } // namespace without_effect
  } // namespace move_generator
} // namespace osl

template <osl::Player P,osl::Ptype T,bool isAttackToKing>
template <class State,class Action>
void osl::move_generator::AddEffectShort<P,T,isAttackToKing>::
generate(const State& state,Position target,Action& action,NearMask nearMask)
{
  BOOST_STATIC_ASSERT((PtypeTraits<T>::isBasic));
  BOOST_STATIC_ASSERT((PtypeTraits<T>::canPromote));
  typedef without_effect::ShortPieceAction<State,P,T,Action> action_t;
  action_t gkAction(state,target,action,nearMask);
  state.template forEachOnBoard<P,T,action_t>(gkAction);
  /** drop move */
  if (state.template hasPieceOnStand<P,T>())
  {
    without_effect::generateDrop<State,P,T,Action,false,isAttackToKing>(state,target,nearMask,action);
  }
}

template <osl::Player P,osl::Ptype T,bool isAttackToKing>
template <class State,class Action>
void osl::move_generator::AddEffectLong<P,T,isAttackToKing>::
generate(const State& state,Position target,Action& action,NearMask nearMask)
{
  typedef without_effect::LongPieceAction<State,P,T,Action,isAttackToKing> action_t;
  action_t gkAction(state,target,action,nearMask);
  state.template forEachOnBoard<P,T,action_t>(gkAction);
  /** drop move */
  if (state.template hasPieceOnStand<P,T>())
  {
    without_effect::generateDrop<State,P,T,Action,true,isAttackToKing>(state,target,nearMask,action);
  }
}

template <osl::Player P,bool isAttackToKing>
template <class State,class Action>
void osl::move_generator::AddEffect<P,isAttackToKing>::
generateKing(const State& state,Position target,Action& action,NearMask nearMask)
{
  typedef without_effect::GoldKingAction<State,P,KING,Action> action_t;
  action_t gkAction(state,target,action,nearMask);
  state.template forEachOnBoard<P,KING,action_t>(gkAction);
}

template <osl::Player P,bool isAttackToKing>
template <class State,class Action>
void osl::move_generator::AddEffect<P,isAttackToKing>::
generateGold(const State& state,Position target,Action& action,NearMask nearMask)
{
  typedef without_effect::GoldKingAction<State,P,GOLD,Action> action_t;
  action_t gkAction(state,target,action,nearMask);
  state.template forEachOnBoard<P,GOLD,action_t>(gkAction);
  /** drop move */
  if (state.template hasPieceOnStand<P,GOLD>())
  {
    without_effect::generateDrop<State,P,GOLD,Action,false,isAttackToKing>(state,target,nearMask,action);
  }
}

template <osl::Player P,bool isAttackToKing>
template <class State,class Action>
void osl::move_generator::AddEffect<P,isAttackToKing>::
generate(const State& state,Position target,Action& action)
{
  BOOST_STATIC_ASSERT(! boost::is_pointer<State>::value);
  NearMask nearMask=NearMask::make<State,P>(state,target);
  /* Ƽˤmove */
  AddEffectShort<P,PAWN,isAttackToKing>::generate(state,target,action,nearMask);
  AddEffectLong<P,LANCE,isAttackToKing>::generate(state,target,action,nearMask);
  AddEffectShort<P,KNIGHT,isAttackToKing>::generate(state,target,action,nearMask);
  AddEffectShort<P,SILVER,isAttackToKing>::generate(state,target,action,nearMask);
  generateGold(state,target,action,nearMask);
  if (!isAttackToKing)
    generateKing(state,target,action,nearMask);
  AddEffectLong<P,BISHOP,isAttackToKing>::generate(state,target,action,nearMask);
  AddEffectLong<P,ROOK,isAttackToKing>::generate(state,target,action,nearMask);
}

template<bool isAttackToKing>
template<class State>
void osl::move_generator::GenerateAddEffect<isAttackToKing>::
generate(Player player, const State& state, Position target, 
	 move_action::Store& store)
{
  BOOST_STATIC_ASSERT(! boost::is_pointer<State>::value);
  using namespace osl::move_action;
  if(player==BLACK)
  {
    SafeFilter<BLACK,State,Store> filter(state, store);
    AddEffect<BLACK,isAttackToKing>::generate(state,target,filter);
  }
  else
  {
    SafeFilter<WHITE,State,Store> filter(state, store);
    AddEffect<WHITE,isAttackToKing>::generate(state,target,filter);
  }
}

#endif /* _MOVE_GENERATOR_ADD_EFFECT_TCC */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
