#ifndef _GENERATER_ADD_EFFECT_WITH_EFFECT_TCC
#define _GENERATER_ADD_EFFECT_WITH_EFFECT_TCC

#include "osl/move_generator/addEffectWithEffect.h"
#include "osl/move_generator/open.h"

namespace osl
{
  namespace move_generator
  {
    template<class State,class Action> 
    template<Player P>
    void AddEffectWithEffect<State,Action>::generateMoves(const State& state,Position pos,Action& action)
    {
      generateEffectDir<P,U>(state,pos,action);
      generateEffectDir<P,UL>(state,pos,action);
      generateEffectDir<P,UR>(state,pos,action);
      generateEffectDir<P,L>(state,pos,action);
      generateEffectDir<P,R>(state,pos,action);
      generateEffectDir<P,DL>(state,pos,action);
      generateEffectDir<P,D>(state,pos,action);
      generateEffectDir<P,DR>(state,pos,action);
      generateEffectDirKnight<P,UUL>(state,pos,action);
      generateEffectDirKnight<P,UUR>(state,pos,action);
    }

    /**
     * add effect to position "to"
     * the direction of the result effect is "Dir"
     * Dir is one of {UL, U, UR, L, R, DL, D, DR}
     * may produce duplicate moves 
     */
    template<class State,class Action> 
    template<Player P,Direction Dir>
    void AddEffectWithEffect<State,Action>::generateEffectDir(const State& state,Position to,Action& action){
      BOOST_STATIC_ASSERT( (Dir != UUL && Dir!= UUR ));
      Position from=to-DirectionPlayerTraits<Dir,P>::offset();
      Piece p=state.getPieceAt(from);
      if(p.isEmpty()){
	// short֤
	generateDrop<P,Dir,PAWN>(state,from,action);
	generateDrop<P,Dir,SILVER>(state,from,action);
	generateDrop<P,Dir,GOLD>(state,from,action);
	generateDrop<P,Dir,LANCE>(state,from,action);
	generateDrop<P,Dir,BISHOP>(state,from,action);
	generateDrop<P,Dir,ROOK>(state,from,action);
	// ư
	generateSimpleMovesTo<P,Dir,PAWN>(state,from,action);
	generateSimpleMovesTo<P,Dir,KNIGHT>(state,from,action);
	generateSimpleMovesTo<P,Dir,SILVER>(state,from,action);
	generateSimpleMovesTo<P,Dir,GOLD>(state,from,action);
	generateSimpleMovesTo<P,Dir,KING>(state,from,action);
	generateSimpleMovesTo<P,Dir,LANCE>(state,from,action);
	generateSimpleMovesTo<P,Dir,BISHOP>(state,from,action);
	generateSimpleMovesTo<P,Dir,ROOK>	(state,from,action);
	// BISHOPʤмФϤʤ
	// BISHOP̤ˤ뤫Ǥ̤Ǥ
	from-=DirectionPlayerTraits<Dir,P>::offset();
	p=state.getPieceAt(from);
	while(p.isEmpty()){
	  // long֤
	  generateDrop<P,Dir,LANCE>(state,from,action);
	  generateDrop<P,Dir,BISHOP>(state,from,action);
	  generateDrop<P,Dir,ROOK>(state,from,action);
	  // longshort direction
	  // longˤ
	  generateSimpleMovesTo<P,DirectionTraits<Dir>::longDir,BISHOP>
	    (state,from,action);
	  generateSimpleMovesTo<P,DirectionTraits<Dir>::longDir,ROOK>
	    (state,from,action);
	  from-=DirectionPlayerTraits<Dir,P>::offset();
	  p=state.getPieceAt(from);
	}
	if(canMoveOn<P>(p)){
	  // ư꤬뤫ɤ
	  // ưƤ꤬뤫ɤ
	  generateCaptureMovesTo<P,DirectionTraits<Dir>::longDir,LANCE>
	    (state,from,action);
	  generateCaptureMovesTo<P,DirectionTraits<Dir>::longDir,BISHOP>
	    (state,from,action);
	  generateCaptureMovesTo<P,DirectionTraits<Dir>::longDir,ROOK>
	    (state,from,action);
	}
	else if(!isEdge(p)){
	  assert(p.owner()==P);
	  // open attack
	  if (state.template hasEffectDir<DirectionTraits<Dir>::longDir,P>(from))
	  {
	    typedef typename State::simple_board_t simple_board_t;
	    Open<P>::template
	      generateOpenAttack<simple_board_t,Action,DirectionTraitsGen<Dir>::primDir>
	      (state,from,p,action,to);
	  }
	}
      }
      else if(canMoveOn<P>(p)){ 
	assert(p.owner()==alt(P));
	// ư꤬뤫ɤ
	// ưƤ꤬뤫ɤ
	generateCaptureMovesTo<P,Dir,PAWN>(state,from,action);
	generateCaptureMovesTo<P,Dir,KNIGHT>(state,from,action);
	generateCaptureMovesTo<P,Dir,SILVER>(state,from,action);
	generateCaptureMovesTo<P,Dir,GOLD>(state,from,action);
	generateCaptureMovesTo<P,Dir,KING>(state,from,action);
	//
	generateCaptureMovesTo<P,Dir,LANCE>(state,from,action);
	generateCaptureMovesTo<P,Dir,BISHOP>(state,from,action);
	generateCaptureMovesTo<P,Dir,ROOK>(state,from,action);
      }
      else if(!isEdge(p)){ 
	// open attack
	assert(p.owner()==P);
	if (state.template hasEffectDir<DirectionTraits<Dir>::longDir,P>(from))
	{
	  typedef typename State::simple_board_t simple_board_t;
	  Open<P>::template 
	    generateOpenAttack<simple_board_t,Action,DirectionTraitsGen<Dir>::primDir>
	    (state,from,p,action,to);
	}
      }
    }
  } // namespace move_generator
} // namespace osl
#endif /* _GENERATER_ADD_EFFECT_WITH_EFFECT_TCC */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
