#include "osl/search/alphaBeta.h"
#include "osl/search/nullWindowSearch.h"
#include "osl/search/nullWindowSearchPolicy.h"
#include "osl/search/simpleHashRecord.h"
#include "osl/search/searchMoveVector.h"
#include "osl/search/shouldPromoteCut.h"
#include "osl/category/categoryListUtil.h"
#include "osl/record/csa.h"
#include "osl/move_classifier/pawnDropCheckmate.h"
#include "osl/move_classifier/moveAdaptor.h"
#ifdef OSL_SMP
#include "osl/search/parallelSearch.h"
#endif
#include <iostream>

#ifndef _ALPHABETA_TCC
#define _ALPHABETA_TCC

/** ơõ򤵤ʤȤ */
// #define NO_BEST_MOVE_EXTENSION


/* ------------------------------------------------------------------------- */
/*	Constructors							     */
/* ------------------------------------------------------------------------- */

template <class Eval,typename MoveGenerator, typename Table, typename Recorder, typename Probabilities>
osl::search::AlphaBeta<Eval,MoveGenerator,Table,Recorder,Probabilities>::
AlphaBeta(const HashEffectState& s, typename framework_t::checkmate_t& c, 
	  Table *t, Recorder& r)
  : base_t(s, c, t, r), timer(60)
{
  assert(base_t::hasLastRecord());
}
  
template <class Eval,typename MoveGenerator, typename Table, typename Recorder, typename Probabilities>
osl::search::AlphaBeta<Eval,MoveGenerator,Table,Recorder,Probabilities>::
~AlphaBeta()
{
}

/* ------------------------------------------------------------------------- */
/*	Methods for alpha beta search					     */
/* ------------------------------------------------------------------------- */

template <class Eval,typename MoveGenerator, typename Table, typename Recorder, typename Probabilities>
osl::Move osl::search::AlphaBeta<Eval,MoveGenerator,Table,Recorder,Probabilities>::
computeBestMoveIteratively(int limit, const int step, 
			   const int initial_limit, int node_limit,
			   const misc::RealTime& initial_stop_shedule)
{
  setStopSchedule(initial_stop_shedule);

  const bool use_signal_timer = hasSchedule();
  if (use_signal_timer)
  {
    this->timer = stopSchedule();
    base_t::setTimeOut(timer.getTimeLeftInSeconds());
  }
  else
  {
    this->timer = misc::RealTime(6000);
  }

  base_t::recorder.resetNodeCount();

  double last_iteration_consumed = 0;
  double total_consumed = 0;
  int limit_iterative = initial_limit;
  Move last_best_move = Move::INVALID();

  try
  {
    MoveLogProb search_move;
    while (limit_iterative < limit)
    {
      this->timer = stopSchedule();
      if (base_t::table->isVerbose())
	std::cerr << "iteration " << limit_iterative 
		  << " " << timer.getTimeLeftInDouble() << " left\n";
      base_t::recorder.startSearch(limit_iterative);
      alphaBetaSearch(search_move, limit_iterative);
      last_iteration_consumed = timer.getConsumedInDouble();
      total_consumed += last_iteration_consumed;
      base_t::recorder.setCheckmateCount
	(base_t::checkmateSearcher().totalNodeCount());
      base_t::recorder.finishSearch(search_move.getMove(), 
				    last_iteration_consumed,
				    base_t::table->isVerbose());
      base_t::testTimeOut();

      limit_iterative += step;
      last_best_move = search_move.getMove();
      bool time_over = false;
      if (hasSchedule())
      {
	const double current_time_left = stopSchedule().getTimeLeftInDouble();
	if (current_time_left 
	    < last_iteration_consumed * base_t::next_iteration_coefficient)
	  time_over = true;
	if (! time_over)
	{
	  SimpleHashRecord *record
	    = base_t::table->find(base_t::currentHash());
	  if (record)
	  {
	    record->setSumNodeCountOfChildren();
	    if (base_t::table->isVerbose())
	      record->dumpNodeCount(std::cerr, 3);
	  }
	}
      }
      if (time_over || (base_t::recorder.nodeCount() *8 > node_limit))
      {
	if (base_t::table->isVerbose())
	  std::cerr << "iteration stop at " << limit_iterative << "\n";
	goto finish;
      }
      if (base_t::table->isVerbose())
	std::cerr << "selected " 
		  << record::csa::show(search_move.getMove()) << "\n";
    }
    if (base_t::table->isVerbose())
      std::cerr << "go to final iteration\n";

    while (true)
    {
      base_t::recorder.startSearch(limit);
      alphaBetaSearch(search_move, limit);
      const double consumed = timer.getConsumedInDouble();
      base_t::recorder.setCheckmateCount
	(base_t::checkmateSearcher().totalNodeCount());
      base_t::recorder.finishSearch(search_move.getMove(), consumed,
				    base_t::table->isVerbose());
      last_best_move = search_move.getMove();

      if (last_best_move.isNormal())
	break;
      base_t::testTimeOut();

      // ۤäƤλ
      if (limit >= 2000)
	break;

      limit += 200;
      if (base_t::table->isVerbose())
	std::cerr << "extend limit to " << limit << " before resign\n";
    }
  }
  catch (std::exception& e)
  {
    std::cerr << "std exception " << e.what() << "\n";
  }
  catch (...)
  {
    std::cerr << "unknown exception\n";
#ifndef NDEBUG
    throw;
#endif
  }
finish:
  if (use_signal_timer)
    base_t::resetTimeOut();
  if (base_t::table->isVerbose())
    std::cerr << "selected " << record::csa::show(last_best_move) << "\n";
  return last_best_move;
}

namespace osl
{
  namespace search
  {
    /** AlphaBeta ˴ط륵֥饹 */
    namespace alpha_beta
    {
      /**
       * examineMoves ͳǼ¹Ԥ륳
       * (alpha beta window search ξ)
       */
      template <typename AlphaBeta, Player P>
      struct AlphaBetaSearchPolicy
      {
	typedef AlphaBetaWindow window_t;
	typedef AlphaBeta searcher_t;
	
	AlphaBeta *searcher;
	mutable AlphaBetaWindow ab_window;	
	AlphaBetaSearchPolicy(AlphaBeta *s, AlphaBetaWindow w) 
	  : searcher(s), ab_window(w) 
	{
	}
	/** @return cut Ǥ뤫 */
	bool searchWithMove(const SearchMove& ml, int& max_value, 
			    SearchMove& best_move) const
	{
	  return (*searcher).template
	    alphaBetaSearch<P>(ml, ab_window, max_value, best_move);
	}
	static const bool isBestMoveExtension = false;
	AlphaBetaWindow& window() const { return ab_window; }
	void setSearcher(AlphaBeta *new_searcher){
	  searcher = new_searcher;
	}
	template <class Recorder>
	TableHit isOutOfWindow(Player p, const SimpleHashRecord& record, 
			       int limit, int& val, const Recorder& recorder) const
	{
	  assert(P == p);
	  return AlphaBetaWindowUtil<P,AlphaBeta>::
	    isOutOfWindow(record, limit, ab_window, val, recorder);
	}
	void recordToTable(Player p, SimpleHashRecord *record, int limit, 
			   const SearchMove& best_move, int val) const
	{
	  assert(P == p);
	  assert(record);
	  assert(limit >= 0);
	  if (EvalTraits<P>::betterThan(ab_window.beta(), val))
	  {
	    searcher->recordUpperBound(P, record, limit, best_move, val);
	  }
	  if (EvalTraits<P>::betterThan(ab_window.alpha(), val))
	  {
	    searcher->recordLowerBound(P, record, limit, best_move, val);
	  }
	}
      };

      /**
       * searchWithMove ƤФ
       * @param P move ؤ(ؤ)
       */
      template <typename AlphaBeta, Player P>
      struct DoUndoAlphaBeta
      {
	AlphaBeta *searcher;
	const SearchMove& moved;
	AlphaBetaWindow window;
	int *result;
	DoUndoAlphaBeta(AlphaBeta *s, const SearchMove& md, AlphaBetaWindow w)
	  : searcher(s), moved(md), window(w)
	{
	  assert(P == md.getMove().player());
	}
	void setResultAddress(int *result)
	{
	  this->result = result;
	}
	void operator()(Position /*lastTo*/)
	{
#ifndef NDEBUG
	  const int cur_limit = searcher->curLimit();
#endif
	  SearchMove best_move;
	  *result = (*searcher).template 
	    alphaBetaSearchAfterMove<P>(moved, window, best_move);
	  assert(cur_limit == searcher->curLimit());
	}
      };
    } // namespace search
  } // namespace alpha_beta
} // namespace osl

template <class Eval,typename MoveGenerator, typename Table, typename Recorder, typename Probabilities>
template <osl::Player P>
int osl::search::AlphaBeta<Eval,MoveGenerator,Table,Recorder,Probabilities>::
alphaBetaSearch(AlphaBetaWindow window, MoveLogProb& best_move, int limit)
{
  assert(P == base_t::state().getTurn());
  assert(window.alpha() % 2);
  assert(window.beta() % 2);
  base_t::setRoot(limit);
  assert(base_t::curDepth() == 0);

  // ޤơ֥򸣤
  SimpleHashRecord *record_in_table
    = base_t::table->allocate(base_t::currentHash(), limit);
  SimpleHashRecord *record = record_in_table;
  boost::scoped_ptr<SimpleHashRecord> record_if_not_allocated;
  if (! record)
  {
    record_if_not_allocated.reset(new SimpleHashRecord());
    record = record_if_not_allocated.get();
  }
  assert(record);
  assert(base_t::hasLastRecord());
  assert(base_t::rootRecord() == 0 || base_t::rootRecord() == record);
  base_t::setRootRecord(record);

  if (record_in_table)
  {
    int table_value;
    if (! record_in_table->bestMove().getMove().isNormal())
      record_in_table->resetValue();
    else if (AlphaBetaWindowUtil<P,base_t>::isOutOfWindow
	     (*record, limit, window, table_value, base_t::recorder))
    {
      best_move = record->bestMove().moveLogProb();
      return table_value;
    }
  }
#ifndef DONT_USE_CHECKMATE
  // ;ƤǤߤ root ǤƤǤʤ
  const int check_limit 
    = ((limit < 500) 
       ? 800 
       : ((limit < 700) ? 3800 : 4000));
  const int checkmate_max = checkmate::limitToCheckCount(check_limit);
  const int checkmate_node = record->qrecord.checkmateNodesLeft(checkmate_max);
  if (checkmate_node > 0)
  {
    PathEncoding path(base_t::path());
    assert(path.turn() == base_t::state().getTurn());
    const bool my_king_in_check
      = base_t::state().hasEffectBy(alt(P),base_t::state().getKingPosition(P));
    if (my_king_in_check)
    {
      // ꤫鲦꤬äƤ
      base_t::recorder.gotoCheckmateSearch(base_t::state(), checkmate_node);
      const bool lose = base_t::template isLosingState<P>(checkmate_node);
      base_t::recorder.backFromCheckmateSearch();
      if (lose)
      {
	best_move = MoveLogProb(Move::INVALID(),100);
	base_t::recordLoseByCheckmate(P, record);
	return base_t::winByCheckmate(alt(P));
      }
    }
    // ͤޤʤСͤޤȤͤ 
    {
      Move checkmate_move;
      base_t::recorder.gotoCheckmateSearch(base_t::state(), checkmate_node);
      const bool win = base_t::template isWinningState<P>
	(checkmate_node, checkmate_move, record->qrecord.attack_oracle);
      base_t::recorder.backFromCheckmateSearch();
      if (win)
      {
	best_move = MoveLogProb(checkmate_move,100);
	base_t::recordWinByCheckmate(P, record, checkmate_move);
	return base_t::winByCheckmate(P);
      }
    }
    // ͤͤ
    if ((! my_king_in_check)
	&& (! (record->threatmate().isThreatmate(P))))
    {
      Move threatmate_move;
      base_t::recorder.gotoCheckmateSearch(base_t::state(), checkmate_node);
      const bool threatmate 
	= base_t::template isThreatmateState<P>
	(checkmate_node, threatmate_move, record->qrecord.threatmate_oracle);
      base_t::recorder.backFromCheckmateSearch();
      if (threatmate)
      {
	if (record)
	  record->threatmate().setThreatmate(P, threatmate_move);
	if (base_t::table->isVerbose())
	  std::cerr << "root threatmate " << threatmate_move << "\n";
      }
    }

  }
#endif
  // 礤κƤ
  base_t::template quiescenceValue<P>(base_t::winThreshold(alt(P)),
				      base_t::winThreshold(P));

  SearchMoveSet& move_set = record->moves();
  if (record->qrecord.bestMove().isNormal())
  {
    const MoveLogProb quiescence_move(record->qrecord.bestMove(),
				      Probabilities::KillerMove);
    move_set.assignIfBetter(quiescence_move);
  }

  SearchMove best_search_move;

  while (true)
  {
    const category::CategoryEnv env(&base_t::state(), limit, &base_t::history(),
				    base_t::getEval().progress32(),
				    record, base_t::curDepth());
    CategoryListUtil::gatherAllMoves<MoveGenerator>(env, move_set);
    if ((move_set.size() > 1) || (limit > 2000))
      break;
    limit += 100;
  }
  if (move_set.size() == 1)
  {
    best_move = move_set.front().moveLogProb();
    return 0;
  }
  SearchMoveVector all_moves;
  {
    SearchMoveSet::range r(move_set);
    for (SearchMoveSet::iterator p=r.first; p!=r.last; ++p)
    {
      if (move_classifier::MoveAdaptor<move_classifier::PawnDropCheckmate<P> >
	  ::isMember(base_t::state(), p->getMove()))
	continue; // 
      if ((! p->getMove().isNormal())
	  || ShouldPromoteCut::canIgnoreAndNotDrop(p->getMove()))
	continue;
      p->setLogProbAtMost(1000);	// ϳ̵褦
      if (p->getLogProb() <= base_t::curLimit())
	all_moves.push_back(&*p);
    }
  }
  all_moves.sortByValue<P>(window.alpha()
			   + Eval::captureValue(newPtypeO(P,PAWN)));
  base_t::recorder.newCategory("root", base_t::curLimit());
  int result = base_t::winByCheckmate(alt(P));
  for (SearchMoveVector::const_iterator p=all_moves.begin(); 
       p!=all_moves.end(); p++)
  {
    const int last_value = result;
    const int cut = alphaBetaSearch<P>(**p, window, result, best_search_move);
    if (base_t::table->isVerbose()
	&& EvalTraits<P>::betterThan(result, last_value))
      std::cerr << record::csa::show((*p)->getMove())
		<< " " << (*p)->getLogProb() << " = " << result << "\n";
    if (cut)
    {
      assert(! base_t::isWinValue(alt(P), result));
      break;
    }
  }
  record->setLowerBound(P, base_t::curLimit(), best_search_move, result);

  best_move = best_search_move.moveLogProb();
  return result;
}

template <class Eval,typename MoveGenerator, typename Table, typename Recorder, typename Probabilities>
template <osl::Player P>
bool osl::search::AlphaBeta<Eval,MoveGenerator,Table,Recorder,Probabilities>::
alphaBetaSearch(const SearchMove& move, 
		AlphaBetaWindow& w, int& base_value, SearchMove& best_move)
{
  // base_value ϡlow fail ξǤ⤽餷뤿ɬ
  assert(w.alpha() % 2);
  assert(w.beta() % 2);
  assert(P == move.getMove().player());
  assert(P == base_t::state().getTurn());
  // std::cerr << P << " " << w.beta() << " " << w.alpha() << "\n";
  assert(eval::notLessThan(P, w.beta(), w.alpha()));
  assert(eval::betterThan(P, w.alpha(), base_value));

  if ((! base_t::timerAvailable()) && (! timer.isInvalid()) && (! timer.timeLeft()))
    throw misc::NoMoreTime();
#ifdef OSL_SMP
  parallelSearch.checkStop();
#endif

  typedef alpha_beta::DoUndoAlphaBeta<AlphaBeta,P> helper_t;
  helper_t helper(this, move, w);
  const int value
    = framework_t::template searchWithMove<P,helper_t>(move, helper);

  if (eval::betterThan(P, value, base_value))
  {
    base_value = value;
    best_move = move;
    if (eval::betterThan(P, value, w.alpha()))
    {
      w.alpha() = value + EvalTraits<P>::delta;
      if (eval::betterThan(P, value, w.beta()))
	return true;
    }
  }
  return false;
}

template <class Eval,typename MoveGenerator, typename Table, typename Recorder, typename Probabilities>
template <osl::Player P>
int osl::search::AlphaBeta<Eval,MoveGenerator,Table,Recorder,Probabilities>::
alphaBetaSearchAfterMove(const SearchMove& moved, AlphaBetaWindow w,
			 SearchMove& best_move)
{
  assert(w.alpha() % 2);
  assert(w.beta() % 2);
  assert(alt(P) == base_t::state().getTurn());
  assert(P == moved.getMove().player());
  assert(eval::notLessThan(P, w.beta(), w.alpha()));

  const Player Turn = PlayerTraits<P>::opponent;
  typedef null_window::NullWindowSearchPolicy<AlphaBeta,Turn,false>
    null_policy_t;
  typedef alpha_beta::AlphaBetaSearchPolicy<AlphaBeta,Turn>
    ab_policy_t;

  base_t::null_window.alpha() = w.alpha();
  assert(base_t::null_window.alpha() % 2);
  // null window search
  base_t::recorder.tryMove(moved.moveLogProb(),
			   base_t::null_window.alpha(), base_t::curLimit());
  base_t::subLimit(moved.getLogProb());

  null_policy_t nullPolicy(this, base_t::nullWindow());
  int val = framework_t::template searchAllMoves<Turn,null_policy_t>
    (moved.record, nullPolicy, best_move);

  base_t::addLimit(moved.getLogProb());
  base_t::recorder.recordValue(moved.moveLogProb(),
			       val,eval::betterThan(P, val, w.alpha()),
			       base_t::curLimit());

  if (! eval::betterThan(P, val, w.alpha()))
    return val;

  // 򹹿õ

  // ¿ȿ
  base_t::recorder.retryMove(moved.moveLogProb(), w.alpha(), base_t::curLimit(), 1);
  if (moved.getLogProb() 
      > Probabilities::ReSearch*4 + framework_t::ReSearchLimitMargin) 
  {
    ab_policy_t policy(this, w.flipPlayer());
    base_t::subLimit(Probabilities::ReSearch*4);
    
    val = framework_t::template searchAllMoves<Turn,ab_policy_t>
      (moved.record, policy, best_move);

    base_t::addLimit(Probabilities::ReSearch*4);
    if (! eval::betterThan(P, val, w.alpha()))
      goto last;
  }
  if (moved.getLogProb() 
      > Probabilities::ReSearch*2 + framework_t::ReSearchLimitMargin) 
  {
    ab_policy_t policy(this, w.flipPlayer());
    base_t::subLimit(Probabilities::ReSearch*2);
    
    val = framework_t::template searchAllMoves<Turn,ab_policy_t>
      (moved.record, policy, best_move);

    base_t::addLimit(Probabilities::ReSearch*2);
    if (! eval::betterThan(P, val, w.alpha()))
      goto last;
  }
  
  {
  const int final_limit 
    = std::min(moved.getLogProb(), (int)Probabilities::ReSearch);
  base_t::subLimit(final_limit);

  ab_policy_t abPolicy(this, w.flipPlayer());
  val = framework_t::template searchAllMoves<Turn,ab_policy_t>
    (moved.record, abPolicy, best_move);

  base_t::addLimit(final_limit);
  }
last:
  base_t::recorder.recordValue(moved.moveLogProb(), val, 
			       eval::betterThan(P, val, w.alpha()),
			       base_t::curLimit());
  return val;
}

/* ------------------------------------------------------------------------- */

#endif /* _ALPHABETA_TCC */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
