/* checkmateSearcherHolder.tcc
 */
#ifndef OSL_CHECKMATESEARCHERHOLDER_TCC
#define OSL_CHECKMATESEARCHERHOLDER_TCC

#include "osl/search/checkmateSearcherHolder.h"
#include "osl/search/parallelSearch.h"
#include <boost/thread/mutex.hpp>
#include <boost/shared_ptr.hpp>
#include <deque>

template <class Checkmate>
struct osl::search::CheckmateSearcherHolder<Checkmate>::Table
  : public std::deque<boost::shared_ptr<Checkmate> >
{
  typedef boost::shared_ptr<Checkmate> element_t;
  typedef std::deque<element_t> base_t;
  typedef boost::mutex Mutex;
  mutable Mutex mutex;
  const Checkmate *seed;

  explicit Table(const Checkmate& seed)
    : base_t(parallelSearch.numCpus()*8), seed(&seed)
  {
  }
  using base_t::size;
  using base_t::operator[];
  Checkmate *clone(const Worker *key)
  {
    if (key->threadId() >= size())
    {
      Mutex::scoped_lock lk(mutex);
      do 
      {
	base_t::push_back(element_t());
      } while (key->threadId() >= size());
    }
    element_t& result = operator[](key->threadId());
    if (! result)
      result.reset(new Checkmate(*seed));
    return result.get();
  }
  void clear()
  {
    Mutex::scoped_lock lk(mutex);
    // ̥åɹԤɤ
    base_t::clear();
  }
  size_t totalNodeCount() const
  {
    size_t result = seed->totalNodeCount();
    Mutex::scoped_lock lk(mutex);
    for (size_t i=0; i<size(); ++i) {
      const element_t& checkmate = operator[](i);
      if (checkmate)
	result += checkmate->totalNodeCount();
    }
    return result;
  }
};

template <class Checkmate>
osl::search::CheckmateSearcherHolder<Checkmate>::
CheckmateSearcherHolder(const Checkmate& seed)
  : table(new Table(seed))
{
}

template <class Checkmate>
osl::search::CheckmateSearcherHolder<Checkmate>::
~CheckmateSearcherHolder()
{
}

template <class Checkmate>
Checkmate *
osl::search::CheckmateSearcherHolder<Checkmate>::
clone(const Worker *key)
{
  return table->clone(key);
}

template <class Checkmate>
void osl::search::CheckmateSearcherHolder<Checkmate>::
clear()
{
  table->clear();
}

template <class Checkmate>
size_t osl::search::CheckmateSearcherHolder<Checkmate>::
totalNodeCount() const
{
  return table->totalNodeCount();
}

#endif /* OSL_CHECKMATESEARCHERHOLDER_TCC */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
