/* quiescenceGenerator.h
 */
#ifndef QUIESCENCEGENERATOR_H
#define QUIESCENCEGENERATOR_H

#include "osl/search/shouldPromoteCut.h"
#include "osl/move_generator/addEffect.h"
#include "osl/move_generator/capture.h"
#include "osl/move_generator/escape.h"
#include "osl/move_generator/promote.h"
#include "osl/move_generator/safeDropMajorPiece.h"
#include "osl/move_generator/addEffect8.h"
#include "osl/move_generator/additionalLance.h"
#include "osl/move_action/store.h"
#include "osl/eval/pieceEval.h"
#include "osl/state/numEffectState.h"
#include "osl/container/position8.h"

namespace osl
{
  namespace search
  {
    /**
     * QuiescenceSearch ǻȤؼ
     */
    template <Player P>
    struct QuiescenceGenerator
    {
      /**
       * P Ũ PTYPE 롥
       * @param dont_capture Ͻ
       */
      template <Ptype PTYPE, bool has_dont_capture>
      static void capture(const NumEffectState&,
			  MoveVector& moves, Piece dont_capture);
      static void captureAll(const NumEffectState&, MoveVector& moves);
      static void captureAll(const NumEffectState&,
			     MoveVector& moves, Piece dont_capture);
      /**
       * P Ũ target 롥
       */
      static void capture(const NumEffectState&,
			  Position target, MoveVector& moves);
      static void promote(const NumEffectState&, PieceMask pins,
			  MoveVector& moves);
      template <Ptype PTYPE>
      static void promote(const NumEffectState&, MoveVector& moves);
      static void check(const NumEffectState&, PieceMask pins,
			MoveVector& moves, bool no_liberty=false);
      static void check(const NumEffectState&, PieceMask pins, bool no_liberty,
			const Position8& sendoffs, MoveVector& moves);
      static void escapeKing(const NumEffectState& state, MoveVector& moves); 
      /**
       * @return existance of safe (in takeback) move
       */
      static bool escapeKingInTakeBack(const NumEffectState& state, MoveVector& moves, bool check_by_lance); 
      static void dropMajorPiece(const NumEffectState& state, MoveVector& moves); 

      static void attackMajorPiece(const NumEffectState& state, PieceMask pins,
				   MoveVector& moves); 
      static void escapeAll(const NumEffectState& state, MoveVector& moves); 
      /**
       * @param escape KINGʳζ
       */
      static void escapeNormalPiece(const NumEffectState& state, 
				    Piece escape, MoveVector& moves,
				    bool add_support_only=false); 
      /**
       * ľ˻ؼ꤫ƨ
       */
      static void escapeFromLastMove(const NumEffectState& state, 
				     Move last_move, MoveVector& moves); 
      static void escapeFromLastMoveOtherThanPawn(const NumEffectState& state, 
				     Move last_move, MoveVector& moves); 
      /**
       * @return Τʤ˰ưǽ
       */
      static bool escapeByMoveOnly(const NumEffectState& state, 
				   Piece piece, MoveVector& moves); 
      static void attackGoldWithPawn(const NumEffectState& state,
				     PieceMask pins, MoveVector& moves); 
      static void attackWithKnight(const NumEffectState& state,
				       PieceMask pins, Position attack_from, 
				       Position attack_to,
				       bool has_knight, MoveVector& moves); 
      static void attackSilverWithPawn(const NumEffectState& state,
				       PieceMask pins, MoveVector& moves); 
      static void attackKnightWithPawn(const NumEffectState& state,
				       PieceMask pins, MoveVector& moves); 
      /**
       * Ѥ˿ʤ. ˤɤȴɤ
       */
      static void advanceBishop(const NumEffectState& state, 
				MoveVector& moves); 
      template <Direction DIR>
      static void advanceBishop(const NumEffectState& state, 
				const Position from, MoveVector& moves); 
      static void attackKing8(const NumEffectState& state, PieceMask pins,
			      MoveVector& moves); 
      static void attackToPinned(const NumEffectState& state, PieceMask pins,
				 MoveVector& moves); 

      static void utilizePromoted(const NumEffectState& state, 
				  Piece target,
				  MoveVector& moves); 

      static void breakThreatmate(const NumEffectState& state, 
				  Move threatmate, PieceMask pins,
				  MoveVector& moves);
      static void kingWalk(const NumEffectState& state, 
			   MoveVector& moves);

      struct CaptureGenerator
      {
	const NumEffectState *state;
	MoveVector& out;
	void operator()(Piece target_piece)
	{
	  QuiescenceGenerator<P>::capture(*state, target_piece.position(), out);
	}
      };
      struct CaptureGeneratorWithCheck
      {
	const NumEffectState *state;
	MoveVector& out;
	Piece dont_capture;
	void operator()(Piece target_piece)
	{
	  if (target_piece != dont_capture)
	    QuiescenceGenerator<P>::capture(*state, target_piece.position(), out);
	}
      };

    private:
      static void attackMajorPieceSecondSelection(const NumEffectState& state,
						  bool target_has_support,
						  const MoveVector& src,
						  MoveVector& out);
      static void attackMajorPieceFirstSelection(const NumEffectState& state,
						 PieceMask pins,
						 const MoveVector& all_moves,
						 MoveVector& moves,
						 MoveVector& expensive_drops);
      static void attackMajorPieceZerothSelection(const NumEffectState& state,
						  const MoveVector& src,
						  Position target,
						  MoveVector& open_out,
						  MoveVector& out);
    };
  } // namespace search
} // namespace osl




template <osl::Player P> 
inline
void osl::search::QuiescenceGenerator<P>::
capture(const NumEffectState& state, Position target, MoveVector& moves)
{
  using namespace move_action;
  MoveVector all_moves;
  Store store(all_moves);
  move_generator::Capture<P>::generate(state, target, store);
  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end(); ++p)
  {
    if (ShouldPromoteCut::canIgnoreMove<P>(*p))
      continue;
    moves.push_back(*p);
  }
}

template <osl::Player P> 
template <osl::Ptype PTYPE> 
inline
void osl::search::QuiescenceGenerator<P>::
promote(const NumEffectState& state, MoveVector& moves)
{
  assert(moves.empty());
  using namespace move_action;
  Store store(moves);
  move_generator::Promote<P,NumEffectState,Store>
    ::template generateMovesPtype<PTYPE>(state, store);
  return;
}

#endif /* QUIESCENCEGENERATOR_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
