/* quiescenceGenerator.tcc
 */
#ifndef QUIESCENCEGENERATOR_TCC
#define QUIESCENCEGENERATOR_TCC

#include "osl/search/quiescenceGenerator.h"
#include "osl/effect_util/unblockableEffect.h"
#include "osl/effect_util/sendOffPosition.h"
#include "osl/effect_util/additionalEffect.h"
#include "osl/effect_util/shadowEffect.h"
#include "osl/effect_util/neighboring8Direct.h"
#include "osl/effect_util/effectUtil.h"
#include "osl/effect_util/pin.h"
#include "osl/move_generator/attackToPinned.h"
#include "osl/neighboring8.h"

template <osl::Player P> 
void osl::search::QuiescenceGenerator<P>::
captureAll(const NumEffectState& state, MoveVector& moves)
{
  capture<PAWN,  false>(state, moves, Piece::EMPTY());
  capture<LANCE, false>(state, moves, Piece::EMPTY());
  capture<KNIGHT,false>(state, moves, Piece::EMPTY());
  capture<SILVER,false>(state, moves, Piece::EMPTY());
  capture<GOLD,  false>(state, moves, Piece::EMPTY());
  capture<BISHOP,false>(state, moves, Piece::EMPTY());
  capture<ROOK,  false>(state, moves, Piece::EMPTY());
}

template <osl::Player P> 
void osl::search::QuiescenceGenerator<P>::
captureAll(const NumEffectState& state,
	   MoveVector& moves, Piece dont_capture)
{
  capture<PAWN,  true>(state, moves, dont_capture);
  capture<LANCE, true>(state, moves, dont_capture);
  capture<KNIGHT,true>(state, moves, dont_capture);
  capture<SILVER,true>(state, moves, dont_capture);
  capture<GOLD,  true>(state, moves, dont_capture);
  capture<BISHOP,true>(state, moves, dont_capture);
  capture<ROOK,  true>(state, moves, dont_capture);
}

template <osl::Player P> 
template <osl::Ptype PTYPE, bool has_dont_capture> 
void osl::search::QuiescenceGenerator<P>::
capture(const NumEffectState& state, MoveVector& moves, Piece dont_capture)
{
  if (has_dont_capture && dont_capture.isPiece() && (dont_capture.ptype() == PTYPE))
  {
    typedef CaptureGeneratorWithCheck generator_t;
    generator_t generator = {&state, moves, dont_capture};
    state.template forEachOnBoard<PlayerTraits<P>::opponent,PTYPE,generator_t>
      (generator);
  }
  else
  {
    typedef CaptureGenerator generator_t;
    generator_t generator = {&state, moves};
    state.template forEachOnBoard<PlayerTraits<P>::opponent,PTYPE,generator_t>
      (generator);
  }
  if (((PTYPE == KNIGHT) || (PTYPE == BISHOP))
      && state.hasPieceOnStand<P,LANCE>())
  {
    // Ƕ꤬ϡɲĤ

    // NOTE: MoveVector push_backƤiteratorinvalidate
    // ʤȤͤ˰¸
    MoveVector::const_iterator original_end = moves.end();
    for (MoveVector::const_iterator p=moves.begin(); p!=original_end; ++p)
    {
      const Position from = p->from();
      if ((p->oldPtype() == PAWN)
	  && (state.hasEffectBy<PlayerTraits<P>::opponent>(p->to())))
      {
	move_generator::AdditionalLance<P>::generate(state, from, moves);
      }
    }
  }
}

template <osl::Player P> inline
void osl::search::QuiescenceGenerator<P>::
attackMajorPieceFirstSelection(const NumEffectState& state,
			       PieceMask pins, const MoveVector& all_moves,
			       MoveVector& moves,
			       MoveVector& expensive_drops)
{
  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end();
       ++p)
  {
    const Position to = p->to();
    {
      const int defense = state.countEffect(alt(P),to, pins);
      int offense = state.countEffect(P,to) + (p->isDrop() ? 1 : 0);
      if (defense >= offense)
	offense += AdditionalEffect::count2(state, to, P);
      if (p->ptype() != PAWN)
      {
	if (defense > offense)
	  continue;
      }
      else
      {
	if (defense && (offense==1))
	{
	  if (! (state.hasEffectByPtype<ROOK>(alt(P),to)
		 && state.hasEffectByPtype<BISHOP>(alt(P),to)))
	    continue;
	}
      }
      const Ptype ptype = p->ptype();
      if ((defense >= offense)
	  && (unpromote(ptype) != PAWN))
	continue;
      if (ptype != PAWN)
      {
	const Position front_position
	  = to + DirectionPlayerTraits<U,P>::offset();
	const Piece front_piece = state.getPieceAt(front_position);
	if (front_piece.ptypeO() == newPtypeO(alt(P),PAWN))
	  continue;
      }
      if (p->isDrop() && (p->ptype() == PAWN)
	  && to.canPromote<P>())
      {
	// 
	const Position back_position
	  = to + DirectionPlayerTraits<D,P>::offset();
	if (state.getPieceOnBoard(back_position).isEmpty())
	  moves.push_back(Move(back_position, PAWN, P));
      }
      const int ptype_value = eval::Ptype_Eval_Table.value(ptype);
      const bool is_large_piece
	= (ptype_value > eval::PtypeEvalTraits<BISHOP>::val);
      if ((p->isDrop() 
	   && (ptype_value > eval::PtypeEvalTraits<KNIGHT>::val))
	  || (is_large_piece && defense))
	expensive_drops.push_back(*p);
      else
	moves.push_back(*p);
    }
  }
}

template <osl::Player P> inline
void osl::search::QuiescenceGenerator<P>::
attackMajorPieceSecondSelection(const NumEffectState& state,
				bool target_has_support,
				const MoveVector& src,
				MoveVector& out)
{
  for (MoveVector::const_iterator p=src.begin(); p!=src.end(); ++p)
  {
    const Ptype ptype = p->ptype();
    if (target_has_support
	&& (eval::Ptype_Eval_Table.value(ptype) 
	    >= eval::PtypeEvalTraits<GOLD>::val))
      continue;

    out.push_back(*p);
  }
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
attackMajorPieceZerothSelection(const NumEffectState& state,
				const MoveVector& src,
				Position target,
				MoveVector& open_out,
				MoveVector& out)
{
  for (MoveVector::const_iterator p=src.begin(); p!=src.end(); ++p)
  {
    if (ShouldPromoteCut::canIgnoreAndNotDrop<P>(*p))
      continue;
    const bool is_open
      = (! p->isDrop())
      && state.hasEffectBy<P>(p->from())
      && (state.hasEffectByPtype<LANCE>(P, p->from())
	  || state.hasEffectByPtype<BISHOP>(P, p->from())
	  || state.hasEffectByPtype<ROOK>(P, p->from()))
      && ! state.hasEffectFromTo(p->ptypeO(), p->to(), target);
    // Ͻʣ
    const bool may_overlap = (p->capturePtype() != PTYPE_EMPTY
			      || p->isPromote());
    if (is_open) 
    {
      if (! may_overlap		// ֤Ϥ֤ʣʤ
	  || state.hasEffectBy<PlayerTraits<P>::opponent>(p->to()))
	open_out.push_back(*p);
    }
    else 
    {
      if (! may_overlap)
	out.push_back(*p);
    }
  }
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
attackMajorPiece(const NumEffectState& state, PieceMask pins,
		 MoveVector& moves) 
{
  assert(moves.empty());
  using namespace move_action;
  MoveVector work;
  Store store(work);
  MoveVector unsupported, supported;
  assert(PtypeTraits<BISHOP>::indexLimit == PtypeTraits<ROOK>::indexMin);
  for (int i = PtypeTraits<BISHOP>::indexMin;
       i < PtypeTraits<ROOK>::indexLimit; i++)
  {
    const Piece p = state.getPieceOf(i);
    if (p.isOnBoardByOwner<PlayerTraits<P>::opponent>())
    {
      const Position target = p.position();
      assert(isMajor(p.ptype()));
      const bool unstable_rook = (p.ptype() == ROOK)
	&& (state.getPieceAt(Board_Table.nextPosition(alt(P), target, U))
	    != Piece::EMPTY());	// ˿ʤʤ
      work.clear();
      if (state.hasEffectBy(alt(P), target) && (! unstable_rook))
      {
	move_generator::GenerateAddEffect<false>::generate<NumEffectState>
	  (P, state, target, store);
	attackMajorPieceZerothSelection(state, work, target, moves, supported);	
      }
      else if (unstable_rook || (! state.hasEffectBy(P, target)))
      {
	// ǼˤĤʤ
	move_generator::GenerateAddEffect<false>::generate<NumEffectState>
	  (P, state, target, store);
	attackMajorPieceZerothSelection(state, work, target, moves, unsupported);
      }
    }
  }

  // first pass
  MoveVector drops_supported, drops_unsupported;
  attackMajorPieceFirstSelection(state, pins, unsupported, moves, 
				 drops_unsupported);
  attackMajorPieceFirstSelection(state, pins, supported, moves, 
				 drops_supported);
  // second pass
  if (moves.size() > 5)
    return;
  attackMajorPieceSecondSelection(state, false, drops_unsupported, moves);
  if (moves.size() > 5)
    return;
  attackMajorPieceSecondSelection(state, true, drops_supported, moves);
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
escapeNormalPiece(const NumEffectState& state, 
		  Piece escape, MoveVector& moves, bool add_support_only)
{
  assert(escape.ptype() != KING);
  using namespace move_action;
  MoveVector all_moves;
  Store store(all_moves);
  const Piece attack_piece
    = state.hasEffectPiece<PlayerTraits<P>::opponent>(escape.position());
  const int attack_ptype_value 
    = eval::Ptype_Eval_Table.value(unpromote(attack_piece.ptype()));
  const Position escape_from = escape.position();
  if (! add_support_only)
  {
    GenerateEscape<P>::generate(state, escape, store);
    for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end();
	 ++p)
    {
      const Position to = p->to();
      if (p->isDrop())
      {
	if (! state.hasEffectBy<P>(to))
	  continue;	// 
#ifdef QSEARCH_EXPENSIVE_BLOCK
	if (eval::Ptype_Eval_Table.value(p->ptype()) > attack_ptype_value)
	  continue;
#else
	if (p->ptype() != PAWN)
	  continue;
#endif
	moves.push_back(*p);
      }
      else
      {
	if (p->from() != escape_from)
	{
	  // ư
	  if (! state.hasMultipleEffectBy(P, to))
	    continue;
	  if (eval::Ptype_Eval_Table.value(p->ptype()) > attack_ptype_value)
	    continue;
	}
	// Ͻʣ
	// TODO: KnightCaptureDepth
	if ((p->capturePtype() == PTYPE_EMPTY)
	    && !p->isPromote()
	    && (! ShouldPromoteCut::canIgnoreMove<P>(*p)))
	{
	  moves.push_back(*p);
	}
      }
    }
  }
  // ɳĤ
  if (unpromote(attack_piece.ptype()) == PAWN)
    return;
  if ((eval::Ptype_Eval_Table.value(escape.ptype()) - attack_ptype_value)
      >= (eval::PtypeEvalTraits<BISHOP>::val - eval::PtypeEvalTraits<KNIGHT>::val))
    return;
  if (state.hasEffectBy<P>(escape_from)
      || (state.countEffect(alt(P), escape_from) != 1))
    return;
  all_moves.clear();
  move_generator::GenerateAddEffect<>::generate<NumEffectState>
    (P, state, escape_from, store);
  const size_t escape_moves = moves.size();
  MoveVector not_drop;
  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end();
       ++p)
  {
    const Position to = p->to();
    if (p->isDrop())
    {
      if (state.hasEffectBy<PlayerTraits<P>::opponent>(to)
	  && (! state.hasEffectBy<P>(to)))
	continue;
      // Ǥļ⤫֤
      if ((p->ptype() != PAWN) && (p->ptype() != LANCE))
	continue;
      moves.push_back(*p);
    }
    else
    {
      // ư
      const int defense = state.countEffect(P,to);
      const int offense = state.countEffect(alt(P),to);
      if (offense >= defense)
	continue;
      // Ͻʣ
      // TODO: KnightCaptureDepth
      if ((p->capturePtype() == PTYPE_EMPTY)
	  && !p->isPromote()
	  && (! ShouldPromoteCut::canIgnoreMove<P>(*p)))
      {
	moves.push_back(*p);
      }
    }
  }
  // 碌
  if (state.hasEffectByPtype<BISHOP>(alt(P), escape_from)
      && state.hasPieceOnStand<P,BISHOP>())
  {
    const Piece bishop
      = state.hasEffectPiece<PlayerTraits<P>::opponent>(escape_from);
    if (bishop.ptype() == BISHOP)
    {
      Position p = bishop.position();
      const Offset offset = Board_Table.getShortOffset(Offset32(p,escape_from));
      p += offset;
      while (state.getPieceAt(p).isEmpty())
      {
	if (state.hasEffectBy<P>(p))
	{
	  moves.push_back(Move(p, BISHOP, P));
	  break;
	}
	p += offset;
      }
    }
  }
  
  if (escape_moves == moves.size()) // drop not found
    moves.push_back(not_drop.begin(), not_drop.end());
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
escapeAll(const NumEffectState& state, MoveVector& moves) 
{
  assert(moves.empty());

  bool found_attacked_piece = false;
  // ƨΤ𤫲ͤΰֹ⤤Τ
  for (int i = PtypeTraits<ROOK>::indexMin;
       i < PtypeTraits<ROOK>::indexLimit; ++i)
  {
    const Piece p = state.getPieceOf(i);
    if (p.isOnBoardByOwner<P>()
	&& state.hasEffectBy<PlayerTraits<P>::opponent>(p.position()))
    {
      found_attacked_piece = true;
      escapeNormalPiece(state, p, moves);
    }
  }
  for (int i = PtypeTraits<BISHOP>::indexMin;
       i < PtypeTraits<BISHOP>::indexLimit; ++i)
  {
    const Piece p = state.getPieceOf(i);
    if (p.isOnBoardByOwner<P>()
	&& state.hasEffectBy<PlayerTraits<P>::opponent>(p.position()))
    {
      found_attacked_piece = true;
      escapeNormalPiece(state, p, moves);
    }
  }

  if (found_attacked_piece)
    goto finish;
  for (int i = PtypeTraits<GOLD>::indexMin;
       i < PtypeTraits<GOLD>::indexLimit; ++i)
  {
    const Piece p = state.getPieceOf(i);
    if (p.isOnBoardByOwner<P>()
	&& state.hasEffectBy<PlayerTraits<P>::opponent>(p.position()))
    {
      escapeNormalPiece(state, p, moves);
      goto finish;
    }
  }
  for (int i = PtypeTraits<SILVER>::indexMin;
       i < PtypeTraits<SILVER>::indexLimit; ++i)
  {
    const Piece p = state.getPieceOf(i);
    if (p.isOnBoardByOwner<P>()
	&& state.hasEffectBy<PlayerTraits<P>::opponent>(p.position()))
    {
      escapeNormalPiece(state, p, moves);
      goto finish;
    }
  }
  for (int i = PtypeTraits<KNIGHT>::indexMin;
       i < PtypeTraits<KNIGHT>::indexLimit; ++i)
  {
    // ˤƨʤ
    const Piece p = state.getPieceOf(i);
    if (p.isOnBoardByOwner<P>()
	&& state.hasEffectBy<PlayerTraits<P>::opponent>(p.position()))
    {
      escapeNormalPiece(state, p, moves, p.ptype()==PKNIGHT);
      goto finish;
    }
  }
  for (int i = PtypeTraits<LANCE>::indexMin;
       i < PtypeTraits<LANCE>::indexLimit; ++i)
  {
    // ᡤƨʤ
    const Piece p = state.getPieceOf(i);
    if (p.isOnBoardByOwner<P>()
	&& state.hasEffectBy<PlayerTraits<P>::opponent>(p.position()))
    {
      escapeNormalPiece(state, p, moves, true);
      goto finish;
    }
  }
  for (int i = PtypeTraits<PAWN>::indexMin;
       i < PtypeTraits<PAWN>::indexLimit; ++i)
  {
    // ⡤ȶƨʤ
    const Piece p = state.getPieceOf(i);
    if (p.isOnBoardByOwner<P>()
	&& state.hasEffectBy<PlayerTraits<P>::opponent>(p.position()))
    {
      escapeNormalPiece(state, p, moves, true);
      goto finish;
    }
  }
finish:
  return;
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
escapeFromLastMoveOtherThanPawn(const NumEffectState& state, Move last_move, 
				MoveVector& moves)
{
  if (! last_move.isNormal())
    return;
  assert(last_move.ptype() != PAWN);

  PieceVector targets;
  const Position from = last_move.to();
  EffectUtil::findThreat(state, from, last_move.ptypeO(), targets);
  if (targets.empty())
    return;
  assert(targets[0].ptype() != KING);

  escapeNormalPiece(state, targets[0], moves, 
		    (last_move.ptype() != PAWN)
		    && (eval::Ptype_Eval_Table.value(targets[0].ptype())
			<= eval::Ptype_Eval_Table.value(KNIGHT)));
  if (targets.size() > 1)
    escapeNormalPiece(state, targets[1], moves, 
		      (last_move.ptype() != PAWN)
		      && (eval::Ptype_Eval_Table.value(targets[1].ptype())
			  <= eval::Ptype_Eval_Table.value(KNIGHT)));
}

template <osl::Player P>
bool osl::search::QuiescenceGenerator<P>::
escapeByMoveOnly(const NumEffectState& state, Piece piece, MoveVector& moves)
{
  assert(piece.isOnBoardByOwner<P>());

  using namespace move_action;
  MoveVector all_moves;
  Store store(all_moves);
  move_generator::PieceOnBoard<P>::generate(state, piece, store);

  const Player Opponent = PlayerTraits<P>::opponent;
  const Position from = piece.position();
  
  const bool consider_shadowing
    = state.hasEffectByPtype<LANCE>(Opponent, from)
    || state.hasEffectByPtype<ROOK>(Opponent, from)
    || state.hasEffectByPtype<BISHOP>(Opponent, from);
  const bool is_major = isMajor(piece.ptype());
  const bool chase_danger
    = (! state.hasEffectBy(P, from)
       && (state.hasMultipleEffectBy(alt(P), from)
	   || AdditionalEffect::hasEffect(state, from, alt(P))));
  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end();
       ++p)
  {
    assert(p->from() == piece.position());
    if (p->capturePtype() != PTYPE_EMPTY)
      continue;
    const Position to = p->to();
    const bool safe_position
      = (consider_shadowing
	 ? (! state.hasEffectByWithRemove<Opponent>(to, from))
	 : (! state.hasEffectBy<Opponent>(to)));
    if (safe_position)
    {
      if (! is_major || ! chase_danger)
	return true;
      if (! to.canPromote<Opponent>())
	return true; // ذʳоݤˤ QuiescenceSearch Ĵ
      if ((to - from) != Board_Table.getShortOffset(Offset32(to, from)))
	return true;		// ʾƨɤ
    }
    moves.push_back(*p);
  }
  return false;
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
check(const NumEffectState& state, PieceMask pins, MoveVector& moves,
      bool no_liberty)
{
  Position8 sendoffs;
  const Position king_position = state.getKingPosition(alt(P));
  effect_util::SendOffPosition::find<P>(state, king_position, sendoffs);
  check(state, pins, no_liberty, sendoffs, moves);
}


template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
check(const NumEffectState& state, PieceMask pins, bool no_liberty, 
      const Position8& sendoffs, MoveVector& moves)
{
  assert(moves.empty());
  using namespace move_action;
  MoveVector all_moves;
  Store store(all_moves);
  const Position king_position = state.getKingPosition(alt(P));
  move_generator::GenerateAddEffect<true>::generate<NumEffectState>
    (P, state, king_position, store);

  MoveVector merginal_moves;
  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end();
       ++p)
  {
    // ꡤϽʣ
    if (p->isPromote() || ShouldPromoteCut::canIgnoreAndNotDrop<P>(*p))
      continue;
    const Position to = p->to();
    const Ptype captured = p->capturePtype();
    
    if ((captured != PTYPE_EMPTY)
	&& (eval::Ptype_Eval_Table.value(captured)
	    >= eval::PtypeEvalTraits<KNIGHT>::val))
      continue;
    const bool is_open
      = (! p->isDrop())
      && state.hasEffectBy<P>(p->from())
      && (state.hasEffectByPtype<LANCE>(P, p->from())
	  || state.hasEffectByPtype<BISHOP>(P, p->from())
	  || state.hasEffectByPtype<ROOK>(P, p->from()))
      && ! state.hasEffectFromTo(p->ptypeO(), p->to(), king_position);
    if (! is_open)
    {
      if ((p->ptype() != PAWN) && (p->ptype() != KNIGHT))
      {
	// ɤޤʤ
	const Position front_position
	  = to + DirectionPlayerTraits<U,P>::offset();
	const Piece front_piece = state.getPieceAt(front_position);
	if (front_piece.ptypeO() == newPtypeO(alt(P),PAWN))
	  continue;
      }
      if (! sendoffs.isMember(to))
      {
	const int defense = state.countEffect(alt(P),to, pins);
	int offense = state.countEffect(P,to) + no_liberty;
	const bool may_effective = offense && (! sendoffs.empty());
	if (p->isDrop())
	  ++offense;
	if (defense >= offense)
	  offense += AdditionalEffect::count2(state, to, P);
	if (defense >= offense)
	  offense += ShadowEffect::count2(state, to, P);
	if (p->ptype() == KNIGHT)
	{
	  const Position front_position
	    = to + DirectionPlayerTraits<U,P>::offset();
	  if (state.hasEffectBy<P>(front_position))
	    offense+=1;		// pin ⤷϶˲ǤƤ뤫
	}
	if (defense > offense)
	{
	  const bool side_attack
	    = (king_position.x() == 1 || king_position.x() == 9)
	    && p->ptype() == PAWN;
	  if (! side_attack)
	    continue;		// ˽ʲϻؤʤ
	}
	else if (defense == offense)
	{
	  if ((unpromote(p->ptype()) == PAWN)
	      || state.hasEffectByPtype<PAWN>(P, to)
	      || state.hasEffectByPtype<LANCE>(P, to))
	    moves.push_back(*p);
	  else if (may_effective)
	    merginal_moves.push_back(*p);
	  continue;
	}
      }
    }
    if (p->isDrop() && (p->ptype() == PAWN)
	&& to.canPromote<P>())
    {
      // 
      const Position back_position
	= to + DirectionPlayerTraits<D,P>::offset();
      if (state.getPieceOnBoard(back_position).isEmpty())
	moves.push_back(Move(back_position, PAWN, P));
    }
    moves.push_back(*p);
  }
  if (moves.size() < 3)
    moves.push_back(merginal_moves.begin(), merginal_moves.end());
}

template <osl::Player P> 
void osl::search::QuiescenceGenerator<P>::
promote(const NumEffectState& state, PieceMask pins, MoveVector& moves)
{
  assert(moves.empty());
  using namespace move_action;
  MoveVector all_moves;
  Store store(all_moves);
  move_generator::Promote<P,NumEffectState,Store>::generateMoves(state, store);

  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end(); ++p)
  {
    const Position to = p->to();
    const int defense = state.countEffect(alt(P),to, pins);
    int offense = state.countEffect(P,to);
    if (defense >= offense)
      offense += AdditionalEffect::count2(state, to, P);
    if (p->ptype() == PPAWN
	&& defense && offense + 1 >= defense)
    {
      // ⤬ɲĤƤߤ?
      if (p->ptype() == PPAWN)
      {
	// ֤ư
	for (int i = PtypeTraits<ROOK>::indexMin;
	     i < PtypeTraits<ROOK>::indexLimit; ++i)
	{
	  const Piece rook = state.getPieceOf(i);
	  if (! rook.isOnBoardByOwner<P>() || rook.ptype() == PROOK
	      || rook.position().template canPromote<P>())
	    continue;
	  const Position mid(p->from().x(), rook.position().y());
	  if (state.getPieceOnBoard(mid).isEmpty()
	      && state.isEmptyBetween(p->from(), mid)
	      && state.isEmptyBetween(rook.position(), mid))
	  {
	    const Move m(rook.position(), mid, ROOK, PTYPE_EMPTY, false, P);
	    moves.push_back(m);
	  }
	}
	if (state.hasPieceOnStand<P,LANCE>())
	  move_generator::AdditionalLance<P>::generate(state, p->from(), moves);
	// ȤϷϤ餤?
      }
    }
    if (defense > offense)
    {
      continue;     // ǤϤޤ˽ʼϻؤʤ
    }
    if ((defense == offense)
	&& (p->ptype() != PPAWN))
      continue;

    moves.push_back(*p);
  }
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
attackKing8(const NumEffectState& state, PieceMask pins, MoveVector& moves) 
{
  using namespace move_action;
  MoveVector all_moves;
  Store store(all_moves);

  MoveVector not8;
  MoveVector not_drop;
  MoveVector major_drop;
  move_generator::AddEffect8<P>::generate(state, store);
  const Position king_position = state.getKingPosition(alt(P));
  const int king_x = king_position.x();
  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end();
       ++p)
  {
    // ꡤϽʣ
    if (p->isPromote() || ShouldPromoteCut::canIgnoreAndNotDrop<P>(*p))
      continue;
    const Position to = p->to();
    const Ptype captured = p->capturePtype();
    
    if (captured != PTYPE_EMPTY)
      continue;

    const Ptype ptype = p->ptype();

    const int defense = state.countEffect(alt(P),to,pins);
    int offense = state.countEffect(P,to) + (p->isDrop() ? 1 : 0);
    if (defense >= offense)
      offense += AdditionalEffect::count2(state, to, P);
    if (ptype == PAWN)
    {
      const Position head = (Board_Table.nextPosition(P, to, U));
      if (state.hasEffectBy<P>(head))
	++offense;
    }
    if (defense > offense)
      continue;
    if (defense == offense)
    {
      if (ptype != PAWN)
	continue;
    }
    if (isMajor(ptype))
    {
      if (defense == 0)
      {
	// ΤȤˤϤʤ TODO: kingΤߵ?
	if (isPromoted(ptype) && to.x() == king_x 
	    && abs(to.y() - king_position.y()) <= 2)
	  moves.push_back(*p);
	else if (p->isDrop())
	  major_drop.push_back(*p);
	else
	  not_drop.push_back(*p);
      }
      continue;
    } 
    if (Ptype_Table.getMoveMask(p->ptype()) == PtypeTraits<GOLD>::moveMask)
    {
      const int y = to.y();
      if (((P == BLACK) && (y == 1))
	  || ((P == WHITE) && (y == 9)))
	continue;		// 1ܤζ
    }
    // ɤޤʤ TODO: Ϥ?
    if (ptype != PAWN)
    {
      if (state.hasEffectByPtype<PAWN>(alt(P), to))
	continue;
      if (state.hasEffectByPtype<LANCE>(alt(P), to))
      {
	not_drop.push_back(*p);
	continue;
      }
    }
    if (! Neighboring8::isNeighboring8(king_position, to))
      not8.push_back(*p);
    else if (! p->isDrop())
      not_drop.push_back(*p);
    else
      moves.push_back(*p);
  }
  const size_t minimum_moves
    = (king_position.positionForBlack<P>().y() == 1) ? 3 : 2;
  for (MoveVector::const_iterator p=not8.begin();
       (p!=not8.end()) && (moves.size() <= minimum_moves); ++p)
  {
    moves.push_back(*p);
  }
  for (MoveVector::const_iterator p=not_drop.begin();
       (p!=not_drop.end()) && (moves.size() <= minimum_moves); ++p)
  {
    moves.push_back(*p);
  }
  for (MoveVector::const_iterator p=major_drop.begin();
       (p!=major_drop.end()) && (moves.size() <= minimum_moves); ++p)
  {
    moves.push_back(*p);
  }
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
attackToPinned(const NumEffectState& state, PieceMask pins, MoveVector& moves)
{
  using namespace move_action;
  MoveVector all_moves;
  Store store(all_moves);
  move_generator::AttackToPinned<P>::generate(state, store);
  
  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end();
       ++p)
  {
    // ꡤϽʣ
    if (p->isPromote() || ShouldPromoteCut::canIgnoreAndNotDrop<P>(*p))
      continue;
    const Position to = p->to();
    const Ptype captured = p->capturePtype();
    
    if (captured != PTYPE_EMPTY)
      continue;

    const Ptype ptype = p->ptype();

    const int defense = state.countEffect(alt(P),to,pins);
    int offense = state.countEffect(P,to) + (p->isDrop() ? 1 : 0);
    if (defense >= offense)
      offense += AdditionalEffect::count2(state, to, P);
    if (ptype == PAWN)
    {
      const Position head = (Board_Table.nextPosition(P, to, U));
      if (state.hasEffectBy<P>(head))
	++offense;
    }
    if (defense > offense)
      continue;
    if (defense == offense)
    {
      if (ptype != PAWN)
	continue;
    }
    // ɤޤʤ TODO: Ϥ?
    if (ptype != PAWN)
    {
      const Position front_position
	= to + DirectionPlayerTraits<U,P>::offset();
      const Piece front_piece = state.getPieceAt(front_position);
      if (front_piece.ptypeO() == newPtypeO(alt(P),PAWN))
	continue;
    }
    moves.push_back(*p);
  }
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
utilizePromoted(const NumEffectState& state, Piece target,
		MoveVector& moves)
{
  if (! target.isPromoted())
    return;
  if (/*isMajor*/
      target.ptype() == ROOK
      || target.ptype() == BISHOP)
    return;

  MoveVector all_moves;
  move_action::Store store(all_moves);
  move_generator::PieceOnBoard<P>::generate(state, target, store);

  MoveVector others;
  const Player Opponent = PlayerTraits<P>::opponent;
  bool has_good_capture = false;
  bool may_have_additional = false; // ֤Ȥ
  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end();
       ++p)
  {
    assert(p->from() == target.position());
    if (p->capturePtype() != PTYPE_EMPTY)
    {
      if (p->capturePtype() != PAWN)
	has_good_capture =true;
      continue;
    }
    
    const Position to = p->to();
    int offense = state.countEffect(P, to);
    const int defense = state.countEffect(Opponent, to);
    const int additional = AdditionalEffect::count2(state, to, P);
    if (defense >= offense)
    {
      offense += additional;
      may_have_additional |= (additional > 0);
    }
    if (defense >= offense)
    {
      others.push_back(*p);
      continue;
    }
    moves.push_back(*p);
  }
  if ((! has_good_capture) && may_have_additional)
    moves.push_back(others.begin(), others.end());
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
breakThreatmate(const NumEffectState& state, 
		Move threatmate, PieceMask pins, MoveVector& moves)
{
  assert(moves.empty());
  MoveVector all_moves, major_piece, major_sacrifice;
  move_action::Store store(all_moves);
  assert(threatmate.isNormal());
  const Position target = threatmate.to();

  // defense by add effect
  move_generator::GenerateAddEffect<false>::generate<NumEffectState>
    (P, state, target, store);
  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end(); ++p)
  {
    const Ptype ptype = p->ptype();
    if (ptype == KING)
      continue;			// KING_WALK will generate
    if (! p->isDrop())
    {
      if (p->isPromote() 
	  || ShouldPromoteCut::canIgnoreAndNotDrop<P>(*p))
	continue;
      if (isMajor(ptype)
	  && state.hasEffectByPiece(state.getPieceOnBoard(p->from()), target))
	continue;			// already have effect
    }
    const Position to = p->to();
    const int me = state.countEffect(P, to) + (p->isDrop() ? 1 : 0);
    const int op = state.countEffect(alt(P), to, pins);
    if ((me >= 2) || (op == 0))
    {
      if (isMajor(ptype))
      {
	if (op)
	  major_sacrifice.push_back(*p);
	else 
	  major_piece.push_back(*p);
      }
      else			// ! major
      {
	if (p->isDrop() && (ptype == GOLD || ptype == SILVER)
	    && (to.x() == 1 || to.x() == 9))
	  major_piece.push_back(*p);
	else
	  moves.push_back(*p);
      }
    }
  }
  all_moves.clear();
  
  if (threatmate.isDrop())
  {
    ;				// ꤷơDrop ˡ?
  }
  // Ĺblock¾

  const Position king_position = state.getKingPosition(P);
  // make a space for king
  {
    for (int i=UL; i<=DR; ++i)
    {
      const Piece p = state.getPieceAt(Board_Table.nextPosition
				       (P,king_position,Direction(i)));
      if (! (p.isPiece() && p.owner() == P))
	continue;
      if (state.hasEffectBy(alt(P), p.position()))
	continue;
      move_generator::PieceOnBoard<P>::generate(state, p, store);
    }
    for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end(); ++p)
    {
      assert(! p->isDrop());
      if (p->isPromote() 
	  || ShouldPromoteCut::canIgnoreAndNotDrop<P>(*p))
	continue;
      const Ptype captured = p->capturePtype();
      if ((captured != PTYPE_EMPTY)
	  && (eval::Ptype_Eval_Table.value(captured)
	      >= eval::PtypeEvalTraits<KNIGHT>::val))
	continue;
      moves.push_back(*p);
    }
  }
  // defence by add effect: unusual moves
  const size_t minimum_moves = 8;
  for (MoveVector::const_iterator p=major_piece.begin(); 
       p!=major_piece.end() && moves.size() < minimum_moves; ++p)
  {
    moves.push_back(*p);
  }
  for (MoveVector::const_iterator p=major_sacrifice.begin(); 
       p!=major_sacrifice.end() && moves.size() < minimum_moves; ++p)
  {
    moves.push_back(*p);
  }
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
attackGoldWithPawn(const NumEffectState& state, PieceMask pins, 
		   MoveVector& moves) 
{
  assert(moves.empty());
  using namespace move_action;

  const bool has_pawn = state.hasPieceOnStand<P, PAWN>();
  const bool has_lance = state.hasPieceOnStand<P, LANCE>();
  const bool has_knight = state.hasPieceOnStand<P, KNIGHT>();
  const bool has_silver = state.hasPieceOnStand<P, SILVER>();
  for (int i = PtypeTraits<GOLD>::indexMin;
       i < PtypeTraits<GOLD>::indexLimit; ++i)
  {
    const Piece p = state.getPieceOf(i);
    if (! p.isOnBoardByOwner<PlayerTraits<P>::opponent>())
      continue;
    const Position head = (Board_Table.nextPosition(P, p.position(), D));
    if (state.getPieceAt(head).isEmpty())
    {
      // TODO: Ϥ˵
      const int defense = state.countEffect(alt(P), head, pins);
      int attack = state.countEffect(P, head)
	+ state.hasEffectByPtype<BISHOP>(P, p.position());
      if (defense >= attack)
	attack += AdditionalEffect::count2(state, head, P);

      // move
      if (! head.canPromote<P>()) // ϤǤƤϤ
      {
	const Position origin = (Board_Table.nextPosition(P, head, D));
	if (defense <= attack
	    || state.hasEffectByPtype<BISHOP>(P, origin)) // open attack
	{
	  const Piece candidate = state.getPieceAt(origin);
	  if (candidate.ptype() == PAWN && candidate.owner() == P)
	  {
	    const Move move(origin, head, PAWN, PTYPE_EMPTY, false, P);
	    moves.push_back(move);
	    ++attack;		// 
	  }
	}
      }      
      // drop
      if (defense <= attack)
      {
	if (has_pawn && !state.template isPawnMaskSet<P>(head.x()))
	{
	  const Move move(head, PAWN, P);
	  moves.push_back(move);
	}
	if (has_lance)
	{
	  const Move move(head, LANCE, P);
	  moves.push_back(move);
	}
	if (has_silver)
	{
	  const Move move(head, SILVER, P);
	  moves.push_back(move);
	}
      }
      const bool generate_long_lance = has_lance 
	&& (! state.hasPieceOnStand<PlayerTraits<P>::opponent, PAWN>()
	    || state.template isPawnMaskSet<PlayerTraits<P>::opponent>(head.x()));
      if (generate_long_lance) 
      {
	for (Position to=Board_Table.nextPosition(P, head, D); 
	     state.getPieceAt(to).isEmpty(); 
	     to=Board_Table.nextPosition(P, to, D))
	{
	  const int defense = state.countEffect(alt(P), to, pins);
	  const int attack = state.countEffect(P, to);
	  if (defense > attack)
	    continue;
	  const Move move(to, LANCE, P);
	  moves.push_back(move);
	}
      }
    }
    // knight
    if (! state.getPieceAt(head).isEdge())
    {
      const Position knight_l = Board_Table.nextPosition(P, head, DL);
      attackWithKnight(state, pins, knight_l, p.position(), has_knight, moves);
      const Position knight_r = Board_Table.nextPosition(P, head, DR);
      attackWithKnight(state, pins, knight_r, p.position(), has_knight, moves);
    }
  }
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
attackWithKnight(const NumEffectState& state, PieceMask pins, 
		     Position attack_from, Position attack_to,
		     bool has_knight, MoveVector& moves) 
{
  if (state.getPieceAt(attack_from) != Piece::EMPTY())
    return;
  
  const int defense = state.countEffect(alt(P), attack_from, pins);
  int attack = state.countEffect(P, attack_from);

  if (defense == 1 
      && (attack == 1 || (has_knight && attack == 0)))
  {
    const Piece gold = state.hasEffectPiece(alt(P), attack_from);
    if (gold.ptype() == GOLD)
    {
      // Ũζμ꤬
      const Position guarded
	= Board_Table.nextPosition(P, gold.position(), U);
      if (state.getPieceAt(guarded).isOnBoardByOwner(alt(P))
	  && (state.countEffect(alt(P), guarded) 
	      <= state.countEffect(P, guarded)))
	++attack;
    }
  }
  
  if (defense > attack)
    return;
  
  if (has_knight)
  {
    const Move drop(attack_from, KNIGHT, P);
    moves.push_back(drop);
  }

  if (defense < attack)
  {
    mask_t mask=state.getEffect(attack_from).getMask(PtypeFuns<KNIGHT>::indexNum);
    mask &= mask_t::makeDirect(PtypeFuns<KNIGHT>::indexMask);
    mask &= state.getOnBoardMask(P).getMask(PtypeFuns<KNIGHT>::indexNum);
    BOOST_STATIC_ASSERT(PtypeTraits<KNIGHT>::indexLimit < 32);
    while (mask.any())
    {
      const int n = mask.takeOneBit();
      const Piece knight = state.getPieceOf(n);
      if (knight.isPromoted())
	continue;
      
      const Move move(knight.position(), attack_from, KNIGHT, PTYPE_EMPTY, false, P);
      assert(state.isAlmostValidMove<false>(move));
      moves.push_back(move);
    }
  }
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
attackKnightWithPawn(const NumEffectState& state, PieceMask pins,
		     MoveVector& moves) 
{
  assert(moves.empty());
  using namespace move_action;

  const bool has_pawn = state.hasPieceOnStand<P, PAWN>();
  for (int i = PtypeTraits<KNIGHT>::indexMin;
       i < PtypeTraits<KNIGHT>::indexLimit; ++i)
  {
    const Piece p = state.getPieceOf(i);
    if (p.isOnBoard() && p.ptype() == KNIGHT && p.owner() == alt(P))
    {
      const Position head = (Board_Table.nextPosition(P, p.position(), D));
      // ɤ...
      if (state.getPieceAt(head).isEmpty())
      {
	const int defense = state.countEffect(alt(P), head, pins);
	const int offense = state.countEffect(P, head);
	if ((defense <= offense)
	    && (! head.canPromote<P>())) // ϤǤƤϤ
	{
	  const Position origin = (Board_Table.nextPosition(P, head, D));
	  const Piece candidate = state.getPieceAt(origin);
	  if (candidate.ptype() == PAWN && candidate.owner() == P)
	  {
	    const Move move(origin, head, PAWN, PTYPE_EMPTY, false, P);
	    moves.push_back(move);
	  }
	}
	if (has_pawn && !state.template isPawnMaskSet<P>(head.x())
	    && (defense <= offense+1))
	{
	  const Move move(head, PAWN, P);
	  moves.push_back(move);
	}
      }
    }
  }
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
attackSilverWithPawn(const NumEffectState& state, PieceMask pins,
		     MoveVector& moves) 
{
  // assert(moves.empty());
  using namespace move_action;

  const bool has_pawn = state.hasPieceOnStand<P, PAWN>();
  const bool has_lance = state.hasPieceOnStand<P, LANCE>();
  const bool has_knight = state.hasPieceOnStand<P, KNIGHT>();
  const bool has_silver = state.hasPieceOnStand<P, SILVER>();
  const Position opponent_king = state.getKingPosition(alt(P));
  for (int i = PtypeTraits<SILVER>::indexMin;
       i < PtypeTraits<SILVER>::indexLimit; ++i)
  {
    const Piece p = state.getPieceOf(i);
    if (! p.isOnBoardByOwner<PlayerTraits<P>::opponent>())
      continue;

    const Position head = (Board_Table.nextPosition(P, p.position(), D));
    if (state.getPieceAt(head).isEmpty())
    {
      const int defense = state.countEffect(alt(P), head, pins);
      int attack = state.countEffect(P, head) 
	+ state.hasEffectByPtype<BISHOP>(P, p.position());
      if (defense >= attack)
	attack += AdditionalEffect::count2(state, head, P);
      const bool near_king 
	= Neighboring8Direct::hasEffect(state, p.ptypeO(), p.position(),
					opponent_king);
      if (defense > attack)
      {
	if (! near_king)
	  continue;
	// TODO: Ϥ˵
      }
      if (! head.canPromote<P>()) // ϤǤƤϤ
      {
	const Position origin = (Board_Table.nextPosition(P, head, D));
	const Piece candidate = state.getPieceAt(origin);
	if (candidate.ptype() == PAWN && candidate.owner() == P)
	{
	  const Move move(origin, head, PAWN, PTYPE_EMPTY, false, P);
	  moves.push_back(move);
	}
      }
      if (has_pawn && !state.template isPawnMaskSet<P>(head.x()))
      {
	const Move move(head, PAWN, P);
	moves.push_back(move);
      }
      if (! near_king)
	continue;
      if (defense <= attack)
      {
	if (has_lance)
	{
	  const Move move(head, LANCE, P);
	  moves.push_back(move);
	}
	if (has_silver)
	{
	  const Move move(head, SILVER, P);
	  moves.push_back(move);
	}
      }
    }
    // knight
    if (! state.getPieceAt(head).isEdge())
    {
      const Position knight_l = Board_Table.nextPosition(P, head, DL);
      attackWithKnight(state, pins, knight_l, p.position(), has_knight, moves);
      const Position knight_r = Board_Table.nextPosition(P, head, DR);
      attackWithKnight(state, pins, knight_r, p.position(), has_knight, moves);
    }
  }
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
kingWalk(const NumEffectState& state, MoveVector& moves)
{
  const Piece my_king = state.getKingPiece<P>();
  MoveVector all_moves;
  move_action::Store store(all_moves);
  move_generator::PieceOnBoard<P>::generate(state, my_king, store);
  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end(); ++p)
  {
    if (state.hasEffectBy(alt(P), p->to()))
      continue;
    moves.push_back(*p);
  }
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
escapeKing(const NumEffectState& state, MoveVector& moves) 
{
  assert(moves.empty());
  MoveVector all_moves;
  move_action::Store store(all_moves);
  GenerateEscape<P>::generate(state, state.template getKingPiece<P>(), store);
  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end();
       ++p)
  {
    const Position to = p->to();
    if (p->isDrop())
    {
      // ʳɤޤʤ
      if (! state.hasEffectBy<P>(to))
      {
	if (! ((p->ptype() == PAWN)
	       && state.hasMultipleEffectBy(alt(P), to)))
	  continue;
      }
    }
    else if (p->ptype() != KING)
    {
      // ʳɤޤʤ
      if (p->capturePtype() == PTYPE_EMPTY)
      {
	if (! state.hasMultipleEffectBy(P, to))
	{
	  if (! ((p->ptype() == PAWN)
		 && state.hasMultipleEffectBy(alt(P), to)))
	    continue;
	}
      }
    }
    moves.push_back(*p);
  }
}

template <osl::Player P>
bool osl::search::QuiescenceGenerator<P>::
escapeKingInTakeBack(const NumEffectState& state, MoveVector& moves,
		     bool check_by_lance) 
{
  bool has_safe_move = false;
  assert(moves.empty());
  MoveVector all_moves;
  move_action::Store store(all_moves);
  GenerateEscape<P>::generate(state, state.template getKingPiece<P>(), store);
  Position last_drop_to = Position::STAND();
  MoveVector drops;
  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end();
       ++p)
  {
    const Position to = p->to();
    if (p->ptype() == KING)
    {
      // TODO: Ϥ
      has_safe_move = true; // KFEND ή: ƨ꤬ɤȤ
      continue;
    }
    if (p->capturePtype() != PTYPE_EMPTY)
    {
      moves.push_back(*p);
      continue;
    }
    // ̵̹ɤޤʤ
    const int attack_count = state.countEffect(alt(P),to);
    const int defense_count 
      = state.countEffect(P,to) + (p->isDrop() ? 1 : 0);
    if (defense_count <= attack_count)
      continue;
    if ((attack_count == 1) && (! check_by_lance))
    {
      // ιФƼʬʬʤɤȤ
      has_safe_move = true;
      continue;
    }

    // ʾdrop β򤫤ʤΤǹ1ǵͤȽˤϽʬ
    // TODO: ְ¤ˤ
    if (to != last_drop_to)
    {
      last_drop_to = to;
      drops.push_back(*p);
    }
  }
  if (! has_safe_move)
    moves.push_back(drops.begin(), drops.end());
  return has_safe_move;
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
advanceBishop(const NumEffectState& state, MoveVector& moves) 
{
  for (int i = PtypeTraits<BISHOP>::indexMin;
       i < PtypeTraits<BISHOP>::indexLimit; ++i)
  {
    const Piece bishop = state.getPieceOf(i);
    if (! bishop.isOnBoardByOwner<P>())
      continue;
    if (bishop.ptype() != BISHOP)
      continue;
    const Position from = bishop.position();
    if (state.hasEffectBy<PlayerTraits<P>::opponent>(from))
      continue;			// escape Ͻʣ
    advanceBishop<UL>(state, from, moves);
    advanceBishop<UR>(state, from, moves);
  }
}

template <osl::Player P>
template <osl::Direction DIR>
void osl::search::QuiescenceGenerator<P>::
advanceBishop(const NumEffectState& state, 
	      const Position from, MoveVector& moves)
{
  const Offset offset = DirectionPlayerTraits<DIR,P>::offset();
  for (Position to=from+offset;; to+=offset)
  {
    if (! state.getPieceAt(to).isEmpty())
      break;		// capture ϽʣĤǤ׳Ƚ
    if (to.canPromote<P>())
      break;		// promote Ͻʣ
    if (state.hasEffectBy<PlayerTraits<P>::opponent>(to))
      continue;
    const Move move(from, to, BISHOP, PTYPE_EMPTY, false, P);
    assert(state.isAlmostValidMove<false>(move));
    moves.push_back(move);
  }
}
      
template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
escapeFromLastMove(const NumEffectState& state, Move last_move, MoveVector& moves)
{
  if (! last_move.isNormal())
    return;
  if (last_move.ptype() != PAWN)
  {
    escapeFromLastMoveOtherThanPawn(state, last_move, moves);
    return;
  }
  const Position attack_from = last_move.to();
  const Position attack_to = (Board_Table.nextPosition(alt(P),attack_from, U));
  const Piece target = state.getPieceOnBoard(attack_to);
  if (! target.isOnBoardByOwner<P>())
    return;

  using namespace move_action;
  MoveVector all_moves;
  Store store(all_moves);
  
  GenerateEscape<P>::generate(state, target, store);
  for (MoveVector::const_iterator p=all_moves.begin(); p!=all_moves.end();
	 ++p)
  {
    assert(! p->isDrop());
    const Position to = p->to();
    // ƨˤνʬʤȤ
    const int defense = state.countEffect(P, to);
    const int offense = state.countEffect(alt(P), to);
    if (offense > defense)
      continue;
    if (to == attack_from)
    {
      // ϽʣξϿˤΤƤ
      if (offense == defense)
	continue;
      assert(p->capturePtype() == PAWN);
      if (! ShouldPromoteCut::canIgnoreMove<P>(*p))
	moves.push_back(*p);
      continue;
    }
    assert(p->from() == attack_to);
    if ((offense == defense)
	&& (p->capturePtype() == PTYPE_EMPTY))
      continue;

    // TODO: KnightCaptureDepth
    if ((! p->isPromote())
	&& (! ShouldPromoteCut::canIgnoreMove<P>(*p)))
    {
      moves.push_back(*p);
    }
  }
}

template <osl::Player P>
void osl::search::QuiescenceGenerator<P>::
dropMajorPiece(const NumEffectState& state, MoveVector& moves) 
{
  assert(moves.empty());
  using namespace move_action;
  Store store(moves);
  move_generator::SafeDropMajorPiece<P>::generate(state, store);
}

#endif /* QUIESCENCEGENERATOR_TCC */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
