/* quiescenceRecord.h
 */
#ifndef _QUIESCENCERECORD_H
#define _QUIESCENCERECORD_H

#include "osl/search/dualThreatmateState.h"
#include "osl/checkmate/oracleAges.h"
#include "osl/checkmate/king8Info.h"
#include "osl/effect_util/sendOffPosition.h"
#include "osl/ptype.h"
#include "osl/move.h"
#include "osl/container/moveVector.h"
#include "osl/stl/vector.h"
#include <iosfwd>
#ifdef OSL_SMP
#  include "osl/misc/lightMutex.h"
#endif

namespace osl
{
  namespace search
  {
    struct QSearchTraits
    {
      enum { 
	/** ̾õκ */
	MaxDepth = 8,
	/** ¨ͤο */
	CheckmateSpecialDepth = 127,
	/** »롼פο */
	HistorySpecialDepth = 126,
      };
      enum { FirstThreat = 6, SecondThreat = 2 };
      enum MoveType { 
	UNKNOWN, KING_ESCAPE, CAPTURE, PROMOTE, CHECK,
	ESCAPE, ATTACK, OTHER, 
      };
    };

    struct QuiescenceFlags
    {
      unsigned int data;
      enum Flags { 
	/** 
	 * ؼȽŤʤʤ褦ˡʳλؼϼPTYPE
	 * Ťʤʤ褦ꤹɬפ
	 */
	KING_ESCAPE = KING, PROMOTE = PTYPE_EMPTY, CHECK = PTYPE_EDGE,
	DROP = PTYPE_SIZE, ALL_ESCAPE, MAJOR_PIECE_ATTACK, KNIGHT_ATTACK,
	GOLDSILVER_ATTACK,
	ADVANCE_BISHOP, KING8_ATTACK, ESCAPE_FROM_LAST_MOVE,
	PINNED_ATTACK, UTILIZE_PROMOTED, BREAK_THREATMATE, KING_WALK,
	//
	LAST_FLAG,
      };      
      QuiescenceFlags() : data(0)
      {
      }
    private:
      bool isGenerated(int type) const
      {
	return data & (1u << type);
      }
      void recordGeneration(int type)
      {
	data |= (1u << type);
      }
    public:
      bool isGenerated(Ptype ptype) const
      {
	return isGenerated(static_cast<int>(ptype));
      }
      void recordGeneration(Ptype ptype)
      {
	recordGeneration(static_cast<int>(ptype));
      }
      bool isGenerated(Flags f) const
      {
	return isGenerated(static_cast<int>(f));
      }
      void recordGeneration(Flags f)
      {
	recordGeneration(static_cast<int>(f));
      }
      void clear() 
      {
	data = 0;
      }
    };

    /**
     * QuiescenceSearch ǥѥͭϤʻؼ
     */
    struct QuiescenceThreat
    {
      int value;
      Move move;
      explicit QuiescenceThreat(int v=0, Move m=Move::INVALID()) 
	: value(v), move(m)
      {
      }
    };    

    /**
     * QuiescenceRecord ΥǡΤlock, public ʥǡʳ
     */
    struct QuiescenceRecordBase
    {
      int upper_bound, lower_bound;
      vector<Move> move_cache;
      QuiescenceFlags flags;
      Move best_move;
      int static_value;
      /** static_value ؤζ */
      QuiescenceThreat threat1, threat2;
      int checkmate_nodes;
      int threatmate_nodes;
      mutable Position8 sendoffs;
      mutable King8Info king8info;
      mutable PieceMask pin_cache;
      /** upper_bound, lower_bound, static_value õ */
      char upper_depth, lower_depth, static_value_depth;
      char static_value_type;
      mutable bool sendoffs_initialized, king8info_initialized;
      bool visited;
    public:
      /**
       * Ȥʤ.
       * CAVEAT: -1 ȲĹ depth -1ˤʤäˤϤޤ
       */
      enum { InitialDepth = -128, };
      enum StaticValueType { UNKNOWN, UPPER_BOUND, EXACT };
    protected:
      QuiescenceRecordBase() 
	: best_move(Move::INVALID()), checkmate_nodes(0), threatmate_nodes(0),
	  king8info(0),
	  upper_depth(InitialDepth), lower_depth(InitialDepth),
	  static_value_depth(InitialDepth), 
	  static_value_type(UNKNOWN), sendoffs_initialized(false), 
	  king8info_initialized(false), visited(false)
      {
      }
      ~QuiescenceRecordBase() {}
    };
    
    /**
     * QuiescenceSearch ɽ¸ǡ
     */
    class QuiescenceRecord : public QuiescenceRecordBase
    {
    public:
      static const char *toString(StaticValueType);
    private:
#ifdef OSL_SMP
      typedef osl::misc::LightMutex Mutex;
      mutable Mutex mutex;
#endif
    public:
      DualThreatmateState threatmate;
      /**  oracle  */
      AttackOracleAges attack_oracle, threatmate_oracle;
      QuiescenceRecord()
      {
      }
      /**
       * copy constructor: copy everything except for mutex
       */
      QuiescenceRecord(const QuiescenceRecord& src)
	: QuiescenceRecordBase(src), threatmate(src.threatmate),
	  attack_oracle(src.attack_oracle),
	  threatmate_oracle(src.threatmate_oracle)
      {
      }
      QuiescenceRecord& operator=(const QuiescenceRecord& src)
      {
	if (this == &src)
	  return *this;
	
	QuiescenceRecordBase::operator=(src);
	threatmate = src.threatmate;
	attack_oracle = src.attack_oracle;
	threatmate_oracle = src.threatmate_oracle;
	return *this;
      }

      template <Player Turn>
      const Position8& sendOffPosition(const NumEffectState& state) const
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	assert(Turn == state.getTurn());
	if (! sendoffs_initialized)
	{
	  const Position king_position = state.getKingPosition(alt(Turn));
	  effect_util::SendOffPosition::find<Turn>(state, king_position, 
						   sendoffs);
	  sendoffs_initialized = true;
	}
	return sendoffs;
      }
      const Position8& 
      sendOffPosition(Player turn, const NumEffectState& state) const
      {
	if (turn == BLACK)
	  return sendOffPosition<BLACK>(state);
	else
	  return sendOffPosition<WHITE>(state);
      }
    private:
      void makeKing8InfoInLock(const NumEffectState& state) const
      {
	if (! king8info_initialized)
	{
	  king8info = King8Info::makeAndSetPin(state.getTurn(), state, pin_cache);
	  king8info_initialized = true;
	}
      }
    public:
      const King8Info& 
      king8Info(const NumEffectState& state) const
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	makeKing8InfoInLock(state);
	return king8info;
      }
      const PieceMask 
      pins(const NumEffectState& state) const
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	makeKing8InfoInLock(state);
	return pin_cache;
      }

      /**
       * @param max recordǻȤäɤΡɿ
       * @return ;˻ȤΡɿ֤
       */
      int checkmateNodesLeft(int max)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	if (max > checkmate_nodes)
	{
	  const int left = max - checkmate_nodes;
	  checkmate_nodes = max;
	  return left;
	}
	return 0;
      }
      /**
       * @param max recordǻȤäɤΡɿ
       * @return ͤǧε;˻ȤΡɿ֤
       */
      int threatmateNodesLeft(int max)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	if (max > threatmate_nodes)
	{
	  const int left = max - threatmate_nodes;
	  threatmate_nodes = max;
	  return left;
	}
	return 0;
      }
      /** ޤǤ˵;õΡɿ */
      int checkmateNodes() const { return checkmate_nodes; }
      int threatmateNodes() const { return threatmate_nodes; }

      void setVisited(bool value=true) { 
#ifndef OSL_SMP
	assert(! (visited && value));
#endif
	visited = value;
      }
      bool isVisited() const { return visited; }
      void clear()
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	move_cache.clear();
	flags.clear();
	upper_depth = lower_depth = static_value_depth = InitialDepth;
      }
      void setStaticValue(StaticValueType type, int value, int depth,
			  const QuiescenceThreat& t1=QuiescenceThreat(),
			  const QuiescenceThreat& t2=QuiescenceThreat())
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	assert((depth <= QSearchTraits::MaxDepth)
	       || (depth == QSearchTraits::CheckmateSpecialDepth));
	static_value_type = type;
	static_value = value;
	static_value_depth = depth;
	threat1 = t1;
	threat2 = t2;
      }
      void setLowerBound(int depth, int bound, Move best_move)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	assert((depth <= QSearchTraits::MaxDepth)
	       || (depth == QSearchTraits::CheckmateSpecialDepth));
	if (depth >= lower_depth)
	{
	  lower_depth = depth;
	  lower_bound = bound;
	  this->best_move = best_move;
	}
      }
      void setUpperBound(int depth, int bound)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	assert((depth <= QSearchTraits::MaxDepth)
	       || (depth == QSearchTraits::CheckmateSpecialDepth));
	if (depth >= upper_depth)
	{
	  upper_depth = depth;
	  upper_bound = bound;
	}
      }
      void setHistoryValue(int value)
      {
	lower_bound = upper_bound = value;
	lower_depth = upper_depth = QSearchTraits::HistorySpecialDepth;
      }
      void setHistoryValue(Move best_move, int value)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	this->best_move = best_move;
	setHistoryValue(value);
      }
    private:
      void reserveMovesInLock(size_t new_size)
      {
	move_cache.reserve(move_cache.size()+new_size);
      }
    public:
      void reserveMoves(size_t new_size)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	reserveMovesInLock(new_size);
      }
    private:
      void addMovesReservedInLock(const MoveVector& new_moves)
      {
	move_cache.insert(move_cache.end(), 
			  new_moves.begin(), new_moves.end());
      }
      void addMovesReserved(const MoveVector& new_moves)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	addMovesReservedInLock(new_moves);
      }
      void addMovesInLock(const MoveVector& new_moves)
      {
	reserveMovesInLock(new_moves.size());
	addMovesReservedInLock(new_moves);
      }
      void addMoves(const MoveVector& new_moves)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	addMovesInLock(new_moves);
      }
    public:
      void addKillerMoves(const MoveVector& new_moves)
      {
	addMoves(new_moves);
      }
      void addKillerMovesReserved(const MoveVector& new_moves)
      {
	assert(move_cache.capacity() >= move_cache.size() + new_moves.size());
	addMovesReserved(new_moves);
      }
      template <class MOVE_TYPE>
      void addMoves(MOVE_TYPE type, const MoveVector& new_moves)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
	if(flags.isGenerated(type)) return;
#else
	assert(! flags.isGenerated(type));
#endif
	flags.recordGeneration(type);
	addMovesInLock(new_moves);
      }

      StaticValueType staticValueType() const { 
	return static_cast<StaticValueType>(static_value_type); 
      }
      bool hasStaticValue() const { return staticValueType() != UNKNOWN; }
      int staticValue() const { assert(hasStaticValue()); return static_value; }
      int staticValueDepth() const { return static_value_depth; }
      int upperDepth() const { return upper_depth; }
      int lowerDepth() const { return lower_depth; }
      int upperBound() const { return upper_bound; }
      int lowerBound() const { return lower_bound; }
      const Move bestMove() const { return best_move; }
      int moves_size() const { 
	return move_cache.size(); 
      }
      void loadMoves(MoveVector& dst) const{
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	dst.clear();
	dst.push_back(move_cache.begin(),move_cache.end());
      }
      QuiescenceFlags cacheFlags() const { return flags; }

      void dump(std::ostream&) const;
    };
    
  } // namespace search
} // namespace osl

#endif /* _QUIESCENCERECORD_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
