/* quiescenceSearch.h
 */
#ifndef _QUIESCENCESEARCH_H
#define _QUIESCENCESEARCH_H

#include "osl/search/fixedEval.h"
#include "osl/search/quiescenceRecord.h"
#include "osl/search/searchState.h"
#include "osl/eval/pieceEval.h"
#include "osl/state/numEffectState.h"
#include "osl/pathEncoding.h"
namespace osl
{
  namespace container
  {    
    class MoveVector;
  }
  namespace hash
  {
    class HashKey;
  }
  namespace search
  {
    class SimpleHashTable;

    /**
     * 礤õ (Żõ).
     *
     * - ΤϼΤ
     * - KFENDή http://www31.ocn.ne.jp/~kfend/inside_kfend/quiescence.html
     * - ®Τüμ֤ϡľɤ
     * - ̵̤ɤθ
     * - ƨ뤳ȤǤʤҡפϤޤƤʤ
     * - üϡ֤ȳѤǡ֤ʤ
     *
     * TODO:
     * - ϡƨꡤͭ?
     *
     */
    template <class EvalT>
    class QuiescenceSearch : protected FixedEval, private QSearchTraits
    {
      typedef FixedEval base_t;
      SearchStateCore& state;
      SimpleHashTable& table;
      int root_depth;
      int max_depth;
      /**
       * õΡɿ
       */
      int node_count;
      /** rootο */
      int depthFromRoot() const 
      {
	return state.path().getDepth() - root_depth;
      }
      /** Ĥ꿼 */
      int depth() const 
      {
	return max_depth - depthFromRoot();
      }
    public:
      typedef EvalT eval_t;
      typedef NumEffectState effect_state_t;
      using base_t::isWinValue;

      typedef container::MoveVector MoveVector;
    
      QuiescenceSearch(SearchStateCore& s, SimpleHashTable& t)
	: state(s), table(t), root_depth(s.path().getDepth()),
	  max_depth(QSearchTraits::MaxDepth), node_count(0)
      {
      }
      template <Player P>
      int search(eval_t const& ev, Move last_move,
		 int depth=QSearchTraits::MaxDepth)
      {
	assert(last_move.player() == alt(P));
	assert(state.state().getTurn() == P);
	max_depth = depth;
	return searchInternal<P>(base_t::winThreshold(alt(P)), 
				 base_t::winThreshold(P), 
				 ev, last_move);
      }
      int search(Player P, eval_t const& ev, Move last_move,
		 int depth=QSearchTraits::MaxDepth)
      {
	if (P == BLACK)
	  return search<BLACK>(ev, last_move, depth);
	else
	  return search<WHITE>(ev, last_move, depth);
      }
      template <Player P>
      int searchIteratively(eval_t const& ev, Move last_move,
			    int depth=QSearchTraits::MaxDepth)
      {
	assert(last_move.player() == alt(P));
	assert(state.state().getTurn() == P);
	return searchIteratively<P>(base_t::winThreshold(alt(P)), 
				    base_t::winThreshold(P), 
				    ev, last_move, depth);
      }
      int searchIteratively(Player P, eval_t const& ev, Move last_move,
			    int depth=QSearchTraits::MaxDepth)
      {
	if (P == BLACK)
	  return searchIteratively<BLACK>(ev, last_move, depth);
	else
	  return searchIteratively<WHITE>(ev, last_move, depth);
      }

      template <Player P>
      int searchIteratively(int alpha, int beta, eval_t const& ev, Move last_move,
			    int depth)
      {
	assert(depth >= 2);
	int result=0;
	for (int i=2; i<=depth; i+=2)
	{
	  max_depth = i;
	  result=searchInternal<P>(alpha, beta, ev, last_move);
	}
	return result;
      }
      template <Player P>
      int search(int alpha, int beta, eval_t const& ev, Move last_move,
		 int depth=QSearchTraits::MaxDepth)
      {
	max_depth = depth;
	return searchInternal<P>(alpha, beta, ev, last_move);
      }
      int search(Player P, int alpha, int beta, eval_t const& ev, Move last_move, int depth){
	if (P == BLACK)
	  return search<BLACK>(alpha, beta, ev, last_move, depth);
	else
	  return search<WHITE>(alpha, beta, ev, last_move, depth);
      }
      template <Player P>
      int searchProbCut(int alpha, int beta, eval_t const& ev, Move last_move);
      int searchProbCut(Player P, int alpha, int beta, eval_t const& ev, Move last_move)
      {
	if (P == BLACK)
	  return searchProbCut<BLACK>(alpha, beta, ev, last_move);
	else
	  return searchProbCut<WHITE>(alpha, beta, ev, last_move);
      }

      template <Player P>
      int searchInternal(int alpha, int beta, eval_t const& ev, Move last_move,
			 int additional_depth=0);
    private:
      template <Player P, bool has_record> inline
      int searchMain(QuiescenceRecord *record, 
		     int alpha, int beta, eval_t const& ev, Move last_move,
		     int additional_depth);
      template <bool has_record, class TYPE> inline
      void recordGeneration(QuiescenceRecord *record, TYPE type, 
			    const MoveVector& moves);
      /**
       * PTYPE  examineMoves Ƥ
       * @return cut Ǥ뤫
       */
      template <Player P, Ptype PTYPE, bool has_record>
      bool examineCapture(QuiescenceRecord *record,
			  int& curVal, MoveVector& working, int& alpha, 
			  int beta, eval_t const& ev, Piece last_piece,
			  int additional_depth);
      template <Player P, bool has_record>
      int staticValue(eval_t const& ev, int alpha, QuiescenceRecord *record);
      template <Player P>
      int passValue(int alpha, int beta, eval_t const& ev);
      int currentValueWithLastThreat(eval_t const& ev, Piece last_move_piece);
    public:
      /**
       * @return cut Ǥ뤫
       * @param dont_capture TakeBack ǻ夫ʣƻʤ˻Ȥ.
       *   template parameter  has_dont_capture true λ
       */
      template <Player P, bool has_record, bool has_dont_capture, MoveType move_type>
      bool examineMoves(QuiescenceRecord *record, int& curVal, 
			const Move *first, const Move *last, 
			int& alpha, int beta, eval_t const& ev,
			int additional_depth,
			Position dont_capture=Position::STAND());
      /**
       * ñʼ֤õ.
       * PieceEval::computeDiffAfterMoveForRP Ȱۤʤꡤ̤ư
       * ΤȽꤷ⿭Ӥ
       * @param attack_piece last_move λ˲򤫤Ƥ
       */
      template <Player P>
      int takeBackValue(int alpha, int beta, eval_t const& ev, Move last_move,
			Piece last_attack_piece=Piece::EMPTY());
      template <Player P>
      bool examineTakeBack(const MoveVector& moves,
			   int& cur_val, int& alpha, int beta, eval_t const& ev,
			   Piece king_attack_piece=Piece::EMPTY());

      /**
       * üμ֤. threat
       * @param threat1 ζ
       * @param threat2 2ܤζ
       * @param calm_move_only ޥˤϤʤ
       * @param king_attack_piece P ˲򤫤Ƥ(Τΰ)
       */
      template <Player P, bool calm_move_only, bool first_nolmal_move_only>
      bool examineTakeBack2(const MoveVector& moves,
			    QuiescenceThreat& threat2, 
			    QuiescenceThreat& threat1,
			    int beta, int beta2, eval_t const& ev);
      /** üμ֤.
       * Ƽ˻ؼ examineTakeBack2 Ƥ
       */
      template <Player P, Ptype PTYPE>
      bool generateAndExamineTakeBack2(MoveVector& moves, 
				       QuiescenceThreat& threat2, 
				       QuiescenceThreat& threat1, 
				       int beta1, int beta2, eval_t const& ev);

      /**
       * last_move ƨǡƨμ֤ޤɷβͤȽ
       */
      template <Player P>
      int takeBackOrChase(int alpha, int beta, eval_t const& ev, Move last_move);

      template <Player P>
      int staticValueWithThreat(eval_t const& ev, int alpha, 
				QuiescenceThreat& threat1, 
				QuiescenceThreat& threat2);
      template <Player P>
      int staticValueWithThreat(eval_t const& ev)
      {
	QuiescenceThreat t1, t2;
	return staticValueWithThreat<P>(ev, base_t::winThreshold(alt(P)), 
					t1, t2);
      }
      int staticValueWithThreat(eval_t const& ev)
      {
	if (state.state().getTurn() == BLACK)
	  return staticValueWithThreat<BLACK>(ev);
	else
	  return staticValueWithThreat<WHITE>(ev);
      }
      int nodeCount() const { return node_count; }
      const NumEffectState& currentState() const { return state.state(); }
    };

  } // namespace search
  using search::QuiescenceSearch;
} // namespace osl


#endif /* _QUIESCENCESEARCH_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
