/* searchMoveSet.h
 */
#ifndef _SEARCHMOVESET_H
#define _SEARCHMOVESET_H

#include "osl/container/linearMoveMap.h"
#include "osl/search/searchMove.h"
#ifdef OSL_SMP
#  include "osl/misc/lightMutex.h"
#endif
#include <iosfwd>

namespace osl
{
  namespace search
  {
    class SearchMoveSet : private LinearMoveMap<search::SearchMove>
    {
#ifdef OSL_SMP
      typedef osl::misc::LightMutex Mutex;
      mutable Mutex mutex;
#endif
    public:
      typedef LinearMoveMap<search::SearchMove> base_t;
      typedef search::SearchMove SearchMove;

      SearchMoveSet() {}
      SearchMoveSet(const SearchMoveSet& src);
      ~SearchMoveSet();
      SearchMoveSet& operator=(const SearchMoveSet& src);

      SearchMove* find(Move key) 
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	return base_t::find(key);
      }
      const SearchMove* find(Move key) const
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	return base_t::find(key);
      }
      /** 
       * ̤Ͽäꡤ¸Ψ⤫äϿ
       * @return Ͽɤ. 
       * ֤pointerϤdangling ˤʤ뤳Ȥ(ƥ饹vectorȤ)
       */
      SearchMove *assignIfBetter(const MoveLogProb& move, int margin=0)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	SearchMove *recorded_move = base_t::find(move.getMove());
	if (recorded_move)
	{
	  // ˤ⤤Ψμ꤬äƤϿʤ
	  // 㤤ΨμϾ񤭤
	  if (recorded_move->moveLogProb().getLogProb() 
	      <= (move.getLogProb() + margin))
	    return 0;
	  *recorded_move = SearchMove(move, recorded_move->record);
	  return recorded_move;
	}
	return insertNoCheck(SearchMove(move));
      }
      bool isValidAll() const;
      void dump(std::ostream&) const;
      bool hasOnlyPass() const
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	const_iterator p = begin();
	if (p == end())
	  return false;
	return (p->getMove().isPass() && (++p == end()));
      }

      const SearchMove& front() const
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	assert(! empty());
	return *begin();
      }
      
      struct const_range
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lock;
#endif
	const const_iterator first, last;
	explicit const_range(const SearchMoveSet& parent)
	  : 
#ifdef OSL_SMP
	  lock(parent.mutex),
#endif
	  first(parent.begin()), last(parent.end())
	{
	}
      };
      struct range
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lock;
#endif
	const iterator first, last;
	explicit range(SearchMoveSet& parent)
	  : 
#ifdef OSL_SMP
	  lock(parent.mutex),
#endif
	  first(parent.begin()), last(parent.end())
	{
	}
      };
      friend class const_range;
      friend class range;
      using base_t::const_iterator;
      using base_t::iterator;
      using base_t::size;
      using base_t::empty;
    };

  } // namespace search
  using search::SearchMoveSet;
} // namespace osl


#endif /* _SEARCHMOVESET_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
