/* pool_allocator.h
 */
#ifndef POOL_ALLOCATOR_H
#define POOL_ALLOCATOR_H

#ifdef USE_GPL_POOL_ALLOCATOR
// gpsshogi Ȱ
#  include "osl/stl/gnu_pool_alloc.h"
#else
#  include <memory>
#endif

namespace osl
{
  /**
   * stl ƥʤallocatorؤΤ󶡤.
   * gcc 3.4 ɸallocatornew/deleteƤ֤褦ˤʤäᡤ
   * GPLʤϰgcc°pool_allocatorȤȤǸΨݤ
   */
  namespace stl
  {
    extern const int gpl_pool_allocator_used;
    struct ConsistencyCheck
    {
      ConsistencyCheck(int);
    };
#ifndef USE_GPL_POOL_ALLOCATOR
    static ConsistencyCheck consistency_check(0);
    template <class T>
    struct pool_allocator : std::allocator<T>
    {
      pool_allocator() {}
      template <class T2>
      pool_allocator(const std::allocator<T2>&) {}
    };
#else
    static ConsistencyCheck consistency_check(1);
#  if __GNUC__ >= 4 || (__GNUC__ == 3 && __GNUC_MINOR__ > 3)
    template <class T>
    struct pool_allocator : __gnu_cxx::__pool_alloc<T>
    {
      pool_allocator() {}
      template <class T2>
      pool_allocator(const __gnu_cxx::__pool_alloc<T2>&) {}
    };
#  else
    template <class T, bool STL_MT=true>
    struct pool_allocator 
    {
      typedef typename ::osl::__gnu_cxx::__pool_alloc<STL_MT,0> allocator_t;
    public:
      typedef T* pointer;
      typedef const T* const_pointer;
      typedef T& reference;
      typedef const T& const_reference;
      typedef size_t size_type;
      typedef size_t difference_type;

      pool_allocator() {}
      template <class T2>
      pool_allocator(const pool_allocator<T2>&) {}
    
      static T* allocate(size_t n)
      {
	T* ret = 0;
	if (n)
	  ret = static_cast<T*>(allocator_t::allocate(n * sizeof(T)));
	return ret;
      }
  
      static T* allocate() 
      {
	return static_cast<T*>(allocator_t::allocate(sizeof(T))); 
      }
  
      static void deallocate(T* p, size_t n)
      { 
	if (n) 
	  allocator_t::deallocate(p, n * sizeof(T)); 
      }
      static void deallocate(T* p) { allocator_t::deallocate(p, sizeof(T)); }

      template<typename T2>
      struct rebind { typedef pool_allocator<T2> other; };
    };
    template <class T>
    inline bool
    operator==(const pool_allocator<T>&,const pool_allocator<T>&){
      return true;
    }
#  endif
#endif
  } // namespace stl
} // namespace stl

#endif /* POOL_ALLOCATOR_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
