/* addEffect.cc
 */
#include "osl/category/addEffect.h"
#include "osl/category/probTable.tcc"
#include "osl/category/moveEvaluation.h"
#include "osl/category/probTableDefine.h"
#include "osl/move_generator/addEffect.h"
#include "osl/move_action/store.h"
#include "osl/move_classifier/moveAdaptor.h"

namespace osl
{
  namespace category
  {
    DEF_CATEGORY_TABLE_IP(PtypePtype2ProbTable, AddEffect);
    DEF_CATEGORY_TABLE_IP(PtypePtypeProbTable, AddEffectToLastMove);
    DEF_CATEGORY_TABLE_IP(PieceValueTable, OpenBishop);
  }
}

const int osl::category::AddEffectToLastMove::pieceValueLimit
= -eval::PtypeEvalTraits<PAWN>::val*2;

void osl::category::AddEffectToLastMove::
generate(const CategoryEnv& env, MoveLogProbVector& out, int *indices)
{
  const Move last_move = env.history->lastMove();
  if (! last_move.isNormal())
    return;
  const Ptype target = last_move.ptype();
  MoveVector moves;
  move_action::Store store(moves);
  move_generator::GenerateAddEffect<false>::generate
    (env.state->getTurn(), *env.state, last_move.to(), store);

  for (size_t i=0; i<moves.size(); ++i)
  {
    const int pieceValue = MoveEvaluation::evalLight(*env.state, moves[i]);
    if (pieceValue < pieceValueLimit)
      continue;
    const bool supported 
      = env.state->hasEffectBy(alt(last_move.player()),last_move.to());
    const size_t index = 
      probTable.getIndex(target,supported, moves[i].ptype(),pieceValue);
    const int prob = probTable[index];
    if (prob <= env.limit)
    {
      out.push_back(moves[i], prob);
      if (indices)		// FIXME: ٤ɤޤ
	*indices++ = index;
    }
  }
}

void osl::category::AddEffectToLastMove::
generate(const CategoryEnv& env, MoveLogProbVector& out)
{
  generate(env, out, 0);
}

const char * osl::category::AddEffectToLastMove::
getName() 
{
  return "AddEffect!";
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
