/* fullWidth.cc
 */

#include "osl/category/fullWidth.h"
#ifdef NEED_EXTRA_TEMPLATE_INSTANTIATION
#  include "osl/category/categoryListUtil.h"
#  include "osl/category/categoryListUtil.tcc"
#endif
#include "osl/search/realizationProbability.h"
#include "osl/move_generator/escape.h"
#include "osl/move_generator/allMoves.h"
#include "osl/move_action/store.h"
#include "osl/container/moveVector.h"

namespace osl
{
  namespace category
  {
    const FixedValueTable 
    FullWidth::probTable(search::FullWidthMoveProbability::DefaultProb);
#ifdef NEED_EXTRA_TEMPLATE_INSTANTIATION
    template void CategoryListUtil::generate<FullWidth>(CategoryEnv const&, MoveLogProbVector&);
#endif
  }
}

void osl::category::FullWidth::
generate(const CategoryEnv& env, MoveVector& out)
{
  move_action::Store store(out);
  const Player turn = env.state->getTurn();
  const Piece kingPiece=env.state->getKingPiece(turn);
  const Position kingPosition=kingPiece.position();
  if (env.state->hasEffectBy(alt(turn), kingPosition))
  {
    assert(kingPiece == (*env.state).getPieceAt(kingPosition));
    if (turn == BLACK)
      move_generator::Escape<BLACK,CategoryEnv::effect_state_t,move_action::Store>::
	generateMoves(*env.state,kingPiece,store);
    else
      move_generator::Escape<WHITE,CategoryEnv::effect_state_t,move_action::Store>::
	generateMoves(*env.state,kingPiece,store);
  }
  else
  {
    move_generator::AllMoves<NumEffectState,move_action::Store>::
      generateMoves(env.state->getTurn(),*env.state, store);
  }
}


/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
