/* eachPlayerAttackDefense.cc
 */
#include "osl/eval/eachPlayerAttackDefense.h"
#include "osl/container/pieceValues.h"
#include <iostream>
void osl::eval::
EachPlayerAttackDefense::setValues(const SimpleState& state, PieceValues& values)
{
  values.fill(0);
  additional_eval_t::setValues(state, values);
  progress_t progress((NumEffectState(state)));
  // ®٤̵
  const CArray<Piece,2> kings
    = {{ state.getKingPiece(BLACK), state.getKingPiece(WHITE) }};
  const CArray<Progress16,2> dangers
    = {{ progress.progress16(BLACK), progress.progress16(WHITE) }};
  
  for (int i=0; i<Piece::SIZE; i++) {
    const Piece target = state.getPieceOf(i);
    values[i] += valueOf(kings, dangers, target);
  }
}

osl::eval::
EachPlayerAttackDefense::EachPlayerAttackDefense(const NumEffectState& state)
  : main_value(0), additional_eval(state), current_progress(state)
{
  const CArray<Piece,2> kings
    = {{ state.getKingPiece(BLACK), state.getKingPiece(WHITE) }};
  const CArray<Progress16,2> dangers
    = {{ progress16(BLACK), progress16(WHITE) }};

  for (int i=0; i<Piece::SIZE; i++) {
    const Piece target = state.getPieceOf(i);
    const int value = valueOf(kings, dangers, target);
    main_value += value;
  }
}

void osl::eval::
EachPlayerAttackDefense::updateAll(const SimpleState& state, 
				   Position to, Ptype ptype)
{
  main_value = 0;

  CArray<Piece,2> kings
    = {{ state.getKingPiece(BLACK), state.getKingPiece(WHITE) }};
  const CArray<Progress16,2> dangers
    = {{ progress16(BLACK), progress16(WHITE) }};

  assert(ptype != KING);
  for (int i=0; i<Piece::SIZE; i++) {
    const Piece target = state.getPieceOf(i);
    addValue(kings, dangers, target);
  }
  const Player player = state.getTurn();
  const Piece my_king = kings[playerToIndex(player)];
  const Piece op_king = kings[playerToIndex(alt(player))];
  const Progress16 my_danger = dangers[playerToIndex(player)];
  const Progress16 op_danger = dangers[playerToIndex(alt(player))];
  
  main_value -= valueOf(my_king, op_king, my_danger, op_danger,
			newPtypeO(player, ptype), Position::STAND());
  main_value += valueOf(my_king, op_king, my_danger, op_danger,
			newPtypeO(player, ptype), to);
}

void osl::eval::
EachPlayerAttackDefense::updateAll(const SimpleState& state, 
				   Position from, Position to,
				   int promote_mask)
{
  main_value = 0;

  CArray<Piece,2> kings
    = {{ state.getKingPiece(BLACK), state.getKingPiece(WHITE) }};
  const CArray<Progress16,2> dangers
    = {{ progress16(BLACK), progress16(WHITE) }};
  
  const Piece old_piece = state.getPieceOnBoard(from);
  Piece new_piece = old_piece.promoteWithMask(promote_mask);
  new_piece += (to-from);
  const Player player = old_piece.owner();

  if (old_piece.ptype() == KING)
    kings[playerToIndex(player)] = new_piece;

  for (int i=0; i<Piece::SIZE; i++) {
    const Piece target = state.getPieceOf(i);
    addValue(kings, dangers, target);
  }
  main_value -= valueOf(kings, dangers, old_piece); // ­
  main_value += valueOf(kings, dangers, new_piece);
}

void osl::eval::
EachPlayerAttackDefense::updateAll(const SimpleState& state, 
				   Position from, Position to, Piece captured,
				   int promote_mask)
{
  main_value = 0;

  CArray<Piece,2> kings
    = {{ state.getKingPiece(BLACK), state.getKingPiece(WHITE) }};
  const CArray<Progress16,2> dangers
    = {{ progress16(BLACK), progress16(WHITE) }};

  const Piece old_piece = state.getPieceOnBoard(from);
  const Player player = old_piece.owner();
  Piece new_piece = old_piece.promoteWithMask(promote_mask);
  new_piece += (to - from);
  
  if (old_piece.ptype() == KING)
    kings[playerToIndex(player)] = new_piece;

  for (int i=0; i<Piece::SIZE; i++) {
    const Piece target = state.getPieceOf(i);
    if (target == old_piece)
      addValue(kings, dangers, new_piece);
    else if (target == captured)
      addValue(kings, dangers, captured.captured());
    else 
      addValue(kings, dangers, target);
  }
}

void osl::eval::
EachPlayerAttackDefense::dump() const
{
  std::cerr << "EachPlayerAttackDefense(" << main_value
	    << ", " << additional_eval.rawValue() 
	    << ", " << current_progress.progress16(BLACK).value() 
	    << ", " << current_progress.progress16(WHITE).value() 
	    << ")";
}
/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
