/* attackDefense.cc
 */
#include "osl/eval/endgame/attackDefense.h"
#include "osl/container/pieceValues.h"

void osl::eval::endgame::
AttackDefense::setValues(const SimpleState& state, PieceValues& values)
{
  values.fill(0);
  // ®٤̵
  const Piece king_black = state.getKingPiece(BLACK);
  const Piece king_white = state.getKingPiece(WHITE);
  
  for (int i=0; i<Piece::SIZE; i++) {
    const Piece target = state.getPieceOf(i);
    values[i] = valueOf(king_black, king_white, target);
  }
}

osl::eval::endgame::
AttackDefense::AttackDefense(const SimpleState& state)
{
  values.fill(0);
  const Piece king_black = state.getKingPiece(BLACK);
  const Piece king_white = state.getKingPiece(WHITE);
  for (int i=0; i<Piece::SIZE; i++) {
    const Piece target = state.getPieceOf(i);
    addValue(king_black, king_white, target);
  }
}

void osl::eval::endgame::
AttackDefense::updateKingMove(const SimpleState& state, 
			      Position from, Position to)
{
  reset();
  
  const Piece old_king = state.getPieceOnBoard(from);
  const Player player = old_king.owner();
  assert(old_king.ptype() == KING);
  const Piece new_king = Piece::makeKing(player, to);
  
  const Piece king_black
    = (player == BLACK) ? new_king : state.getKingPiece(BLACK);
  const Piece king_white
    = (player == WHITE) ? new_king : state.getKingPiece(WHITE);

  for (int i=0; i<Piece::SIZE; i++) {
    const Piece target = state.getPieceOf(i);
    if (target == old_king)
      addValue(king_black, king_white, new_king);
    else
      addValue(king_black, king_white, target);
  }
}

void osl::eval::endgame::
AttackDefense::updateKingMove(const SimpleState& state, 
			      Position from, Position to, Piece captured)
{
  reset();

  const Piece old_king = state.getPieceOnBoard(from);
  const Player player = old_king.owner();
  assert(old_king.ptype() == KING);
  const Piece new_king = Piece::makeKing(player, to);
  
  const Piece king_black
    = (player == BLACK) ? new_king : state.getKingPiece(BLACK);
  const Piece king_white
    = (player == WHITE) ? new_king : state.getKingPiece(WHITE);

  for (int i=0; i<Piece::SIZE; i++) {
    const Piece target = state.getPieceOf(i);
    if (target == old_king)
      addValue(king_black, king_white, new_king);
    else if (target == captured)
      addValue(king_black, king_white, captured.captured());
    else 
      addValue(king_black, king_white, target);
  }
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
