/**
 * attackKing.cc
 */
#include "osl/eval/endgame/attackKing.h"
#include "osl/eval/pieceEval.h"
#include "osl/misc/carray.h"

// YSSɾͤ򿿻
// http://www32.ocn.ne.jp/~yss/book.html#SEC3
//
// ζ̤Фζа֤ˤ硣ؼˤĤƤоΤǤ롣
//        0   1   2   3   4   5   6   7   8 (صΥ)
// 
//   +8   50  50  50  50  50  50  50  50  50
//   +7   50  50  50  50  50  50  50  50  50
//   +6   62  60  58  52  50  50  50  50  50
//   +5   80  78  72  67  55  51  50  50  50
//   +4  100  99  95  87  78  69  50  50  50
//   +3  140 130 110 100  95  75  54  50  50
//   +2  170 160 142 114  98  80  62  55  50  <--- ʿȤ롣
//   +1  170 165 150 121  94  78  58  52  50
//    0  170 145 137 115  91  75  57  50  50  <--- 濴(0 0)˲Ȥ
//   -1  132 132 129 102  84  71  51  50  50
//   -2  100  97  95  85  70  62  50  50  50
//   -3   90  85  80  68  60  53  50  50  50
//   -4   70  66  62  55  52  50  50  50  50
//   -5   54  53  51  50  50  50  50  50  50
//   -6   50  50  50  50  50  50  50  50  50
//   -7   50  50  50  50  50  50  50  50  50
//   -8   50  50  50  50  50  50  50  50  50

namespace osl
{
  namespace
  {
    const CArray<int,19*9> yss_bonus = {{
      // ˡϰʲ뤿ᡤ+9  -9 ޤǼ
         50,  50,  50,  50,  50,  50,  0,  0,  0, 
         50,  50,  50,  50,  50,  50,  0,  0,  0, 
         50,  50,  50,  50,  50,  50,  0,  0,  0, 
         62,  60,  58,  52,  50,  50,  0,  0,  0, 
         80,  78,  72,  67,  55,  51,  0,  0,  0, 
        100,  99,  95,  87,  78,  69,  0,  0,  0, 
        140, 130, 110, 100,  95,  75,  0,  0,  0, 
        170, 160, 142, 114,  98,  80,  0,  0,  0,
        170, 165, 150, 121,  94,  78,  0,  0,  0, 
        170, 145, 137, 115,  91,  75,  0,  0,  0,
        132, 132, 129, 102,  84,  71,  0,  0,  0, 
        100,  97,  95,  85,  70,  62,  0,  0,  0, 
         90,  85,  80,  68,  60,  53,  0,  0,  0, 
         70,  66,  62,  55,  52,  50,  0,  0,  0, 
         54,  53,  51,  50,  50,  50,  0,  0,  0, 
         50,  50,  50,  50,  50,  50,  0,  0,  0, 
         50,  50,  50,  50,  50,  50,  0,  0,  0, 
         50,  50,  50,  50,  50,  50,  0,  0,  0,
         50,  50,  50,  50,  50,  50,  0,  0,  0, 
    }};
    inline int multiply(int value, int bonus)
    {
      // yss Ǥϻղòͤ2٤뤿1.2ǳ
      return (value * bonus / 120) & (~1); 
    }
    inline void adhoc_adjust(int& value, double scale)
    {
      value = static_cast<int>(value * scale);
      value &= (~1);
    }
  } // anonymous namespace
} // namespace osl

void osl::eval::endgame::AttackKing::
Table::adhoc_edge_king_1(const Player player,
			 const Position king,
			 const Position attack)
{
  for (int ptype = PPAWN; ptype <= ROOK; ptype++)
  {
    if (ptype != KING)
    {
      adhoc_adjust(valueOf(king, player, attack,
			   static_cast<osl::Ptype>(ptype)), 1.25);
    }
  }
}

void osl::eval::endgame::AttackKing::
Table::adhoc_edge_king_2(const Player player,
			 const Position king,
			 const Position attack)
{
  adhoc_adjust(valueOf(king, player, attack,
		       GOLD), 1.25);
  adhoc_adjust(valueOf(king, player, attack,
		       SILVER), 1.25);
  for (int ptype = PPAWN; ptype <= PSILVER; ptype++)
  {
    adhoc_adjust(valueOf(king, player, attack,
			 static_cast<osl::Ptype>(ptype)), 1.25);
  }
}

osl::eval::endgame::AttackKing::
Table::Table()
{
  data.fill(0);
  for (int king_x=1; king_x<=9; ++king_x)
  {
    for (int king_y=1; king_y<=9; ++king_y)
    {
      const Position king(king_x, king_y);
      for (int p=PTYPE_PIECE_MIN; p<=PTYPE_MAX; ++p)
      {
	const Ptype ptype = static_cast<Ptype>(p);
	assert(isPiece(ptype));
	const Ptype basic_type = unpromote(ptype);

	const int ptype_value
	  = Ptype_Eval_Table.value(newPtypeO(BLACK,ptype));
	// 
	const int stand_bonus 
	  = (isMajorBasic(basic_type) 
	     ? Ptype_Eval_Table.value(newPtypeO(BLACK,PAWN))
	     : 0);
	valueOf(king, BLACK, Position::STAND(), ptype) = -ptype_value 
	  - stand_bonus;
	valueOf(king, WHITE, Position::STAND(), ptype) = +ptype_value
	  + stand_bonus;
	if (ptype == KNIGHT || ptype == LANCE)
	{
	  adhoc_adjust(valueOf(king, BLACK, Position::STAND(), ptype), 0.85);
	  adhoc_adjust(valueOf(king, WHITE, Position::STAND(), ptype), 0.85);
	}
	// ׾
	for (int attack_x=1; attack_x<=9; ++attack_x)
	{
	  const int relative_x = abs(king_x - attack_x);
	  for (int attack_y=1; attack_y<=9; ++attack_y)
	  {
	    const Position attacker(attack_x, attack_y);
	    if (basic_type == KING)
	    {
	      // ̤ϤΤޤ
	      valueOf(king, BLACK, attacker, ptype) = -ptype_value;
	      valueOf(king, WHITE, attacker, ptype) = ptype_value;
	      continue;
	    }

	    const int relative_y_black_king 
	      = (((ptype == KNIGHT)||(ptype == LANCE))
		 ? king_y - attack_y - 1 // ˹ϰʱ󤯤ɾ
		 : king_y - attack_y);
	    const int relative_y_white_king 
	      = (((ptype == KNIGHT)||(ptype == LANCE))
		 ? attack_y - king_y - 1 // ˹ϰʱ󤯤ɾ
		 : attack_y - king_y);

	    const int bonus_black_king
	      = yss_bonus[relative_x + (-relative_y_black_king+9)*9];
	    const int bonus_white_king
	      = yss_bonus[relative_x + (-relative_y_white_king+9)*9];

	    const int black_attack =  multiply(ptype_value,bonus_white_king);
	    const int white_attack = -multiply(ptype_value,bonus_black_king);

	    if (isMajorBasic(basic_type))
	    {
	      // ϹƤ max 
	      const int relative_y = abs(king_y - attack_y);
	      const int diff1 = king_x + king_y - (attack_x + attack_y);
	      const int diff2 = king_x - king_y - (attack_x - attack_y);

	      if ((basic_type == ROOK &&
		   relative_y <= 1) ||
		  (basic_type == BISHOP &&
		   ((-1 <= diff1 && diff1 <= 1) ||
		    (-1 <= diff2 && diff2 <= 1))))
	      {
		valueOf(king, BLACK, attacker, ptype) 
		  = EvalTraits<WHITE>::max(-ptype_value, white_attack);
		valueOf(king, WHITE, attacker, ptype) 
		  = EvalTraits<BLACK>::max(ptype_value, black_attack);
		// ̤ƱڤΥܡʥ⤯
		if (relative_y == 0)
		{
		  adhoc_adjust(valueOf(king, BLACK, attacker, ptype),
			       0.98);
		  adhoc_adjust(valueOf(king, WHITE, attacker, ptype),
			       0.98);
		}
	      }
	      else
	      {
		// ¾ 90 
		valueOf(king, BLACK, attacker, ptype) 
		  = -multiply(ptype_value, 90);
		valueOf(king, WHITE, attacker, ptype) 
		  = multiply(ptype_value, 90);
	      }
	      continue;
	    }
	    valueOf(king, BLACK, attacker, ptype) = white_attack;
	    valueOf(king, WHITE, attacker, ptype) = black_attack;
	  }
	}
      }
    }
  }
  for (int king_x=1; king_x<=9; ++king_x)
  {
    for (int king_y=1; king_y<=9; ++king_y)
    {
      const Position king(king_x, king_y);
      // 1ĤưʤƱ
      for (int x=1; x<=9; ++x)
      {
	const Position b(x,2);
	valueOf(king, WHITE, b, LANCE) = valueOf(king, WHITE, b, PAWN);
	const Position w(x,8);
	valueOf(king, BLACK, w, LANCE) = valueOf(king, BLACK, w, PAWN);
      }
      // 1ܤζϲͤȾʬ
      for (int x=1; x<=9; ++x)
      {
	const Position b(x,1);
	if (x <= 2 || x >= 8)
	{
	  adhoc_adjust(valueOf(king, WHITE, b, GOLD),    0.25);
	  adhoc_adjust(valueOf(king, WHITE, b, PSILVER), 0.25);
	}
	else
	{
	  adhoc_adjust(valueOf(king, WHITE, b, GOLD),    0.5);
	  adhoc_adjust(valueOf(king, WHITE, b, PSILVER), 0.5);
	}
	adhoc_adjust(valueOf(king, WHITE, b, PKNIGHT), 0.5);
	adhoc_adjust(valueOf(king, WHITE, b, PLANCE),  0.5);
	adhoc_adjust(valueOf(king, WHITE, b, PPAWN),   0.5);

	const Position w(x,9);
	if (x <= 2 || x >= 8)
	{
	  adhoc_adjust(valueOf(king, BLACK, w, GOLD),    0.25);
	  adhoc_adjust(valueOf(king, BLACK, w, PSILVER), 0.25);
	}
	else
	{
	  adhoc_adjust(valueOf(king, BLACK, w, GOLD),    0.5);
	  adhoc_adjust(valueOf(king, BLACK, w, PSILVER), 0.5);
	}
	adhoc_adjust(valueOf(king, BLACK, w, PKNIGHT), 0.5);
	adhoc_adjust(valueOf(king, BLACK, w, PLANCE),  0.5);
	adhoc_adjust(valueOf(king, BLACK, w, PPAWN),   0.5);
      }
      // 1ʶ̤Ф3ܤζϲͤ1.25ܤ
      if (king_y == 1)
      {
	for (int x = 1; x <= 9; ++x)
	{
	  const Position three(x, 3);
	  for (int ptype = PPAWN; ptype <= ROOK; ptype++)
	  {
	    if (ptype != KING && ptype != ROOK && ptype != PROOK)
	      adhoc_adjust(valueOf(king, WHITE, three,
				   static_cast<osl::Ptype>(ptype)), 1.25);
	  }
	}
      }
      if (king_y == 9)
      {
	for (int x = std::max(king_x - 2, 1);
	     x <= std::min(king_x + 2, 9); ++x)
	{
	  const Position three(x, 7);
	  for (int ptype = PPAWN; ptype <= ROOK; ptype++)
	  {
	    if (ptype != KING && ptype != ROOK && ptype != PROOK)
	      adhoc_adjust(valueOf(king, BLACK, three,
				   static_cast<osl::Ptype>(ptype)), 1.25);
	  }
	}
      }
      if (king_y <= 2)
      {
	for (int x = std::max(king_x - 1, 1);
	     x < std::min(king_x + 1, 9); x++)
	{
	  adhoc_adjust(valueOf(king, WHITE, Position(x, 4), PAWN),
		       1.25);
	}
      }
      if (king_y >= 8)
      {
	for (int x = std::max(king_x - 1, 1);
	     x < std::min(king_x + 1, 9); x++)
	{
	  adhoc_adjust(valueOf(king, BLACK, Position(x, 6), PAWN),
		       1.25);
	}
      }
      // ṳ̈Фܡʥ
      if (king_x == 1 || king_x == 9)
      {
	int next = king_x == 1 ? 2 : 8;
	adhoc_edge_king_1(BLACK, king, Position(next, king_y));
	adhoc_edge_king_1(WHITE, king, Position(next, king_y));
	if (king_y > 1)
	{
	  adhoc_edge_king_1(BLACK, king, Position(next, king_y - 1));
	}
	if (king_y > 2)
	{
	  // 7ܤ 1.25 * 1.25 ܤ뤱ɵˤʤ
	  adhoc_edge_king_1(BLACK, king, Position(next, king_y - 2));
	}
	if (king_y < 9)
	{
	  adhoc_edge_king_1(WHITE, king, Position(next, king_y + 1));
	}
	if (king_y < 8)
	{
	  // 3ܤ 1.25 * 1.25 ܤ뤱ɵˤʤ
	  adhoc_edge_king_1(WHITE, king, Position(next, king_y + 2));
	}
      }
      if (king_x < 6)
      {
	valueOf(king, WHITE, Position(9, 9), LANCE) = 0;
	valueOf(king, WHITE, Position(9, 8), LANCE) = 0;
	valueOf(king, WHITE, Position(9, 7), LANCE) = 0;
	valueOf(king, WHITE, Position(8, 9), KNIGHT) = 0;

	valueOf(king, BLACK, Position(9, 1), LANCE) = 0;
	valueOf(king, BLACK, Position(9, 2), LANCE) = 0;
	valueOf(king, BLACK, Position(9, 3), LANCE) = 0;
	valueOf(king, BLACK, Position(8, 1), KNIGHT) = 0;

	valueOf(king, WHITE, Position(9, 1), GOLD) = 0;
	valueOf(king, WHITE, Position(9, 1), SILVER) = 0;
	valueOf(king, WHITE, Position(9, 1), PSILVER) = 0;
	valueOf(king, WHITE, Position(8, 1), GOLD) = 0;
	valueOf(king, WHITE, Position(8, 1), SILVER) = 0;
	valueOf(king, WHITE, Position(8, 1), PSILVER) = 0;

	valueOf(king, BLACK, Position(9, 9), GOLD) = 0;
	valueOf(king, BLACK, Position(9, 9), SILVER) = 0;
	valueOf(king, BLACK, Position(9, 9), PSILVER) = 0;
	valueOf(king, BLACK, Position(8, 9), GOLD) = 0;
	valueOf(king, BLACK, Position(8, 9), SILVER) = 0;
	valueOf(king, BLACK, Position(8, 9), PSILVER) = 0;
      }

      if (king_x > 4)
      {
	valueOf(king, WHITE, Position(1, 9), LANCE) = 0;
	valueOf(king, WHITE, Position(1, 8), LANCE) = 0;
	valueOf(king, WHITE, Position(1, 7), LANCE) = 0;
	valueOf(king, WHITE, Position(2, 9), KNIGHT) = 0;

	valueOf(king, BLACK, Position(1, 1), LANCE) = 0;
	valueOf(king, BLACK, Position(1, 2), LANCE) = 0;
	valueOf(king, BLACK, Position(1, 3), LANCE) = 0;
	valueOf(king, BLACK, Position(2, 1), KNIGHT) = 0;

	valueOf(king, WHITE, Position(1, 1), GOLD) = 0;
	valueOf(king, WHITE, Position(1, 1), PSILVER) = 0;
	valueOf(king, WHITE, Position(1, 1), SILVER) = 0;
	valueOf(king, WHITE, Position(2, 1), GOLD) = 0;
	valueOf(king, WHITE, Position(2, 1), SILVER) = 0;
	valueOf(king, WHITE, Position(2, 1), PSILVER) = 0;

	valueOf(king, BLACK, Position(1, 9), GOLD) = 0;
	valueOf(king, BLACK, Position(1, 9), SILVER) = 0;
	valueOf(king, BLACK, Position(1, 9), PSILVER) = 0;
	valueOf(king, BLACK, Position(2, 9), GOLD) = 0;
	valueOf(king, BLACK, Position(2, 9), SILVER) = 0;
	valueOf(king, BLACK, Position(2, 9), PSILVER) = 0;
      }

    }
  }
  adhoc_edge_king_2(BLACK, Position(1, 9), Position(3, 8));
  adhoc_edge_king_2(BLACK, Position(1, 8), Position(3, 7));
  adhoc_edge_king_2(BLACK, Position(9, 9), Position(7, 8));
  adhoc_edge_king_2(BLACK, Position(9, 8), Position(7, 7));
  adhoc_edge_king_2(WHITE, Position(1, 1), Position(3, 2));
  adhoc_edge_king_2(WHITE, Position(1, 2), Position(3, 3));
  adhoc_edge_king_2(WHITE, Position(9, 1), Position(7, 2));
  adhoc_edge_king_2(WHITE, Position(9, 2), Position(7, 3));

  for (int x = 1; x <= 9; x++)
  {
    for (int y = 1; y <= 9; y++)
    {
      adhoc_adjust(valueOf(Position(1,1), WHITE, Position(x, y), GOLD), 1.05);
      adhoc_adjust(valueOf(Position(9,1), WHITE, Position(x, y), GOLD), 1.05);
      adhoc_adjust(valueOf(Position(1,1), WHITE, Position(x, y), SILVER), 1.05);
      adhoc_adjust(valueOf(Position(9,1), WHITE, Position(x, y), SILVER), 1.05);
      adhoc_adjust(valueOf(Position(1,1), WHITE, Position(x, y), BISHOP), 0.95);
      adhoc_adjust(valueOf(Position(1,1), WHITE, Position(x, y), PBISHOP), 0.95);
      adhoc_adjust(valueOf(Position(9,1), WHITE, Position(x, y), BISHOP), 0.95);
      adhoc_adjust(valueOf(Position(9,1), WHITE, Position(x, y), PBISHOP), 0.95);
      for (int ptype = PPAWN; ptype <= PSILVER; ptype++)
      {
	adhoc_adjust(valueOf(Position(1, 1), WHITE, Position(x,y),
			     static_cast<osl::Ptype>(ptype)), 1.05);
	adhoc_adjust(valueOf(Position(9, 1), WHITE, Position(x,y),
			     static_cast<osl::Ptype>(ptype)), 1.05);
	adhoc_adjust(valueOf(Position(1, 9), BLACK, Position(x,y),
			     static_cast<osl::Ptype>(ptype)), 1.05);
	adhoc_adjust(valueOf(Position(9, 9), BLACK, Position(x,y),
			     static_cast<osl::Ptype>(ptype)), 1.05);
      }
      adhoc_adjust(valueOf(Position(1,9), BLACK, Position(x, y), GOLD), 1.05);
      adhoc_adjust(valueOf(Position(9,9), BLACK, Position(x, y), GOLD), 1.05);
      adhoc_adjust(valueOf(Position(1,9), BLACK, Position(x, y), SILVER), 1.05);
      adhoc_adjust(valueOf(Position(9,9), BLACK, Position(x, y), SILVER), 1.05);
      adhoc_adjust(valueOf(Position(1,9), BLACK, Position(x, y), BISHOP), 0.95);
      adhoc_adjust(valueOf(Position(1,9), BLACK, Position(x, y), PBISHOP), 0.95);
      adhoc_adjust(valueOf(Position(9,9), BLACK, Position(x, y), BISHOP), 0.95);
      adhoc_adjust(valueOf(Position(9,9), BLACK, Position(x, y), PBISHOP), 0.95);

    }
  }
  adhoc_adjust(valueOf(Position(1,1), WHITE, Position::STAND(), BISHOP), 0.95);
  adhoc_adjust(valueOf(Position(9,1), WHITE, Position::STAND(), BISHOP), 0.95);
  adhoc_adjust(valueOf(Position(1,9), BLACK, Position::STAND(), BISHOP), 0.95);
  adhoc_adjust(valueOf(Position(9,9), BLACK, Position::STAND(), BISHOP), 0.95);
}

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
