/* alphaBetaPlayer.cc
 */
#include "osl/game_playing/alphaBetaPlayer.h"
#include "osl/game_playing/searchPlayer.tcc"
#include "osl/search/alphaBeta.h"
#include "osl/search/simpleHashTable.h"
#include "osl/eval/progressEval.h"
#include "osl/eval/eachPlayerAttackDefense.h"
#include "osl/eval/pieceEval.h"
#include "osl/category/standardMoveGenerator.h"

/* ------------------------------------------------------------------------- */

osl::game_playing::
PlainAlphaBetaPlayer::PlainAlphaBetaPlayer()
{
}

osl::game_playing::
PlainAlphaBetaPlayer::~PlainAlphaBetaPlayer()
{
}

osl::game_playing::ComputerPlayer* osl::game_playing::
PlainAlphaBetaPlayer::clone() const 
{
  return cloneIt(*this);
}

const osl::game_playing::MoveWithComment osl::game_playing::
PlainAlphaBetaPlayer::searchWithSecondsForThisMove(const GameState& gs, int seconds)
{
  typedef AlphaBeta<PieceEval,StandardMoveGenerator,
    SimpleHashTable,CountRecorder,RealizationProbability> searcher_t;

  const int consumed = setUpTable(gs, 0);
  if (seconds > 0)
    seconds = std::max(1, seconds-consumed);
  searcher.reset();
  searcher.reset(new searcher_t(gs.state(), *checkmate_ptr, table_ptr.get(), 
				*recorder_ptr));
  return SearchPlayer::search<searcher_t>(gs, seconds);
}

bool osl::game_playing::
PlainAlphaBetaPlayer::isReasonableMove(const GameState& gs,
				       Move move, int pawn_sacrifice)
{
  typedef AlphaBeta<PieceEval,StandardMoveGenerator,
    SimpleHashTable,CountRecorder,RealizationProbability> searcher_t;
  setUpTable(gs, 0);
  searcher_t searcher(gs.state(), *checkmate_ptr, table_ptr.get(),
		      *recorder_ptr);
  return isReasonableMoveBySearch(searcher, move, pawn_sacrifice);
}

/* ------------------------------------------------------------------------- */
osl::game_playing::
ProgressAlphaBetaPlayer::ProgressAlphaBetaPlayer()
{
}

osl::game_playing::
ProgressAlphaBetaPlayer::~ProgressAlphaBetaPlayer()
{
}

osl::game_playing::ComputerPlayer* osl::game_playing::
ProgressAlphaBetaPlayer::clone() const 
{
  return cloneIt(*this);
}

const osl::game_playing::MoveWithComment osl::game_playing::
ProgressAlphaBetaPlayer::searchWithSecondsForThisMove(const GameState& gs, int seconds)
{
  typedef AlphaBeta<eval::ProgressEval,StandardMoveGenerator,
    SimpleHashTable,CountRecorder,RealizationProbability> searcher_t;

  const int consumed = setUpTable(gs, 0);
  if (seconds > 0)
    seconds = std::max(1, seconds-consumed);
  searcher.reset();
  searcher.reset(new searcher_t(gs.state(), *checkmate_ptr, table_ptr.get(), 
				*recorder_ptr));
  return SearchPlayer::search<searcher_t>(gs, seconds);
}

bool osl::game_playing::
ProgressAlphaBetaPlayer::isReasonableMove(const GameState& gs,
					  Move move, int pawn_sacrifice)
{
  typedef AlphaBeta<PieceEval,StandardMoveGenerator,
    SimpleHashTable,CountRecorder,RealizationProbability> searcher_t;
  setUpTable(gs, 0);
  searcher_t searcher(gs.state(), *checkmate_ptr, table_ptr.get(), 
		      *recorder_ptr);
  return isReasonableMoveBySearch(searcher, move, pawn_sacrifice);
}

/* ------------------------------------------------------------------------- */
#ifdef USE_EACH_PLAYER_PROGRESS_PLAYER

osl::game_playing::
EachPlayerProgressAlphaBetaPlayer::EachPlayerProgressAlphaBetaPlayer()
{
}

osl::game_playing::
EachPlayerProgressAlphaBetaPlayer::~EachPlayerProgressAlphaBetaPlayer()
{
}

osl::game_playing::ComputerPlayer* osl::game_playing::
EachPlayerProgressAlphaBetaPlayer::clone() const 
{
  return cloneIt(*this);
}

const osl::game_playing::MoveWithComment osl::game_playing::
EachPlayerProgressAlphaBetaPlayer::searchWithSecondsForThisMove(const GameState& gs, int seconds)
{
  typedef AlphaBeta<eval::EachPlayerAttackDefense,StandardMoveGenerator,
    SimpleHashTable,CountRecorder,RealizationProbability> searcher_t;

  const int consumed = setUpTable(gs, 0);
  if (seconds > 0)
    seconds = std::max(1, seconds-consumed);
  searcher.reset();
  searcher.reset(new searcher_t(gs.state(), *checkmate_ptr, table_ptr.get(), 
				*recorder_ptr));
  return SearchPlayer::search<searcher_t>(gs, seconds);
}

bool osl::game_playing::
EachPlayerProgressAlphaBetaPlayer::isReasonableMove(const GameState& gs,
						    Move move, int pawn_sacrifice)
{
  typedef AlphaBeta<PieceEval,StandardMoveGenerator,
    SimpleHashTable,CountRecorder,RealizationProbability> searcher_t;
  setUpTable(gs, 0);
  searcher_t searcher(gs.state(), *checkmate_ptr, table_ptr.get(), *recorder_ptr);  
  return SearchPlayer::isReasonableMove(searcher, move, pawn_sacrifice);
}
#endif /* USE_EACH_PLAYER_PROGRESS_PLAYER */
/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
