/* historyToTable.cc
 */
#include "osl/game_playing/historyToTable.h"
#include "osl/game_playing/gameState.h"
#include "osl/hash/hashKeyStack.h"
#include "osl/repetitionCounter.h"
#include "osl/search/simpleHashTable.h"
#include "osl/search/simpleHashRecord.h"
#include "osl/container/moveStack.h"

const int osl::game_playing::HistoryToTable::LIMIT
= osl::search::SearchTable::HistorySpecialDepth;

void osl::game_playing::
HistoryToTable::adjustDominance(const HashKey& key, 
				search::SimpleHashTable& table,
				int black_win, int white_win,
				const Move& good_move)
{
  const PieceStand black_stand = key.getPieceStand();
  const Player turn = key.turn();
  for (size_t i=0; i<PieceStand::order.size(); ++i)
  {
    const Ptype ptype = PieceStand::order[i];
    if (black_stand.get(ptype))
    {
      // white win dominance
      PieceStand new_stand = black_stand;
      new_stand.sub(ptype);
      HashKey new_key = key;
      new_key.setPieceStand(new_stand);
      SimpleHashRecord *record = table.allocate(new_key, LIMIT);
      if (record)
      {
	const Move record_move = (turn == WHITE) ? good_move : Move::INVALID();
	record->setAbsoluteValue(record_move, white_win, LIMIT);
	record->qrecord.setHistoryValue(record_move, white_win);
      }
    }
    if (black_stand.canAdd(ptype))
    {
      // black win dominance
      // TODO:
      // - ä𤬤뤫ɤΥå̤򸫤ɬפ
      // - good_move valid Ǥ뤳Ȥ assert 
      PieceStand new_stand = black_stand;
      new_stand.add(ptype);
      HashKey new_key = key;
      new_key.setPieceStand(new_stand);
      SimpleHashRecord *record = table.allocate(new_key, LIMIT);
      if (record)
      {
	const Move record_move = (turn == BLACK) ? good_move : Move::INVALID();
	record->setAbsoluteValue(record_move, black_win, LIMIT);
	record->qrecord.setHistoryValue(record_move, black_win);
      }
    }
  }
}

void osl::game_playing::
HistoryToTable::adjustTable(const GameState& state, SimpleHashTable& table,
			    int black_win, int draw, int white_win)
{
  const RepetitionCounter& counter = state.counter();
  // ͥ۴ط꤬ͥ
  // Ͽͥ
  HashKeyStack history = state.hashHistory(); // copy
  MoveStack move_history = state.moveHistory();
  move_history.push(Move::INVALID());
  assert(move_history.size() == history.size());

  HashKeyStack reverse_history;
  while (! history.empty())
  {
    const HashKey key = history.top();
    history.pop();
    assert(move_history.hasLastMove());
    const Move last_move = move_history.lastMove();
    move_history.pop();

    if (key != state.state().getHash())
      reverse_history.push(key);

    // set dominance
    adjustDominance(key, table, black_win, white_win, last_move);
  }

  while (! reverse_history.empty())
  {
    // set repetition
    const HashKey key = reverse_history.top();
    reverse_history.pop();

    SimpleHashRecord *record = table.allocate(key, LIMIT);
    Sennichite result = counter.isAlmostSennichite(key);
    if (result.isDraw())
    {
      record->setAbsoluteValue(Move::INVALID(), draw, LIMIT);
      record->qrecord.setHistoryValue(draw);
    }
    else
    {
      assert(result.hasWinner());
      const int value = (result.winner() == BLACK) ? black_win : white_win;
      record->setAbsoluteValue(Move::INVALID(), value, LIMIT);
      record->qrecord.setHistoryValue(value);
    }
  }
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
