#include "osl/hash/hashKey.h"
#include <iomanip>
#include <cstdlib>
#include <iostream>
namespace osl
{
  BOOST_STATIC_ASSERT(sizeof(HashKey) >= sizeof(int)*6);
} // namespace osl

std::ostream& osl::hash::operator<<(std::ostream& os,const osl::HashKey& h)
{
  os << h.getPieceStand();
  os << ':' 
     << std::setfill('0') << std::setbase(16) << std::setw(8)
     << h.getSignature();
  const BoardKey& board_key = h.getSignatureKey().getBoardKey();
  for (size_t i=0; i<board_key.size(); ++i)
  {
    os << ':' 
       << std::setfill('0') << std::setbase(16) << std::setw(8)
       << board_key[i];
  }
  return os << ':' << std::setbase(10);
}

osl::HashKey osl::HashKey::
calcHash(const SimpleState& state)
{
  HashKey v;
  for(int num=0;num<40;num++){
    Piece p=state.getPieceOf(num);
    Hash_Gen_Table.addHashKey(v,p.position(),p.ptypeO());
  }
  v.setPlayer(state.getTurn());
  return v;
}

const osl::HashKey osl::HashKey::
newHashWithMove(Move move) const
{
  HashKey ret(*this);
  if (! move.isPass())
  {
    assert(move.isValid());
    Position from=move.from();
    Position to=move.to();
    Ptype capturePtype=move.capturePtype();
    PtypeO ptypeO=move.ptypeO();
    PtypeO oldPtypeO=move.oldPtypeO();
    if (capturePtype!=PTYPE_EMPTY)
    {
      PtypeO capturePtypeO=newPtypeO(alt(move.player()),capturePtype);
      PtypeO capturedPtypeO=captured(capturePtypeO);

      Hash_Gen_Table.subHashKey(ret,to,capturePtypeO);
      Hash_Gen_Table.addHashKey(ret,Position::STAND(),capturedPtypeO);

      Hash_Gen_Table.subHashKey(ret,from,oldPtypeO);
      Hash_Gen_Table.addHashKey(ret,to,ptypeO);
    }
    else{
      Hash_Gen_Table.subHashKey(ret,from,oldPtypeO);
      Hash_Gen_Table.addHashKey(ret,to,ptypeO);
    }
  }
  ret.changeTurn();
  return ret;
}
  

osl::hash::HashGenTable::HashGenTable()
{
  for(int j=0;j<PTYPEO_SIZE;j++)
  {
    const PtypeO pjo = (PtypeO)(j+PTYPEO_MIN);
    for(int i=0;i<Position::SIZE;i++)
    {
      if (Position::nth(i) == Position::STAND())
      {
	const Ptype pj = getPtype(pjo);
	if (isBasic(pj) && (getOwner(pjo) == BLACK))
	{
	  PieceStand stand;
	  stand.add(pj);
	  hashKey[i][j].setPieceStand(stand);
	}
      }
      else
      {
	hashKey[i][j].setRandom();
      }
    }
  }
}


// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
