/* oslConfig.cc
 */
#include "osl/oslConfig.h"
#include <boost/filesystem/operations.hpp>
#include <iostream>
#include <cstdlib>

bool osl::OslConfig::isGoodHome(const std::string& dir)
{
  return boost::filesystem::exists(dir)
    && boost::filesystem::is_directory(dir);
}

void osl::OslConfig::trySetHome(std::string& home, const std::string& dir)
{
  if (isGoodHome(dir))
  {
    home = dir;
    return;
  }
  std::cerr << "skipping " << dir << std::endl;
}

const std::string osl::OslConfig::makeHome()
{
  std::string result;
  if (const char *env = getenv("GPSSHOGI_HOME"))
    trySetHome(result, env);
  
#if defined GPSSHOGI_HOME
  if (result.empty())
    trySetHome(result, GPSSHOGI_HOME);
#endif

  if (result.empty())
    if (const char *env = getenv("OSL_HOME"))
      trySetHome(result, env);

  if (result.empty())
    result = OSL_HOME;

  std::cerr << "using " << result << " as OSL_HOME" << std::endl;
  return result;
}

const std::string& osl::OslConfig::home()
{
  static const std::string home_directory = makeHome();
  return home_directory;
}

const char * osl::OslConfig::home_c_str()
{
  return home().c_str();
}

const char *osl::OslConfig::openingBook()
{
  static std::string opening = home() + "/data/joseki.dat";
  return opening.c_str();
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
