/* kanjiPrint.cc
 */
#include "osl/record/kanjiPrint.h"
#include "osl/ptype.h"
#include "osl/pieceStand.h"
#include <iostream>


osl::record::KanjiPrint::
KanjiPrint(std::ostream& o) : os(o)
{
}

osl::record::KanjiPrint::
~KanjiPrint()
{
}

namespace osl
{
  namespace
  {
    const size_t NumKanjis = PTYPEO_MAX - PTYPEO_MIN+2;
    const char kanjiPieces[NumKanjis][3] = 
    {
      // 
      "", "-E", 
      "", ""/*""*/, ""/*""*/, "" /*""*/, "", "", "", "", "", "", "", "", "", "", 
      // 
      "", "+E",
      "", "", "", "", "", "ζ", "", "", "", "", "", "", "", "", 
    };
  } // anonymous namespace
} // namespace osl

const char *osl::record::KanjiPrint::kanji(PtypeO ptypeo)
{
  const size_t index = ptypeo -PTYPEO_MIN;
  assert(index < NumKanjis);
  return kanjiPieces[index];
}

void osl::record::KanjiPrint::
print(const SimpleState& state)
{
  for(int y=1;y<=9;y++){
    os << 'P' << y;  
    for(int x=9;x>0;x--){
      const PtypeO ptypeo = state.getPieceOnBoard(Position(x,y)).ptypeO();
      os << kanji(ptypeo);
    }
    os << std::endl;
  }
  // ɽ
  os << " ";
  for (unsigned int i=0; i<PieceStand::order.size(); ++i)
  {
    const Ptype ptype = PieceStand::order[i];
    const int count = state.countPiecesOnStand(BLACK, ptype);
    if (count)
      os << kanji(newPtypeO(BLACK, ptype))
	 << count << " ";
  }
  os << std::endl;  
  os << " ";
  for (unsigned int i=0; i<PieceStand::order.size(); ++i)
  {
    const Ptype ptype = PieceStand::order[i];
    const int count = state.countPiecesOnStand(WHITE, ptype);
    if (count)
      os << kanji(newPtypeO(BLACK, ptype))
	 << count << " ";
  }
  os << std::endl;  
  os << state.getTurn() << std::endl;
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
