/* quiescenceRecord.cc
 */
#include "osl/search/quiescenceRecord.h"
#include "osl/record/csa.h"
#include "osl/ptypeTable.h"
#include <boost/static_assert.hpp>
#include <map>
#include <iostream>

BOOST_STATIC_ASSERT(osl::search::QuiescenceFlags::LAST_FLAG 
		    < sizeof(osl::search::QuiescenceFlags)*8);

namespace osl
{
  namespace search
  {
    namespace
    {
      typedef std::map<int, const char *> name_map_t; // TODO:QuiescenceFlags::lags
      const name_map_t& make_names()
      {
	static name_map_t result;
	if (result.empty())
	{
	  for (int i=PTYPE_BASIC_MIN; i<=PTYPE_MAX; ++i)
	  {
	    const Ptype p = static_cast<Ptype>(i);
	    result[p] = Ptype_Table.getName(p);
	  }
	  result[QuiescenceFlags::KING_ESCAPE] = "KING_ESCAPE";
	  result[QuiescenceFlags::PROMOTE] = "PROMOTE";
	  result[QuiescenceFlags::CHECK] = "CHECK";
	  result[QuiescenceFlags::DROP] = "DROP";
	  result[QuiescenceFlags::ALL_ESCAPE] = "ALL_ESCAPE";
	  result[QuiescenceFlags::MAJOR_PIECE_ATTACK] = "MAJOR_PIECE_ATTACK";
	  result[QuiescenceFlags::KNIGHT_ATTACK] = "KNIGHT_ATTACK";
	  result[QuiescenceFlags::GOLDSILVER_ATTACK] = "GOLDSILVER_ATTACK";
	  result[QuiescenceFlags::ADVANCE_BISHOP] = "ADVANCE_BISHOP";
	  result[QuiescenceFlags::KING8_ATTACK] = "KING8_ATTACK";
	  result[QuiescenceFlags::ESCAPE_FROM_LAST_MOVE] = "ESCAPE!";
	  result[QuiescenceFlags::PINNED_ATTACK] = "PINNED_ATTACK";
	  result[QuiescenceFlags::UTILIZE_PROMOTED] = "UTILIZE_PROMOTED";
	  result[QuiescenceFlags::BREAK_THREATMATE] = "BREAK_THREATMATE";
	  result[QuiescenceFlags::KING_WALK] = "KING_WALK";
	}
	return result;
      }
    } // anonymous namespace
  } // namespace search
} // namespace osl

void osl::search::
QuiescenceRecord::dump(std::ostream& os) const
{
  os << "QuiescenceRecord " << this << "\n";
  os << lower_bound << " (" << (int)lower_depth << ")"
     << upper_bound << " (" << (int)upper_depth << ")";
  if (hasStaticValue())
  {
    os << " s " << static_value << toString(staticValueType());
    os << " t1 " << threat1.value << " " << record::csa::show(threat1.move)
       << " t2 " << threat2.value << " " << record::csa::show(threat2.move);
  }
  os << "\n";
  os << "best move " <<  record::csa::show(best_move) << "\n";
  os << "threatmate " <<  threatmate << "\n";
  os << "king8info " <<  king8info << "\n";
  os << "oracle a " << attack_oracle.proof 
     << " t " << threatmate_oracle.proof << "\n";
  os << "flags: ";
  const name_map_t& names = make_names();
  {
    int printed = 0;
    for (int i=0; i<QuiescenceFlags::LAST_FLAG; ++i)
    {
      if (flags.data & (1 << (i)))  // isGenerated
      {
	++printed;
	const name_map_t::const_iterator p=names.find(i);
	if (p != names.end())
	  os << p->second << " ";
	else
	  os << "(" << i << ") ";
	if (printed % 4 == 0)
	  os << "\n";
      }
    }
    if (printed % 4)
      os << "\n";
  }
  os << "sendoffs " << sendoffs.size();
  for (size_t i=0; i<sendoffs.size(); ++i)
  {
    os << " " << record::csa::show(sendoffs[i]);
  }
  os << "\n";
  os << "moves " << moves_size();
  size_t i=0;
  MoveVector moves_copy;
  loadMoves(moves_copy);
  for (MoveVector::const_iterator p=moves_copy.begin(); p!=moves_copy.end(); ++p, ++i)
  {
    os << " " << record::csa::show(*p);
    if (i % 8 == 7)
      os << "\n";
  }
  if (i % 8 != 7)
    os << "\n";
}

const char *osl::search::
QuiescenceRecord::toString(StaticValueType type)
{
  switch (type)
  {
  case UNKNOWN:
    return "?";
  case UPPER_BOUND:
    return ">";
  case EXACT:
    return "=";
  default:
    assert(0);
  }
  return "!";			// should not occur
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
