#include "osl/ptype.h"
#include "osl/direction.h"
#include "osl/effect/liberty8Table.h"
#include "osl/pieceTable.h"
#include "osl/hash/hashKey.h"

#include "osl/move_generator/addEffectTable.h"
#include "osl/move_generator/addEffect8Table.h"
#include "osl/progress/ptypeProgress.h"
#include "osl/progress/effect5x3Table.h"
#include "osl/pathEncoding.h"
#include "osl/effect/moveSignature.h"

#include "osl/centering3x3.h"
#include "osl/centering5x3.h"
#include "osl/effect_util/neighboring8Direct.h"
#include "osl/effect_util/sendOffPosition.h"

#include "osl/checkmate/immediateCheckmateTable.h"
#include "osl/checkmate/proofNumberTable.h"

#include "osl/eval/pieceEval.h"
#include "osl/eval/ppair/piecePairRawEval.h"
#include "osl/eval/ppair/piecePairPieceEval.h"
#include "osl/eval/endgame/attackKing.h"
#include "osl/eval/endgame/defenseKing.h"

namespace osl
{
  /** y ɸбʹ٤ηǽ0 (Ѥʤ)*/
  const osl::CArray<int,10> osl::progress::PtypeProgressTable::yVals =
  {{   
    0,4,4,4,3,2,1,0,0,0
  }};

  const CArray<Offset,DIRECTION_SIZE> BoardTable::offsets = { {
    DirectionTraits<static_cast<Direction>(0)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(1)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(2)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(3)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(4)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(5)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(6)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(7)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(8)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(9)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(10)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(11)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(12)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(13)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(14)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(15)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(16)>::blackOffset(),
    DirectionTraits<static_cast<Direction>(17)>::blackOffset()
  } };
  const CArray<int,DIRECTION_SIZE> BoardTable::dxs = { {
    DirectionTraits<static_cast<Direction>(0)>::blackDx,
    DirectionTraits<static_cast<Direction>(1)>::blackDx,
    DirectionTraits<static_cast<Direction>(2)>::blackDx,
    DirectionTraits<static_cast<Direction>(3)>::blackDx,
    DirectionTraits<static_cast<Direction>(4)>::blackDx,
    DirectionTraits<static_cast<Direction>(5)>::blackDx,
    DirectionTraits<static_cast<Direction>(6)>::blackDx,
    DirectionTraits<static_cast<Direction>(7)>::blackDx,
    DirectionTraits<static_cast<Direction>(8)>::blackDx,
    DirectionTraits<static_cast<Direction>(9)>::blackDx,
    DirectionTraits<static_cast<Direction>(10)>::blackDx,
    DirectionTraits<static_cast<Direction>(11)>::blackDx,
    DirectionTraits<static_cast<Direction>(12)>::blackDx,
    DirectionTraits<static_cast<Direction>(13)>::blackDx,
    DirectionTraits<static_cast<Direction>(14)>::blackDx,
    DirectionTraits<static_cast<Direction>(15)>::blackDx,
    DirectionTraits<static_cast<Direction>(16)>::blackDx,
    DirectionTraits<static_cast<Direction>(17)>::blackDx
  } };
  const CArray<int,DIRECTION_SIZE> BoardTable::dys = { {
    DirectionTraits<static_cast<Direction>(0)>::blackDy,
    DirectionTraits<static_cast<Direction>(1)>::blackDy,
    DirectionTraits<static_cast<Direction>(2)>::blackDy,
    DirectionTraits<static_cast<Direction>(3)>::blackDy,
    DirectionTraits<static_cast<Direction>(4)>::blackDy,
    DirectionTraits<static_cast<Direction>(5)>::blackDy,
    DirectionTraits<static_cast<Direction>(6)>::blackDy,
    DirectionTraits<static_cast<Direction>(7)>::blackDy,
    DirectionTraits<static_cast<Direction>(8)>::blackDy,
    DirectionTraits<static_cast<Direction>(9)>::blackDy,
    DirectionTraits<static_cast<Direction>(10)>::blackDy,
    DirectionTraits<static_cast<Direction>(11)>::blackDy,
    DirectionTraits<static_cast<Direction>(12)>::blackDy,
    DirectionTraits<static_cast<Direction>(13)>::blackDy,
    DirectionTraits<static_cast<Direction>(14)>::blackDy,
    DirectionTraits<static_cast<Direction>(15)>::blackDy,
    DirectionTraits<static_cast<Direction>(16)>::blackDy,
    DirectionTraits<static_cast<Direction>(17)>::blackDy
  } };
}

namespace osl
{
  const PieceTable Piece_Table;
  const BoardTable Board_Table;
  // PtypeTable depends on BoardTable
  const PtypeTable Ptype_Table;
  // BoardTable, PtypeTable -> Liberty8Table
  const effect::Liberty8Table effect::Liberty8_Table;

  const eval::PtypeEvalTable eval::Ptype_Eval_Table;
  const hash::HashGenTable hash::Hash_Gen_Table;
  // PtypeTable, BoardTable -> AddEffectTable
  const move_generator::AddEffectTable move_generator::Add_Effect_Table;
  // PtypeTable, BoardTable -> AddEffect8Table
  const move_generator::AddEffect8Table move_generator::Add_Effect8_Table;

  const effect::MoveSignatureTable effect::Move_Signature_Table;
  const PathEncodingTable Path_Encoding_Table;

  const progress::PtypeProgressTable progress::Ptype_Progress_Table;

  const Centering3x3::Table Centering3x3::table;
  const Centering5x3::Table Centering5x3::table;
  const effect_util::Neighboring8Direct::Table Neighboring8Direct::table;
  const effect_util::SendOffPosition::Table effect_util::SendOffPosition::table;

  const checkmate::ImmediateCheckmateTable checkmate::Immediate_Checkmate_Table;
  const checkmate::ProofNumberTable checkmate::Proof_Number_Table;

  const eval::ppair::PiecePairRawTable eval::ppair::PiecePairRawTable::Table;
  const eval::ppair::PiecePairPieceTable eval::ppair::PiecePairPieceTable::Table;

  const eval::endgame::AttackKing::Table eval::endgame::AttackKing::table;
  const eval::endgame::DefenseKing::Table eval::endgame::DefenseKing::table;

  const progress::Effect5x3Table progress::Effect5x3_Table;

}


// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
