// make_move.cc
#include "osl/category/analyzer/categoryMoveVector.h"
#include "osl/category/standardMoveGenerator.h"
#include "osl/record/csaRecord.h"
#include "osl/apply_move/applyMove.h"
#include <iostream>

using namespace osl;
using namespace osl::category;
using namespace osl::category::analyzer;

void usage(const char *prog)
{
  using namespace std;
  cerr << "Usage: " << prog << " [-d] [-l limit] csa_files\n"
       << endl;
  exit(1);
}

/** move generation limit */
int limit = 800;
int detailed = false;

void testFile(const char *file_name);

int main(int argc, char **argv)
{
  const char *program_name = argv[0];
  bool error_flag = false;
  extern char *optarg;
  extern int optind;
    
  char c;
  while ((c = getopt(argc, argv, "dl:vh")) != EOF)
  {
    switch(c)
    {
    case 'd':	detailed = true;
      break;
    case 'l':	limit = atoi(optarg);
      break;
    default:	error_flag = true;
    }
  }
  argc -= optind;
  argv += optind;

  if (error_flag || (argc < 1))
    usage(program_name);

  for (int i=0; i<argc; ++i)
  {
    testFile(argv[i]);
  }
}

void testState(const CategoryEnv& env)
{
  std::cout << *env.state;
  CategoryMoveVector moves;
  if (detailed)
    moves.generateAndSetDetailed<StandardMoveGenerator>(env);
  else
    moves.generateAndSet<StandardMoveGenerator>(env);
  for (CategoryMoveVector::const_iterator p=moves.begin();
       p!=moves.end(); ++p)
  {
    std::cout << p->category << "\n";
    for (MoveLogProbVector::const_iterator q=p->moves.begin();
	 q!=p->moves.end(); ++q)
    {
      std::cout << "  " << *q << "\n";
    }
  }
}

void testFile(const char *filename)
{
  const Record record = CsaFile(filename).getRecord();
  NumEffectState state(record.getInitialState());
  const vector<osl::Move> moves=record.getMoves();

  MoveStack history;
  CategoryEnv env(&state, limit, &history);

  testState(env);
  for (size_t i=0; i<moves.size(); ++i)
  {
    const Move move = moves[i];
    assert(state.isValidMove(move));

    ApplyMoveOfTurn::doMove(state, move);
    history.push(move);

    testState(env);
  }
}


// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
