#include "osl/category/kingEscape.h"
#include "osl/record/csaString.h"
#include "osl/state/numEffectState.h"
#include "osl/container/moveLogProbVector.h"
#include "osl/category/categoryUtil.h"
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <iostream>

class KingEscapeTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(KingEscapeTest);
  CPPUNIT_TEST(testBug030926);
  CPPUNIT_TEST(testCount);
  CPPUNIT_TEST_SUITE_END();
public:
  void testBug030926();
  void testCount();
};

CPPUNIT_TEST_SUITE_REGISTRATION(KingEscapeTest);

using namespace osl;
using namespace osl::category;

void KingEscapeTest::testBug030926()
{
  NumEffectState state(CsaString(
			 "P1-KY-KE * -KI *  *  * +RY * \n"
			 "P2 * -OU-GI *  *  *  *  * -KY\n"
			 "P3 * -FU *  * -KI *  * -FU * \n"
			 "P4-FU * -FU-FU *  *  *  * -FU\n"
			 "P5 *  *  *  *  *  *  * +FU * \n"
			 "P6+FU * +FU+GI-FU *  *  * +FU\n"
			 "P7 * +FU * +FU *  * +KI *  * \n"
			 "P8 * +OU *  *  * +FU *  *  * \n"
			 "P9+KY+KE-UM *  *  *  * -RY+KY\n"
			 "P+00FU\n"
			 "P+00FU\n"
			 "P+00FU\n"
			 "P-00FU\n"
			 "P+00KE\n"
			 "P-00KE\n"
			 "P+00GI\n"
			 "P+00GI\n"
			 "P+00KI\n"
			 "P+00KA\n"
			 "+\n").getInitialState());
  MoveLogProbVector moves;
  CategoryEnv env(&state, 213, 0);
  CategoryUtil<KingEscape>::generate(env, moves);
  CPPUNIT_ASSERT(moves.size() >= 2);
}

int countEscapeMoves(const char *str)
{
  NumEffectState neState(CsaString(str).getInitialState());
  MoveVector moves;
  CategoryEnv env(&neState, 1000, 0);
  KingEscape::generate(env,moves);
#if 0
  std::cerr << std::endl;
  for (size_t i=0; i<moves.size(); ++i)
    std::cerr << moves[i]<< "\n";
  std::cerr << std::endl;
#endif
  const size_t generatedSize = moves.size();
  moves.unique();
  CPPUNIT_ASSERT_EQUAL(generatedSize, moves.size());
  return moves.size();
}

void KingEscapeTest::testCount()
{
  const int count1 = countEscapeMoves(
"P1+NY+TO *  *  *  * -OU-KE-KY\n"
"P2 *  *  *  *  * -GI-KI *  *\n"
"P3 * +RY *  * +UM * -KI-FU-FU\n"
"P4 *  * +FU-FU *  *  *  *  *\n"
"P5 *  * -KE * +FU *  * +FU *\n"
"P6-KE *  * +FU+GI-FU *  * +FU\n"
"P7 *  * -UM *  *  *  *  *  *\n"
"P8 * -FU *  *  *  *  *  *  * \n"
"P9 * +OU * -GI *  *  *  * -NG\n"
"P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU\n"
"P-00KI00KY00FU\n"
"P-00AL\n"
"+\n");
  // MoveLogProb(Move(+,PROOK,capture=PAWN,Position(83)->Position(88),promote=0),100)
  // MoveLogProb(Move(+,KING,capture=PAWN,Position(89)->Position(88),promote=0),100)
  // MoveLogProb(Move(+,KING,Position(89)->Position(99),promote=0),100)
  // MoveLogProb(Move(+,KING,Position(89)->Position(79),promote=0),100)
  CPPUNIT_ASSERT_EQUAL(4, count1);

  const int count2 = countEscapeMoves(
"P1+NY+TO *  *  *  * -OU-KE-KY\n"
"P2 *  *  *  *  * -GI-KI *  *\n"
"P3 * +RY *  * +UM * -KI-FU-FU\n"
"P4 *  * +FU-FU *  *  *  *  *\n"
"P5 *  * -KE * +FU *  * +FU *\n"
"P6 *  *  * +FU+GI-FU *  * +FU\n"
"P7 * -KE-UM *  *  *  *  *  *\n"
"P8 *  *  *  *  *  *  *  *  * \n"
"P9+OU *  * -GI *  *  *  * -NG\n"
"P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU-FU\n"
"P-00KI00KY00FU\n"
"P-00AL\n"
"+\n");
  // MoveLogProb(Move(+,KING,Position(99)->Position(98),promote=0),100)
  // MoveLogProb(Move(+,KING,Position(99)->Position(89),promote=0),100)
  CPPUNIT_ASSERT_EQUAL(2, count2);
  }

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
