/* eachPlayerAttackDefense.t.cc
 */
#include "osl/eval/eachPlayerAttackDefense.h"
#include "osl/record/csaString.h"
#include "consistencyTest.h"

#include <cppunit/TestCase.h>

class EachPlayerAttackDefenseTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(EachPlayerAttackDefenseTest);
  CPPUNIT_TEST(testConsistent);
  CPPUNIT_TEST(testSafety);
  CPPUNIT_TEST_SUITE_END();
public:
  void testConsistent();
  void testSafety();
};

CPPUNIT_TEST_SUITE_REGISTRATION(EachPlayerAttackDefenseTest);

using namespace osl;
using namespace osl::eval;

void EachPlayerAttackDefenseTest::testSafety()
{
  {
    const NumEffectState danger(CsaString(
				  "P1-KY * -KI *  *  *  * +UM-KY\n"
				  "P2 *  * -FU * +KI *  *  *  * \n"
				  "P3 * +HI-KA * +GI-FU-OU-FU-FU\n"
				  "P4 *  *  * -FU-FU *  * -GI * \n"
				  "P5-FU-FU+KE *  *  * -FU *  * \n"
				  "P6 *  *  * +FU+GI+KE * +FU+FU\n"
				  "P7+FU *  *  * +FU+FU+FU+GI * \n"
				  "P8 * -RY+FU * +OU * +KI *  * \n"
				  "P9+KY *  *  *  *  *  * +KE+KY\n"
				  "P+00KI\n"
				  "P-00KE00FU\n"
				  "+\n").getInitialState());
    const NumEffectState safe(CsaString( // ζ
				"P1-KY * -KI *  *  *  * +UM-KY\n"
				"P2 *  * -FU * +KI *  *  *  * \n"
				"P3 * +HI-KA * +GI-FU-OU-FU-FU\n"
				"P4-RY *  * -FU-FU *  * -GI * \n"
				"P5-FU-FU+KE *  *  * -FU *  * \n"
				"P6 *  *  * +FU+GI+KE * +FU+FU\n"
				"P7+FU *  *  * +FU+FU+FU+GI * \n"
				"P8 *  * +FU * +OU * +KI *  * \n"
				"P9+KY *  *  *  *  *  * +KE+KY\n"
				"P+00KI\n"
				"P-00KE00FU\n"
				"+\n").getInitialState());
    const EachPlayerAttackDefense ev_danger(danger);
    const EachPlayerAttackDefense ev_safe(safe);
    CPPUNIT_ASSERT(ev_safe.value() - ev_danger.value() > 1600);
  }
}

void EachPlayerAttackDefenseTest::testConsistent()
{
  consistencyTest<EachPlayerAttackDefense>();
}


/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
