/* pieceEval.t.cc
 */
#include "osl/eval/pieceEval.h"
#include "osl/eval/pieceEval.tcc"
#include "osl/state/simpleState.tcc"
#include "osl/state/numEffectState.h"
#include "osl/effect_util/effectUtil.h"
#include "consistencyTest.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <iostream>
#include <fstream>

class PieceEvalTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(PieceEvalTest);
  CPPUNIT_TEST(testBug);
  CPPUNIT_TEST(testIsEvalValueConsistent);
  CPPUNIT_TEST(testComputeDiffAfterMoveForRPBug030910);
  CPPUNIT_TEST(testComputeDiffAfterMoveForRP);
  CPPUNIT_TEST(testComputeDiffAfterTakeBack);
  CPPUNIT_TEST(testComputeDiffAfterMoveMulti);
  CPPUNIT_TEST(testComputeDiffAfterTakeBackMulti);
  CPPUNIT_TEST(testIsEvalValueConsistent);
  CPPUNIT_TEST(testCannotTake);
  CPPUNIT_TEST(testConsistent);
  CPPUNIT_TEST(testSoma);
  CPPUNIT_TEST_SUITE_END();
public:
  void testBug();
  void testSoma();
  void testCannotTake();
  void testComputeDiffAfterMoveForRP();
  void testComputeDiffAfterMoveForRPBug030910();
  void testComputeDiffAfterTakeBack();
  void testComputeDiffAfterMoveMulti();
  void testComputeDiffAfterTakeBackMulti();
  void testIsEvalValueConsistent();
  void testConsistent();
};

CPPUNIT_TEST_SUITE_REGISTRATION(PieceEvalTest);

using namespace osl;
using namespace osl::eval;
extern bool isShortTest;

const int Gold = PtypeEvalTraits<GOLD>::val;
const int Rook = PtypeEvalTraits<ROOK>::val;
const int Knight = PtypeEvalTraits<KNIGHT>::val;
const int Pawn = PtypeEvalTraits<PAWN>::val;
const int Pknight = PtypeEvalTraits<PKNIGHT>::val;
const int Bishop = PtypeEvalTraits<BISHOP>::val;
const int Pbishop = PtypeEvalTraits<PBISHOP>::val;

void PieceEvalTest::testSoma()
{
  const SimpleState state(CsaString(
			    "P1-KY-KE-GI-KI *  * -HI-KE * \n"
			    "P2 * -HI *  * -KA-OU *  *  * \n"
			    "P3-FU * -FU-FU-FU * -GI-FU-KY\n"
			    "P4 * +KE *  *  * -FU *  * -FU\n"
			    "P5 * -FU+KE+KA *  *  *  * +FU\n"
			    "P6 *  * +FU *  *  * +FU *  * \n"
			    "P7+FU+FU * +FU+FU+FU *  *  * \n"
			    "P8 * +GI+KI *  *  * +GI *  * \n"
			    "P9+KY *  *  * +OU+KI *  * +KY\n"
			    "P+00FU\n"
			    "P+00FU\n"
			    "P+00KI\n"
			    "+\n").getInitialState());
  const PieceEval ev(state);
  const int base_value = ev.value();
  {
    int ret=ev.computeValWithMove
      (state, Move(Position(1,5),Position(1,4),PAWN,PAWN,false,BLACK));
    /** ꤬ؤʤΤ */
    CPPUNIT_ASSERT_EQUAL(base_value+Pawn*2, ret);
  }
  {
    int ret=ev.computeValWithMove
      (state, Move(Position(6,5),Position(9,2),PBISHOP,PTYPE_EMPTY,true,BLACK));
    /** ּ֤ only */
    CPPUNIT_ASSERT_EQUAL(base_value-Bishop*2, ret);
  }
  {
    int ret=ev.computeValWithMove
      (state, Move(Position(6,5),Position(8,3),PBISHOP,PTYPE_EMPTY,true,BLACK));
    /**  */
    CPPUNIT_ASSERT_EQUAL(base_value+Pbishop-Bishop, ret);
  }
  {
    int ret=ev.computeValWithMove
      (state, Move(Position(6,5),Position(2,1),PBISHOP,KNIGHT,true,BLACK));
    CPPUNIT_ASSERT_EQUAL(base_value+Knight*2-Bishop*2, ret);
  }
  {
    int ret=ev.computeValWithMove
      (state, Move(Position(8,4),Position(7,2),PKNIGHT,PTYPE_EMPTY,true,BLACK));
    CPPUNIT_ASSERT_EQUAL(base_value-Knight*2, ret);
  }
  {
    int ret1=ev.computeValWithMove
      (state, Move(Position(6,5),Position(5,4),BISHOP,PTYPE_EMPTY,false,BLACK));
    CPPUNIT_ASSERT_EQUAL(base_value-Bishop*2, ret1);
  }
}

void PieceEvalTest::testBug()
{
  const SimpleState state=CsaString(
    "P1-KY-KE * -KI *  *  * -KE-KY\n"
    "P2 * -OU-GI * -KI * -HI *  * \n"
    "P3 * -FU *  * -FU-GI-KA * -FU\n"
    "P4 *  * -FU-FU * -FU-FU-FU * \n"
    "P5-FU *  *  *  *  *  *  *  * \n"
    "P6 *  * +FU * +FU+GI+FU * +FU\n"
    "P7 * +FU * +FU * +FU *  *  * \n"
    "P8 * +KA+OU * +KI+GI * +HI * \n"
    "P9+KY+KE * +KI *  *  * +KE+KY\n"
    "P-00FU00FU\n"
    "+\n").getInitialState();
  const PieceEval ev(state);
  int ret=ev.computeValWithMove<SimpleState,BLACK>
    (state, Move(Position(5,6),Position(5,5),PAWN,PTYPE_EMPTY,false,BLACK));
  CPPUNIT_ASSERT_EQUAL(-Pawn*4, ret);
}

void PieceEvalTest::testCannotTake()
{
  CPPUNIT_ASSERT_EQUAL(0, Ptype_Eval_Table.value(PTYPE_EMPTY));
  NumEffectState state(CsaString(
			 "P1 *  *  *  *  *  *  *  *  * \n"
			 "P2 *  * +RY-KI-OU *  *  *  * \n"
			 "P3 *  *  *  *  *  *  *  *  * \n"
			 "P4 *  *  *  *  *  *  *  *  * \n"
			 "P5 *  *  *  *  *  *  *  *  * \n"
			 "P6 *  *  *  *  *  *  *  *  * \n"
			 "P7 *  *  *  *  *  *  *  *  * \n"
			 "P8 *  *  *  *  *  *  *  *  * \n"
			 "P9+OU *  *  *  *  *  *  *  * \n"
			 "P+00KI\n"
			 "P-00AL\n"
			 "+\n").getInitialState());
  {
    Move m = Move(Position(6,3),GOLD,BLACK);
    const int ret
      = PieceEval::computeDiffAfterMoveForRP(state, m);
    // ϼʤ
    CPPUNIT_ASSERT_EQUAL(0, ret);
  }
}

void PieceEvalTest::testComputeDiffAfterMoveForRP()
{
  NumEffectState state(CsaString(
			 "P1-KY-KE-GI-KI *  * -HI-KE * \n"
			 "P2 * -HI *  * -KA-OU *  *  * \n"
			 "P3-FU * -FU-FU-FU * -GI-FU-KY\n"
			 "P4 * +KE *  *  * -FU *  * -FU\n"
			 "P5 * -FU+KE+KA *  *  *  * +FU\n"
			 "P6 *  * +FU *  *  * +FU *  * \n"
			 "P7+FU+FU * +FU+FU+FU *  *  * \n"
			 "P8 * +GI+KI *  *  * +GI *  * \n"
			 "P9+KY *  *  * +OU+KI *  * +KY\n"
			 "P+00FU\n"
			 "P+00FU\n"
			 "P+00KI\n"
			 "+\n").getInitialState());
  {
    Move m = Move(Position(1,5),Position(1,4),PAWN,PAWN,false,BLACK);
    const int ret
      = PieceEval::computeDiffAfterMoveForRP(state, m);
    // ꤬ؤʤΤ
    CPPUNIT_ASSERT_EQUAL(Pawn*2, ret);
  }
  {
    Move m = Move(Position(8,4),Position(7,2),PKNIGHT,PTYPE_EMPTY,true,BLACK);
    const int ret
      = PieceEval::computeDiffAfterMoveForRP(state, m);
    // Τ
    CPPUNIT_ASSERT_EQUAL(-Knight*2, ret);
  }
  state.changeTurn();
  {
    Move m = Move(Position(8,5),Position(8,6),PAWN,PTYPE_EMPTY,false,WHITE);
    const int ret
      = PieceEval::computeDiffAfterMoveForRP(state, m);
    // Τ
    CPPUNIT_ASSERT_EQUAL(-Pawn*2, ret);
  }
  {
    Move m = Move(Position(8,2),Position(8,4),ROOK,KNIGHT,false,WHITE);
    const int ret
      = PieceEval::computeDiffAfterMoveForRP(state, m);
    // 
    CPPUNIT_ASSERT_EQUAL(Knight*2, ret);
  }
}

void PieceEvalTest::testComputeDiffAfterMoveForRPBug030910()
{
  NumEffectState state(CsaString(
			 "P1 *  *  *  *  *  * -HI-KE-KY\n"
			 "P2 *  *  * +RY+UM-GI-KI-OU * \n"
			 "P3+TO *  *  * -FU * -GI *  * \n"
			 "P4-FU *  * -FU * -GI+KY-FU * \n"
			 "P5 * -FU+FU-KI * -FU-FU *  * \n"
			 "P6+FU+FU *  * +KY * +FU * -FU\n"
			 "P7 *  *  *  * +FU+FU * +FU * \n"
			 "P8 *  *  *  * +KI * +GI+OU+FU\n"
			 "P9 *  *  *  *  * +KI * +KE+KY\n"
			 "P-00FU\n"
			 "P-00KE\n"
			 "P-00KE\n"
			 "P+00KA\n"
			 "-\n").getInitialState());
  {
    const Position p86 = Position(8,6);
    Move m = Move(Position(8,5),p86,PAWN,PAWN,false,WHITE);
    // ̤
    const int ret = PieceEval::computeDiffAfterMoveForRP(state, m);
    CPPUNIT_ASSERT_EQUAL(Pawn*2, ret);
    const int rets = 
      PieceEval::computeDiffAfterMoveForRP((const PawnMaskState&)state, m);
    CPPUNIT_ASSERT_EQUAL(Pawn*2, rets);

  }
  {
    const Position p36 = Position(3,6);
    Move m = Move(Position(3,5),p36,PAWN,PAWN,false,WHITE);
    // EffectUtil::showEffect(state, p36, std::cerr);
    CPPUNIT_ASSERT_EQUAL(-Pawn*2, 
			 PieceEval::diffWithMove(state, m));

    const int ret = PieceEval::computeDiffAfterMoveForRP(state, m);
    // ʤΤϤ
    CPPUNIT_ASSERT_EQUAL(Pawn*2, ret);
    const int rets = 
      PieceEval::computeDiffAfterMoveForRP((const PawnMaskState&)state, m);
    CPPUNIT_ASSERT_EQUAL(Pawn*2, rets);
  }
}

void PieceEvalTest::testComputeDiffAfterTakeBack()
{
  NumEffectState state(CsaString(
			 "P1-KY-KE *  *  *  * -HI-KE * \n"
			 "P2 * -HI *  * -KA-OU *  *  * \n"
			 "P3-FU-KI-FU-FU-FU * -GI-FU-KY\n"
			 "P4 * +KE *  *  * -FU * +FU-FU\n"
			 "P5 *  * +KE+KA * -GI *  * +FU\n"
			 "P6 * -FU+FU *  *  * +FU *  * \n"
			 "P7+FU+FU * +FU+FU+FU+GI *  * \n"
			 "P8 * +GI+KI *  *  *  *  *  * \n"
			 "P9+KY *  *  * +OU+KI *  * +KY\n"
			 "P+00FU\n"
			 "P+00KI\n"
			 "+\n").getInitialState());
  {
    const Position p44 = Position(4,4);
    // EffectUtil::showEffect(state, p44, std::cerr);
    // ʤ
    const int ret
      = PieceEval::computeDiffAfterTakeBackLimited(WHITE, state, p44);
    CPPUNIT_ASSERT_EQUAL(0, ret);
  }
  {
    const int ret
      = PieceEval::computeDiffAfterTakeBackLimited(WHITE, state, Position(8,6));
    CPPUNIT_ASSERT_EQUAL(Pawn*2, ret);
  }
  {
    // Ȥʤ
    const int ret
      = PieceEval::computeDiffAfterTakeBackLimited
      (WHITE, state, Position(2,1));
    CPPUNIT_ASSERT_EQUAL(0, ret);
  }
  {
    // Ȥʤ
    const int ret
      = PieceEval::computeDiffAfterTakeBackLimited(BLACK, state, Position(3,6));
    CPPUNIT_ASSERT_EQUAL(0, ret);
  }
  {
    // Ȥʤ
    const int ret
      = PieceEval::computeDiffAfterTakeBackLimited(WHITE, state, Position(6,3));
    CPPUNIT_ASSERT_EQUAL(0, ret);
  }
  {
    // Ȥ ⤬ + 
    const int ret
      = PieceEval::computeDiffAfterTakeBackLimited(WHITE, state, Position(8,3));
    CPPUNIT_ASSERT_EQUAL(Gold*2+Pknight-Knight, ret);
  }
  {
    const int ret
      = PieceEval::computeDiffAfterTakeBackLimited(BLACK, state, Position(2,4));
    CPPUNIT_ASSERT_EQUAL(-Pawn*2, ret);
  }
  {
    const Position p15 = Position(1,5);
    // EffectUtil::showEffect(state, p15, std::cerr);
    // 11 ι֤ӤʤΤ -200 ǤϤʤ 0 ˤʤ
    const int ret
      = PieceEval::computeDiffAfterTakeBackLimited(BLACK, state, p15);
    CPPUNIT_ASSERT_EQUAL(0, ret);
  }

  NumEffectState state2(CsaString(
			  "P1 *  *  *  *  *  * -HI-KE-KY\n"
			  "P2 *  *  * +RY+UM-GI-KI-OU * \n"
			  "P3+TO *  *  * -FU * -GI *  * \n"
			  "P4-FU *  * -FU * -GI+KY-FU * \n"
			  "P5 * -FU+FU-KI * -FU *  *  * \n"
			  "P6+FU *  *  * +KY * -FU * -FU\n"
			  "P7 *  *  *  * +FU+FU * +FU * \n"
			  "P8 *  *  *  * +KI * +GI+OU+FU\n"
			  "P9 *  *  *  *  * +KI * +KE+KY\n"
			  "P+00FU\n"
			  "P-00FU\n"
			  "P-00FU\n"
			  "P-00KE\n"
			  "P-00KE\n"
			  "P+00KA\n"
			  "+\n").getInitialState());
  {
    const Position p36 = Position(3,6);
    EffectUtil::showEffect(state2, p36, std::cerr);
    // ʤ
    const int ret
      = PieceEval::computeDiffAfterTakeBackLimited(WHITE, state2, p36);
    CPPUNIT_ASSERT_EQUAL(0, ret);
  }
}

int computeDiffAfterMoveMulti(const char *stateStr, Move m)
{
  NumEffectState state(CsaString(stateStr).getInitialState());
  return PieceEval::computeDiffAfterMoveMulti(state.getTurn(), state, m);
}

void PieceEvalTest::testComputeDiffAfterMoveMulti()
{
  const char *state1 = 
    "P1+NY * +TO *  *  * -OU-KE-KY\n"
    "P2 *  *  *  *  * -GI-KI *  *\n"
    "P3 * +RY *  * +UM * -KI-FU-FU\n"
    "P4 *  * +FU-FU *  *  *  *  *\n"
    "P5 *  * -KE *  *  *  * +FU *\n"
    "P6-KE *  * +FU+GI-FU *  * +FU\n"
    "P7 *  * -UM *  *  *  *  *  *\n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 * +OU * -GI *  *  *  * -NG\n"
    "P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU00FU\n"
    "P-00KI00KY00FU\n"
    "P-00AL\n"
    "+\n";
  const int test1 = computeDiffAfterMoveMulti
    (state1, 
     Move(Position(7,1),Position(6,1),PPAWN,PTYPE_EMPTY,false,BLACK));
  // 53ϤȤ
  CPPUNIT_ASSERT_EQUAL(- PtypeEvalTraits<PBISHOP>::val
		       - PtypeEvalTraits<BISHOP>::val
		       + PtypeEvalTraits<SILVER>::val*2,
		       test1);
  const int test2 = computeDiffAfterMoveMulti
    (state1, 
     Move(Position(8,3),Position(8,8),PROOK,PTYPE_EMPTY,false,BLACK));
  // ζȤ
  CPPUNIT_ASSERT_EQUAL(- PtypeEvalTraits<PROOK>::val
		       - PtypeEvalTraits<ROOK>::val
		       - PtypeEvalTraits<PKNIGHT>::val
		       + PtypeEvalTraits<KNIGHT>::val,
		       test2);
  const int test3 = computeDiffAfterMoveMulti
    (state1, 
     Move(Position(5,3),Position(6,3),PBISHOP,PTYPE_EMPTY,false,BLACK));
  // 66뤬Ϥϼʤ
  CPPUNIT_ASSERT_EQUAL(- PtypeEvalTraits<PAWN>::val*2, test3);
  const int test4 = computeDiffAfterMoveMulti
    (state1, 
     Move(Position(5,3),Position(6,4),PBISHOP,PAWN,false,BLACK));
  // 6466äƺ0
  CPPUNIT_ASSERT_EQUAL(0, test4);

  const char *state5 = 
    "P1+NY * +TO *  *  * -OU-KE-KY\n"
    "P2 *  *  *  *  * -GI-KI *  *\n"
    "P3 * +RY * +UM *  * -KI-FU-FU\n"
    "P4 *  * +FU-FU *  *  *  *  *\n"
    "P5 *  * -KE-GI+FU *  * +FU *\n"
    "P6-KE *  * +FU+GI-FU *  * +FU\n"
    "P7 *  * -UM *  *  *  *  *  *\n"
    "P8 *  * +KI *  *  *  *  *  * \n"
    "P9 * +OU *  *  *  *  *  * -NG\n"
    "P+00HI00KE00KY00FU00FU00FU00FU00FU00FU00FU\n"
    "P-00KI00KY00FU\n"
    "P-00AL\n"
    "-\n";
  const int test5 = computeDiffAfterMoveMulti
    (state5, 
     Move(Position(3,1),Position(2,2),KING,PTYPE_EMPTY,false,WHITE));
  // 77ϤȤ
  CPPUNIT_ASSERT_EQUAL(PtypeEvalTraits<PBISHOP>::val
		       + PtypeEvalTraits<BISHOP>::val,
		       test5);

  const int test6 = computeDiffAfterMoveMulti
    (state5, 
     Move(Position(7,7),Position(6,6),PBISHOP,PAWN,false,WHITE));
  // 6466äƺ0
  CPPUNIT_ASSERT_EQUAL(0, test6);

  const int test7 = computeDiffAfterMoveMulti
    (state5, 
     Move(Position(7,7),Position(5,9),PBISHOP,PTYPE_EMPTY,false,WHITE));
  // 65ζ
  CPPUNIT_ASSERT_EQUAL(PtypeEvalTraits<SILVER>::val*2, test7);
}

void PieceEvalTest::testComputeDiffAfterTakeBackMulti()
{
  const NumEffectState state1(
    CsaString(
      "P1+NY * +TO *  *  * -OU-KE-KY\n"
      "P2 *  *  *  *  * -GI-KI *  *\n"
      "P3 * +RY *  * +UM * -KI-FU-FU\n"
      "P4 *  * +FU-FU *  *  *  *  *\n"
      "P5 * -KE *  * +FU *  * +FU *\n"
      "P6-KE *  *  * +GI-FU *  * +FU\n"
      "P7 *  * -UM+FU *  *  *  *  *\n"
      "P8 *  *  *  *  *  *  *  *  * \n"
      "P9 * +OU * -GI *  *  *  * -NG\n"
      "P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU00FU\n"
      "P-00KI00KY00FU\n"
      "P-00AL\n"
      "+\n").getInitialState());
  const int test1 = PieceEval::
    computeDiffAfterTakeBackMulti<NumEffectState,BLACK>(state1);
  // 53ϤȤ
  CPPUNIT_ASSERT_EQUAL(- PtypeEvalTraits<PBISHOP>::val
		       - PtypeEvalTraits<BISHOP>::val
		       + PtypeEvalTraits<SILVER>::val*2,
		       test1);
  const int test2 = PieceEval::
    computeDiffAfterTakeBackMulti<NumEffectState,WHITE>(state1);
  // 85ˤ
  CPPUNIT_ASSERT_EQUAL(PtypeEvalTraits<KNIGHT>::val*2,
		       test2);
}

static void testIsEvalValueConsistent(std::string const& fileName){
  Record rec=CsaFile(fileName).getRecord();
  NumEffectState state(rec.getInitialState());
  const vector<osl::Move> moves=rec.getMoves();
  PieceEval pb(state);
  for(unsigned int i=0;i<moves.size();i++)
  {
    const Move move=moves[i];
    CPPUNIT_ASSERT_EQUAL(state.getTurn(), move.player());
    const int valWithMove = pb.computeValWithMove(state, move);
    const int valWithMoveMulti
      = pb.value() 
      + PieceEval::computeDiffAfterMoveMulti(move.player(), state, move);
    ApplyMoveOfTurn::doMove(pb, state, move);
    ApplyMoveOfTurn::doMove(state, move);
    
    const int valAfterMove = pb.computeValWithTakeBackLimited
      (move.player(), state, move.to());
    const int valAfterMoveMulti = pb.computeValWithTakeBackMulti
      (move.player(), state);
    if ((! isShortTest)
	&& (valWithMoveMulti != valAfterMoveMulti))
    {
      static int shown=0;
      if (++shown < 2)
      {
	std::cerr << valWithMove << " " << valAfterMove << " "
		  << valWithMoveMulti << " " << valAfterMoveMulti << "\n";
	std::cerr << move << "\n" << state << "\n";
      }
    }
    if (valWithMove != valAfterMove)
    {
	std::cerr << valWithMove << " " << valAfterMove << " "
		  << valWithMoveMulti << " " << valAfterMoveMulti << "\n";
	std::cerr << move << "\n" << state << "\n";
    }
    CPPUNIT_ASSERT_EQUAL(valWithMove, valAfterMove);
    // ޤĹդꡤӤꤷ˰פʤ
    // CPPUNIT_ASSERT_EQUAL(valWithMoveMulti, valAfterMoveMulti);
  }
}

void PieceEvalTest::testIsEvalValueConsistent()
{
  std::ifstream ifs(CSAFILES "/FILES");
  CPPUNIT_ASSERT(ifs);
  std::string fileName;
  for(int i=0;i<100 && (ifs >> fileName); i++){
    if(fileName == "") 
      break;
    ::testIsEvalValueConsistent(CSAFILES "/" + fileName);
  }
}

void PieceEvalTest::testConsistent()
{
  consistencyTest<PieceEval>();
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
