#include "osl/ntesuki/idNtesukiResult.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <string>
#include <sstream>

using namespace osl;
using namespace osl::ntesuki;

class IdNtesukiResultTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(IdNtesukiResultTest);
  CPPUNIT_TEST(testGenerate);
  CPPUNIT_TEST(testEqual);
  CPPUNIT_TEST(testNotEqual);
  CPPUNIT_TEST(testIsUnknown);
  CPPUNIT_TEST(testGetDepth);
  CPPUNIT_TEST(testOutput);
  CPPUNIT_TEST_SUITE_END();

public:
  void testGenerate()
  {
    IdNtesukiResult nr;
    nr = NTESUKI_YES;
    nr = NTESUKI_NO;
    nr = newIdNtesukiUnknown(1);
  }

  void testEqual()
  {
    CPPUNIT_ASSERT(NTESUKI_YES == NTESUKI_YES);
    CPPUNIT_ASSERT(NTESUKI_NO == NTESUKI_NO);
    CPPUNIT_ASSERT(newIdNtesukiUnknown(0) == newIdNtesukiUnknown(0));
    CPPUNIT_ASSERT(newIdNtesukiUnknown(1) == newIdNtesukiUnknown(1));
    CPPUNIT_ASSERT(newIdNtesukiUnknown(10) == newIdNtesukiUnknown(10));
  }

  void testNotEqual()
  {
    CPPUNIT_ASSERT(NTESUKI_YES != NTESUKI_NO);
    CPPUNIT_ASSERT(NTESUKI_YES != newIdNtesukiUnknown(0));
    CPPUNIT_ASSERT(NTESUKI_YES != newIdNtesukiUnknown(1));
    CPPUNIT_ASSERT(NTESUKI_YES != newIdNtesukiUnknown(10));
    CPPUNIT_ASSERT(NTESUKI_NO != newIdNtesukiUnknown(0));
    CPPUNIT_ASSERT(NTESUKI_NO != newIdNtesukiUnknown(1));
    CPPUNIT_ASSERT(NTESUKI_NO != newIdNtesukiUnknown(10));
    CPPUNIT_ASSERT(newIdNtesukiUnknown(0) != newIdNtesukiUnknown(1));
    CPPUNIT_ASSERT(newIdNtesukiUnknown(0) != newIdNtesukiUnknown(10));
    CPPUNIT_ASSERT(newIdNtesukiUnknown(1) != newIdNtesukiUnknown(10));
  }

  void testIsUnknown()
  {
    CPPUNIT_ASSERT(!isUnknown(NTESUKI_YES));
    CPPUNIT_ASSERT(!isUnknown(NTESUKI_NO));
    CPPUNIT_ASSERT(isUnknown(newIdNtesukiUnknown(0)));
    CPPUNIT_ASSERT(isUnknown(newIdNtesukiUnknown(1)));
    CPPUNIT_ASSERT(isUnknown(newIdNtesukiUnknown(10)));
  }

  void testGetDepth()
  {
    CPPUNIT_ASSERT_EQUAL(0, getDepth(newIdNtesukiUnknown(0)));
    CPPUNIT_ASSERT_EQUAL(1, getDepth(newIdNtesukiUnknown(1)));
    CPPUNIT_ASSERT_EQUAL(10, getDepth(newIdNtesukiUnknown(10)));
    CPPUNIT_ASSERT_EQUAL(100, getDepth(newIdNtesukiUnknown(100)));
  }

  void testOutput1(std::string correct, IdNtesukiResult nr)
  {
    std::ostringstream oss;

    oss << nr;
    CPPUNIT_ASSERT_EQUAL(correct, oss.str());
  }
  
  void testOutput()
  {
    testOutput1("NTESUKI_YES", NTESUKI_YES);
    testOutput1("NTESUKI_NO", NTESUKI_NO);
    testOutput1("NTESUKI_UNKNOWN(0)", newIdNtesukiUnknown(0));
    testOutput1("NTESUKI_UNKNOWN(10)", newIdNtesukiUnknown(10));
    testOutput1("NTESUKI_UNKNOWN(100)", newIdNtesukiUnknown(100));
  }
};

CPPUNIT_TEST_SUITE_REGISTRATION(IdNtesukiResultTest);

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
