/* idntesuki.cc
 */
#include "osl/ntesuki/idntesuki.h"
#include "osl/state/hashEffectState.h"
#include "osl/record/csaString.h"
#include "osl/ntesuki/ntesukiMoveGenerator.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

#include <iostream>


class idNtesukiTest : public CppUnit::TestFixture {
  CPPUNIT_TEST_SUITE(idNtesukiTest);

  CPPUNIT_TEST(test0tesuki0);
  CPPUNIT_TEST(test0tesuki1);
  CPPUNIT_TEST(test1tesuki0);
  CPPUNIT_TEST(test1tesuki1);
  CPPUNIT_TEST(test2tesuki0);
  CPPUNIT_TEST(test2tesuki1);
  CPPUNIT_TEST(test3tesuki0);
  CPPUNIT_TEST(test3tesuki1);
  //CPPUNIT_TEST(test2tesukiWithEscape);

  CPPUNIT_TEST_SUITE_END();
public:
  void test0tesuki0();
  void test0tesuki1();
  void test1tesuki0();
  void test1tesuki1();
  void test2tesuki0();
  void test2tesuki1();
  void test3tesuki0();
  void test3tesuki1();
  //void test2tesukiWithEscape();
};
CPPUNIT_TEST_SUITE_REGISTRATION(idNtesukiTest);

using namespace osl;
using namespace osl::ntesuki;
extern bool isShortTest;
const size_t limit = 400000;

typedef ntesuki::GetMultipleAttackMoves attack_t;
typedef ntesuki::GetNoMoves defense_t;

void idNtesukiTest::test0tesuki0()
{
  SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  * +FU * \n"
    "P4 *  *  *  *  *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  *  *  * \n"
    "P7 *  *  *  *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  * +OU *  *  *  *  *  * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
  HashEffectState hState(state);
  IdNtesukiSearcher<HashEffectState, attack_t, defense_t> searcher(hState, limit, !isShortTest);

  const int ntesuki_num = searcher.searchSlow (hState.getTurn());
  CPPUNIT_ASSERT_EQUAL(0, ntesuki_num);
}

void idNtesukiTest::test0tesuki1()
{
  SimpleState state=CsaString(
    "P1-KY-KE-GI-KI-OU-KI+UM-KE-KY\n"
    "P2 * -HI *  * +KA *  *  *  * \n"
    "P3-FU-FU-FU-FU-FU-FU * -FU-FU\n"
    "P4 *  *  *  *  *  * -FU *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  * +FU *  *  *  *  *  * \n"
    "P7+FU+FU * +FU+FU+FU+FU+FU+FU\n"
    "P8 *  *  *  *  *  *  * +HI * \n"
    "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
    "P+00GI\n"
    "+\n").getInitialState();
  HashEffectState hState(state);
  IdNtesukiSearcher<HashEffectState, attack_t, defense_t> searcher (hState, limit, !isShortTest);

  const int ntesuki_num = searcher.searchSlow (hState.getTurn());
  CPPUNIT_ASSERT_EQUAL(0, ntesuki_num);
}

void idNtesukiTest::test1tesuki0()
{
  SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  * +FU * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  *  *  * \n"
    "P7 *  *  *  *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  * +OU *  *  *  *  *  * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
  HashEffectState hState(state);
  IdNtesukiSearcher<HashEffectState, attack_t, defense_t> searcher (hState, limit, !isShortTest);

  const int ntesuki_num = searcher.searchSlow (hState.getTurn());
  CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
}

void idNtesukiTest::test1tesuki1()
{
  SimpleState state=CsaString(
    "P1-KY-KE-GI-KI-OU-KI+UM-KE-KY\n"
    "P2 * -HI *  *  *  *  *  *  * \n"
    "P3-FU-FU-FU-FU-FU-FU * -FU-FU\n"
    "P4 *  *  *  *  *  * -FU *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  * +FU *  *  *  *  *  * \n"
    "P7+FU+FU * +FU+FU+FU+FU+FU+FU\n"
    "P8 *  *  *  *  *  *  * +HI * \n"
    "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
    "P+00GI00KA\n"
    "+\n").getInitialState();
  HashEffectState hState(state);
  IdNtesukiSearcher<HashEffectState, attack_t, defense_t> searcher (hState, limit, !isShortTest);

  const int ntesuki_num = searcher.searchSlow (hState.getTurn());
  CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
}

void idNtesukiTest::test2tesuki0()
{
  SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  * +FU * \n"
    "P6 *  *  *  *  *  *  *  *  * \n"
    "P7 *  *  *  *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  * +OU *  *  *  *  *  * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
  HashEffectState hState(state);
  IdNtesukiSearcher<HashEffectState, attack_t, defense_t> searcher (hState, limit, !isShortTest);

  const int ntesuki_num = searcher.searchSlow (hState.getTurn());
  CPPUNIT_ASSERT_EQUAL(2, ntesuki_num);
}

void idNtesukiTest::test2tesuki1()
{
  SimpleState state=CsaString(
    "P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
    "P2 * -HI *  *  *  *  * +UM * \n"
    "P3-FU-FU-FU-FU-FU-FU * -FU-FU\n"
    "P4 *  *  *  *  *  * -FU *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  * +FU *  *  *  *  *  * \n"
    "P7+FU+FU * +FU+FU+FU+FU+FU+FU\n"
    "P8 *  *  *  *  *  *  * +HI * \n"
    "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
    "P+00KA\n"
    "+\n").getInitialState();
  HashEffectState hState(state);
  IdNtesukiSearcher<HashEffectState, attack_t, defense_t> searcher (hState, limit, !isShortTest);

  const int ntesuki_num = searcher.searchSlow (hState.getTurn());
  CPPUNIT_ASSERT_EQUAL(2, ntesuki_num);
}

void idNtesukiTest::test3tesuki0()
{
  SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  * +FU * \n"
    "P7 *  *  *  *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  * +OU *  *  *  *  *  * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
  HashEffectState hState(state);
  IdNtesukiSearcher<HashEffectState, attack_t, defense_t> searcher (hState, limit, !isShortTest);

  const int ntesuki_num = searcher.searchSlow (hState.getTurn());
  CPPUNIT_ASSERT_EQUAL(3, ntesuki_num);
}

void idNtesukiTest::test3tesuki1()
{
  SimpleState state=CsaString(
    "P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
    "P2 * -HI *  *  *  *  * -KA * \n"
    "P3-FU-FU-FU-FU-FU-FU * -FU-FU\n"
    "P4 *  *  *  *  *  * -FU *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  * +FU *  *  *  *  *  * \n"
    "P7+FU+FU * +FU+FU+FU+FU+FU+FU\n"
    "P8 * +KA *  *  *  *  * +HI * \n"
    "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
    "+\n").getInitialState();
  HashEffectState hState(state);
  IdNtesukiSearcher<HashEffectState, attack_t, defense_t> searcher (hState, limit, !isShortTest);

  const int ntesuki_num = searcher.searchSlow (hState.getTurn());
  CPPUNIT_ASSERT_EQUAL(3, ntesuki_num);
}
#if 0
void idNtesukiTest::test2tesukiWithEscape()
{
  if (isShortTest) return;
  SimpleState state=CsaString(
    "P1-KY-KE-GI-KI-OU * -GI-KE-KY\n"
    "P2 *  *  *  *  *  * -KI-KA * \n"
    "P3-FU * -FU-FU-FU-FU *  * -FU\n"
    "P4 *  *  *  *  *  * -FU+HI * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 * -HI+FU *  *  *  *  *  * \n"
    "P7+FU *  * +FU+FU+FU+FU * +FU\n"
    "P8 * +KA+KI *  *  *  *  *  * \n"
    "P9+KY+KE+GI * +OU+KI+GI+KE+KY\n"
    "P-00FU00FU\n"
    "P+00FU00FU\n"
    "+\n").getInitialState();
  HashEffectState hState(state);
  IdNtesukiSearcher<HashEffectState,
    attack_t,
    GenerateEscapeKing> searcher (hState, limit, !isShortTest);
  //defense_t> searcher (hState, limit, !isShortTest);

  const int ntesuki_num = searcher.searchSlow (hState.getTurn());
  CPPUNIT_ASSERT_EQUAL(ntesuki_num, 2);
}
#endif
/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
