#include "osl/ntesuki/ntesukiCheckmateReason.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace osl;
using namespace osl::ntesuki;

class NtesukiCheckmateReasonTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(NtesukiCheckmateReasonTest);
  CPPUNIT_TEST(testProof);
  CPPUNIT_TEST(testDisproof);
  CPPUNIT_TEST_SUITE_END();
public:
  void testProof();
  void testDisproof();
};

void NtesukiCheckmateReasonTest::
testProof()
{
  NtesukiCheckmateReason ntesukiCheckmateReason;

  HashKey key_empty;
  {
    HashKey key =
      key_empty.newHashWithMove(Move(Position(7,7), Position(7,6),
				     PAWN, PTYPE_EMPTY, false, BLACK));
    const int passLimit = 1;
    const bool inSimulation = false;

    ntesukiCheckmateReason.proofByChildCache (key, passLimit, inSimulation);
    CPPUNIT_ASSERT(ntesukiCheckmateReason.isProofByChildCache(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isProofByDoUndo(key));
    CPPUNIT_ASSERT_EQUAL(passLimit, ntesukiCheckmateReason.passLimit(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isInSimulation(key));
  }
  {
    const HashKey key =
      key_empty.newHashWithMove(Move(Position(8,7), Position(8,6),
				     PAWN, PTYPE_EMPTY, false, BLACK));
    const int passLimit = 2;
    const bool inSimulation = true;

    ntesukiCheckmateReason.proofByDoUndo (key, passLimit, inSimulation);
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isProofByChildCache(key));
    CPPUNIT_ASSERT(ntesukiCheckmateReason.isProofByDoUndo(key));
    CPPUNIT_ASSERT_EQUAL(passLimit, ntesukiCheckmateReason.passLimit(key));
    CPPUNIT_ASSERT(ntesukiCheckmateReason.isInSimulation(key));
  }
  
}

void NtesukiCheckmateReasonTest::
testDisproof()
{
  NtesukiCheckmateReason ntesukiCheckmateReason;

  HashKey key_empty;
  const int passLimit = 1;
  const bool inSimulation = false;

  {
    const HashKey key =
      key_empty.newHashWithMove(Move(Position(1,7), Position(1,6),
				     PAWN, PTYPE_EMPTY, false, BLACK));
    ntesukiCheckmateReason.disproofByChildCache (key, passLimit, inSimulation);
    CPPUNIT_ASSERT(ntesukiCheckmateReason.isDisproofByChildCache(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByPass(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByLoop(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByAttackBack(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByDoUndo(key));
  }

  {
    const HashKey key =
      key_empty.newHashWithMove(Move(Position(2,7), Position(2,6),
				     PAWN, PTYPE_EMPTY, false, BLACK));
    ntesukiCheckmateReason.disproofByPass (key, passLimit, inSimulation);
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByChildCache(key));
    CPPUNIT_ASSERT(ntesukiCheckmateReason.isDisproofByPass(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByLoop(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByAttackBack(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByDoUndo(key));
  }

  {
    const HashKey key =
      key_empty.newHashWithMove(Move(Position(3,7), Position(3,6),
				     PAWN, PTYPE_EMPTY, false, BLACK));
    ntesukiCheckmateReason.disproofByLoop (key, passLimit, inSimulation);
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByChildCache(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByPass(key));
    CPPUNIT_ASSERT(ntesukiCheckmateReason.isDisproofByLoop(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByAttackBack(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByDoUndo(key));
  }

  {
    const HashKey key =
      key_empty.newHashWithMove(Move(Position(4,7), Position(4,6),
				     PAWN, PTYPE_EMPTY, false, BLACK));
    ntesukiCheckmateReason.disproofByAttackBack (key, passLimit, inSimulation);
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByChildCache(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByPass(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByLoop(key));
    CPPUNIT_ASSERT(ntesukiCheckmateReason.isDisproofByAttackBack(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByDoUndo(key));
  }

  {
    const HashKey key =
      key_empty.newHashWithMove(Move(Position(5,7), Position(5,6),
				     PAWN, PTYPE_EMPTY, false, BLACK));
    ntesukiCheckmateReason.disproofByDoUndo (key, passLimit, inSimulation);
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByChildCache(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByPass(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByLoop(key));
    CPPUNIT_ASSERT(!ntesukiCheckmateReason.isDisproofByAttackBack(key));
    CPPUNIT_ASSERT(ntesukiCheckmateReason.isDisproofByDoUndo(key));
  }
}

CPPUNIT_TEST_SUITE_REGISTRATION(NtesukiCheckmateReasonTest);

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
