#include "osl/ntesuki/ntesukiMoveGenerator.h"
#include "osl/record/csaString.h"
#include "osl/record/csaRecord.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

#include <iostream>

using namespace osl;
using namespace osl::ntesuki;

class NtesukiMoveGeneratorTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(NtesukiMoveGeneratorTest);
  CPPUNIT_TEST(testGetAllAttackMoves);
  CPPUNIT_TEST(testGetAttackMoves);
  CPPUNIT_TEST(testGetMultipleAttackMoves);
  CPPUNIT_TEST(testGetAllDefenseMoves);
  CPPUNIT_TEST(testGetDefenseMoves);
  CPPUNIT_TEST_SUITE_END();
public:
  void testGetAllAttackMoves();
  void testGetAttackMoves();
  void testGetMultipleAttackMoves();
  void testGetAllDefenseMoves();
  void testGetDefenseMoves();
};

extern bool isShortTest;

//-----------------------------------------------------------------------------
// Attack
//-----------------------------------------------------------------------------

void NtesukiMoveGeneratorTest::
testGetAllAttackMoves()
{
  GetAllAttackMoves gam(!isShortTest);

  SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  * +FU * \n"
    "P7 *  *  *  *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  * +OU *  *  *  *  *  * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
  NumEffectState nState(state);

  {
    NtesukiMoveList moves;

    gam.generate<BLACK>(nState, moves);
    CPPUNIT_ASSERT(moves.size() == 84);
  }

}

void NtesukiMoveGeneratorTest::
testGetAttackMoves()
{
  GetAttackMoves gam(!isShortTest);
  {
    SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  * +FU * \n"
    "P7 *  *  *  *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  * +OU *  *  *  *  *  * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
    NumEffectState nState(state);
  
    {
      NtesukiMoveList moves;
      osl::Position lastTo = Position::STAND();
      
      gam.generate<BLACK>(nState, moves);
      CPPUNIT_ASSERT(moves.size() == 19);
    }
  }
  {
    SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  * +FU * \n"
    "P7 *  *  *  *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  * +OU *  *  *  * +HI * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
    NumEffectState nState(state);
    GetAttackMoves gam(!isShortTest);
    
    {
      NtesukiMoveList moves;
      osl::Position lastTo = Position::STAND();
      
      gam.generate<BLACK>(nState, moves);
      CPPUNIT_ASSERT(moves.size() == 28);
    }
  }
  {
    SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  * +FU * \n"
    "P7 *  *  *  *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  * +OU *  * -HI * +HI * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
    NumEffectState nState(state);
    GetAttackMoves gam(!isShortTest);
    
    {
      NtesukiMoveList moves;
      osl::Position lastTo = Position::STAND();
      
      gam.generate<BLACK>(nState, moves);
      CPPUNIT_ASSERT(moves.size() == 1);
    }
  }
  {
    SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  * +FU  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  *  *  * \n"
    "P7 *  *  *  *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  * +OU *  *  *  *  *  * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
    NumEffectState nState(state);
    GetAttackMoves gam(!isShortTest);
    
    NtesukiMoveList moves;
      
    gam.generate<BLACK>(nState, moves);
    CPPUNIT_ASSERT(moves.size() == 20);
  }
}

void NtesukiMoveGeneratorTest::
testGetMultipleAttackMoves()
{
  GetMultipleAttackMoves gam(!isShortTest);
  {
    SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  * +FU * \n"
    "P7 *  *  *  *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  * +OU *  *  *  *  *  * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
    NumEffectState nState(state);

    NtesukiMoveList moves;
      
    gam.generate<BLACK>(nState, moves);
    CPPUNIT_ASSERT(moves.size() == 41);
  }
}

//-----------------------------------------------------------------------------
// Defense
//-----------------------------------------------------------------------------
void NtesukiMoveGeneratorTest::
testGetAllDefenseMoves()
{
  SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  * +FU * \n"
    "P7 *  *  *  *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  * +OU *  *  *  *  *  * \n"
    "P+00KI\n"
    "P-00AL\n"
    "-\n").getInitialState();
  NumEffectState nState(state);
  GetAllDefenseMoves gdm(!isShortTest);

  NtesukiMoveList moves;
  gdm.generate<WHITE>(nState, moves, Position::STAND());
  CPPUNIT_ASSERT(moves.size() == 5 + 78 * 4 + 70 + 70 + 61);
}

void NtesukiMoveGeneratorTest::
testGetDefenseMoves()
{
  /*
   * -- ̤ư
   * -- ̤Τޤ 8˵˶Ǥ
   * -- ̤ƨƻ     
   * -- ̤Τޤ8˵ĤƤ(25˵ζ?)
   * -- ̤Τޤ 8˵()Ĥˤ defense
   * -- ľư
   */
  SimpleState state=CsaString(
			      "P1 *  *  *  *  *  *  * -OU * \n"
			      "P2 *  *  *  *  *  *  * -FU * \n"
			      "P3 *  *  *  *  *  *  *  *  * \n"
			      "P4 *  *  *  * -KI *  *  *  * \n"
			      "P5 *  *  *  *  * +KA * -KI * \n"
			      "P6 *  *  *  *  *  *  * +FU * \n"
			      "P7 *  *  *  *  *  *  *  *  * \n"
			      "P8 *  *  *  *  *  *  *  *  * \n"
			      "P9 *  * +OU *  *  *  *  *  * \n"
			      "P-00KI00GI\n"
			      "P+00AL\n"
			      "-\n").getInitialState();
  NumEffectState nState(state);
  GetDefenseMoves gdm(!isShortTest);

  NtesukiMoveList moves;
  gdm.generate<WHITE>(nState, moves, Position(2, 6));
  CPPUNIT_ASSERT_EQUAL( /* -- ̤ư (12ϼ) */
		       4 - 1 +
		        /* -- ̤Τޤ 8˵˶Ǥ */
		       7 * 2 +
		       /* -- ̤ƨƻ */
		       1 +
		       /* -- ̤Τޤ8˵ĤƤ(25˵ζ?) */
		       1 +
		       /* -- ̤Τޤ 8˵()Ĥˤ defense */
		       6 + 7 + 1 +
		       /* -- ľư */
		       1,
		       int(moves.size()));
}


CPPUNIT_TEST_SUITE_REGISTRATION(NtesukiMoveGeneratorTest);

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
