/* ntesukiSeacher.cc
 */
#include "osl/ntesuki/ntesukiSearcher.h"
#include "osl/state/hashEffectState.h"
#include "osl/record/csaString.h"
#include "osl/record/csaRecord.h"
#include "osl/move_generator/legalMoves.h"
#include "osl/move_generator/escape.h"
#include "osl/ntesuki/ntesukiMoveGenerator.h"
#include "osl/ntesuki/pdNtesukiTable.h"
#include "osl/checkmate/proofDisproof.h"
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

#include <iostream>

class dfpnNtesukiTest : public CppUnit::TestFixture {
  CPPUNIT_TEST_SUITE(dfpnNtesukiTest);
  CPPUNIT_TEST(test0tesuki0);
  CPPUNIT_TEST(test1tesuki0);

  CPPUNIT_TEST(test0tesuki1);
#if 0
  CPPUNIT_TEST(test1tesuki1);
#endif

  CPPUNIT_TEST(testRecognizeAttackBack0);
  CPPUNIT_TEST(testRecognizeAttackBack1);

  CPPUNIT_TEST(test1tesukiTest13);
  CPPUNIT_TEST(test1tesukiTest25);
  CPPUNIT_TEST(test1tesukiTest29);
  CPPUNIT_TEST(test1tesukiTest56);
  CPPUNIT_TEST(test1tesukiTest76);
  CPPUNIT_TEST(test1tesukiTest94);
#if 0
  CPPUNIT_TEST(test1tesukiTest01);
  CPPUNIT_TEST(test1tesukiTest22);
  CPPUNIT_TEST(test2tesuki0);
  CPPUNIT_TEST(test2tesuki1);
  CPPUNIT_TEST(test3tesuki0);
  CPPUNIT_TEST(test3tesuki1);
#endif

  CPPUNIT_TEST_SUITE_END();
public:
  void test0tesuki0();
  void test0tesuki1();
  void test1tesuki0();
  void test1tesuki1();
  void test2tesuki0();
  void test2tesuki1();
  void testRecognizeAttackBack0();
  void testRecognizeAttackBack1();
  void test1tesukiTest01();
  void test1tesukiTest22();
  void test1tesukiTest13();
  void test1tesukiTest25();
  void test1tesukiTest29();
  void test1tesukiTest56();
  void test1tesukiTest76();
  void test1tesukiTest94();
  void test3tesuki0();
  void test3tesuki1();
};
CPPUNIT_TEST_SUITE_REGISTRATION(dfpnNtesukiTest);

using namespace osl;
using namespace osl::ntesuki;
extern bool isShortTest;
const size_t TABLE_LIMIT = 1000000;
const size_t READ_LIMIT = 2000000;

typedef  GetMultipleAttackMoves more_attack_t;
typedef  NtesukiDefenseMoveGenerator no_defense_t;
typedef  GetAttackMoves attack_t;
typedef  GetDefenseMoves defense_t;

volatile bool stop_flag = false;

void dfpnNtesukiTest::test0tesuki0()
{
  SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  * +FU * \n"
    "P4 *  *  *  *  *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  *  *  * \n"
    "P7 *  *  *  *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  * +OU *  *  *  *  *  * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new no_defense_t());
  NumEffectState nState(state);
  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);

  const int ntesuki_num = searcher.searchSlow(nState.getTurn());
  CPPUNIT_ASSERT_EQUAL(0, ntesuki_num);
}

void dfpnNtesukiTest::test1tesuki0()
{
  SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  * +FU * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  *  *  * \n"
    "P7 * +FU+FU+FU *  *  *  *  * \n"
    "P8 * +GI+KI+GI *  *  *  *  * \n"
    "P9 * +KI+OU+KI *  *  *  *  * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
  NumEffectState nState(state);
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new no_defense_t());
  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);

  const int ntesuki_num = searcher.searchSlow(nState.getTurn());
  CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
}

void dfpnNtesukiTest::test2tesuki0()
{
  SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  * +FU * \n"
    "P6 *  *  *  *  *  *  *  *  * \n"
    "P7 * +FU+FU+FU *  *  *  *  * \n"
    "P8 * +GI+KI+GI *  *  *  *  * \n"
    "P9 * +KI+OU+KI *  *  *  *  * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new no_defense_t());
  NumEffectState nState(state);
  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);

  const int ntesuki_num = searcher.searchSlow(nState.getTurn());
  CPPUNIT_ASSERT_EQUAL(2, ntesuki_num);
}

void dfpnNtesukiTest::test0tesuki1()
{
  SimpleState state=CsaString(
    "P1-KY-KE-GI-KI-OU-KI+UM-KE-KY\n"
    "P2 * -HI *  * +KA *  *  *  * \n"
    "P3-FU-FU-FU-FU-FU-FU * -FU-FU\n"
    "P4 *  *  *  *  *  * -FU *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  * +FU *  *  *  *  *  * \n"
    "P7+FU+FU * +FU+FU+FU+FU+FU+FU\n"
    "P8 *  *  *  *  *  *  * +HI * \n"
    "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
    "P+00GI\n"
    "+\n").getInitialState();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new no_defense_t());
  NumEffectState nState(state);
  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag,!isShortTest);

  const int ntesuki_num = searcher.searchSlow(nState.getTurn());
  CPPUNIT_ASSERT_EQUAL(0, ntesuki_num);
}

void dfpnNtesukiTest::test1tesuki1()
{
  SimpleState state=CsaString(
    "P1-KY-KE-GI-KI-OU-KI+UM-KE-KY\n"
    "P2 * -HI *  *  *  *  *  *  * \n"
    "P3-FU-FU-FU-FU-FU-FU * -FU-FU\n"
    "P4 *  *  *  *  *  * -FU *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  * +FU *  *  *  *  *  * \n"
    "P7+FU+FU * +FU+FU+FU+FU+FU+FU\n"
    "P8 *  *  *  *  *  *  * +HI * \n"
    "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
    "P+00GI00KA\n"
    "+\n").getInitialState();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new no_defense_t());
  NumEffectState nState(state);
  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);

  const int ntesuki_num = searcher.searchSlow(nState.getTurn());
  CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
}


void dfpnNtesukiTest::test2tesuki1()
{
  SimpleState state=CsaString(
    "P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
    "P2 * -HI *  *  *  *  * +UM * \n"
    "P3-FU-FU-FU-FU-FU-FU * -FU-FU\n"
    "P4 *  *  *  *  *  * -FU *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  * +FU *  *  *  *  *  * \n"
    "P7+FU+FU * +FU+FU+FU+FU+FU+FU\n"
    "P8 *  *  *  *  *  *  * +HI * \n"
    "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
    "P+00KA\n"
    "+\n").getInitialState();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new no_defense_t());
  NumEffectState nState(state);
  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);

  const int ntesuki_num = searcher.searchSlow(nState.getTurn());
  CPPUNIT_ASSERT_EQUAL(2, ntesuki_num);
}

void dfpnNtesukiTest::testRecognizeAttackBack0()
{
  SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  * +FU * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  *  *  * \n"
    "P7 *  * -FU *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  * +OU *  *  *  *  *  * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new no_defense_t());
  NumEffectState nState(state);
  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);

  const int ntesuki_num = searcher.searchSlow(nState.getTurn());
  CPPUNIT_ASSERT_EQUAL(-1, ntesuki_num);
}

void dfpnNtesukiTest::testRecognizeAttackBack1()
{
  CsaFile file(OSL_DATADIR "grimbergen_testset/test12.csa");
  SimpleState state = file.getInitialState ();
  const osl::vector<Move> moves=file.getRecord().getMoves();
  NumEffectState nState(state);
  const HashKey key = HashKey::calcHash(nState).
    newHashWithMove(Move(Position(6,2), BISHOP, BLACK));

  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new defense_t());
  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);

  try
  {
    const int ntesuki_num = searcher.searchSlow(nState.getTurn());
    CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
  }
  catch (ReadNodeLimit err)
  {
  }
  const PdNtesukiTable& table = searcher.getTableSlow(BLACK);
  const PdNtesukiRecord *record = table.find(key);
  if (record)
  {
    CPPUNIT_ASSERT(!record->getPdNtesukiResult(1).isCheckmateSuccess());
  }
  else
  {
    std::cerr << "node not searched\n";
  }
}

void dfpnNtesukiTest::
test1tesukiTest01()
{
  if (isShortTest) return;
  CsaFile file(OSL_DATADIR "grimbergen_testset/test1.csa");
  SimpleState state = file.getInitialState ();
  const osl::vector<Move> moves=file.getRecord().getMoves();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new defense_t());
  NumEffectState nState(state);

  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
  CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
}

void dfpnNtesukiTest::
test1tesukiTest22()
{
  if (isShortTest) return;
  CsaFile file(OSL_DATADIR "grimbergen_testset/test22.csa");
  SimpleState state = file.getInitialState ();
  const osl::vector<Move> moves=file.getRecord().getMoves();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new defense_t());
  NumEffectState nState(state);

  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
  CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
}

void dfpnNtesukiTest::
test1tesukiTest13()
{
  if (isShortTest) return;
  CsaFile file(OSL_DATADIR "grimbergen_testset/test13.csa");
  SimpleState state = file.getInitialState ();
  const osl::vector<Move> moves=file.getRecord().getMoves();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new defense_t());
  NumEffectState nState(state);

  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
  CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
}

void dfpnNtesukiTest::
test1tesukiTest25()
{
  if (isShortTest) return;
  CsaFile file(OSL_DATADIR "grimbergen_testset/test25.csa");
  SimpleState state = file.getInitialState ();
  const osl::vector<Move> moves=file.getRecord().getMoves();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new defense_t());
  NumEffectState nState(state);

  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
  CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
}

void dfpnNtesukiTest::
test1tesukiTest29()
{
  if (isShortTest) return;
  CsaFile file(OSL_DATADIR "grimbergen_testset/test29.csa");
  SimpleState state = file.getInitialState ();
  const osl::vector<Move> moves=file.getRecord().getMoves();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new defense_t());
  NumEffectState nState(state);

  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
  CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
}

void dfpnNtesukiTest::
test1tesukiTest56()
{
  if (isShortTest) return;
  CsaFile file(OSL_DATADIR "grimbergen_testset/test56.csa");
  SimpleState state = file.getInitialState ();
  const osl::vector<Move> moves=file.getRecord().getMoves();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new defense_t());
  NumEffectState nState(state);

  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
  CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
}

void dfpnNtesukiTest::
test1tesukiTest76()
{
  if (isShortTest) return;
  CsaFile file(OSL_DATADIR "grimbergen_testset/test76.csa");
  SimpleState state = file.getInitialState ();
  const osl::vector<Move> moves=file.getRecord().getMoves();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new defense_t());
  NumEffectState nState(state);

  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
  CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
}

void dfpnNtesukiTest::
test1tesukiTest94()
{
  if (isShortTest) return;
  CsaFile file(OSL_DATADIR "grimbergen_testset/test94.csa");
  SimpleState state = file.getInitialState ();
  const osl::vector<Move> moves=file.getRecord().getMoves();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new defense_t());
  NumEffectState nState(state);

  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
  CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
}


void dfpnNtesukiTest::test3tesuki0()
{
  SimpleState state=CsaString(
    "P1 *  *  *  *  *  *  * -OU * \n"
    "P2 *  *  *  *  *  *  *  *  * \n"
    "P3 *  *  *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  * +FU * \n"
    "P7 * +FU+FU+FU *  *  *  *  * \n"
    "P8 * +GI+KI+GI *  *  *  *  * \n"
    "P9 * +KI+OU+KI *  *  *  *  * \n"
    "P+00KI\n"
    "P-00AL\n"
    "+\n").getInitialState();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new more_attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new defense_t());
  NumEffectState nState(state);
  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);

  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
  CPPUNIT_ASSERT_EQUAL(3, ntesuki_num);
}

void dfpnNtesukiTest::test3tesuki1()
{
  SimpleState state=CsaString(
			      "P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
			      "P2 * -HI *  *  *  *  * -KA * \n"
			      "P3-FU-FU-FU-FU-FU-FU * -FU-FU\n"
			      "P4 *  *  *  *  *  * -FU *  * \n"
			      "P5 *  *  *  *  *  *  *  *  * \n"
			      "P6 *  * +FU *  *  *  *  *  * \n"
			      "P7+FU+FU * +FU+FU+FU+FU+FU+FU\n"
			      "P8 * +KA *  *  *  *  * +HI * \n"
			      "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
			      "+\n").getInitialState();
  boost::scoped_ptr<osl::ntesuki::NtesukiAttackMoveGenerator>
    gam(new more_attack_t());
  boost::scoped_ptr<osl::ntesuki::NtesukiDefenseMoveGenerator>
    gdm(new defense_t());
  NumEffectState nState(state);
  NtesukiSearcher searcher(nState, gam.get(), gdm.get(), TABLE_LIMIT, &stop_flag, !isShortTest);

  const int ntesuki_num =
    searcher.searchSlow(nState.getTurn());

  CPPUNIT_ASSERT_EQUAL(3, ntesuki_num);

}
/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
