/* consistencyTest.h
 */
#ifndef TEST_PROGRESS_CONSISTENCYTEST_H
#define TEST_PROGRESS_CONSISTENCYTEST_H

#include "osl/state/numEffectState.h"
#include "osl/apply_move/applyMove.h"
#include "osl/record/csaString.h"
#include "osl/record/csaRecord.h"
#include <fstream>
#include <string>

#include <cppunit/extensions/HelperMacros.h>

template <class Progress>
void consistencyTest()
{
  using namespace osl;

  extern bool isShortTest;
  std::ifstream ifs(CSAFILES "/FILES");
  CPPUNIT_ASSERT(ifs);
  std::string file_name;
  for (int i=0;i<(isShortTest ? 10 : 200) && (ifs >> file_name) ; i++)
  {
    if (file_name == "") 
      break;
    file_name = CSAFILES "/" + file_name;

    const Record rec=CsaFile(file_name).getRecord();
    const vector<osl::Move> moves=rec.getMoves();

    NumEffectState state(rec.getInitialState());
    Progress progress(state);
    
    for (unsigned int i=0; i<moves.size(); i++)
    {
      const Move m = moves[i];
      ApplyMoveOfTurn::doMove(progress, state, m);
      ApplyMoveOfTurn::doMove(state, m);
      const Progress new_progress(state);
      CPPUNIT_ASSERT_EQUAL(new_progress.progress(BLACK), 
			   progress.progress(BLACK));
      CPPUNIT_ASSERT_EQUAL(new_progress.progress(WHITE), 
			   progress.progress(WHITE));
      CPPUNIT_ASSERT(new_progress.progress16().isValid());
      CPPUNIT_ASSERT(progress.progress16().isValid());
    }
  }
}


#endif /* TEST_PROGRESS_CONSISTENCYTEST_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
