#include "osl/record/opening/openingBook.h"
#include "osl/record/csaRecord.h"
#include "osl/record/compactBoard.h"
#include "osl/apply_move/applyMove.h"
#include "osl/oslConfig.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <iostream>

class OpeningBookTest : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE(OpeningBookTest);
  CPPUNIT_TEST(testOpen);
  CPPUNIT_TEST(testMove);
  CPPUNIT_TEST(testCompactBoard);
  CPPUNIT_TEST(testCompactBoardOrder);
  CPPUNIT_TEST_SUITE_END();
public:
  void testOpen();
  void testMove();
  void testCompactBoard();
  void testCompactBoardOrder();
};

CPPUNIT_TEST_SUITE_REGISTRATION(OpeningBookTest);

void OpeningBookTest::testOpen()
{
  osl::record::opening::WeightedBook book(osl::OslConfig::openingBook());
}

void OpeningBookTest::testMove()
{
  extern bool isShortTest;
  std::ifstream ifs(CSAFILES "/FILES");
  CPPUNIT_ASSERT(ifs);
  int max=100;
  if (isShortTest)
      max=10;
  std::string fileName;
  int i=0;
  while((ifs >> fileName) && (++i<max)) 
  {
    if (! isShortTest)
      std::cerr << fileName << " ";
    std::string fullFileName = CSAFILES "/";
    fullFileName += fileName;

    osl::Record rec=osl::CsaFile(fullFileName).getRecord();
    const osl::vector<osl::Move> moves=rec.getMoves();
    for (size_t i=0; i<moves.size(); ++i)
    {
      const osl::Move& m = moves[i];
      osl::record::opening::OMove om(m);
      CPPUNIT_ASSERT_EQUAL(m.from(), om.getFrom());
      CPPUNIT_ASSERT_EQUAL(m.to(), om.getTo());
      CPPUNIT_ASSERT_EQUAL(m.isPromote(), om.isPromote());
      CPPUNIT_ASSERT_EQUAL(m.capturePtype(), om.getCapturePtype());
      CPPUNIT_ASSERT_EQUAL(m.ptype(), om.getPtype());
      CPPUNIT_ASSERT_EQUAL(m.player(), om.getPlayer());
      CPPUNIT_ASSERT_EQUAL(m, om.operator osl::Move());
    }
  }
}

void OpeningBookTest::testCompactBoard()
{
  extern bool isShortTest;
  std::ifstream ifs(CSAFILES "/FILES");
  CPPUNIT_ASSERT(ifs);
  int max=100;
  if (isShortTest)
      max=10;
  std::string fileName;
  int i=0;
  while((ifs >> fileName) && (++i<max)) 
  {
    if (! isShortTest)
      std::cerr << fileName << " ";
    std::string fullFileName = CSAFILES "/";
    fullFileName += fileName;

    osl::Record rec=osl::CsaFile(fullFileName).getRecord();
    const osl::vector<osl::Move> moves=rec.getMoves();
    osl::SimpleState state(rec.getInitialState());
    // This doesn't test initial state itself but oh well...
    for (size_t i=0; i<moves.size(); ++i)
    {
      const osl::Move& m = moves[i];
      osl::ApplyMoveOfTurn::doMove(state, m);
      for (int p_index = 0; p_index < osl::Piece::SIZE; p_index++)
      {
	osl::Piece piece = state.getPieceOf(p_index);
	osl::record::OPiece opiece(piece);
	CPPUNIT_ASSERT_EQUAL(piece.position(), opiece.getPosition());
	CPPUNIT_ASSERT_EQUAL(piece.ptype(), opiece.getPtype());
	CPPUNIT_ASSERT_EQUAL(piece.owner(), opiece.getOwner());
      }
      osl::record::CompactBoard board(state);
      CPPUNIT_ASSERT_EQUAL(state, board.getState());

      std::ostringstream oss(std::ostringstream::out);
      osl::record::CompactBoard reconstructed_board;
      oss << board;
      std::istringstream iss(oss.str());
      iss >> reconstructed_board;
      CPPUNIT_ASSERT_EQUAL(board.getState(), reconstructed_board.getState());
    }
  }
}

void OpeningBookTest::testCompactBoardOrder()
{
  extern bool isShortTest;
  std::ifstream ifs(CSAFILES "/FILES");
  CPPUNIT_ASSERT(ifs);
  int max=100;
  if (isShortTest)
      max=10;
  std::string fileName;
  int i=0;
  while((ifs >> fileName) && (++i<max)) 
  {
    if (! isShortTest)
      std::cerr << fileName << " ";
    std::string fullFileName = CSAFILES "/";
    fullFileName += fileName;

    osl::Record rec=osl::CsaFile(fullFileName).getRecord();
    const osl::vector<osl::Move> moves=rec.getMoves();
    osl::SimpleState state(rec.getInitialState());
    // This doesn't test initial state itself but oh well...
    for (size_t i=0; i<moves.size(); ++i)
    {
      const osl::Move& m = moves[i];
      osl::ApplyMoveOfTurn::doMove(state, m);
      osl::record::CompactBoard board(state);

      std::ostringstream oss(std::ostringstream::out);
      oss << board;
      // test uniquness of CompactBoard representationg for given state.
      const std::string s = oss.str();
      char str[s.length() + 1];
      s.copy(str, s.length());
      str[s.length()] = '\0';
      // Swap pieces
      for (int i = 0; i < 1; i++)
      {
	for (int j = 0; j < 4; j++)
	{
	  char tmp = str[i * 4 + j];
	  str[i * 4 + j] = str[(40 - 1 - i) * 4 + j];
	  str[(40 - 1 - i) * 4 + j] = tmp;
	}
       }
      std::string board_string(str, s.length());
      std::istringstream iss(board_string);
      osl::record::CompactBoard tmp_board;
      iss >> tmp_board;
      // We need to do SimpleState -> CompactBoard convesion because
      // that's where sorting happens.
      osl::record::CompactBoard reconstructed_board(tmp_board.getState());
      std::ostringstream oss2(std::ostringstream::out);
      oss2 << reconstructed_board;
      CPPUNIT_ASSERT_EQUAL(oss.str(), oss2.str());
    }
  }
}
// ;;; local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
