/* historyTable.t.cc
 */
#include "osl/search/historyTable.h"
#include "osl/state/numEffectState.h"
#include "osl/record/csaString.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

namespace osl
{
  namespace search
  {
    class HistoryTableTest : public CppUnit::TestFixture 
    {
      CPPUNIT_TEST_SUITE(HistoryTableTest);
      CPPUNIT_TEST(testAdd);
      CPPUNIT_TEST_SUITE_END();
    public:
      void testAdd();
    };
  }
} // namespace osl

CPPUNIT_TEST_SUITE_REGISTRATION(osl::search::HistoryTableTest);

void osl::search::HistoryTableTest::testAdd()
{
  NumEffectState state(CsaString(
			 "P1 *  *  *  *  *  *  *  * -OU\n"
			 "P2 *  *  *  *  *  *  *  *  * \n"
			 "P3 *  *  *  *  *  *  *  *  * \n"
			 "P4 *  *  *  *  *  *  *  *  * \n"
			 "P5 *  *  *  *  *  *  *  *  * \n"
			 "P6 *  *  *  *  *  *  *  *  * \n"
			 "P7 *  *  *  *  *  *  *  *  * \n"
			 "P8 *  *  *  *  *  *  *  *  * \n"
			 "P9+OU *  *  *  *  *  *  *  * \n"
			 "P+00KI00GI00FU\n"
			 "P-00AL\n"
			 "+\n").getInitialState());
  HistoryTable table;
  const Move m23ki(Position(2,3), GOLD, BLACK);
  const Move m23gi(Position(2,3), SILVER, BLACK);
  const Move m23fu(Position(2,3), PAWN, BLACK);
  table.setMove(400, m23ki);

  MoveLogProbVector moves;
  table.getMoves(state, moves);
  CPPUNIT_ASSERT(moves.size() == 1);
  CPPUNIT_ASSERT_EQUAL(m23ki, moves[0].getMove());
  CPPUNIT_ASSERT(moves[0].getLogProb() <= 200);
  
  table.setMove(200, m23gi);
  table.setMove(100, m23fu);
  table.getMoves(state, moves);
  CPPUNIT_ASSERT(moves.size() == 3);
  CPPUNIT_ASSERT_EQUAL(m23ki, moves[0].getMove());
  CPPUNIT_ASSERT_EQUAL(m23gi, moves[1].getMove());
  CPPUNIT_ASSERT_EQUAL(m23fu, moves[2].getMove());
  CPPUNIT_ASSERT(moves[0].getLogProb() <= moves[1].getLogProb());
  CPPUNIT_ASSERT(moves[1].getLogProb() <= moves[2].getLogProb());

}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
