#include "mtdfdebug.h"
#include "problems.h"
#include "osl/search/searchRecorder.h"
#include "osl/search/simpleHashTable.h"
#include "osl/record/csaString.h"
#include "osl/state/hashEffectState.h"
#include "osl/category/fWMoveGen.h"
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace osl;
class MtdfIterativeDeepeningTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(MtdfIterativeDeepeningTest);
  CPPUNIT_TEST(testProblems1);
  CPPUNIT_TEST(testProblems2);
  CPPUNIT_TEST(testProblems3);
  CPPUNIT_TEST_SUITE_END();
  void testProblems(SearchRecorder& recorder,
		    const Problem *problems, int numProblems, int depth);
public:
  void testProblems1();
  void testProblems2();
  void testProblems3();
  bool verbose() const
  {
    extern bool isShortTest;
    return ! isShortTest;
  }
};

CPPUNIT_TEST_SUITE_REGISTRATION(MtdfIterativeDeepeningTest);

extern bool isShortTest;

void MtdfIterativeDeepeningTest::
testProblems(SearchRecorder& recorder,
	     const Problem *problems, int numProblems, int depth)
{
  for (int i=0; i<numProblems; ++i)
  {
    recorder.setLogMargin(100);
    HashEffectState eState((CsaString(problems[i].state).getInitialState()));
    SimpleHashTable table(100000,-4,!isShortTest);
    Mtdf::checkmate_t checkmate;
    Mtdf mtdf_searcher(eState, checkmate, table, recorder);

    recorder.stream() << "\nfile " << i << " limit " << depth << "\n";
    recorder.stream() << eState;
    mtdf_searcher.setInitialGuess();

    const Move bestMove = 
      mtdf_searcher.computeBestMoveIteratively(depth, 500, 500);
    CPPUNIT_ASSERT(problems[i].acceptable(bestMove));
  }
}

void MtdfIterativeDeepeningTest::testProblems1()
{
  osl::SearchRecorder recorder("MtdfIterativeDeepeningTest.testProblems500");
  testProblems(recorder, problems1, numProblems1, 500);
}
void MtdfIterativeDeepeningTest::testProblems2()
{
  osl::SearchRecorder recorder("MtdfIterativeDeepeningTest.testProblems1000");
  testProblems(recorder, problems1, numProblems1, 1000);
}
void MtdfIterativeDeepeningTest::testProblems3()
{
  osl::SearchRecorder recorder("MtdfIterativeDeepeningTest.testProblems1500");
  testProblems(recorder, problems1, numProblems1, 1500);
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
