#include "problems.h"
#include "osl/search/mtdf.h"

#include "osl/search/mtdf.tcc"
#include "osl/search/nullWindowSearch.tcc"

#include "osl/search/searchRecorder.h"
#include "osl/search/simpleHashTable.h"
#include "osl/search/analyzer/tableAnalyzer.h"
#include "osl/search/analyzer/dotAnalyzerAll.h"
#include "osl/category/standardMoveGenerator.h"
#include "osl/state/hashEffectState.h"
#include "osl/category/fWMoveGen.h"
#include "osl/record/csaString.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <fstream>
using namespace osl;

class MtdfRpIDTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(MtdfRpIDTest);
  CPPUNIT_TEST(testProblems1);
  CPPUNIT_TEST(testProblems2);
  CPPUNIT_TEST(testProblems3);
  CPPUNIT_TEST_SUITE_END();
  void testProblems(SearchRecorder& recorder,
		    const Problem *problems, int numProblems, int depth);
public:
  void testProblems1();
  void testProblems2();
  void testProblems3();
  bool verbose() const
  {
    extern bool isShortTest;
    return ! isShortTest;
  }
};

CPPUNIT_TEST_SUITE_REGISTRATION(MtdfRpIDTest);

typedef MTDF<PieceEval,StandardMoveGenerator,
	     SimpleHashTable,SearchRecorder,RealizationProbability> RpMtdf;

void MtdfRpIDTest::
testProblems(SearchRecorder& recorder,
	     const Problem *problems, int numProblems, int depth)
{
  for (int i=0; i<numProblems; ++i)
  {
    HashEffectState state((CsaString(problems[i].state).getInitialState()));
    SimpleHashTable table(100000,-4,verbose());
    RpMtdf::checkmate_t checkmate;
    RpMtdf mtdf_searcher(state, checkmate, table, recorder);
    recorder.stream() << "\nfile " << i << " limit " << depth << "\n";
    recorder.stream() << state;
    mtdf_searcher.setInitialGuess();
    const Move bestMove = mtdf_searcher.computeBestMoveIteratively(depth);

    if (! problems[i].acceptable(bestMove))
    {
      std::ofstream os("debug.dot");
      search::analyzer::DotAnalyzerAll analyzer(table, os);
      analyzer.analyze(state.getHash(), 1000);
    }

    CPPUNIT_ASSERT(problems[i].acceptable(bestMove));
  }
}

void MtdfRpIDTest::testProblems1()
{
  osl::SearchRecorder recorder("MtdfRpIDTest.testProblems500");
  recorder.setLogMargin(100);
  testProblems(recorder, problems1, numProblems1, 500);
}
void MtdfRpIDTest::testProblems2()
{
  osl::SearchRecorder recorder("MtdfRpIDTest.testProblems800");
  recorder.setLogMargin(100);
  testProblems(recorder, problems1, numProblems1, 800);
}
void MtdfRpIDTest::testProblems3()
{
  osl::SearchRecorder recorder("MtdfRpIDTest.testProblems1200");
  recorder.setLogMargin(100);
  testProblems(recorder, problems1, numProblems1, 1000);
  recorder.setLogMargin(100);
  testProblems(recorder, problems3, numProblems3, 1200);
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
